Package: pscl
Version: 1.5.9
Date: 2024-01-14
Title: Political Science Computational Laboratory
Authors@R: c(
  person("Simon", "Jackman",
         role = c("aut", "cre"),
         email = "simon.jackman@sydney.edu.au",
         comment = c(ORCID = "0000-0001-7421-4034")
         ),
  person("Alex", "Tahk", role = c("ctb"), comment = c(ORCID = "0000-0001-7895-9420")),
  person("Achim", "Zeileis", role = c("ctb"), comment = c(ORCID = "0000-0003-0918-3766")),
  person("Christina", "Maimone", role = c("ctb"), comment = c(ORCID = "0000-0002-0402-6297")),
  person("James", "Fearon", role = c("ctb")),
  person("Zoe", "Meers", role = c("ctb"), comment = c(ORCID = "0000-0001-8045-6531"))
  )
Imports: MASS, datasets, grDevices, graphics, stats, utils
Suggests: lattice, MCMCpack, car, lmtest, sandwich, zoo, coda, vcd,
        mvtnorm, mgcv
Description: Bayesian analysis of item-response theory (IRT) models,
  roll call analysis; computing highest density regions; 
  maximum likelihood estimation of zero-inflated and hurdle models for count data;
  goodness-of-fit measures for GLMs;
  data sets used in writing	and teaching; seats-votes curves.
LazyData: true
License: GPL-2
URL: https://github.com/atahk/pscl
RoxygenNote: 7.3.0
NeedsCompilation: yes
Packaged: 2024-01-16 06:19:34 UTC; jackman
Author: Simon Jackman [aut, cre] (<https://orcid.org/0000-0001-7421-4034>),
  Alex Tahk [ctb] (<https://orcid.org/0000-0001-7895-9420>),
  Achim Zeileis [ctb] (<https://orcid.org/0000-0003-0918-3766>),
  Christina Maimone [ctb] (<https://orcid.org/0000-0002-0402-6297>),
  James Fearon [ctb],
  Zoe Meers [ctb] (<https://orcid.org/0000-0001-8045-6531>)
Maintainer: Simon Jackman <simon.jackman@sydney.edu.au>
Repository: CRAN
Date/Publication: 2024-01-31 08:00:03 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-25 21:52:30 UTC; unix
Archs: pscl.so.dSYM

useDynLib("pscl")

export("hurdle", "hurdle.control", "hurdletest",
       "zeroinfl", "zeroinfl.control",
       "odTest",
       "predprob", "vuong", "ntable", "betaHPD")

export("densigamma", "pigamma", "qigamma", "rigamma", "igammaHDR")

export("computeMargins",
       "constrain.items", "constrain.legis",
       "convertCodes",
       "dropRollCall",
       "dropUnanimous", 
       "extractRollCallObject",
       "ideal", "idealToMCMC",
       "readKH",
       "rollcall", "summary.rollcall",
       "plot.predict.ideal",
       "plot.ideal",
       "postProcess",
       "simpi",
       "tracex",
       "vectorRepresentation")

export("seatsVotes", "plot.seatsVotes")

export("hitmiss", "pR2")

importFrom("MASS", "glm.nb", "polr")
importFrom("grDevices", "dev.interactive", "gray", "rainbow")
importFrom("graphics", "abline", "axis", "layout", "legend", "lines",
           "mtext", "pairs", "par", "plot", "points", "polygon", "rug",
           "text", "title")
importFrom("stats", ".getXlevels", "AIC", "binomial", "coef", "cor",
           "dbeta", "delete.response", "density", "dnbinom", "dpois",
           "family", "glm", "glm.fit", "lm", "loess", "logLik",
           "lm.wfit",
           "make.link", "model.frame", "model.matrix",
           "model.response", "model.weights", "na.omit", "na.pass",
           "nobs", "optim", "optimize", "pbeta", "pchisq", "pgamma",
           "pnbinom", "pnorm", "poisson", "ppois", "predict",
           "printCoefmat", "qbeta", "qchisq", "qgamma", "quantile",
           "residuals", "rgamma", "rnorm", "sd", "terms", "uniroot",
           "update")
importFrom("utils", "tail")

## methods for class zeroinfl
  S3method("print", "zeroinfl")
  S3method("print", "summary.zeroinfl")
  S3method("summary", "zeroinfl")
  S3method("coef", "zeroinfl")
  S3method("vcov", "zeroinfl")
  S3method("logLik", "zeroinfl")
  S3method("predict", "zeroinfl")
  S3method("residuals", "zeroinfl")
  S3method("fitted", "zeroinfl")
  S3method("predprob", "zeroinfl")
  S3method("terms", "zeroinfl")
  S3method("model.matrix", "zeroinfl")
  S3method("extractAIC", "zeroinfl")

## methods for class hurdle
  S3method("print", "hurdle")
  S3method("print", "summary.hurdle")
  S3method("summary", "hurdle")
  S3method("coef", "hurdle")
  S3method("vcov", "hurdle")
  S3method("logLik", "hurdle")
  S3method("predict", "hurdle")
  S3method("residuals", "hurdle")
  S3method("fitted", "hurdle")
  S3method("predprob", "hurdle")
  S3method("terms", "hurdle")
  S3method("model.matrix", "hurdle")
  S3method("extractAIC", "hurdle")

## methods for class ideal
  S3method("plot", "ideal")
  S3method("plot", "predict.ideal")
  S3method("predict", "ideal")
  S3method("predprob", "ideal")
  S3method("print", "ideal")
  S3method("print", "predict.ideal")
  S3method("print", "summary.ideal")
  S3method("summary", "ideal")

## methods for class rollcall
  S3method("dropUnanimous", "rollcall")
  S3method("print", "rollcall")
  S3method("print", "summary.rollcall")
  S3method("summary", "rollcall")

## misc methods
  S3method("dropUnanimous", "matrix")
  S3method("predprob", "glm")

## methods for class seatsVotes
   S3method("print", "seatsVotes")
   S3method("plot", "seatsVotes")
   S3method("summary", "seatsVotes")

## hitmiss methods
   S3method("hitmiss", "glm")
   S3method("hitmiss", "polr")
   S3method("hitmiss", "multinom")

## pR2 methods
   S3method("pR2", "default")

# Political Science Computational Laboratory

## Description

**pscl** is an R package providing classes, methods and test data for

- Bayesian analysis of roll call data and other binary, item-response type data (e.g., from surveys or educational testing).

- elementary Bayesian statistics

- maximum likelihood estimation of zero-inflated and hurdle models for count data

- utility functions

## Historical note

The Political Science Computational Laboratory was the name of Simon Jackman's research group at Stanford University's Department of Political Science (2002-2014), where this package was first developed.

The hurdle and count data models were extensively re-written and updated by Achim Zeileis and Christian Kleiber.  

## Installation

Most users should use latest stable release of the package, which can be installed from [CRAN](https://cran.r-project.org/) by running
```R
install.packages("pscl")
```

The development version can be installed directly from [GitHub](https://github.com/atahk/pscl) by running
```R
install.packages("devtools") ## if not already installed
library(devtools)
install_github("atahk/pscl")
```

betaHPD <- function(alpha,beta,p=.95,plot=FALSE,xlim=NULL,debug=FALSE){
  
  if(is.na(p) | is.nan(p) | p > 1 | p < 0)
    stop("p not between 0 and 1\n")
  
    if(alpha<=1 | beta <=1)
      stop("betaHPD only implemented for alpha and beta both > 1\n")

  ## initialize internal logical flags
  compute <- TRUE
  swap <- FALSE

  if(alpha==beta){
    if(debug)
      cat("symmetric case, alpha=",alpha,"beta=",beta,"\n")
    out <- qbeta((1 + c(-1,1)*p)/2,
                 alpha,beta)
    compute <- FALSE
  }
  if(alpha>beta){
    swap <- TRUE
    alphaStar <- beta
    betaStar <- alpha
  }
  else if(beta>alpha){
    swap <- FALSE
    alphaStar <- alpha
    betaStar <- beta
  }
  if(debug)
    cat("swap=",swap,"\n")
  
  func <- function(x0,alpha,beta){
    y0 <- dbeta(x0,alpha,beta)
    p0 <- pbeta(x0,alpha,beta)
    x1 <- qbeta(p0+p,alpha,beta)
    y1 <- dbeta(x1,alpha,beta)
    out <- abs(y0-y1)
    out
  }
  
  if(compute){
    foo <- try(optimize(f=func,alpha=alphaStar,beta=betaStar,
                        tol=.Machine$double.eps^(.6),
                        interval=c(.Machine$double.eps,
                          qbeta(1-p,
                                alphaStar,betaStar))))
    if(inherits(foo,"try-error")){
      warning("optimization in betaHPD failed\n")
      out <- rep(NA,2)
    }
    else{
      if(debug){
        cat("results of optimization:\n")
        print(foo)
      }
      out <- c(foo$minimum,
               qbeta(pbeta(foo$minimum,alphaStar,betaStar)+p,
                     alphaStar,betaStar)
               )
    }
    if(swap){
      out <- 1-out
      out <- sort(out)
      if(debug){
        cat("swapped back\n")
        print(out)
      }
    }
  }

  ## plotting
  if(plot & all(!is.na(out))){
    xseq <- NULL
    if(length(xlim)==2 & all(!is.na(xlim))){
      if(xlim[2]>xlim[1] & xlim[1] >= 0 & xlim[2] <= 1){
        xseq <- seq(xlim[1]+(.Machine$double.eps^(.25)),
                    xlim[2]+(.Machine$double.eps^(.25)),
                    length=1000)
      }
    }
    if(is.null(xseq))
      xseq <- seq(min(qbeta(.0001,alpha,beta),out[1]),
                  max(qbeta(.9999,alpha,beta),out[2]),
                  length=1000)
    
    plot(xseq,dbeta(xseq,alpha,beta),
         xlab=expression(theta),
         ylab="",
         axes=F,
         type="n")
    axis(1)

    ## get polygon for HDR
    dseq <- seq(out[1],out[2],length=250)
    fx <- dbeta(dseq,alpha,beta)
    polygon(x=c(out[1],dseq,rev(dseq)),
            y=c(0,fx,rep(0,250)),
            border=F,col=gray(.45))
    lines(xseq,dbeta(xseq,alpha,beta))
  }

  out
}

densigamma <- function(x,alpha,beta){
  if(alpha > 0 & beta > 0 & all(x>0))
    (beta^alpha)/gamma(alpha) * x^(-alpha-1) * exp(-beta/x)
  else
    stop("densigamma: invalid parameters\n")
}

pigamma <- function(q,alpha,beta){
  if(alpha > 0 & beta > 0 & all(q>0))
    1-pgamma(1/q,alpha,beta)
  else
    stop("pigamma: invalid parameters\n")
}

qigamma <- function(p,alpha,beta){
  if(alpha > 0 & beta > 0 & all(p>0) & all(p<1)){
    if((1-p)<=.Machine$double.eps){
      out <- Inf
    }
    else{
      out <- 1/qgamma(1-p,alpha,beta)
    }
  }
  else
    stop("qigamma: invalid parameters\n")
  return(out)
}

rigamma <- function(n,alpha,beta){
  if(alpha > 0 & beta > 0)
    1/rgamma(n=n,alpha,beta)
  else
    stop("rigamma: invalid parameters\n")
}

igammaHDR <- function(alpha,beta,content=.95,debug=FALSE){
  ok <- alpha>0 & beta>0 & content>0 & content<1
  if(!ok)
    stop("igammaHDR: invalid parameters\n")

  func <- function(x0,alpha,beta,content){
    y0 <- densigamma(x0,alpha,beta)
    p0 <- pigamma(x0,alpha,beta)
    p1 <- p0 + content - .Machine$double.eps
    if(p1<1){
      x1 <- qigamma(p0+content,alpha,beta)
      y1 <- densigamma(x1,alpha,beta)
      out <- y0-y1
    }
    else{
       if(debug)
         cat(paste("igammaHDR: upper bound too large",p1,"\n"))
       out <- NaN
     }
    out
  }
 
  tryContent <- content
  flag <- FALSE
  while(!flag){
    if(debug)
      cat(paste("igammaHPR: checking search bounds with content=",
                tryContent,
                "\n"))
    try <- rep(NA,2)
    eps <- .Machine$double.eps
    bounds <- c(eps,qigamma(1-tryContent-eps,alpha,beta))
    try[1] <- func(bounds[1],alpha,beta,content=content)
    try[2] <- func(bounds[2],alpha,beta,content=content)
    if(any(is.nan(try)))
      stop("igammaHPR failed with NaN in func\n")
    if(sign(try[1])!=sign(try[2]))
      flag <- TRUE
    else{
      if(debug){
        cat("igammaHPR: bad bounds\n")
        print(try)
        cat("\n")
      }
      tryContent <- tryContent + .01*(1-tryContent)
    }
  }

  if(debug)
    cat("igammaHPD: done refining search bounds...now optimizing")
  foo <- uniroot(f=func,
                 interval=bounds,
                 tol=1e-12,
                 alpha=alpha,
                 beta=beta,
                 content=content)$root
  if(debug)
    cat("...done\n")

  hpd <- c(foo,
           qigamma(pigamma(foo,alpha,beta)+content,
                   alpha,beta)
           )
  hpd
}

## given an object of class ideal
## recover the rollcall object used in model fitting
## after applying the dropList etc

extractRollCallObject <- function(object){
  if(!inherits(object,"ideal"))
    stop("extractRollCallObject only defined for objects of class ideal")

  rcObj <- eval(object$call$object)
  dropList <- eval(object$call$dropList)

  tmpObj <- dropRollCall(rcObj,dropList)
  tmpObj
}

## predict method for class ideal
predict.ideal <- function(object,
                          cutoff=0.5,
                          burnin=NULL,
                          ...) {
  if(!inherits(object, "ideal"))
    stop("predict.ideal only defined for objects of class ideal\n")
  
  if(is.null(object$beta)){
    cat("Beta values must have be stored in ideal object to make predictions")
    stop("try re-fitting model with store.item=TRUE.")
  }

  if(is.null(burnin))
    keep <- checkBurnIn(object,eval(object$call$burnin))
  else
    keep <- checkBurnIn(object,burnin)  ## check that start is valid

  ## get votes into shape for prediction from ideal
  cat(paste("predict.ideal: Working with rollcall object",
            object$call$object,
            "\n"))
  rcObj <- try(dropRollCall(eval(object$call$object),
                            eval(object$call$dropList)),
               silent=TRUE)
  if(inherits(rcObj,"try-error") | is.null(rcObj$votes)){
    cat(paste("The ideal object ",
               as.name(object),
              " was fitted using the\n",
              "rollcall object" ,
              as.name(object$call$object),
              " which can no longer be found,\n",
              "or does not have a votes matrix as one of its components.",
              sep=""))
    stop("Prediction can not proceed.")
  }
  
  if(checkVotes(rcObj))  ## codes ok, should be ok
    stop("bad votes in rollcall object, can't generate predictions")
  cat("\n")
  votes <- convertCodes(rcObj)       ## convert to 0, 1 and NAs 
  
  ## predictions at posterior means
  predprob <- matrix(NA, ncol=ncol(votes), nrow=nrow(votes))
  dimnames(predprob) <- dimnames(votes)
  pred <- predprob
  correct <- predprob
  if(!is.null(burnin)){
    cat("Computing posterior means using ideal object.\n")
    x1 <- matrix(apply(object$x[keep,],
                       c(2,3),
                       mean),
                 nrow=object$n,
                 ncol=object$d,
                 byrow=TRUE)
    x1 <- cbind(x1,-1)            ## negative intercept !!! SDJ 05/15/07
    b <- matrix(apply(object$beta[keep,-1],2,mean),
                nrow=object$m,ncol=object$d+1,byrow=TRUE)
  }
  else{
    cat("Using posterior means in ideal object.\n")
    x1 <- cbind(object$xbar,-1.0)   ## negative intercept !!! SDJ 01/22/07
    b <- object$betabar
  }
  mu <- tcrossprod(x1,b)        ## this should be n by (d+1) times (d+1) by m
  predprob <- pnorm(mu)
  pred <- predprob >= cutoff
  correct <- votes==pred
  correct[is.na(votes)] <- NA

  lp <- apply(correct,1,tally)*100   ## legislator-specific
  pp <- NULL                         ## by party
  party <- eval(object$call$object)$legis.data$party
  if(!is.null(party))
    pp <- tapply(lp,party,mean)
  
  out <- list(pred.probs=predprob,
              prediction=pred,
              correct=correct,
              legis.percent=lp,
              vote.percent=apply(correct,2,tally)*100,
              yea.percent=(sum(correct[votes==1],na.rm=T)/
                           sum(!is.na(correct[votes==1])))*100,
              nay.percent=(sum(correct[votes==0],na.rm=T)/
                           sum(!is.na(correct[votes==0])))*100,
              party.percent=pp,
              overall.percent=(sum(correct,na.rm=T)/sum(!is.na(correct)))*100,
              ideal=match.call()$object)
  class(out) <- "predict.ideal"
  out
}

tally <- function(x){
  sum(x,na.rm=T)/sum(!is.na(x))
}

print.predict.ideal <- function(x,digits=2,...) {
  cat(paste("Predictions using ideal object",
            x$ideal,
            "\n"))
  cat(paste(x$ideal,"uses rollcall object",eval(x$ideal)$call$object,"\n"))
  rcObj <- eval(x$ideal)$call$object
  printHeader(eval(rcObj))
            
  cat("Predictions calculated using posterior means for\n")
  cat("legislators ideal points and bill parameters\n\n")
  cat("Percent correctly predicted:\n")
  cat(paste("\tOverall:\t",round(x$overall.percent,digits),"%\n",sep=""))
  cat(paste("\tYeas:\t\t",round(x$yea.percent,digits),"%\n",sep=""))
  cat(paste("\tNays:\t\t",round(x$nay.percent,digits),"%\n\n",sep=""))
  cat("Percent Correctly Predicted by Legislator\n")
  mat <- round(as.matrix(x$legis.percent),digits)
  colnames(mat) <- "Percent"
  print(mat)
  if(!is.null(x$party.percent)) {
    cat("\nPercent Correctly Predicted by Legislator, Party Average\n")
    print(round(x$party.percent,digits))
  }
  mat <- round(as.matrix(x$vote.percent),digits)
  colnames(mat) <- "Percent"
  cat("\nPercent Correctly Predicted by Vote\n")
  print(mat)
  cat("\n")
  invisible(NULL)
}

plot.predict.ideal <- function(x,
                               type=c("legis","votes"),
                               ...){
  if(!inherits(x, "predict.ideal"))
    stop("plot.predict.ideal only defined for objects of class predict.ideal")

  localType <- match.arg(type)

  d <- eval(x$ideal)$d
  desc <- eval(eval(x$ideal)$call$object)$desc
  rcObj <- extractRollCallObject(eval(x$ideal))
  
  if(localType=="legis"){
    ## plot percent correctly predicted against posterior mean of ideal point
    ## dimension by dimension
    xbar <- eval(x$ideal)$xbar
    n <- eval(x$ideal)$n
    
    ## party colors
    party <- rcObj$legis.data$party
    if(!is.null(party)){
      tbl <- table(party,exclude=NULL)
      cl <- rainbow(length(tbl))
      grp <- match(party,names(tbl))
      col <- cl[grp]
    }
    else
      col <- rep("black",n)

    for(b in 1:d){
      plot(y=x$legis.percent,
           x=xbar[,b],
           col=col,
           xlab="Ideal Point (Posterior Mean)",
           ylab="Voting Decisions Correctly Predicted (%)",
           pch=16)
      titleString <- paste(desc,"\n",
                           "Percent Correctly Predicted, by Legislator's Ideal Point")  
      if(d>1)
        titleString <- paste(titleString,
                             "(Dimension",b,")")
      title(titleString)
    }
  }

  #########################################################################
  ## type = votes
  #########################################################################
  if(localType=="votes"){
    if(is.null(rcObj$voteMargins))
      rcObj <- computeMargins(rcObj)
    margin <- rcObj$voteMargins
    
    if(is.null(margin))
      stop("failed to find or computes votes margins")
    
    ## margin information
    vote.percent <- t(apply(margin,1,function(x)x/sum(x[1:2])*100))
    margin <- cbind(margin,vote.percent[,1:2])
    
    #dimnames(margin)[[2]] <- c("Yea","Nay","NA",
    #                           "Yea (proportion of those voting)",
    #                           "Nay (proportion of those voting)")
    plot(x=jitter(margin[,4]),   ## percent voting yes
         y=x$vote.percent,
         xlab="Losing Coalition (%, excluding NAs, jittered)",
         ylab="Voting Decisions Correctly Predicted (%, excluding NA)")
    titleString <- paste(desc,"\n",
                         "Percent Correctly Predicted, by Vote Margin")
    title(titleString)
  }

  invisible(NULL)
}

zeroinfl <- function(formula, data, subset, na.action, weights, offset,
                     dist = c("poisson", "negbin", "geometric"),
                     link = c("logit", "probit", "cloglog", "cauchit", "log"),
                     control = zeroinfl.control(...),
                     model = TRUE, y = TRUE, x = FALSE, ...)
{
  ## set up likelihood
  ziPoisson <- function(parms, trunc.start=FALSE) {
    ## count mean
    mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
    ## binary mean
    if (trunc.start)
      phi <- rep(0, length(mu))
    else
      phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
    
    ## log-likelihood for y = 0 and y >= 1
    loglik0 <- log( phi + exp( log(1-phi) - mu ) ) ## -mu = dpois(0, lambda = mu, log = TRUE)
    loglik1 <- log(1-phi) + dpois(Y, lambda = mu, log = TRUE)
    
    ## collect and return
    if (trunc.start)
      sum(weights[Y1] * loglik1[Y1]) - sum(weights[Y1] * log(1 - exp(loglik0[Y1])))
    else
      sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * loglik1[Y1])
  }
  
  ziNegBin <- function(parms, trunc.start=FALSE) {
    ## count mean
    mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
    ## binary mean
    if (trunc.start)
      phi <- rep(0, length(mu))
    else
      phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
    ## negbin size
    theta <- exp(parms[(kx+kz)+1])
    
    ## log-likelihood for y = 0 and y >= 1
    loglik0 <- log( phi + exp( log(1-phi) + suppressWarnings(dnbinom(0, size = theta, mu = mu, log = TRUE)) ) )
    loglik1 <- log(1-phi) + suppressWarnings(dnbinom(Y, size = theta, mu = mu, log = TRUE))
    
    ## collect and return
    if (trunc.start)
      sum(weights[Y1] * loglik1[Y1]) - sum(weights[Y1] * log(1 - exp(loglik0[Y1])))
    else
      sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * loglik1[Y1])
  }
  
  ziGeom <- function(parms, trunc.start=FALSE)
    ziNegBin(c(parms, 0), trunc.start)
  
  countGradPoisson <- function(parms) {
    eta <- as.vector(X %*% parms[1:kx] + offsetx)[Y1]
    mu <- exp(eta)
    colSums(((Y[Y1] - mu) - exp(ppois(0, lambda = mu, log.p = TRUE) -
                                  ppois(0, lambda = mu, lower.tail = FALSE, log.p = TRUE) + eta)) * weights[Y1] * X[Y1, , drop = FALSE])
  }
  
  countGradGeom <- function(parms) {
    eta <- as.vector(X %*% parms[1:kx] + offsetx)[Y1]
    mu <- exp(eta)      
    colSums(((Y[Y1] - mu * (Y[Y1] + 1)/(mu + 1)) -
               exp(pnbinom(0, mu = mu, size = 1, log.p = TRUE) -
                     pnbinom(0, mu = mu, size = 1, lower.tail = FALSE, log.p = TRUE) -
                     log(mu + 1) + eta)) * weights[Y1] * X[Y1, , drop = FALSE])
  }
  
  countGradNegBin <- function(parms) {
    eta <- as.vector(X %*% parms[1:kx] + offsetx)[Y1]
    mu <- exp(eta)      
    theta <- exp(parms[kx+1])
    logratio <- pnbinom(0, mu = mu, size = theta, log.p = TRUE) -
      pnbinom(0, mu = mu, size = theta, lower.tail = FALSE, log.p = TRUE)
    rval <- colSums(((Y[Y1] - mu * (Y[Y1] + theta)/(mu + theta)) -
                       exp(logratio + log(theta) - log(mu + theta) + eta)) * weights[Y1] * X[Y1, , drop = FALSE])
    rval2 <- sum((digamma(Y[Y1] + theta) - digamma(theta) +    
                    log(theta) - log(mu + theta) + 1 - (Y[Y1] + theta)/(mu + theta) +
                    exp(logratio) * (log(theta) - log(mu + theta) + 1 - theta/(mu + theta))) * weights[Y1]) * theta
    c(rval, rval2)
  }  
  
  
  gradPoisson <- function(parms) {
    ## count mean
    eta <- as.vector(X %*% parms[1:kx] + offsetx)
    mu <- exp(eta)
    ## binary mean
    etaz <- as.vector(Z %*% parms[(kx+1):(kx+kz)] + offsetz)
    muz <- linkinv(etaz)
    
    ## densities at 0
    clogdens0 <- -mu
    dens0 <- muz * (1 - as.numeric(Y1)) + exp(log(1 - muz) + clogdens0)
    
    ## working residuals  
    wres_count <- ifelse(Y1, Y - mu, -exp(-log(dens0) + log(1 - muz) + clogdens0 + log(mu)))
    wres_zero <- ifelse(Y1, -1/(1-muz) * linkobj$mu.eta(etaz),
                        (linkobj$mu.eta(etaz) - exp(clogdens0) * linkobj$mu.eta(etaz))/dens0)
    
    colSums(cbind(wres_count * weights * X, wres_zero * weights * Z))
  }
  
  gradGeom <- function(parms) {
    ## count mean
    eta <- as.vector(X %*% parms[1:kx] + offsetx)
    mu <- exp(eta)
    ## binary mean
    etaz <- as.vector(Z %*% parms[(kx+1):(kx+kz)] + offsetz)
    muz <- linkinv(etaz)
    
    ## densities at 0
    clogdens0 <- dnbinom(0, size = 1, mu = mu, log = TRUE)
    dens0 <- muz * (1 - as.numeric(Y1)) + exp(log(1 - muz) + clogdens0)
    
    ## working residuals  
    wres_count <- ifelse(Y1, Y - mu * (Y + 1)/(mu + 1), -exp(-log(dens0) +
                                                               log(1 - muz) + clogdens0 - log(mu + 1) + log(mu)))
    wres_zero <- ifelse(Y1, -1/(1-muz) * linkobj$mu.eta(etaz),
                        (linkobj$mu.eta(etaz) - exp(clogdens0) * linkobj$mu.eta(etaz))/dens0)
    
    colSums(cbind(wres_count * weights * X, wres_zero * weights * Z))
  }
  
  gradNegBin <- function(parms) {
    ## count mean
    eta <- as.vector(X %*% parms[1:kx] + offsetx)
    mu <- exp(eta)
    ## binary mean
    etaz <- as.vector(Z %*% parms[(kx+1):(kx+kz)] + offsetz)
    muz <- linkinv(etaz)
    ## negbin size
    theta <- exp(parms[(kx+kz)+1])
    
    ## densities at 0
    clogdens0 <- dnbinom(0, size = theta, mu = mu, log = TRUE)
    dens0 <- muz * (1 - as.numeric(Y1)) + exp(log(1 - muz) + clogdens0)
    
    ## working residuals  
    wres_count <- ifelse(Y1, Y - mu * (Y + theta)/(mu + theta), -exp(-log(dens0) +
                                                                       log(1 - muz) + clogdens0 + log(theta) - log(mu + theta) + log(mu)))
    wres_zero <- ifelse(Y1, -1/(1-muz) * linkobj$mu.eta(etaz),
                        (linkobj$mu.eta(etaz) - exp(clogdens0) * linkobj$mu.eta(etaz))/dens0)
    wres_theta <- theta * ifelse(Y1, digamma(Y + theta) - digamma(theta) +
                                   log(theta) - log(mu + theta) + 1 - (Y + theta)/(mu + theta),
                                 exp(-log(dens0) + log(1 - muz) + clogdens0) *
                                   (log(theta) - log(mu + theta) + 1 - theta/(mu + theta)))
    
    colSums(cbind(wres_count * weights * X, wres_zero * weights * Z, wres_theta))
  }
  
  dist <- match.arg(dist)
  loglikfun <- switch(dist,
                      "poisson" = ziPoisson,
                      "geometric" = ziGeom,
                      "negbin" = ziNegBin)
  gradfun <- switch(dist,
                    "poisson" = gradPoisson,
                    "geometric" = gradGeom,
                    "negbin" = gradNegBin)
  
  ## binary link processing
  linkstr <- match.arg(link)
  linkobj <- make.link(linkstr)
  linkinv <- linkobj$linkinv
  
  if(control$trace) cat("Zero-inflated Count Model\n",
                        paste("count model:", dist, "with log link\n"),
                        paste("zero-inflation model: binomial with", linkstr, "link\n"), sep = "")
  
  
  ## call and formula
  cl <- match.call()
  if(missing(data)) data <- environment(formula)
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("formula", "data", "subset", "na.action", "weights", "offset"), names(mf), 0)
  mf <- mf[c(1, m)]
  mf$drop.unused.levels <- TRUE
  
  ## extended formula processing
  if(length(formula[[3]]) > 1 && identical(formula[[3]][[1]], as.name("|")))
  {
    ff <- formula
    formula[[3]][1] <- call("+")
    mf$formula <- formula
    ffc <- . ~ .
    ffz <- ~ .
    ffc[[2]] <- ff[[2]]
    ffc[[3]] <- ff[[3]][[2]]
    ffz[[3]] <- ff[[3]][[3]]
    ffz[[2]] <- NULL
  } else {
    ffz <- ffc <- ff <- formula
    ffz[[2]] <- NULL
  }
  if(inherits(try(terms(ffz), silent = TRUE), "try-error")) {
    ffz <- eval(parse(text = sprintf( paste("%s -", deparse(ffc[[2]])), deparse(ffz) )))
  }
  
  ## call model.frame()
  mf[[1]] <- as.name("model.frame")
  mf <- eval(mf, parent.frame())
  
  ## extract terms, model matrices, response
  mt <- attr(mf, "terms")
  mtX <- terms(ffc, data = data)
  X <- model.matrix(mtX, mf)
  mtZ <- terms(ffz, data = data)
  mtZ <- terms(update(mtZ, ~ .), data = data)
  Z <- model.matrix(mtZ, mf)
  Y <- model.response(mf, "numeric")
  
  
  ## sanity checks
  if(length(Y) < 1) stop("empty model")
  if(all(Y > 0)) stop("invalid dependent variable, minimum count is not zero")  
  if(!isTRUE(all.equal(as.vector(Y), as.integer(round(Y + 0.001)))))
    stop("invalid dependent variable, non-integer values")
  Y <- as.integer(round(Y + 0.001))
  if(any(Y < 0)) stop("invalid dependent variable, negative counts")
  
  if(control$trace) {
    cat("dependent variable:\n")
    tab <- table(factor(Y, levels = 0:max(Y)), exclude = NULL)
    names(dimnames(tab)) <- NULL
    print(tab)
  }
  
  ## convenience variables
  n <- length(Y)
  kx <- NCOL(X)
  kz <- NCOL(Z)
  Y0 <- Y <= 0
  Y1 <- Y > 0
  
  ## weights and offset
  weights <- model.weights(mf)
  if(is.null(weights)) weights <- 1
  if(length(weights) == 1) weights <- rep.int(weights, n)
  weights <- as.vector(weights)
  names(weights) <- rownames(mf)
  
  offsetx <- model_offset_2(mf, terms = mtX, offset = TRUE)
  if(is.null(offsetx)) offsetx <- 0
  if(length(offsetx) == 1) offsetx <- rep.int(offsetx, n)
  offsetx <- as.vector(offsetx)
  offsetz <- model_offset_2(mf, terms = mtZ, offset = FALSE)
  if(is.null(offsetz)) offsetz <- 0
  if(length(offsetz) == 1) offsetz <- rep.int(offsetz, n)
  offsetz <- as.vector(offsetz)
  
  ## starting values
  start <- control$start
  if(!is.null(start)) {
    valid <- TRUE
    if(!("count" %in% names(start))) {
      valid <- FALSE
      warning("invalid starting values, count model coefficients not specified")
      start$count <- rep.int(0, kx)
    }
    if(!("zero" %in% names(start))) {
      valid <- FALSE
      warning("invalid starting values, zero-inflation model coefficients not specified")
      start$zero <- rep.int(0, kz)
    }
    if(length(start$count) != kx) {
      valid <- FALSE
      warning("invalid starting values, wrong number of count model coefficients")
    }
    if(length(start$zero) != kz) {
      valid <- FALSE
      warning("invalid starting values, wrong number of zero-inflation model coefficients")
    }
    if(dist == "negbin") {
      if(!("theta" %in% names(start))) start$theta <- 1
      start <- list(count = start$count, zero = start$zero, theta = as.vector(start$theta[1]))
    } else {
      start <- list(count = start$count, zero = start$zero)
    }
    if(!valid) start <- NULL
  }
  
  method <- control$method
  hessian <- control$hessian
  ocontrol <- control
  control$method <- control$hessian <- control$EM <- control$start <- NULL
  
  if(is.null(start)) {
    if(control$trace) cat("generating starting values...")
    model_zero <- glm.fit(Z, as.integer(Y0), weights = weights, family = binomial(link = linkstr), offset = offsetz)
    
    countloglikfun <- function(parms)
      loglikfun(c(parms[1:kx], rep(0, kz), parms[-(1:kx)]),
                trunc.start = TRUE)
    countgradfun <- switch(dist,
                           "poisson" = countGradPoisson,
                           "geometric" = countGradGeom,
                           "negbin" = countGradNegBin)
    
    lmstart <- lm.wfit(X[Y1,,drop=FALSE],
                       log(Y[Y1]) - offsetx[Y1],
                       weights[Y1])$coefficients
    lmstart <- ifelse(is.na(lmstart), 0, lmstart)
    
    fit <- tryCatch(optim(fn = countloglikfun, gr = countgradfun,
                          par = c(lmstart, if(dist == "negbin") 0 else NULL),
                          method = method, hessian = FALSE, control = control),
                    error = function(e) list(convergence=1))
    
    if(fit$convergence == 0) {
      model_count <- glm.fit(X, Y, family = poisson(), weights = weights, offset = offsetx)
      start <- list(count = model_count$coefficients, zero = model_zero$coefficients)
      if(dist == "negbin") start$theta <- 1
    } else {
      start <- list(count = fit$par[1:kx], zero = model_zero$coefficients)
      if (length(fit$par) > kx)
        start$theta <- exp(fit$par[-(1:kx)])
    }
    
    ## EM estimation of starting values
    if(ocontrol$EM & dist == "poisson" & fit$convergence == 0) {
      mui <- model_count$fitted
      probi <- model_zero$fitted
      probi <- probi/(probi + (1-probi) * dpois(0, mui))
      probi[Y1] <- 0
      
      ll_new <- loglikfun(c(start$count, start$zero))
      ll_old <- 2 * ll_new
      
      while(abs((ll_old - ll_new)/ll_old) > control$reltol) {
        ll_old <- ll_new
        model_count <- glm.fit(X, Y, weights = weights * (1-probi), offset = offsetx,
                               family = poisson(), start = start$count)
        model_zero <- suppressWarnings(glm.fit(Z, probi, weights = weights, offset = offsetz,
                                               family = binomial(link = linkstr), start = start$zero))
        mui <- model_count$fitted
        probi <- model_zero$fitted
        probi <- probi/(probi + (1-probi) * dpois(0, mui))
        probi[Y1] <- 0
        start <- list(count = model_count$coefficients, zero = model_zero$coefficients)
        ll_new <- loglikfun(c(start$count, start$zero))
      }
    }
    
    if(ocontrol$EM & dist == "geometric" & fit$convergence == 0) {
      mui <- model_count$fitted
      probi <- model_zero$fitted
      probi <- probi/(probi + (1-probi) * dnbinom(0, size = 1, mu = mui))
      probi[Y1] <- 0
      
      ll_new <- loglikfun(c(start$count, start$zero))
      ll_old <- 2 * ll_new      
      ##if(!require("MASS")) {
      ##  ll_old <- ll_new
      ##	warning("EM estimation of starting values not available")
      ## }
      
      while(abs((ll_old - ll_new)/ll_old) > control$reltol) {
        ll_old <- ll_new
        model_count <- suppressWarnings(glm.fit(X, Y, weights = weights * (1-probi),
                                                offset = offsetx, family = MASS::negative.binomial(1), start = start$count))
        model_zero <- suppressWarnings(glm.fit(Z, probi, weights = weights,
                                               offset = offsetz, family = binomial(link = linkstr), start = start$zero))
        start <- list(count = model_count$coefficients, zero = model_zero$coefficients)
        mui <- model_count$fitted
        probi <- model_zero$fitted
        probi <- probi/(probi + (1-probi) * dnbinom(0, size = 1, mu = mui))
        probi[Y1] <- 0
        ll_new <- loglikfun(c(start$count, start$zero))
      }
    }
    
    if(ocontrol$EM & dist == "negbin" & fit$convergence == 0) {
      mui <- model_count$fitted
      probi <- model_zero$fitted
      probi <- probi/(probi + (1-probi) * dnbinom(0, size = start$theta, mu = mui))
      probi[Y1] <- 0
      
      ll_new <- loglikfun(c(start$count, start$zero, log(start$theta)))      
      ll_old <- 2 * ll_new      
      ## if(!require("MASS")) {
      ##   ll_old <- ll_new
      ##   warning("EM estimation of starting values not available")
      ## }
      
      ## offset handling in glm.nb is sub-optimal, hence...
      offset <- offsetx
      
      while(abs((ll_old - ll_new)/ll_old) > control$reltol) {
        ll_old <- ll_new
        model_count <- suppressWarnings(glm.nb(Y ~ 0 + X + offset(offset), weights = weights * (1-probi),
                                               start = start$count, init.theta = start$theta))
        model_zero <- suppressWarnings(glm.fit(Z, probi, weights = weights, offset = offsetz,
                                               family = binomial(link = linkstr), start = start$zero))
        start <- list(count = model_count$coefficients, zero = model_zero$coefficients, theta = model_count$theta)
        mui <- model_count$fitted
        probi <- model_zero$fitted
        probi <- probi/(probi + (1-probi) * dnbinom(0, size = start$theta, mu = mui))
        probi[Y1] <- 0
        ll_new <- loglikfun(c(start$count, start$zero, log(start$theta)))
      }
    }
    
    if(control$trace) cat("done\n")
  }
  
  
  ## ML estimation
  if(control$trace) cat("calling optim() for ML estimation:\n")
  
  fit <- optim(fn = loglikfun, gr = gradfun,
               par = c(start$count, start$zero, if(dist == "negbin") log(start$theta) else NULL),
               method = method, hessian = hessian, control = control)
  if(fit$convergence > 0) warning("optimization failed to converge")
  
  ## coefficients and covariances
  coefc <- fit$par[1:kx]
  names(coefc) <- names(start$count) <- colnames(X)
  coefz <- fit$par[(kx+1):(kx+kz)]
  names(coefz) <- names(start$zero) <- colnames(Z)
  
  vc <- tryCatch(-solve(as.matrix(fit$hessian)),
                 error=function(e) {
                   warning(e$message, call=FALSE)
                   k <- nrow(as.matrix(fit$hessian))
                   return(matrix(NA, k, k))
                 })
  
  if(dist == "negbin") {
    np <- kx + kz + 1
    theta <- as.vector(exp(fit$par[np]))
    SE.logtheta <- as.vector(sqrt(diag(vc)[np]))
    vc <- vc[-np, -np, drop = FALSE]
  } else {
    theta <- NULL
    SE.logtheta <- NULL
  }
  colnames(vc) <- rownames(vc) <- c(paste("count", colnames(X), sep = "_"),
                                    paste("zero",  colnames(Z), sep = "_"))
  
  ## fitted and residuals
  mu <- exp(X %*% coefc + offsetx)[,1]
  phi <- linkinv(Z %*% coefz + offsetz)[,1]
  Yhat <- (1-phi) * mu
  res <- sqrt(weights) * (Y - Yhat)
  
  ## effective observations
  nobs <- sum(weights > 0) ## = n - sum(weights == 0)
  
  rval <- list(coefficients = list(count = coefc, zero = coefz),
               residuals = res,
               fitted.values = Yhat,
               optim = fit,
               method = method,
               control = ocontrol,
               start = start,
               weights = if(identical(as.vector(weights), rep.int(1L, n))) NULL else weights,
               offset = list(count = if(identical(offsetx, rep.int(0, n))) NULL else offsetx,
                             zero = if(identical(offsetz, rep.int(0, n))) NULL else offsetz),
               n = nobs,
               df.null = nobs - 2,
               df.residual = nobs - (kx + kz + (dist == "negbin")),
               terms = list(count = mtX, zero = mtZ, full = mt),
               theta = theta,
               SE.logtheta = SE.logtheta,
               loglik = fit$value,
               vcov = vc,
               dist = dist,
               link = linkstr,
               linkinv = linkinv,
               converged = fit$convergence < 1,
               call = cl,
               formula = ff,
               levels = .getXlevels(mt, mf),
               contrasts = list(count = attr(X, "contrasts"), zero = attr(Z, "contrasts"))
  )
  if(model) rval$model <- mf
  if(y) rval$y <- Y
  if(x) rval$x <- list(count = X, zero = Z)
  
  class(rval) <- "zeroinfl"
  return(rval)
}

zeroinfl.control <- function(method = "BFGS", maxit = 10000, trace = FALSE, EM = FALSE, start = NULL, ...) {
  rval <- list(method = method, maxit = maxit, trace = trace, EM = EM, start = start)
  rval <- c(rval, list(...))
  if(!is.null(rval$fnscale)) warning("fnscale must not be modified")
  rval$fnscale <- -1
  if(!is.null(rval$hessian)) warning("hessian must not be modified")
  rval$hessian <- TRUE
  if(is.null(rval$reltol)) rval$reltol <- .Machine$double.eps^(1/1.6)
  rval
}

coef.zeroinfl <- function(object, model = c("full", "count", "zero"), ...) {
  model <- match.arg(model)
  rval <- object$coefficients
  rval <- switch(model,
                 "full" = structure(c(rval$count, rval$zero),
                                    .Names = c(paste("count", names(rval$count), sep = "_"),
                                               paste("zero", names(rval$zero), sep = "_"))),
                 "count" = rval$count,
                 "zero" = rval$zero)
  rval
}

vcov.zeroinfl <- function(object, model = c("full", "count", "zero"), ...) {
  model <- match.arg(model)
  rval <- object$vcov
  if(model == "full") return(rval)
  
  cf <- object$coefficients[[model]]
  wi <- seq(along = object$coefficients$count)
  rval <- if(model == "count") rval[wi, wi] else rval[-wi, -wi]
  colnames(rval) <- rownames(rval) <- names(cf)
  return(rval)
}

logLik.zeroinfl <- function(object, ...) {
  structure(object$loglik, df = object$n - object$df.residual, nobs = object$n, class = "logLik")
}

print.zeroinfl <- function(x, digits = max(3, getOption("digits") - 3), ...)
{
  
  cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
  
  if(!x$converged) {
    cat("model did not converge\n")
  } else {
    cat(paste("Count model coefficients (", x$dist, " with log link):\n", sep = ""))
    print.default(format(x$coefficients$count, digits = digits), print.gap = 2, quote = FALSE)
    if(x$dist == "negbin") cat(paste("Theta =", round(x$theta, digits), "\n"))
    
    cat(paste("\nZero-inflation model coefficients (binomial with ", x$link, " link):\n", sep = ""))
    print.default(format(x$coefficients$zero, digits = digits), print.gap = 2, quote = FALSE)
    cat("\n")
  }
  
  invisible(x)
}

summary.zeroinfl <- function(object,...)
{
  ## residuals
  object$residuals <- residuals(object, type = "pearson")
  
  ## compute z statistics
  kc <- length(object$coefficients$count)
  kz <- length(object$coefficients$zero)
  se <- sqrt(diag(object$vcov))
  coef <- c(object$coefficients$count, object$coefficients$zero)  
  if(object$dist == "negbin") {
    coef <- c(coef[1:kc], "Log(theta)" = log(object$theta), coef[(kc+1):(kc+kz)])
    se <- c(se[1:kc], object$SE.logtheta, se[(kc+1):(kc+kz)])
    kc <- kc+1
  }
  zstat <- coef/se
  pval <- 2*pnorm(-abs(zstat))
  coef <- cbind(coef, se, zstat, pval)
  colnames(coef) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
  object$coefficients$count <- coef[1:kc,,drop = FALSE]
  object$coefficients$zero <- coef[(kc+1):(kc+kz),,drop = FALSE]
  
  ## delete some slots
  object$fitted.values <- object$terms <- object$model <- object$y <-
    object$x <- object$levels <- object$contrasts <- object$start <- NULL
  
  ## return
  class(object) <- "summary.zeroinfl"
  object
}

print.summary.zeroinfl <- function(x, digits = max(3, getOption("digits") - 3), ...)
{
  
  cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
  
  if(!x$converged) {
    cat("model did not converge\n")
  } else {
    
    cat("Pearson residuals:\n")
    print(structure(quantile(x$residuals),
                    names = c("Min", "1Q", "Median", "3Q", "Max")), digits = digits, ...)  
    
    cat(paste("\nCount model coefficients (", x$dist, " with log link):\n", sep = ""))
    printCoefmat(x$coefficients$count, digits = digits, signif.legend = FALSE)
    
    cat(paste("\nZero-inflation model coefficients (binomial with ", x$link, " link):\n", sep = ""))
    printCoefmat(x$coefficients$zero, digits = digits, signif.legend = FALSE)
    
    if(getOption("show.signif.stars") & any(rbind(x$coefficients$count, x$coefficients$zero)[,4] < 0.1, na.rm=TRUE))
      cat("---\nSignif. codes: ", "0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1", "\n")
    
    if(x$dist == "negbin") cat(paste("\nTheta =", round(x$theta, digits), "\n")) else cat("\n")
    cat(paste("Number of iterations in", x$method, "optimization:", tail(na.omit(x$optim$count), 1), "\n"))
    cat("Log-likelihood:", formatC(x$loglik, digits = digits), "on", x$n - x$df.residual, "Df\n")
  }
  
  invisible(x)
}

predict.zeroinfl <- function(object, newdata, type = c("response", "prob", "count", "zero"),
                             na.action = na.pass, at = NULL, ...)
{
  type <- match.arg(type)
  
  ## if no new data supplied
  if(missing(newdata)) {
    rval <- object$fitted.values
    if(type != "response") {
      if(!is.null(object$x)) {
        X <- object$x$count
        Z <- object$x$zero
      } else if(!is.null(object$model)) {
        X <- model.matrix(object$terms$count, object$model, contrasts = object$contrasts$count)
        Z <- model.matrix(object$terms$zero,  object$model, contrasts = object$contrasts$zero)	
      } else {
        stop("predicted probabilities cannot be computed with missing newdata")
      }
      offsetx <- if(is.null(object$offset$count)) rep.int(0, NROW(X)) else object$offset$count
      offsetz <- if(is.null(object$offset$zero))  rep.int(0, NROW(Z)) else object$offset$zero
      mu <- exp(X %*% object$coefficients$count + offsetx)[,1]
      phi <- object$linkinv(Z %*% object$coefficients$zero + offsetz)[,1]
    }
  } else {
    mf <- model.frame(delete.response(object$terms$full), newdata, na.action = na.action, xlev = object$levels)
    X <- model.matrix(delete.response(object$terms$count), mf, contrasts = object$contrasts$count)
    Z <- model.matrix(delete.response(object$terms$zero),  mf, contrasts = object$contrasts$zero)
    offsetx <- model_offset_2(mf, terms = object$terms$count, offset = FALSE)
    offsetz <- model_offset_2(mf, terms = object$terms$zero,  offset = FALSE)
    if(is.null(offsetx)) offsetx <- rep.int(0, NROW(X))
    if(is.null(offsetz)) offsetz <- rep.int(0, NROW(Z))
    if(!is.null(object$call$offset)) offsetx <- offsetx + eval(object$call$offset, newdata)
    
    mu <- exp(X %*% object$coefficients$count + offsetx)[,1]
    phi <- object$linkinv(Z %*% object$coefficients$zero + offsetz)[,1]
    rval <- (1-phi) * mu
  }
  
  ## predicted means for count/zero component
  if(type == "count") rval <- mu
  if(type == "zero") rval <- phi
  
  ## predicted probabilities
  if(type == "prob") {
    if(!is.null(object$y)) y <- object$y
    else if(!is.null(object$model)) y <- model.response(object$model)
    else stop("predicted probabilities cannot be computed for fits with y = FALSE and model = FALSE")
    
    yUnique <- if(is.null(at)) 0:max(y) else at
    nUnique <- length(yUnique)
    rval <- matrix(NA, nrow = length(rval), ncol = nUnique)
    dimnames(rval) <- list(rownames(X), yUnique)
    
    switch(object$dist,
           "poisson" = {
             rval[, 1] <- phi + (1-phi) * exp(-mu)
             for(i in 2:nUnique) rval[,i] <- (1-phi) * dpois(yUnique[i], lambda = mu)
           },
           "negbin" = {
             theta <- object$theta
             rval[, 1] <- phi + (1-phi) * dnbinom(0, mu = mu, size = theta)
             for(i in 2:nUnique) rval[,i] <- (1-phi) * dnbinom(yUnique[i], mu = mu, size = theta)
           },
           "geometric" = {
             rval[, 1] <- phi + (1-phi) * dnbinom(0, mu = mu, size = 1)
             for(i in 2:nUnique) rval[,i] <- (1-phi) * dnbinom(yUnique[i], mu = mu, size = 1)
           })
  }
  
  rval
}

fitted.zeroinfl <- function(object, ...) {
  object$fitted.values
}

residuals.zeroinfl <- function(object, type = c("pearson", "response"), ...) {
  
  type <- match.arg(type)
  res <- object$residuals
  
  switch(type,
         
         "response" = {
           return(res)
         },
         
         "pearson" = {
           mu <- predict(object, type = "count")
           phi <- predict(object, type = "zero")
           theta1 <- switch(object$dist,
                            "poisson" = 0,
                            "geometric" = 1,
                            "negbin" = 1/object$theta)
           vv <- object$fitted.values * (1 + (phi + theta1) * mu)
           return(res/sqrt(vv))  
         })
}

terms.zeroinfl <- function(x, model = c("count", "zero"), ...) {
  x$terms[[match.arg(model)]]
}

model.matrix.zeroinfl <- function(object, model = c("count", "zero"), ...) {
  model <- match.arg(model)
  if(!is.null(object$x)) rval <- object$x[[model]]
  else if(!is.null(object$model)) rval <- model.matrix(object$terms[[model]], object$model, contrasts = object$contrasts[[model]])
  else stop("not enough information in fitted model to return model.matrix")
  return(rval)
}

predprob.zeroinfl <- function(obj, ...) {
  predict(obj, type = "prob", ...)
}

extractAIC.zeroinfl <- function(fit, scale = NULL, k = 2, ...) {
  c(attr(logLik(fit), "df"), AIC(fit, k = k))
}

## ## for creating a package
.package.Name <- "pscl"

##.First.lib <- function(lib,pkg){
##  library.dynam(.package.Name,
##                pkg,
##                lib)
##}

##.Last.lib <- function(libpath){
##  library.dynam.unload(chname="pscl",libpath=libpath)
##}

.onAttach <- function(...){
  packageStartupMessage("Classes and Methods for R developed in the\n",
                        "Political Science Computational Laboratory\n",
                        "Department of Political Science\n",
                        "Stanford University\n",
                        "Simon Jackman\n",
                        "hurdle and zeroinfl functions by Achim Zeileis")
}

.onUnload <- function(libpath){
  library.dynam.unload("pscl",libpath=libpath)
}

## drop unanimous cols of a (rollcall) matrix
dropUnanimous <- function(obj, lop = 0) {
  UseMethod("dropUnanimous")
}

dropUnanimous.rollcall <- function(obj, lop = 0) {
  if (!inherits(obj, "rollcall")) {
    stop("dropUnanimous.rollcall only defined for objects of class rollcall")
  }
  dropRollCall(obj,
    dropList = list(lop = lop)
  )
}

dropUnanimous.matrix <- function(obj, lop = 0) {
  if (!is.matrix(obj)) {
    stop("dropUnanimous.matrix only defined for objects of class matrix")
  }

  if (lop > 1 | lop < 0 | is.na(lop) | !is.numeric(lop) | length(lop) != 1) {
    stop("bad value for lop, must be a single proportion")
  }

  goodObj <- !is.na(obj)
  if (!all(as.vector(obj[goodObj]) %in% c(0, 1, NA))) {
    stop("rollcall matrix contains codes other than 0, 1, and NA.")
  }

  m <- apply(obj, 2, minMargin)
  drop <- m <= lop
  out <- obj[, !drop]
  out
}

printHeaderIdeal <- function(x){
  cat(paste("ideal was called as follows:\n"))
  print(x$call)
  cat("\n")

  cat(paste("Number of Legislators:\t",x$n,"\n"))
  cat(paste("Number of Votes:\t",x$m,"\n"))
  cat(paste("Number of Dimensions:\t",x$d,"\n"))
  cat(paste("Number of Iterations:\t",
            eval(x$call$maxiter,envir=.GlobalEnv),
            "\n"))
  cat(paste("\tThinned By:\t",
            eval(x$call$thin,envir=.GlobalEnv),"\n"))
  cat(paste("\tBurn-in:\t",
            eval(x$call$burnin,envir=.GlobalEnv),"\n\n"))

  invisible(NULL)
}

## summary and print functions
print.ideal <- function(x, ...) {
  if(!inherits(x, "ideal"))
    stop("object passed to print.ideal is not of class ideal\n")

  cat("Markov chain Monte Carlo Analysis of Roll Call Data\n")
  if(x$d==1)
    cat("       (2-parameter item-response modeling)        \n")
  else
    cat("     (multidimensional item-response modeling)     \n")
  cat("===================================================\n\n")

  printHeader(eval(x$call$object))
  printHeaderIdeal(x)

  if(is.null(x$call$file)){
    cat("Ideal Points: Posterior Means\n")
    print(round(x$xbar,2))
    cat("\n")
  }
  invisible(NULL)
}

summary.ideal <- function(object,
                          prob=.95,
                          burnin=NULL,
                          sort=TRUE,
                          include.beta=FALSE,
                          ...){

  if(!inherits(object, "ideal"))
    stop("summary.ideal only defined for objects of class ideal")

  if(!is.null(object$call$file)){
    print(object)
    cat("\n")
    cat(paste("MCMC output was directed to file:",
              object$call$file,
              "\n"))
    cat(paste("no output to summarize in the ideal object",
              match.call()$object,
              "\n"))
    return(invisible(NULL))
  }

  if(is.null(burnin)){
    keep <- checkBurnIn(object,eval(object$call$burnin,envir=.GlobalEnv))
  } else {
    keep <- checkBurnIn(object,burnin)
  }
  
  xm <- NULL
  xsd <- NULL
  bm <- NULL
  bsd <- NULL
  xHDR <- NULL
  bHDR <- NULL
  ##xResults <- list()
  ##bResults <- list()
  bSig <- list()

  myHPD <- function(x,prob){
    tmp <- coda::as.mcmc(x)
    return(coda::HPDinterval(tmp,prob))
  }
  
  ## get HPD of x
  xKeep <- object$x[keep,,,drop=FALSE]
  xHDR <- apply(xKeep,
                c(2,3),
                myHPD,
                prob=prob)
  xm <- apply(xKeep,c(2,3),mean)   ## means
  xsd <- apply(xKeep,c(2,3),sd)    ## standard deviations
  dimnames(xHDR)[[1]] <- c("lower","upper") 
  if(length(dim(xHDR))>2)
    xHDR <- aperm(xHDR,c(2,1,3))
  if (length(dim(xHDR))==2)
    xHDR <- t(xHDR)

  ## ################################################################
  ## get beta summaries
  if ((!is.null(object$beta)) && (include.beta)){
    bKeep <- object$beta[keep,,,drop=FALSE]
    bHDR <- apply(bKeep,
                  c(2,3),
                  myHPD,
                  prob=prob)
    dimnames(bHDR)[[1]] <- c("lower","upper")
    if (length(dim(bHDR))>2)
      bHDR <- aperm(bHDR,c(2,1,3))
    
    bm <- apply(bKeep,c(2,3),mean)
    bsd <- apply(bKeep,c(2,3),sd)
    
    ## "significance tests" for discrimination parameters
    ## we have HDR interval of content prob
    sigFunc <- function(x){
      out <- sign(x[1])==sign(x[2])
      labs <- rep(paste(round(prob*100),"% CI",sep=""),2)
      labs[1] <- paste(labs[1],"does NOT overlap 0")
      labs[2] <- paste(labs[2],"overlaps 0")
      out <- factor(out,
                    levels=c("TRUE","FALSE"),
                    labels=labs)
      out
    }
    bSig <- NULL
    bSig <- apply(bHDR,c(1,3),sigFunc)
    bSig <- bSig[,-grep("Difficulty",dimnames(bSig)[[2]])]
  }

  #####################################################################
  ## summarize by party
  pall.final <- NULL
  if(!is.null(object$call$dropList)){
    party <- dropRollCall(eval(object$call$object),
                          eval(object$call$dropList))$legis.data$partyName
    if(is.null(party))
      party <- dropRollCall(eval(object$call$object),
                            eval(object$call$dropList))$legis.data$party
  } else {
    party <- eval(object$call$object)$legis.data$partyName
    if(is.null(party))
      party <- eval(object$call$object)$legis.data$party
  }
  if(!is.null(party)){                       ## we have some party info
    nms <- NULL
    for (b in 1:object$d){       ## loop over dimensions
      pall <- NULL
      pm <- tapply(xm[,b],party,mean)
      pq <- tapply(xm[,b],party,
                   quantile,
                   probs=c(0,prob) + (1-prob)/2)
      for(j in 1:length(pq)){
        pall <- rbind(pall,pq[[j]])
      }
      pall <- cbind(pm,pall)
      pall.final <- rbind(pall.final,pall)
      nms <- c(nms,paste(rownames(pall),": Dimension ",b,sep=""))
    }
    
    colnames(pall.final)[1] <- "Mean"
    rownames(pall.final) <- nms
  }
  
  ## ###################################################################
  ## gather for output
  out <- list(object=match.call()$object,
              xm=xm,xsd=xsd,xHDR=xHDR,
              bm=bm,bsd=bsd,bHDR=bHDR,
              bSig=bSig,
              party.quant=pall.final,
              sort=sort,
              prob=prob)
  
  class(out) <- "summary.ideal"
  
  return(out)
}

print.summary.ideal <- function(x, digits=3, ...){ 
  if (!inherits(x, "summary.ideal")){
    stop("object passed to print.summary.ideal must be of class summary.ideal")
  }
  
  cat("Markov chain Monte Carlo Analysis of Roll Call Data\n")
  m <- eval(x$object)$m
  d <- eval(x$object)$d
  if(d==1)
    cat("       (2-parameter item-response modeling)        \n")
  else
    cat("     (multidimensional item-response modeling)     \n")

  printHeader(eval(eval(x$object)$call$object))
  printHeaderIdeal(eval(x$object))
  
  if(!is.null(x$party.quant)) {
    cat("Ideal Points (Posterior Means), by Party\n")
    print(round(x$party.quant,digits))
    cat("\n")
  }  

  ## loop over dimensions
  xResults <- list()
  for(j in 1:d){
    xResults[[j]] <- cbind(x$xm[,j],
                           x$xsd[,j],
                           x$xHDR[,,j])
    cNames <- c("Mean","Std.Dev.","lower","upper")
    dimnames(xResults[[j]])[[2]] <- cNames
    if(x$sort)
      xResults[[j]] <- xResults[[j]][order(xResults[[j]][,1]),]
  }

  for(j in 1:d){
    if(x$sort)
      cat(paste("Ideal Points, Dimension ",j," ",
                "(sorted by posterior means):\n",sep=""))
    else
      cat(paste("Ideal Points, Dimension ",j,":\n",sep=""))
    print(round(xResults[[j]],digits))
    cat("\n")
  }

  ## loop over dimensions
  bResults <- list()
  if(!is.null(x$bm)){
    for(b in 1:(d+1)){
      bResults[[b]] <- cbind(x$bm[,b],
                             x$bsd[,b],
                             x$bHDR[,,b])
      dimnames(bResults[[b]])[[2]][1] <- "Mean"
      dimnames(bResults[[b]])[[2]][2] <- "sd"

      tmpRollCall <- computeMargins(object=eval(eval(x$object)$call$object),
                                    dropList=eval(eval(x$object)$call$dropList))
      
      ## if available, tack on margins data
      if(!is.null(tmpRollCall$voteMargins)){
        bResults[[b]] <- cbind(bResults[[b]],
                               tmpRollCall$voteMargins)
      }
    }
    names(bResults) <- c(paste("Discrimination D",
                               1:d),
                         "Difficulty")
    
    
    ## report statistical tests of significance
    if(length(x$bSig)!=0){
      cat("Statistical tests of discrimination parameters:\n")
      if(d==2){ ## do a cross-tabulation
        cat("dimension 1 (rows) against dimension 2 (columns)\n")
        print(table(x$bSig[[1]],x$bSig[[2]]))
      } else{
        for (j in 1:d){
          cat("Dimension:",j)
          print(table(x$bSig[[j]]))
          cat("\n")
        }
      }
    }
    
    for(j in 1:d){
      cat(paste(names(bResults[[j]]),":\n"))
      theseResults <- bResults[[j]]
      foo <- x$bSig[[j]] == (levels(x$bSig[[j]])[2])
      fooChar <- rep("  ",m)
      fooChar[foo] <- "NS"
      dimnames(theseResults)[[1]] <- paste(dimnames(theseResults)[[1]],
                                           fooChar)
      print(round(theseResults,digits))
      cat("\n")
    }
    cat(paste(names(bResults)[d+1],":\n"))
    print(round(bResults[[d+1]],digits))
  }
  
  cat("\n")
  invisible(NULL)
}

rollcall <- function(data,
                     yea=1,nay=0,
                     missing=NA,
                     notInLegis=9,
                     legis.names=NULL,
                     vote.names=NULL,
                     legis.data=NULL,
                     vote.data=NULL,
                     desc=NULL,
                     source=NULL){
  
  ## codes, check and package
  codes <- list()
  if(!is.null(yea))
    codes$yea <- yea
  else
    codes$yea <- NULL
  if(!is.null(nay))
    codes$nay <- nay
  else
    codes$nay <- NULL
  if(!is.null(notInLegis))
    codes$notInLegis <- notInLegis
  else
    codes$notInLegis <- NULL
  if(!is.null(missing))
    codes$missing <- missing
  else
    codes$missing <- NULL
  if(checkCodes(codes))
    stop("codes are not unique\n")

  ## get a roll call matrix from input
  v <- NULL
  if((is.list(data)) && ("votes" %in% names(data))){ 
    v <- data$votes
  }
  else {
    v <- data
  }
  
  if (!is.matrix(v)) {
    v <- as.matrix(v)
  }
  
  ## check votes
  if(checkVotes(v,codes))
    stop("rollcall: bad votes")
  
  ## identifying tags for legislators
  nm <- legis.names
  if(is.null(nm)) ## look for legis.names var in data
    if(is.list(data))
      if(!is.null(data$legis.names))
        nm <- data$legis.names
  
  if(!is.null(nm)){  ## check that any names found by here are ok
    if(length(unique(nm))!=nrow(v)){
      cat("supplied legislator names do not match number of rows\n")
      cat("in roll call matrix; will use default names\n")
      nm <- NULL
    }
  }
  
  if(is.null(nm)){  ## make names
    nm <- paste("Legislator",1:nrow(v))
  }
  rownames(v) <- nm
  
  ## vote labels
  lbl <- vote.names
  if(is.null(lbl))
    if(is.list(data))
      if(!is.null(data$vote.names))  
        lbl <- data$vote.names

  if(!is.null(lbl)){  ## check that vote names are ok
    if(length(unique(lbl))!=ncol(v)){
      cat("supplied vote names do not match number of columns\n")
      cat("in roll call matrix; will use default names\n")
      lbl <- NULL
    }
  }
  if(is.null(lbl)){  ## make name
    lbl <- paste("Vote",1:ncol(v))
  }
  colnames(v) <- lbl
  
  ## legislator attributes
  if(!is.null(legis.data)){
    if(nrow(legis.data)!=nrow(v))
      stop("legislator data does not match number of legislators in roll call matrix")
    else
      rownames(legis.data) <- nm
  }

  if(!is.null(vote.data)){
    if(nrow(vote.data)!=ncol(v))
      stop("rows in vote.data does not match number of votes in roll call matrix")
    else
      rownames(vote.data) <- lbl
  }

  ## description of roll call voting matrix
  dsc <- desc
  if(is.list(data))
    if(!is.null(data$desc))
      dsc <- data$desc

  ## package up for output
  out <- list(votes=v,
              codes=codes,
              n=dim(v)[1],
              m=dim(v)[2],
              legis.data=legis.data,
              vote.data=vote.data,
              desc=dsc,
              source=source)

  class(out) <- c("rollcall")
  out
}

## loop over roll call matrix
## find which votes to DROP (lop-sided)
lopfunc <- function(x,lop=NULL){
  n <- sum(!is.na(x))
  if(is.null(lop))  ## throw away unanimous votes
    toss <- (sum(x, na.rm=T)==n) || (sum(x,na.rm=T)==0)
  else
    toss <- (sum(x==1,na.rm=T)/n <= lop) || (sum(x==0,na.rm=T)/n <= lop)
  
  toss
}

printDescription <- function(object){
  if(inherits(object,"rollcall") & !is.null(object$desc))
    cat(paste("Description:\t",object$desc,"\n"))
  invisible(NULL)
}

printSource <- function(object){
  if(inherits(object,"rollcall") & !is.null(object$source))
    cat(paste("Source:\t\t",object$source,"\n"))
  invisible(NULL)
}

printHeader <- function(object){
  if(inherits(object,"rollcall")){
    printDescription(object)
    printSource(object)
  }
  invisible(NULL)
}

print.rollcall <- function(x,print.votes=FALSE, ...){
  printHeader(x)
  
  cat(paste("Number of Legislators:\t",x$n,"\n"))
  cat(paste("Number of Votes:\t",x$m,"\n"))

  cat("\n")
  printCodes(x$codes)
  cat("\n")

  if(!is.null(x$legis.data)){
    cat("Legislator-specific variables:\n")
    print(names(x$legis.data))
  }
  if(!is.null(x$vote.data)){
    cat("Vote-specific variables:\n")
    print(names(x$vote.data))
  }
  
  if (print.votes)
   print(x$votes) 

  cat(paste("Detailed information is available via the summary function.\n"))
  
  invisible(NULL)
}

## check Votes
checkVotes <- function(object,codes=object$codes){
  if(inherits(object, "rollcall")){
    mat <- object$votes
  } else {
    if(inherits(object, "matrix")) {
      mat <- object
    }
  }
  if(is.null(codes))
    stop("checkVotes: no codes supplied")
  
  flag <- FALSE
  if(!all(mat[!is.na(mat)] %in% unlist(codes))){
    cat("checkVotes: Your data contains values other than the codes\n")
    cat("checkVotes: you supplied as representing Yea, Nay, Missing or\n")
    cat("checkVotes: not in legislature.\n")
    cat("checkVotes: You specified:\n")
    cat(paste("Yea: ",
              paste(as.character(codes$yea),collapse=" "),
              "\n",
              "Nay: ",
              paste(as.character(codes$nay),collapse=" "),
              "\n",
              "Missing: ",
              paste(as.character(codes$missing),collapse=" "),
              "\n",
              "Not In Legislature: ",
              paste(as.character(codes$notInLegis),collapse=" "),
              "\n",sep=""))
    cat("checkVotes: Your data has the following unique values and frequency counts:\n")
    print(table(mat,exclude=NULL))
    cat("\n")
    flag <- TRUE
  }
  flag
}

## check Codes
checkCodes <- function(codes){
  flag <- FALSE
  n <- length(codes)
  for(i in 1:(n-1)){
    for(j in (i+1):n){
      common <- intersect(codes[[i]],codes[[j]])
      if(length(common)!=0)
        flag <- TRUE
    }
  }
  flag
}

## show codes on-screen
printCodes <- function(codes){
  cat("Using the following codes to represent roll call votes:\n")
  if(!is.null(codes$yea))
    cat(paste("Yea:\t\t",
              paste(as.character(codes$yea),collapse=" "),
              "\n"))
  else
    cat(paste("Yea: <no code supplied -- this is almost surely an error>\n"))
  if(!is.null(codes$nay))
    cat(paste("Nay:\t\t",
              paste(as.character(codes$nay),collapse=" "),
              "\n"))
  else
    cat(paste("Nay: <no code supplied -- this is almost surely an error>\n"))
  if(!is.null(codes$missing))
    cat(paste("Abstentions:\t",
              paste(as.character(codes$missing),collapse=" "),
              "\n"))
  else
    cat(paste("Abstentions: <no code supplied -- this is almost surely an error>\n"))
  if(!is.null(codes$notInLegis))
    cat(paste("Not In Legislature:\t",
              paste(as.character(codes$notInLegis),collapse=" "),
              "\n"))
  invisible(NULL)
}

## convert codes to 0 and 1 etc
convertCodes <- function(object,codes=object$codes){
  if(!is.list(codes))
    stop("convertCodes: codes needs to be a list")
  if(is.null(codes$yea) | is.null(codes$nay))
    stop("convertCodes: no Yea and/or Nay code supplied")
  if(!is.matrix(object$votes))
    stop("convertCodes: supplied rollcalls are not in a matrix")

  ## conversions
  theCodes <- codes
  tmp <- matrix(-999,
                dim(object$votes)[1],
                dim(object$votes)[2])
  dimnames(tmp) <- dimnames(object$votes)
  tmp[object$votes %in% theCodes$yea] <- 1
  tmp[object$votes %in% theCodes$nay] <- 0
  if(!is.null(theCodes$missing)){
    if(!any(is.na(theCodes$missing)))
      theCodes$missing <- c(theCodes$missing,NA)
    tmp[object$votes %in% theCodes$missing] <- NA
  }
  if(!is.null(theCodes$notInLegis))
    tmp[object$votes %in% theCodes$notInLegis] <- NA
  
  bad <- tmp[!is.na(tmp)] == -999
  if(any(bad)){
    cat("convertCodes: not all rollcall votes converted to 0, 1, NA.\n")
    cat("convertCodes: information in codes not exhaustive.\n")
    cat(paste("convertCodes: setting remaining",sum(bad),"votes to NA\n"))
    tmp[bad] <- NA
  }
  tmp
}

## for each legislator compute how often they vote with the direction
## in which a majority of their party voted
partyLoyalty <- function(object){
  theParties <- unique(object$legis.data$party)
  nParties <- length(theParties)

  ## what was the majority direction by vote, by party
  majOutcome <- function(x){
    as.numeric(names(which.max(table(x))))
  }
  
  partyDirections <- matrix(NA,object$m,nParties)
  for(p in 1:nParties){
    thisParty <- object$legis.data$party==theParties[p]
    ## only do this if the party is bigger than two legislators
    if(sum(thisParty,na.rm=TRUE)>2){
      foo <- apply(object$votes[thisParty,],2,
                   majOutcome)
      if(is.list(foo)){
        foo[which(lapply(foo,length)==0)] <- NA
        foo <- unlist(foo)
      }
      partyDirections[,p] <- foo
     }
   }
  dimnames(partyDirections) <- list(dimnames(object$votes)[[2]],
                                    theParties)
  
  ## now compare individual voting histories
  ## with voting scores
  partyLoyalty <- rep(NA,object$n)
  legisParty <- match(x=object$legis.data$party,
                      table=theParties)
  
  goodCompare <- function(x,y){
    ok <- !is.na(x) & !is.na(y)
    goodmatch <- sum(x[ok] == y[ok]) + sum(is.na(x) & is.na(y))
    out <- goodmatch/length(x) * 100
    out
  }
  
  for(i in 1:object$n){   ## loop over legislators
    ## dont do it where partyDirections are undefined
    ## (i.e., small numbers of indeps etc)
    if(!all(is.na(partyDirections[,legisParty[i]]))){  
      theDirections <- partyDirections[,legisParty[i]]  
      partyLoyalty[i] <- goodCompare(theDirections,
                                     object$votes[i,])
    }
  }
  partyLoyalty
}

lopLook <- function(margins,cutOff){
  extremeMat <- rep(NA,cutOff+1)
  for(j in 0:cutOff){
    extremeMat[j+1] <- sum(margins[,1]==j | margins[,2]==j)
  }
  extremeMat
}

vectorRepresentation <- function(object,
                                 dropList=list(codes=c("missing",
                                                 "notInLegis"))){
  if(!inherits(object, "rollcall"))
    stop("vectorRepresentation only defined for objects of class rollcall")
  if(is.null(object$codes))
    stop("no rollcall codes")
  else
    codes <- object$codes
  
  if(is.null(dropList) |
     length(dropList)==0 |
     is.null(dropList$codes) |
     length(dropList$codes)==0){
    cat("missing arguments for drop, vectorRepresentation will use defaults")
    dL <- list(codes=c("missing","notInLegis"))
  }
  else
    dL <- dropList
  tmpRollCall <- dropRollCall(object,dropList=dL)

  badCodes <- match(dL$codes,names(codes))
  if(any(is.na(badCodes)))
    stop("couldn't find codes to drop\n")
  else{
    dropCodes <- NULL
    for(j in badCodes)
      dropCodes <- c(dropCodes,codes[[j]])
  }
  cat(paste("vectorRepresentation: dropCodes=",
            paste(dropCodes,collapse=", "),
            "\n"))
  
  n <- tmpRollCall$n
  m <- tmpRollCall$m
  v <- tmpRollCall$votes
  y <- matrix(NA,n*m,3)
  dimnames(y)[[2]] <- list("vote","i","j")

  z <- 1
  for(i in 1:n){
    for(j in 1:m){
      vij <- v[i,j]
      if(!(vij %in% dropCodes)){
        if(vij %in% codes$yea)
          y[z,1] <- 1
        if(vij %in% codes$nay)
          y[z,1] <- 0
        y[z,2] <- i
        y[z,3] <- j
        z <- z + 1
      }
    }
  }
  y <- y[1:(z-1),]
  y
}

matchDimnames <- function(labs,codes){
  codesNames <- names(codes)
  nLabs <- length(labs)
  whichCode <- rep(NA,nLabs)
  for(i in 1:nLabs){
    tmp <- unlist(lapply(codes,function(x)as.numeric(labs[i]) %in% x))
    #print(tmp)
    if(sum(tmp)==1)
      whichCode[i] <- which.max(tmp)
  }
  out <- codesNames[whichCode]
  out <- paste(labs," (",out,")",sep="")
  out
}

summary.rollcall <- function(object,
                             dropList=NULL,
                             ##list(codes="notInLegis",
                             ##  lop=0),
                             verbose=FALSE,
                             debug=FALSE,
                             ...){
  if(!inherits(object, "rollcall"))
    stop("summary.rollcall only operates on objects of class rollcall")

  mc <- match.call()   ## how were we called
  if(is.null(mc$dropList))
    mc$dropList <- dropList
  if(is.null(mc$verbose))
    mc$verbose <- verbose
  
  legisTab <- NULL
  voteTab <- NULL
  dropTab <- NULL
  partyLoyaltyScores <- NULL
  lopSided <- NULL

  if(!is.null(object$dropInfo)){
    cat(paste("The input rollcall object already has a dropInfo component,\n",
              "meaning that it is the product of dropRollCall.\n",
              "This summary and the execution of any current dropList\n",
              "proceeds conditional on the previous dropList.\n"),
        sep="")
  }   

  ## process user options re dropping votes/legislators etc
  if(!is.null(dropList)){
    tmpRollCall <- dropRollCall(object,dropList,debug=debug)
  }
  else
    tmpRollCall <- object

  v <- tmpRollCall$votes
  
  ## party breakdown, if available
  haveParty <- !is.null(tmpRollCall$legis.data$party)
  partyTab <- NULL
  if(haveParty)
    partyTab <- table(tmpRollCall$legis.data$party,exclude=NULL)

  ## get any exclude codes
  
  allVotes <- table(v)
  allVotes <- cbind(allVotes,
                    allVotes/sum(allVotes)*100)
  dimnames(allVotes)[[2]] <- c("Count","Percent")
  dimnames(allVotes)[[1]] <- matchDimnames(dimnames(allVotes)[[1]],
                                           tmpRollCall$codes)
  
  ## what was clobbered by dropRollCall
  if(!is.null(tmpRollCall$dropInfo))
    if("new" %in% names(tmpRollCall$dropInfo))
      dropTab <- lapply(tmpRollCall$dropInfo$new[c("legislators","votes")],
                        table,exclude=NULL)
    else
      dropTab <- lapply(tmpRollCall$dropInfo[c("legislators","votes")],
                        table,exclude=NULL)
  
  if(verbose){
    ## breakdowns by legislator
    if(debug)
      cat("computing breakdowns by legislator...")
    legisTab <- t(apply(v,1,
                        marginWithCodes,
                        codes=tmpRollCall$codes))
    dimnames(legisTab)[[2]] <- c(names(tmpRollCall$codes),
                                 "Total",
                                 paste(names(tmpRollCall$codes),"%",sep=""))
    ## breakdowns by vote
    cat("by vote...")
    voteTab <- t(apply(v,2,
                       marginWithCodes,
                       codes=tmpRollCall$codes))
    dimnames(voteTab)[[2]] <- dimnames(legisTab)[[2]]
    
    lopSided <- lopLook(voteTab,floor(.05*tmpRollCall$n))
    names(lopSided) <- as.character(0:floor(.05*tmpRollCall$n))

    ## party loyalty
    if(haveParty){
      cat("and party loyalty scores")
      partyLoyaltyScores <- partyLoyalty(tmpRollCall)
    }
    cat("\n")
  }
  
  out <- list(n=tmpRollCall$n,
              m=tmpRollCall$m,
              codes=tmpRollCall$codes,
              allVotes=allVotes,
              partyTab=partyTab,
              lopSided=lopSided,
              legisTab=legisTab,
              dropTab=dropTab,
              partyLoyalty=partyLoyaltyScores,
              voteTab=voteTab,
              call=mc)
  class(out) <- "summary.rollcall"
  out
}

printDropTab <- function(x){
  for(i in 1:length(x))
    if("FALSE" %in% names(x[[i]])){
      cat(paste("dropRollCall deleted",
                x[[i]]["FALSE"],
                "of",
                sum(x[[i]]),
                names(x)[[i]],
                "\n"))
    }
    else{
      cat(paste("dropRollCall deleted no",names(x)[[i]],"\n"))
    }
  invisible(NULL)
}

print.summary.rollcall <- function(x, digits=1, ...){
  if(!inherits(x, "summary.rollcall"))
    stop("print.summary.rollcall only defined for objects of class summary.rollcall")
    
  rcObj <- x$call$object
  verbose <- x$call$verbose
  if(is.null(eval(rcObj)))
    stop("can't find rollcall object")

  if(length(rcObj)>1)
    rcObjName <- format(rcObj)
  else
    rcObjName <- rcObj
  if(!is.null(eval(rcObj)$desc))
    cat(paste("\nSummary of rollcall object",
              rcObjName,
              "\n\n"))
  printHeader(eval(rcObj))

  cat(paste("\nNumber of Legislators:\t\t",x$n))
  cat(paste("\nNumber of Roll Call Votes:\t",x$m))
  cat("\n\n")

  ## was summary called from dropRollCall directly? 
  ## if so then dump the drop info from the object 
  
  if(!is.null(x$dropTab))
    printDropTab(x$dropTab)

  ## if(!is.null(x$call$dropList))
##     cat(paste("This summary ignores voting decisions that are coded ",
##               paste(x$call$dropList$codes,collapse=" or "),
##               ".\n",sep=""))
##   if(!is.null(x$call$dropList$lop))
##     if(x$call$dropList$lop==0)
##       cat("This summary computed after dropping unanimous roll calls.\n")
##     else
##       cat(paste("This summary computed after dropping roll calls with",
##                 x$call$dropList$lop,
##                 "or fewer legislators voting\nin the minority.\n"))
##   if(length(x$call$dropList)>2){
##     cat("Other restrictions are being applied. The full dropList is:\n")
##     print(x$call$dropList)
##   }
     
  cat("\n")
  printCodes(x$codes)
  cat("\n")
  
  if(!is.null(x$partyTab)){
    cat("Party Composition:")
    print(x$partyTab)
  }
  cat("\nVote Summary:\n")
  print(round(x$allVotes,1))

  if(!is.null(x$lopSided) | !all(x$lopSided==0)){
    cat("\nLop-sided Votes (Number Voting in Minority), and Frequencies:\n")
    print(x$lopSided)
  }

  if(!verbose)
    cat(paste("\nUse summary(",
              rcObjName,
              ",verbose=TRUE) for more detailed information.\n",sep=""))
  
  if(verbose){
    if(!is.null(x$partyLoyalty)){
      cat("\nSummary By Legislator: Counts, Percentages and Party Loyalty\n")
      foo <- cbind(round(x$legisTab,digits),
                   round(x$partyLoyalty))
      dimnames(foo)[[2]][ncol(foo)] <- "Party Loyalty"
    }
    else{
      cat("\nSummary By Legislator: Counts and Percentages\n")
      foo <- round(x$legisTab,digits)
    }
    print(foo)
    
    cat("\nSummary By Vote\n")
    print(round(x$voteTab,digits))
    cat("\n")
  }

  invisible(NULL)
}

predprob.glm <- function(obj, newdata = NULL, at = NULL, ...){
    if(!inherits(obj,"glm"))
        stop("predprob.glm only available for glm objects (including class negbin)\n")

    isNegBin <- inherits(obj, "negbin")
    isPoisson <- family(obj)$family=="poisson"
    isBinomial <- family(obj)$family=="binomial"
    if(!isNegBin & !isPoisson & !isBinomial)
        stop(paste("your object of class",class(obj),"is unsupported by predprob.glm"))
    
    if(is.null(newdata))
        yhat <- predict(obj,
                        type="response")
    else
        yhat <- predict(obj,
                        newdata=newdata,
                        type="response")

    y <- obj$y
    yUnique <- if(is.null(at)) 0:max(y) else at
    nUnique <- length(yUnique)
    p <- matrix(NA,length(yhat),nUnique)
    dimnames(p) <- list(NULL,yUnique)

    if(isNegBin){
        for(i in 1:nUnique){
            p[,i] <- dnbinom(mu=yhat,
                             size=obj$theta,
                             x=yUnique[i])
        }
    }

    if(isPoisson){
        for(i in 1:nUnique){
            p[,i] <- dpois(lambda=yhat,
                           x=yUnique[i])
        }
    }

    if(isBinomial){
        if(is.null(newdata))
            p <- predict(obj,
                            type="response")
        else
            p <- predict(obj,
                            newdata=newdata,
                            type="response")
        p <- cbind(1-p,p)
        dimnames(p) <- list(NULL,c("0","1"))
    }

    p
    
}

## convert ideal object to MCMC object

idealToMCMC <- function(object, burnin=NULL){
  if(!inherits(object, "ideal"))
    stop("idealToMCMC only defined for objects of class ideal")
  
  if(is.null(burnin))
    b <- eval(object$call$burnin)
  keep <- checkBurnIn(object,b)

  iters <- as.numeric(dimnames(object$x[keep,,])[[1]])

  out <- object$x[keep,,]
  if(!is.null(object$beta)){
    J <- dim(object$beta)[3]
    for(j in 1:J){
      thisBeta <- object$beta[keep,,j]
      dimnames(thisBeta)[[2]] <- paste(dimnames(thisBeta[[2]]),
                                       dimnames(object$beta[[3]])[j])
      out <- cbind(out,thisBeta)
    }
  }

  return(coda::mcmc(data=out,
                    start=iters[1],
                    thin=eval(object$call$thin),
                    end=iters[length(iters)]
                    )
         )
}

## parse and execute drop list used by summary.rollcall and ideal
printDropList <- function(list){
  cat("Dropping elements of rollcall matrix using the following dropList:\n")
  if(!is.null(list$codes)){
    cat("  Voting decisions with the following codes will be set to NA:\n")
    print(list$codes)
    cat("\n")
  }

  if(!is.null(list$lop) & is.numeric(list$lop)){
    if(list$lop==0)
      cat("  Unanimous votes will be dropped.\n")
  else
    cat("  Votes with",list$lop,
        "or fewer legislators voting in the minority",
        "will be dropped.\n")
  }

  if(!is.null(list$legisMin) & is.numeric(list$legisMin))
    cat("  Legislators with",list$legisMin,
        "or fewer non-missing voting decisions",
        "will be dropped.\n")

  if(!is.null(list$dropLegis))
    cat(paste("  Legislators for whom the condition\n",
              "  ",
              deparse(list$dropLegis),
              "\n",
              "  is true (evaluated in the legis.data data frame) will be dropped.\n",
              sep=""))

  if(!is.null(list$dropVotes))
    cat(paste("  Votes for which the condition\n",
              deparse(list$dropVotes),
              "\n",
              "  is true (evaluated in the vote.data data frame) will be dropped.\n",
              sep=""))
  cat("\n")

  invisible(NULL)
}

compareRollCallObjects <- function(old,new){
  legislators <- dimnames(old$votes)[[1]]
  n <- length(legislators)
  votes <- dimnames(old$votes)[[2]]
  m <- length(votes)
  
  if(all(dim(old$votes)==dim(new$votes)))
    keep <- list(legislators=rep(TRUE,n),votes=rep(TRUE,m))
  else{
    newLegis <- dimnames(new$votes)[[1]]
    newVotes <- dimnames(new$votes)[[2]]
    keep <- list(legislators=legislators%in%newLegis,
                 votes=votes%in%newVotes)
  }
  names(keep$legislators) <- legislators
  names(keep$votes) <- votes

  keep 
}


dropRollCall <- function(object,dropList=NULL,debug=FALSE){
  if(!inherits(object, "rollcall")){
    stop("dropRollCall only works for objects of class rollcall.")
  }

  tmpRollCall <- object
  
  if(!is.list(dropList) | is.null(dropList)){
    cat("dropList must be a non-null list or alist.\nNo subsetting will occur.\n")
    return(object)
  }

  if(debug)
    printDropList(dropList)

  flag <- TRUE   ## raise the flag
  counter <- 1
  while(flag){   ## loop until the flag goes down
    if(debug)
      cat(paste("\ndropRollCall: Pass number",counter,"over roll call object\n"))

    v <- tmpRollCall$votes
    dimOld <- dim(v)               ## store this
    if(debug)
      cat(paste("  The roll call matrix has dimension",
                dimOld[1],"legislators",
                "and",
                dimOld[2],"rollcalls.\n"))

    ## strip out user-designated votes of a particular code
    if(!is.null(dropList$codes) & length(dropList$codes)>0){
      if(debug)
        cat("  Processing dropList voting codes...\n")
      dc <- dropList$codes
      dCodes <- NULL
      if(all(is.character(dc))){   ## named element of codes list?
        dropCodes <- match(dc,names(tmpRollCall$codes))
        dropCodes <- dropCodes[!is.na(dropCodes)]
        
        if(length(dropCodes)>0){
          for(j in dropCodes)
            dCodes <- c(dCodes,tmpRollCall$codes[j])  ## drop these
          keepCodes <- !(names(tmpRollCall$codes) %in% dc)
          keepCodes <- tmpRollCall$codes[keepCodes]
          tmpRollCall$codes <- keepCodes    
        }
      }
      if(is.numeric(dc)){    ## or numeric elements
        dCodes <- dc[dc %in% unique(as.vector(v))]
      }
      
      bad <- v %in% dCodes
      if(debug)
        cat(paste("  dropRollCall will set",sum(bad),"voting decisions to NA.\n"))
      tmpRollCall$votes[bad] <- NA
      rm(bad)
    }

    dropLegis <- rep(FALSE,dim(v)[1])
    dropVotes <- rep(FALSE,dim(v)[2])
    
    ## drop legislators if too little data
    if(!is.null(dropList$legisMin)){
      if(debug)
        cat("  dropRollCall processing minimum votes by legislator (legisMin) restrictions...\n")
      legisMin <- dropList$legisMin
      if(length(legisMin)!=1 |
         is.na(legisMin) |
         !is.numeric(legisMin) |
         legisMin >= tmpRollCall$m)
        stop("  Bad value for legisMin in drop list.")
      vtmp <- convertCodes(tmpRollCall)
      goodCount <- apply(vtmp,1,function(x)sum(!is.na(x)))
      dropLegis <- dropLegis | goodCount<legisMin
    }
    
    ## check for subsetting in legis.data
    if(!is.null(dropList$dropLegis)){
      r <- dropRollCallViaData(dropList$dropLegis,
                               object=tmpRollCall,
                               d=expression(legis.data))
      if(!is.null(r))
        dropLegis <- dropLegis | r
    }
    
    ## drop votes by lop-sidedness
    if(!is.null(dropList$lop) & is.numeric(dropList$lop)){
      if(debug)
        cat("  dropRollCall processing lop-sided restrictions...\n")
      lop <- dropList$lop
      if(length(lop)!=1 |
         is.na(lop) |
         !is.numeric(lop) |
         lop < 0 |
         lop >= tmpRollCall$n)
        stop("  Invalid value for lop")
      if(is.null(tmpRollCall$voteMargins)){
        if(debug)
          cat("    Computing vote margins...\n")
        tmpRollCall <- computeMargins(tmpRollCall,dropList=NULL)
      }
      r <- tmpRollCall$voteMargins[,"Min"] <= lop
      if(debug)
        cat(paste("    dropRollCall will drop",sum(r),"roll calls due to lop-sidedness.\n"))
      dropVotes <- dropVotes | r
      if(debug)
        cat("  dropRollCall finished processing lop-sided restrictions.\n")
    }
    
    ## check for subsetting in vote.data
    if(!is.null(dropList$dropVotes) & counter==1){
      r <- dropRollCallViaData(dropList$dropVotes,
                               object=tmpRollCall,
                               d=expression(vote.data))
      if(!is.null(r))
        dropVotes <- dropVotes | r
    }

    ## final processing
    if(debug)
      cat(paste("dropRollCall will drop ",
                sum(dropLegis),
                " legislators & ",
                sum(dropVotes),
                " rollcalls.\n",
                sep=""))
    ## if(sum(dropLegis)>0){
##       cat("Dropped Legislators:\n")
##       print(dimnames(tmpRollCall$votes)[[1]][dropLegis])
##     }
##     if(sum(dropVotes)>0){
##       cat("Dropped Votes:\n")
##       print(dimnames(tmpRollCall$votes)[[2]][dropVotes])
##     }

    if(sum(dropLegis>0) | sum(dropVotes)>0)
      tmpRollCall$votes <- tmpRollCall$votes[!dropLegis,!dropVotes]
    
    if(!is.null(tmpRollCall$legis.data) & sum(dropLegis)>0){
      tmpRollCall$legis.data <- tmpRollCall$legis.data[!dropLegis,]
      if(is.data.frame(object$legis.data)){
        class(tmpRollCall$legis.data) <- "data.frame"
        names(tmpRollCall$legis.data) <- names(object$legis.data)
      }
    }

    if(!is.null(tmpRollCall$vote.data) & sum(dropVotes)>0){
      tmpRollCall$vote.data <- tmpRollCall$vote.data[!dropVotes,]
      if(is.data.frame(object$vote.data)){
        class(tmpRollCall$vote.data) <- "data.frame"
        names(tmpRollCall$vote.data) <- names(object$vote.data)
      }
    }

    if(!is.null(tmpRollCall$voteMargins) & sum(dropVotes)>0)
      tmpRollCall$voteMargins <- tmpRollCall$voteMargins[!dropVotes,]

    dimNew <- dim(tmpRollCall$votes)
    tmpRollCall$n <- dimNew[1]
    tmpRollCall$m <- dimNew[2]

    if(all(dimNew==dimOld)){          ## if no change from previous pass
      if(debug)
        cat("\ndropRollCall has finished processing the rollcall object.\n\n")
      flag <- FALSE                  ## lower the flag, quit loop
    }

    counter <- counter + 1
  }

  ## and finally, add dropped information to rollcall object
  if(!is.null(dropList)){
    newdropInfo <- compareRollCallObjects(object,tmpRollCall)

    if(is.null(tmpRollCall$dropInfo)){
      tmpRollCall$dropInfo <- newdropInfo
      tmpRollCall$dropInfo$dropList <- dropList
    }
    else{
      ## add this dropList to the others
      tmpRollCall$dropInfo <- list(previous=tmpRollCall$dropInfo,
                                   new=list(legislators=newdropInfo$legislators,
                                     votes=newdropInfo$votes,
                                     dropList=dropList))
    }
  }

  return(tmpRollCall)               ## return rollcall object
}

dropRollCallViaData <- function(expr,object,d){
  cf <- match.call()

  f <- try(eval(d,envir=object),silent=TRUE)
  if(inherits(f,"try-error")){
    cat(paste("The data frame ",
              cf$d,
              " was not found in ",
              cf$object,
              ".\n",sep=""))
    cat("Proceeding by ignoring this subsetting restriction.\n")
    return(NULL)
  }

  r <- try(eval(expr,f),silent=TRUE)
  if(inherits(r,"try-error")){
    r <- rep(FALSE,dim(f)[1])
    cat(paste("The assertion ",
              deparse(expr),
              " could not be evaluated in the ",
              cf$d,
              " component of ",
              cf$object,
              ".\n",
              sep=""))
    cat("Proceeding by ignoring this assertion.\n")
  }
  if(!is.logical(r))
    stop("'x' must evaluate to logical")
  r <- r & !is.na(r)
  r
}

## ideal plotting functions
plot.ideal <- function(x, conf.int=0.95, burnin=NULL, ...) {
  if(!inherits(x, "ideal"))
    stop("plot.ideal only available for objects of class ideal")
  
  if(is.null(burnin))
    keep <- checkBurnIn(x,eval(x$call$burnin,envir=.GlobalEnv))
  else
    keep <- checkBurnIn(x,burnin)

  if(x$d == 1)
    plot1d(x,conf.int=conf.int, burnin=burnin, ...)
  else if(x$d==2){
    plot2d(x,burnin=burnin, ...)
  }
  else{ ## more than 2 dimensions    
    xm <- apply(x$x[keep,-1],2,mean,na.rm=T)
    dim <- matrix(rep(NA,(x$n*x$d)),ncol=x$d)
    lbls <- NULL
    for (i in 1:x$d){
      dim[,i] <- xm[seq(i,length(xm),by=x$d)]
      lbls <- c(lbls,paste("Dimension ",i,sep=""))
    }
    pairs(dim,labels=lbls,main="Posterior Mean: Ideal Points")
  }
}

plot1d <- function(x,
                   d=1,
                   conf.int=0.95,
                   burnin=NULL,
                   showAllNames=FALSE,
                   ...){
  if(!inherits(x, "ideal"))
    stop("plot.ideal only available for objects of class ideal")
  
  if(is.null(burnin))
    keep <- checkBurnIn(x,eval(x$call$burnin,envir=.GlobalEnv))
  else
    keep <- checkBurnIn(x,burnin)
  
  checkD(x,d)                    ## check that selected dimension is ok
  checkCI(conf.int)              ## check that confidence interval is ok
  q <- c((1-conf.int)/2, 1-((1-conf.int)/2))  ## quantiles from CI
  
  xm <- x$xbar                   ## xbar
  indx <- order(xm)              ## sort index
  exispar <- par(no.readonly=T)

  myHPD <- function(x,prob){
    tmp <- coda::as.mcmc(x)
    return(coda::HPDinterval(tmp,prob))
  }
  xq <- t(apply(x$x[keep,,1],2,myHPD,prob=conf.int))  ## get HPDs

  ## names etc
  cat(paste("Looking up legislator names and party affiliations\n"))
  cat(paste("in rollcall object",x$call$object,"\n"))
  tmpObject <- dropRollCall(eval(x$call$object),
                            eval(x$call$dropList))
  party <- tmpObject$legis.data$party  ## extract party info
  legis.name <- unclass(dimnames(x$xbar)[[1]])
  longName <- max(nchar(legis.name))
  rm(tmpObject)
  textLoc <- 1.05*min(xq)   ## where to put x labels
  
  if(showAllNames){
    par(mar=c(3,longName*.55,4,2)+0.1,
        oma=rep(0,4))
  } else {
    par(mar=c(3,longName*.75,4,2)+0.1,
        oma=rep(0,4))
  }
  ## title string info
  mainString <- paste("Ideal Points: ",
                      "Posterior Means and ",
                      conf.int*100,
                      "% CIs",sep="")
  if(!is.null(eval(x$call$object)$desc))
    mainString <- paste(eval(x$call$object)$desc,"\n",mainString)
  
  plot(y=c(1-.5,x$n+.5),
       yaxs="i",
       x=1.02*range(xq),
       xaxs="i",
       xlab="",ylab="",
       axes=FALSE,
       type="n",
       ...)
  mtext(mainString,side=3,line=3)
  
  xLims <- 1.01*range(xq)
  
  for (i in 1:x$n){
    if(!showAllNames){
      if((x$n <= 30)||(i %in% as.integer(seq(1,x$n,length=30)))){
        text(x=textLoc,
             y=i,
             labels=legis.name[indx[i]],
             adj=1,xpd=NA)
        lines(x=xLims,y=rep(i,2),lty=1,lwd=.5,col=gray(.45))
      }
    }
    else{
      text(x=textLoc,
           y=i,
           cex=.55,
           labels=legis.name[indx[i]],
           adj=1,xpd=TRUE)
      lines(x=xLims,y=rep(i,2),lty=1,lwd=.5,col=gray(.45))
    }
    
    lines(y=c(i,i),x=xq[indx[i],],lwd=2)
    if (is.null(party)){
      points(y=i,x=xm[indx[i]],col="red",pch=19,xpd=NULL)
    }
    else{
      tbl <- table(party, exclude=NULL)
      cl <- rainbow(length(tbl))
      pt <- xm[indx[i]]
      grp <- match(party[indx[i]],names(tbl))
      points(y=i,
             x=pt,
             pch=19,col=cl[grp],xpd=NULL)    
    }
  }  
  ##par(ps=8)
  ##par(ps=10)
  axis(1)
  axis(3)
  
  par(exispar)
}


plot2d <- function(x,
                   d1=1,
                   d2=2,
                   burnin=NULL,
                   overlayCuttingPlanes=FALSE,
                   ...){
  
  if(!inherits(x, "ideal"))
    stop("plot.ideal only available for objects of class ideal")
  
  if(is.null(burnin))
    keep <- checkBurnIn(x,eval(x$call$burnin,envir=.GlobalEnv))
  else
    keep <- checkBurnIn(x,burnin)

  oCP <- overlayCuttingPlanes   ## local copy of overlayCuttingPlanes
  if(overlayCuttingPlanes){
    if(is.null(x$beta)){
      cat("Item parameters were not stored in ideal object\n")
      cat("No cutting planes will be plotted\n")
      oCP <- FALSE
    }
    if(x$d>2){
      cat("overlay of cutting planes only defined for 2d fits\n")
      oCP <- FALSE
    }
  }
  
  mainString <- paste("Ideal Points:",
                      "Posterior Means")
  if(!is.null(eval(x$call$object)$desc))
    mainString <- paste(eval(x$call$object)$desc,"\n",mainString)
  
  checkD(x,d1)
  checkD(x,d2)
  
  if(d1==d2)
    stop("can't do 2 dimensional summaries of the same dimension\n")
  
  if(is.null(burnin)){    ## use x bar in ideal object
    xm1 <- x$xbar[,d1]
    xm2 <- x$xbar[,d2]
  }
  else{
    xm1 <- apply(x$x[keep,,d1],2,mean)  ## posterior means
    xm2 <- apply(x$x[keep,,d2],2,mean)
  }
  
  if(oCP){
    if(is.null(burnin)){    ## use betabar in ideal object
      b1Bar <- x$betabar[,d1]
      b2Bar <- x$betabar[,d2]
      alphaBar <- x$betabar[,(x$d+1)]
    }
    else{
      bKeep <- x$beta[keep,,,drop=FALSE]
      betaBar <- apply(bKeep,c(2,3),mean)
      b1Bar <- betaBar[,d1]
      b2Bar <- betaBar[,d2]
      alphaBar <- betaBar[,x$d]
    }
  }

  ## get a copy of the rollcall object used by ideal object
  tmpObject <- dropRollCall(eval(x$call$object),
                            eval(x$call$dropList))
  party <- tmpObject$legis.data$party  ## extract party info
  rm(tmpObject)
  if(is.null(party)){
    plot(x=xm1,y=xm2,
         main=mainString,
         type="p",
         xlab=paste("Dimension ",as.character(d1),sep=""),
         ylab=paste("Dimension ",as.character(d2),sep=""),
         xpd=NULL,
         ...)
    if(oCP){
      for(j in 1:x$m)
        abline(a=-alphaBar[j]/b2Bar[j],
               b=-b1Bar[j]/b2Bar[j],
               col=gray(.45))
    }
  }
  else{   ## we have party info
    plot(x=xm1,
         y=xm2,
         main=mainString,
         type="n",
         xlab=paste("Dimension ",as.character(d1),sep=""),
         ylab=paste("Dimension ",as.character(d2),sep=""),
         ...)
    if(overlayCuttingPlanes){
      for(j in 1:x$m)
        abline(a=-alphaBar[j]/b2Bar[j],
               b=-b1Bar[j]/b2Bar[j],
               col=gray(.45),lwd=.5)
    }
    
    tbl <- table(party, exclude=NULL)
    cl <- rainbow(length(tbl))
    for (i in 1:length(tbl)){
      thisParty <- party==names(tbl)[i]
      points(y=xm2[thisParty],
             x=xm1[thisParty],
             pch=16,col=cl[i],
             xpd=NULL)
    }
  }
  
  invisible(NULL)
}

tracex <- function(object,
                   legis=NULL,
                   d=1,
                   conf.int=0.95,
                   multi=FALSE,
                   burnin=NULL,
                   span=.25,
                   legendLoc="topright"){

  warnOption <- options()$warn
  options(warn=-1)
  if(!inherits(object, "ideal"))
    stop("object passed to function tracex must be of class ideal")

  if(is.null(d))
    stop("default value must be supplied for dimension to trace\n")

  if(length(d)>2)
    stop("tracex only works with up to 2 dimensions\n")

  if(!is.character(legis))
    stop("legis must be character (names of legislators)\n")

  Rv <- as.numeric(version$major) + .1*as.numeric(version$minor)
  if(Rv>=2.4)
    old.par <- par(no.readonly=TRUE)  
  else
    old.par <- par()


  ## try matching names
  legis.names <- as.vector(dimnames(object$x)[[2]])
  nLegis <- length(legis)
  p <- list()
  for(j in 1:nLegis){
    p[[j]] <- grep(pattern=paste("^",legis[j],sep=""),
                   x=legis.names)
    if(length(p[[j]])==0){
      cat(paste("could not find legislator",legis[j],"\n"))
      p[[j]] <- NA
    }
    if(!is.null(p[[j]]) & length(p[[j]])>object$d){
      cat(paste("no unique match for legislator",legis[j],"\n"))
      cat("try providing more of the unique identifier for the legislator\n")
      p[[j]] <- NA
    }
  }

  ## process list of matches
  plotName <- rep(NA,length(p))
  for(j in 1:nLegis){    
    if(length(d)==1){
      if(!is.na(p[[j]])){
        p[[j]] <- p[[j]][1]
      }
    }
    if(!is.na(p[[j]])){
      foo <- pmatch(x=legis[j],
                    table=as.vector(dimnames(object$xbar)[[1]]))
      if(!is.na(foo))
        plotName[j] <- dimnames(object$xbar)[[1]][foo]
      cat(paste("matching",
                legis[j],
                "with",
                plotName[j],"\n"))
    }
  }
  p <- p[!is.na(p)]
  plotName <- plotName[!is.na(plotName)]
  names(p) <- plotName
  nLegis <- length(p)

  if(is.null(burnin))
    keep <- checkBurnIn(object,eval(object$call$burnin,envir=.GlobalEnv))
  else
    keep <- checkBurnIn(object,burnin)
  start <- as.numeric(dimnames(object$x)[[1]])[keep][1]
  
  ## #######################################################
  ## one-dimensional stuff
  ## #######################################################
  if(length(d)==1){
    options(warn=0)
    checkD(object,d)

    if(span<=0 | span>=1)
      stop("span must be between 0 and 1")
    
    if((conf.int<=0)||(conf.int>=1))
      stop("conf.int must be between 0 and 1")
    
    rw <- 3
    if (nLegis < 3)
      rw <- nLegis
    count <- 0
    if(rw < 4 & dev.interactive())
      par(mfrow=c(rw,1))
    if(length(legendLoc)==1)
      legendLoc <- rep(legendLoc,nLegis)

    
    for (i in 1:nLegis){
      meat <- object$x[keep,p[[i]],d]
      iter <- as.numeric(dimnames(object$x)[[1]])[keep]
      par(mar=c(4, 4, 4, 2) + 0.1)      

      mainText <- plotName[i]
      if(object$d>1)
        mainText <- paste(mainText,
                          ", Dimension ",d,sep="")
      plot(y=meat,
           x=iter,
           las=1,
           type="l",
           xlab="Iteration",ylab="",
           main=mainText)
      runmean <- cumsum(meat)/1:length(iter)
      lines(iter,runmean,col="red",lwd=3)
      
      lf <- loess(meat~iter,span=span)   ## loess overlay
      lines(iter,predict(lf),col="blue",lwd=3)
      
      xbar <- mean(meat)
      q <- c((1-conf.int)/2, 1-((1-conf.int)/2))
      q <- quantile(meat,q)
      
      abline(h=xbar,lwd=3,col="grey")    ## posterior mean
      abline(h=q[1],lty=2,col="grey")    ## confidence intervals
      abline(h=q[2],lty=2,col="grey")

      count <- count + 1

      ## do legend
      if(!is.null(legendLoc[i]))
        legend(x=legendLoc[i],
               bg="white",
               ncol=1,
               legend=c("Trace",
                 "Cumulative Mean",
                 paste("Moving Average (loess, span=",
                       round(span,2),
                       ")",
                       sep=""),
                 "Posterior Mean",
                 paste(round(100*conf.int),"% Confidence Interval",sep="")),
               lty=c(1,1,1,1,2),
               lwd=c(2,3,3,3,2),
               col=c("black","red","blue","grey","grey"),
               yjust=0,cex=.65)
      
      ## prompt user for more plots if we are in interactive mode
      ##cat("dev.interactive returns ",
      ##    dev.interactive(),"\n")
      ##cat(paste("count=",count,"\n"))
      ##cat(paste("nLegis=",nLegis,"\n"))
      if((count==3) & (nLegis > 3) & (dev.interactive())){
        count <- 0
        readline("Press return/enter to see next set of plots: ")
      }
    }
  }
  
  ## ###################################################################
  ## two-dimensional traceplots
  if(length(d)==2){
    goodD <- d %in% (1:object$d)
    if(!all(goodD))
      stop("invalid dimensions requested in tracex")
    
    col <- rainbow(nLegis)       ## colors
    meat <- list()               ## container for iters to plot
    for(i in 1:nLegis){
      xTraces <- object$x[keep,p[[i]],d[1]]
      yTraces <- object$x[keep,p[[i]],d[2]]
      meat[[i]] <- list(x=xTraces,
                        y=yTraces,
                        col=col[i])
    }
    
    if(!multi){                         ## plot all 2d traces at once
      xRange <- range(unlist(lapply(meat,function(x)x$x)),na.rm=TRUE)
      yRange <- range(unlist(lapply(meat,function(x)x$y)),na.rm=TRUE)

      layout(mat=matrix(c(1,2),1,2,byrow=TRUE),
             widths=c(.7,.3))
      
      par(mar=c(4,4,1,1))
      plot(x=xRange,y=yRange,
           type="n",
           axes=FALSE,
           xlab=paste("Dimension",d[1]),
           ylab=paste("Dimensions",d[2]))
      axis(1,las=1)
      axis(2,las=1)
      lineFunc <- function(obj){
        lines(obj$x,obj$y,col=obj$col)
        points(obj$x[1],obj$y[1],pch=1,col="black",cex=2)
        npoints <- length(obj$x)
        points(obj$x[npoints],obj$y[npoints],
               pch=16,col="black",cex=2)
      }
      
      lapply(meat,lineFunc)
      
      mtext(side=3,outer=FALSE,line=-.5,cex=.75,
            paste("Two-dimensional trace plots, MCMC iterations,\n",
                  eval(object$call$object)$desc,
                  ", Iterations ",
                  start," to ",
                  object$call$maxiter," thinned by ",
                  object$call$thin,sep=""))
      
      ## legend plot, 2nd panel
      par(mar=c(3,0,1,0))
      plot(x=c(0,1),
           y=c(.5,nLegis+.5),
           las=1,
           xlab="",ylab="",xaxs="i",yaxs="i",
           axes=FALSE,type="n")
      ## loop to show lines and legislator name
      for(i in 1:nLegis){
        lines(x=c(0,.15),
              y=rep(i,2),
              lwd=2,
              col=col[i])
        text(x=.25,
             y=i,
             cex=.75,
             plotName[i],
             adj=0)
      }
    }
    
    if(multi){                ## multiple panels, one per legislator
      par(mfrow=c(2,2))
      count <- 0
      for(i in 1:nLegis){
        plot(x=meat[[i]]$x,
             y=meat[[i]]$y,
             type="l",
             las=1,
             xlab=paste("Ideal Point, Dimension ",d[1],sep=""),
             ylab=paste("Ideal Point, Dimension ",d[2],sep=""))
        title(plotName[i])
        count <- count + 1
      }
      if(count==3 & dev.interactive()){
        count <- 0
        readline("Press any key to see next set of plots:  ")
      }
    }
    
  }            ## end 2 dimensional stuff
  
  par(old.par)
  options(warn=warnOption)
  
  invisible(NULL)
}
  

## compute margins in a roll call object, add to the object in vote.data
computeMargins <- function(object,dropList=NULL){
  if(!inherits(object,"rollcall"))
    stop("margins only works on object of class rollcall.")

  tmpRollCall <- object
  if(!is.null(dropList)){
    tmpRollCall <- dropRollCall(object,dropList)
  }

  tab <- t(apply(convertCodes(tmpRollCall),
                 2,
                 marginfunc))
  tab <- cbind(tab,
               apply(tab,1,whichMinMargin))

  rownames(tab) <- dimnames(tmpRollCall$votes)[[2]]
  colnames(tab) <- c("Yea","Nay","NA","Min")
  tmpRollCall$voteMargins <- tab
    
  tmpRollCall
}

marginfunc <- function(x){
  ok <- !is.na(x)
  z <- c(sum(x[ok]==1),   ## Yeas
         sum(x[ok]==0),   ## Nays
         sum(!ok))        ## Missing
  z
}


marginWithCodes <- function(x,codes){
  n <- length(codes)
  tab <- rep(0,n)
  for(i in 1:n){
    if(is.list(codes))
      tab[i] <- sum(x %in% codes[[i]])
    else
      tab[i] <- sum(x %in% codes[i])
  }
  out <- c(tab,
           sum(tab),
           tab/sum(tab)*100)
  out
}


minMargin <- function(x){
  z <- rep(NA,2)
  ok <- !is.na(x)
  z[1] <- sum(x[ok]==0)
  z[2] <- sum(x[ok]==1)
  z <- z/sum(ok)
  out <- min(z)
  out
}

whichMinMargin <- function(x){
  x[which.min(x[1:2])]
}

## read Keith Poole and Howard Rosenthal formatted data
readKH <- function(file,
                   dtl=NULL,
                   yea=c(1,2,3),
                   nay=c(4,5,6),
                   missing=c(7,8,9),
                   notInLegis=0,
                   desc=NULL,
                   debug=FALSE){

  cat("Attempting to read file in Keith Poole/Howard Rosenthal (KH) format.\n")
  warnLevel <- options()$warn
  options(warn=-1)
  data <- try(readLines(con=file),silent=TRUE)
  if(inherits(data,"try-error")){
    cat(paste("Could not read",file,"\n"))
    return(invisible(NULL))
  }
  options(warn=warnLevel)
  
  cat("Attempting to create roll call object\n")
  voteData <- substring(data,37)
  n <- length(voteData)
  m <- nchar(voteData)[1]
  rollCallMatrix <- matrix(NA,n,m)
  for(i in 1:n){
    rollCallMatrix[i,] <- as.numeric(unlist(strsplit(voteData[i],
                                                     split=character(0))))
  }
  rm(voteData)

  if(!is.null(desc))
    cat(paste(desc,"\n"))
  cat(paste(n,"legislators and",m,"roll calls\n"))
  cat("Frequency counts for vote types:\n")
  tab <- table(rollCallMatrix,exclude=NULL)
  print(tab)

  ## unique numeric identifier for each legislator
  icpsrLegis <- as.numeric(substring(data,4,8))
  
  ## party affiliation
  party <- as.numeric(substring(data,21,23))
  ## convert party to label
  partyfunc <- function(x){
    ##data(partycodes)
    party <- partycodes$party[match(x,partycodes$code)]
    party[party=="Democrat"] <- "D"
    party[party=="Republican"] <- "R"
    party[party=="Independent"] <- "Indep"
    party
  }
  partyName <- partyfunc(party)

  ## convert state ICPSR code to abbreviation
  statename <- function(x){
    ##data(state.info)
    state.info$state[match(x,state.info$icpsr)]
  }
  state <- as.numeric(substring(data,9,10))  ## icpsr code
  KHstateName <- substring(data,13,20)
  stateName <- statename(state)  ## covert to name
  stateAbb <- datasets::state.abb[match(stateName,datasets::state.name)]  ## convert to abbrev
  stateAbb[grep(KHstateName,pattern="^USA")] <- "USA"  ## for presidents

  cd <- as.numeric(substring(data,11,12))
  cdChar <- as.character(cd)
  cdChar[cd==0] <- ""

  ## process legislator names
  lnames <- substring(data,26,36)
  for(i in 1:n){
    lnames[i] <- strip.trailing.space(lnames[i])
    lnames[i] <- strip.after.comma(lnames[i])
  }

  ## finally, produce a tag for each legislator
  legisId <- paste(lnames," (",partyName," ",stateAbb,"-",cdChar,")",sep="")
  legisId <- gsub(x=legisId,pattern="-)",replacement=")")

  ## final check for dups
  ## if we find any, pad with icpsrLegis tag
  if(any(duplicated(legisId))){
    dups <- duplicated(legisId)
    legisId[dups] <- paste(legisId[dups],
                           icpsrLegis[dups])
  }

  ## write legis data
  legis.data <- data.frame(state=stateAbb,
                           icpsrState=state,
                           cd=cd,
                           icpsrLegis=icpsrLegis,
                           party=partyName,
                           partyCode=party)
  dimnames(legis.data)[[1]] <- legisId

  ## do we have a dtl file to read?
  vote.data <- NULL
  if(!is.null(dtl)){
    vote.data <- dtlParser(dtl,debug=debug)
  }
  
  ## finally, call rollcall to assemble working object
  rc <- rollcall(data=rollCallMatrix,
                 yea=yea,
                 nay=nay,
                 missing=missing,
                 notInLegis=notInLegis,
                 legis.names=legisId,
                 legis.data=legis.data,
                 vote.data=vote.data,
                 desc=desc,
                 source=file)
  rc
}

## utility functions
strip.after.comma <- function(x){
  indx <- regexpr(",",x)
  if (indx > 0)
    z <- substring(x,1,indx-1)
  else
    z <- x
  z
}

strip.trailing.space <- function(x){
  indx <- regexpr(" ",x)
  if (indx > 0)
    z <- substring(x,1,indx-1)
  else
    z <- x
  z
}

## read from file, possible web
## readFromFunc <- function(file,debug=TRUE){
##   ## check if this is a URL, starting with such as http, https, or ftp
##   urlStrings <- c("http","ftp")
##   netFile <- any(!is.na(pmatch(urlStrings,file)))
##   if(netFile){
##     if(debug)
##       cat(paste("we appear to have a URL:",file,"\n"))
##     slashes <- gregexpr(pattern="/",text=file)[[1]]
##     if(length(slashes)<3)
##       cat(paste("dubious URL, it has only",length(slashes),"slashes\n"))
##     hostname <- substring(file,slashes[2]+1,slashes[3]-1)
##     if(debug)
##       cat(paste("hostname is",hostname,"\n"))

##     ## check that we can actually resolve the hostname
##     w <- options()$warn
##     options(warn=-1)
##     goodNet <- NULL
##     haveNSL <- exists("nsl")
##     if(haveNSL){
##       goodNet <- nsl(hostname)
##     else
      
##     if(debug)
##       if(is.null(goodNet))
##         cat(paste("nsl on",hostname,"returned NULL\n"))
##       else
##         cat(paste("nsl on",hostname,"returned",goodNet,"\n"))
##     if(is.null(goodNet)){
##       options(warn=w)
##       cat("Could not resolve the URL you provided.\n")
##       cat("Check the URL or your internet connection.\n")
##       return(invisible(NULL))
##     }
##     options(warn=w)
##   }

##   ## now actually try to read the data  
##   readResults <- try(readLines(file))
##   if(inherits(readResults, "try-error")){
##     cat(paste("readKH error: could not read from",file,"\n",
##               "execution terminating\n"))
##     data <- NULL
##   }
##   else{
##     data <- readResults
##     nRecs <- length(data)
##       cat(paste("read",file,"ok with",nRecs,"records\n"))
##     }
##   data
## }

dateExtract <- function(string){
  theMonths <- c("JANUARY","FEBRUARY","MARCH",
                 "APRIL","MAY","JUNE",
                 "JULY","AUGUST","SEPTEMBER",
                 "OCTOBER","NOVEMBER","DECEMBER")
  searchStringMonths <- paste(theMonths,collapse="|")
  foo <- unlist(strsplit(string,split=" "))
  foo <- foo[foo!=""]
  nFoo <- length(foo)
  whereMonth <- grep(pattern=searchStringMonths,foo)
  out <- ""
  if(length(whereMonth)==1){
    dateString <- foo[whereMonth:nFoo]
    dateString <- gsub(x=dateString,pattern=",",replacement="")
    ## dateString should be MONTH, DAY, YEAR
    month <- match(foo[whereMonth],table=theMonths)
    out <- paste(dateString[3],month,dateString[2],sep="-")
  }
  out
}

descriptionExtract <- function(recs){
  foo <- substring(recs,13)
  foo <- paste(foo,collapse="")
  foo <- gsub(foo,pattern="\n",replacement="")
  foo <- gsub(foo,pattern='[[:space:]]+',replacement=" ")
  foo <- gsub(foo,pattern='[[:space:]]$',replacement="")
  foo
}

## parse K&H dictionary files
dtlParser <- function(file,debug=TRUE){
  cat(paste("attempting to read dtl file",file,"\n"))
  warnLevel <- options()$warn
  options(warn=-1)
  data <- try(readLines(con=file),silent=TRUE)
  if(inherits(data,"try-error")){
    cat(paste("Could not read",file,"\n"))
    return(invisible(NULL))
  }
  options(warn=warnLevel)
  
  out <- NULL
  if(!is.null(data)){
    number <- as.numeric(substring(data,1,4))
    m <- unique(number)
    date <- rep(NA,length(m))
    description <- rep("",length(m))

    for(j in m){
      if(debug)
        cat(paste("dtlParser: processing record",j,"\n"))
      theRecs <- data[number==j]
      nRecs <- length(theRecs)

      ## extract the date
      date[j] <- dateExtract(theRecs[1])

      ## extract descriptive text
      if(nRecs>2)
        description[j] <- descriptionExtract(theRecs[3:nRecs])
    }
    out <- data.frame(date=date,
                      description=description)
    out$date <- as.Date(out$date,format="%Y-%m-%d")
  }

  out
}

ntable <- function(x,y=NULL,percent=1,digits=2,row=FALSE,col=FALSE){
    if (is.null(y)){
        cat(rownames(table(x)),"\n",sep="\t")
        cat(round(table(x)/sum(table(x)),digits),"\n",sep="\t")
        cat(table(x),"\n",sep="\t")
    }
    else{
        if(length(x)!=length(y))
            stop("x and y do not have same length\n")
        tab <- table(x,y)
        pt <- round(prop.table(tab,percent),digits)
        k <- !is.na(x) & !is.na(y)
        if (row) {
            pt <- cbind(pt,round(table(x)/sum(k),digits))
            tab <- cbind(tab,table(x[k]))
        }
        cat("",colnames(tab),"\n",sep="\t")
        j <- 0
        for (i in rep(c(0,1),nrow(tab))) {
            j <- j + 1 - i
            if (i==0) {
                cat(row.names(pt)[j],pt[j,],"\n",sep="\t")
            }
            else {
                cat("",tab[j,],"\n",sep="\t")
            }
        }
        if (col) {
            cat("\n")
            cat("",round(table(y[!is.na(x)])/sum(k),digits),"\n",sep="\t")
            cat("",table(y[!is.na(x)]),sum(k),"\n",sep="\t")}
    }

    invisible(NULL)
}

## IDEAL
ideal <- function(object,
                  codes=object$codes,
                  dropList=list(codes="notInLegis",lop=0),
                  d=1,
                  maxiter=10000,
                  thin=100,
                  burnin=5000,
                  impute=FALSE,
                  normalize=FALSE,
                  meanzero=normalize,
                  priors=NULL,
                  startvals="eigen",
                  store.item=FALSE,
                  file=NULL,
                  verbose=FALSE,
                  use.voter=NULL){

  cat("ideal: analysis of roll call data via Markov chain Monte Carlo methods.\n\n")

  ## calling args, some evaluated if symbols, for future use
  cl <- match.call()
  
  if(is.null(cl$d) | is.symbol(cl$d))
    cl$d <- eval(d,parent.frame())
  if(is.null(cl$codes))
    cl$codes <- codes
  if(is.null(cl$dropList))
    cl$dropList <- dropList
  if(is.null(cl$maxiter) | is.symbol(cl$maxiter))
    cl$maxiter <- eval(maxiter,parent.frame())
  if(is.null(cl$thin) | is.symbol(cl$thin))
    cl$thin <- eval(thin,parent.frame())
  if(is.null(cl$burnin) | is.symbol(cl$burnin))
    cl$burnin <- eval(burnin,parent.frame())
  if(is.null(cl$impute))
    cl$impute <- impute
  if(is.null(cl$store.item) | is.symbol(cl$store.item))
    cl$store.item <- eval(store.item,parent.frame())
  if(is.null(cl$normalize))
    cl$normalize <- normalize
  if(is.null(cl$verbose))
    cl$verbose <- verbose
  
  mda <- FALSE
  
  ## check validity of user arguments
  if (!("rollcall" %in% class(object)))
    stop("object must be of class rollcall")
  if(((d%%1) != 0) || (d<1)){
    stop("d is not a positive integer")
  }

  if(((thin%%1)!=0) || (thin<1)) {
    stop("thin is not a positive integer")
  }

  if(((maxiter%%1)!=0) || (maxiter<1)) {
    stop("maxiter is not a positive integer")
  }

  if(!is.list(dropList))
    stop("dropList must be a list")

  if(!is.list(codes))
    stop("codes must be a list")

  ##check iterations and thinning
  if ((maxiter%%thin)!=0) {
    stop("maxiter must be a multiple of thin")
  }

  if ((burnin%%thin)!=0) {
    stop("burnin must be a multiple of thin")
  }

  if (burnin >= maxiter)
    stop("burnin must be less than maxiter")

  if(!is.null(normalize) & d>1){
    cat("normalize option is only meaningful when d=1\n")
  }

  if(normalize != meanzero){
    normalize <- meanzero
    cat("meanzero option is being phased out; normalize provides the same functionality\n")
    cat(paste("For now, we will use your supplied value of meanzero, proceeding with normalize=",
              meanzero,"\n"))
  }
              
  ## pre-process rollcall object
  tmpObject <- object
  if(!is.null(codes)){
    tmpObject$codes <- codes
    if(checkCodes(tmpObject$codes))
      stop("supplied codes fail redundancy checks")
  }
  if(!is.null(dropList)){
    if(verbose)
      cat(paste("Subsetting rollcall object",
                as.name(cl$object),
                "using dropList\n"))
    y <- dropRollCall(tmpObject,dropList)  ## any subsetting to do?
  }
  else
    y <- tmpObject
  rm(tmpObject)
  
  n <- dim(y$votes)[1]
  m <- dim(y$votes)[2]
  legis.names <- dimnames(y$votes)[[1]]
  vote.names <- dimnames(y$votes)[[2]]

  if (!is.null(use.voter)) {
      if (!is.vector(use.voter))
          stop("use.voter must be a vector of length n")
      if (n != length(use.voter))
          stop("use.voter must be a vector of length n")
  }

  ## map roll call votes into binary format required by ideal
  if(verbose){
    printCodes(codes)
    cat("\n")
  }
  
  if(checkVotes(y$votes,codes))
    stop("rollcall: can't map all votes using supplied codes")

  v <- convertCodes(y,codes)  ## convert to zeros and ones and NAs
  
  ## using a file for storage
  usefile <- !is.null(file)      

  ## check to see how much information will need to be stored
  numrec <- (maxiter-burnin)/thin+1
  if (interactive() & verbose &
      ((store.item)&&((n+m)*d*numrec>2000000))
      ||
      ((!store.item)&&((n*d*numrec)>2000000))
      ){
    ans <- readline(paste("The current call to ideal will result in a large object that\n",
                          "will take up a large amount of memory.  Do you want to\n",
                          "continue with the current configuation? (y/n): ",
                          sep=""))
    
    if ((substr(ans, 1, 1) == "n")||(substr(ans, 1, 1) == "N"))
      stop("User terminated execution of ideal.")
  }

  if (interactive() & verbose & numrec>1000) {
    ans <- readline(paste("You are attempting to save ",numrec," iterations.  This\n",
                          "could result in a very large object and cause memory problems.\n",
                         "Do you want to continue with the current call to ideal? (y/n): ",
                         sep=""))
    
    if ((substr(ans, 1, 1) == "n")||(substr(ans, 1, 1) == "N"))
      stop("User terminated execution of ideal.")
  }

  cat(paste("Ideal Point Estimation\n\nNumber of Legislators\t\t",
              n,"\nNumber of Items\t\t\t", m, "\n\n"))
  xp <- xpv <- bp <- bpv <- NULL

  ####################################################################
  ## check priors
  ####################################################################
  if(verbose)
    cat("checking for any user-supplied priors...\n")
  if(!is.null(priors)){
    if(!is.list(priors))
      stop("priors must be a list")
    
    if(all(unlist(lapply(priors,is.null))))
      stop("priors supplied in a list, but all elements are NULL")
    
    if(sum(unlist(lapply(priors,is.na)))>0)
      stop("priors contain missing values, which is not allowed")
  
    ## now check individual elements of prior list
    if(!is.null(priors$xp)){
      if(length(priors$xp)==1)     ## user supplied a scalar
        xp <- matrix(priors$xp,n,d)
      ## coerce a vector to a matrix
      if(length(priors$xp)>1 & d==1 & !is.matrix(priors$xp))
        xp <- matrix(priors$xp,n,d)
      if(is.matrix(priors$xp))
        xp <- priors$xp
    }
    else{
      if(verbose)
        cat("no prior means supplied for ideal points,\n",
            "setting to default of 0\n")
      xp <- matrix(0,n,d)
    }
    
    if(!is.null(priors$xpv)){
      if(length(priors$xpv)==1)    ## user supplied a scalar
        xpv <- matrix(priors$xpv,n,d)
      ## coerce a vector to a matrix
      if(length(priors$xpv)>1 & d==1 & !is.matrix(priors$xpv))
        xpv <- matrix(priors$xpv,n,d)
      if(is.matrix(priors$xpv))
        xpv <- priors$xpv
    } else {
      if(verbose)
        cat("no prior precisions supplied for ideal points,\n",
            "setting to default of 1\n")
      xpv <- matrix(1,n,d)
    }
    
    if(!is.null(priors$bp)){
      if(length(priors$bp)==1)    ## user supplied a scalar
        bp <- matrix(priors$bp,m,d+1)
      if(is.matrix(priors$bp))
        bp <- priors$bp
    } else {
      if(verbose)
        cat("no prior means supplied for item parameters,\n",
            "setting to default to 0\n")
      bp <- matrix(0,m,d+1)
    }
    
    if(!is.null(priors$bpv)){
      if(length(priors$bpv)==1){   ## user supplied a scalar
        bpv <- matrix(priors$bpv,m,d+1)
      }
      if(is.matrix(priors$bpv)){
        bpv <- priors$bpv
      }
    } else {
      if(verbose){
        cat("no prior precisions supplied for item parameters,\n",
            "setting to default of .04\n")
      }
      bpv <- matrix(.04,m,d+1)
    }
    
    if (((nrow(xp) != n)||(ncol(xp) != d)) || ((nrow(xpv)!=n)||(ncol(xpv)!=d))) {
      stop("Dimensions of xp or xpv not n by d")
    }
    
    if (((nrow(bp) != m)||(ncol(bp) != (d+1))) || ((nrow(bpv)!=m)||(ncol(bpv)!=(d+1)))) {
      stop("Dimensions of bp or bpv not m by d+1")
    }
  }
  
  ## ##################################################################
  ## if we get this far with priors still NULL
  ## then revert to defaults
  ## ##################################################################
  if(is.null(xp)){
    if(verbose)
      cat("setting prior means for ideal points to all zeros\n")
    xp <- matrix(0,n,d)
  }
  if(is.null(xpv)){
    if(verbose)
      cat("setting prior precisions for ideal points to all 1\n")
    xpv <- matrix(1,n,d)
  }
  if(is.null(bp)){
    if(verbose)
      cat("setting prior means for item parameters to all zeros\n")
    bp <- matrix(0,m,d+1)
  }
  if(is.null(bpv)){
    if(verbose)
      cat("setting prior precisions for item parameters to all 0.04\n")
    bpv <- matrix(0.04,m,d+1)
  }
  
  xp <- as.vector(xp)
  xpv <- as.vector(xpv)
  bp <- as.vector(bp)
  bpv <- as.vector(bpv) 

  ################################################################
  ## check for start values - create if not supplied
  ################################################################
  if(verbose)
    cat("\nchecking start values...\n")
  xstart <- NULL
  bstart <- NULL
  options(warn=-1)

  if(!is.list(startvals)){
    if(startvals=="eigen" | is.null(startvals)){
      xstart <- x.startvalues(v,d=d,verbose=verbose)
      bstart <- b.startvalues(v,xstart,d=d,verbose=verbose)
      bstart <- ifelse(abs(bstart - bp) < 2/sqrt(bpv),
                       bstart,
                       bp + 2*sign(bstart-bp)/sqrt(bpv))
    }

    if(startvals=="random"){
      if(verbose)
        cat("generating start values for ideal points by iid sampling from N(0,1)\n")
      xstart <- matrix(rnorm(n*d),n,d)
      bstart <- b.startvalues(v,xstart,d=d,verbose=verbose)
      bstart <- ifelse(abs(bstart - bp) < 2/sqrt(bpv),
                       bstart,
                       bp + 2*sign(bstart-bp)/sqrt(bpv))
    }
  }

  ## user has passed something in startvals
  if(is.list(startvals)){
    cat("found user-supplied list in startvals\n")
    cat("starvals is a list containing:\n")
    print(names(startvals))
    
    if(!is.null(startvals$x)){
      if(length(startvals$x) != n*d)
        stop("supplied start values for x is not n by d")
      if(d==1)
        xstart <- matrix(startvals$x,ncol=1)
      else
        xstart <- startvals$x
      if (sum(is.na(xstart))!=0)
        stop("xstart contains missing values")
    }

    if(!is.null(startvals$b)){
      if(length(startvals$b) != m*(d+1))
        stop("length of bstart not m by d+1")
      bstart <- startvals$b
      if(sum(is.na(bstart))!=0)
        stop("bstart contains missing values")
    } 
  } 

  ## final check
  if(is.null(xstart)){
    cat("no user-supplied start values found\n")
    xstart <- x.startvalues(v,d,verbose=TRUE)
  }
  if(is.null(bstart)){
    bstart <- b.startvalues(v,xstart,d=d,verbose=verbose)
    bstart <- ifelse(abs(bstart - bp) < 2/sqrt(bpv),
                     bstart,
                     bp + 2*sign(bstart-bp)/sqrt(bpv))
  }

  ## report to user
  if(verbose){
    if(n<501){
      cat("using the following start values for ideal points:\n")      
      print(xstart)
    } else {
      cat("using the following start values for ideal points (summary follows):\n")      
      print(summary(xstart))
    }

    if(m<501){
      cat("using the following start values for item parameters:\n")
      print(bstart)
    } else {
      cat("using the following start values for item parameters (summary follows):\n")
      print(summary(bstart))
    }
  }

  xstart <- as.vector(xstart)
  bstart <- as.vector(bstart)
  
  options(warn=0)

  ##############################################################
  ## end error checking
  ##############################################################

  yToC <- ifelse(is.na(v), 9, v)
  yToC <- as.vector(yToC)
  cat("\nStarting MCMC Iterations...\n")

  ## ############################################
  ## two versions, one with usefile option
  ## ############################################
  if (usefile) {
    if (length(legis.names) == n) {
      cat(paste("\"",c("Iteration",legis.names),"\"", sep="", collapse=","),
          file=file)
    }
    else {
      cat(paste("\"",c("Iteration",paste("x", 1:n, sep="")),"\"",
                sep="", collapse=","),
          file=file)
    }
    
    if (store.item){
      cat(",",
          paste("\"",
                c(paste("b",
                        as.vector(apply(expand.grid(1:m,1:(d+1)),1,paste,collapse=".")),
                        sep=".")),"\"",
                sep="", collapse=","),
          sep="", file=file, append=TRUE)
    }
    cat("\n", file=file, append=TRUE)
    output <- .C("IDEAL",
                 PACKAGE=.package.Name,
                 as.integer(n),           #1
                 as.integer(m),           #2
                 as.integer(d),           #3
                 as.double(yToC),         #4
                 as.integer(maxiter),     #5
                 as.integer(thin),        #6
                 as.integer(impute),      #7
                 as.integer(mda),         #8
                 as.double(xp),           #9
                 as.double(xpv),          #10
                 as.double(bp),           #11
                 as.double(bpv),          #12
                 as.double(xstart),       #13
                 as.double(bstart),       #14
                 xoutput=0,               #15
                 boutput=0,               #16
                 as.integer(burnin),      #17
                 usefile,                 #18
                 as.logical(store.item),              #19
                 as.character(file),      #20
                 as.logical(verbose),     #21
                 as.logical(!is.null(use.voter)),   #22 
                 as.integer(use.voter))   #23
  }
  ## not saving output to file, saving output to memory
  else if (!store.item) {   ## but not saving item parameters
    output <- .C("IDEAL",
                 PACKAGE=.package.Name,
                 as.integer(n), 
                 as.integer(m), 
                 as.integer(d), 
                 as.double(yToC), 
                 as.integer(maxiter), 
                 as.integer(thin), 
                 as.integer(impute),
                 as.integer(mda),
                 as.double(xp), 
                 as.double(xpv), 
                 as.double(bp),
                 as.double(bpv), 
                 as.double(xstart), 
                 as.double(bstart),
                 xoutput=as.double(rep(0,n*d*numrec)),
                 boutput=0,
                 as.integer(burnin),
                 usefile, 
                 as.logical(store.item), 
                 as.character(file),
                 as.logical(verbose), 
                 as.logical(!is.null(use.voter)), 
                 as.integer(use.voter))
  }
  else {
    output <- .C("IDEAL",
                 PACKAGE=.package.Name,
                 as.integer(n), as.integer(m), as.integer(d), as.double(yToC),          
                 as.integer(maxiter), as.integer(thin), as.integer(impute),
                 as.integer(mda),
                 as.double(xp), as.double(xpv), as.double(bp),
                 as.double(bpv), as.double(xstart), as.double(bstart),
                 xoutput=as.double(rep(0,n*d*numrec)),
                 boutput=as.double(rep(0,m*(d+1)*numrec)), 
                 as.integer(burnin),
                 usefile, 
                 as.logical(store.item), 
                 as.character(file),
                 as.logical(verbose), 
                 as.logical(!is.null(use.voter)), 
                 as.integer(use.voter))
  }

  cat("\n")

  ## parse returns from C
  xbar <- NULL
  betabar <- NULL
  if (!usefile) {
    itervec <- seq(burnin,maxiter,by=thin)
    keep <- itervec > burnin

    ## ideal points
    x <- array(output$xoutput,
               c(n,d,numrec))
    
    ## reshape to iteration first format
    x <- aperm(x,c(3,1,2))   
    dimnames(x) <- list(itervec,
                        legis.names,
                        paste("D",1:d,sep=""))
    if(verbose)
      cat("...computing posterior means for ideal points...")
    xbar <- getMean(keep,x)
    if(verbose)
      cat("done\n")

    ###############################################################
    ## item parameters
    if(store.item){
      b <- array(output$boutput,c(m,d+1,numrec))  ## votes by parameters by iters
      dimnames(b) <- list(vote.names,
                          c(paste("Discrimination D",1:d,sep=""),
                            "Difficulty"),
                          itervec)
      ## reshape to iteration first format
      b <- aperm(b,c(3,1,2))                      ## iters by votes by parameters
      if(verbose)
      cat("...computing posterior means for item parameters...")
      betabar <- getMean(keep,b)
      if(verbose)
        cat("done\n")
    } else {
      b <- NULL
    }
  } else {   ## output went to a file
    b <- x <- NULL
  }
 
  ## wrap up for return to user
  out <- list(n=n,m=m,d=d,
              codes=codes,
              x=x,
              beta=b,
              xbar=xbar,
              betabar=betabar,
              call=cl)

  class(out) <- c("ideal")

  ## and, finally, if the user wanted meanzero
  if(normalize){
    if(verbose)
      cat("...normalizing output (post-processing)...")
    out <- postProcess(out,
                       constraints="normalize")
    if(verbose)
      cat("done\n")
  }  
  return(out) 
}

x.startvalues <- function(x,d,scale=TRUE,constraint=NULL,verbose=FALSE){
  if(verbose)
    cat("will use eigen-decomposition method to get start values for ideal points...")

  ## from Jong Hee Park
  row.mean <- apply(x, 1, mean, na.rm=TRUE)
  col.mean <- apply(x, 2, mean, na.rm=TRUE)
  dc1 <- sweep(x, 1, row.mean)
  dc2 <- sweep(dc1, 2, col.mean)
  dc <- dc2 + mean(x, na.rm = T)
  
  r <- cor(t(dc),use="pairwise")
  r[is.na(r)] <- 0
  e <- eigen(r)
  v <- e$vectors[,1:d]
  v <- as.matrix(v)
  if(scale){
    for(i in 1:d){
      v[,i] <- v[,i]*sqrt(e$value[i])
    }
  }

  if (!is.null(constraint)) {
    v <- predict(lm(constraint ~ v), newdata=as.data.frame(v)) 
  }
  if(verbose)
    cat("done\n")
  return(v)
}

probit <- function(y,x){
  glmobj <- glm(y ~ x,
                family=binomial(link=probit))
  b <- coef(glmobj)
  k <- length(b)
  b <- b[c(2:k,1)]   ## put intercept last
  b
}

b.startvalues <- function(v,x,d,verbose=FALSE){
  m <- dim(v)[2]
  if(verbose)
    cat(paste("running",
              m,
              "vote-specific probit GLMs\n",
              "for start values for item/bill parameters\n",
              "conditional on start values for ideal points..."))
  
  b <- matrix(NA,m,d+1)
  for(j in 1:m){
    b[j,] <- probit(y=v[,j],x=x)
  }

  ## check for crazy discrimination parameters
  for(j in 1:d){
    bad <- is.na(b[,j])
    b[bad,j] <- 0
  }
  b[,d+1] <- -b[,d+1]     ## flip the sign on the intercepts, make it a difficulty parameter
  if(verbose)
    cat("done\n")
  b
}

## post-process an ideal object
postProcess <- function(object,
                        constraints="normalize",
                        debug=FALSE){
  if(!inherits(object, "ideal"))
    stop("postProcess only defined for objects of class ideal")

  ## not a list of normalizing constraint (i.e., usually "normalize=TRUE" with d=1)
  if(!is.list(constraints)){
    if(constraints=="normalize"){
      ## get constraints needed for mean zero, standard deviation one restriction
      tMat <- getNormalizingTransform(object)  ## coefficients for linear map
      if(debug){
        cat("transformation matrix is:\n")
        print(tMat)
      }
      newObject <- implementConstraints(object,tMat,debug)
    }
  }
  
  if(is.list(constraints))
    newObject <- postProcessAffine(object,constraints,debug)

  return(newObject)
}

getNormalizingTransform <- function(object){
  ##n <- object$n
  ##m <- object$m
  d <- object$d

  offsets <- apply(object$xbar,2,mean)  
  s <- apply(object$xbar,2,sd)

  coefs <- 1/s * diag(d)               ## d by d
  coefs <- rbind(coefs,-offsets/s)     ## d+1 by d
  return(coefs)
}

affineTrans <- function(x,target){
  d <- dim(x)[2]
  x0 <- cbind(x,1)
  if(d>1){
    zeroMat <- 0*x0
    A <- rbind(cbind(x0,zeroMat),
               cbind(zeroMat,x0))
    b <- as.vector(target)
  }
  if(d==1){
    A <- x0
    b <- target
  }
  foo <- solve(A)%*%b
  foo <- matrix(foo,nrow=d+1)
  return(foo)
}

postProcessAffine <- function(object,constraints,debug){
  d <- object$d
  ##n <- object$n
  ##m <- object$m
  nSavedIters <- dim(object$x)[1]
  theIters <- dimnames(object$x)[[1]]
  keep <- checkBurnIn(object,
                      burnin=object$call$burnin)
  
  nCon <- length(constraints)
  if(nCon != (d+1)){
    cat("postProcess is currently only implements as many constraints\n")
    cat("as there are dimensions plus one.\n")
    stop()
  }
  lengthCon <- lapply(constraints,length)
  if(any(lengthCon != d))
    stop("each constraint must have the same number of dimensions as the fitted model")
  
  ## form target matrix
  target <- matrix(NA,d+1,d)
  for(i in 1:(d+1))
    target[i,] <- constraints[[i]]
  if(debug){
    cat("target:\n")
    print(target)
  }
    
  ## form id vector, where are the named legislators in the ideal object?
  legis.names <- dimnames(eval(object$call$object)$votes)[[1]]
  if(is.null(legis.names)){
    cat("can not find legislator names to match against\n")
    cat(paste("either the original roll call object",
              object$call$object,
              "has been deleted\n"))
    cat("or the vote component of the roll call object has been deleted?\n")
    cat("terminating postProcess with an error\n")
    stop()
  }
  
  legis <- names(constraints)
  ind <- rep(NA,d+1) 
  for(i in 1:nCon){
    p <- grep(pattern=paste("^",legis[i],sep=""),
              x=legis.names)
    if(length(p)==0)
      stop("could not find the named legislator in the rollcall object")
    else
      ind[i] <- p
  }
  cat(paste("matching legislators",legis,"\n"))
  
  ## initialize output objects
  newX <- NA * object$x
  dimnames(newX) <- dimnames(object$x)
  
  haveBeta <- eval(object$call$store.item)
  if(haveBeta){
    cat("will also transform item/bill parameters\n")
    newBeta <- NA * object$beta
    dimnames(newBeta) <- dimnames(object$beta)
  }
  
  ## now loop over iterations
  for(iter in 1:nSavedIters){
    cat(paste("post-processing iteration",theIters[iter],"\n"))

    x0 <- object$x[iter,ind,,drop=TRUE]
    x0 <- matrix(x0,d+1,d)
    tMat <- affineTrans(x0,target=target)
    if(debug){
      cat("transformation matrix:\n")
      print(tMat)
    }
    thisX <- cbind(object$x[iter,,],1)
    tmpX <- thisX%*%tMat
    newX[iter,,] <- tmpX
    
    ## now transform beta (and alpha), if available
    if(haveBeta){
      tMatStar <- rbind(t(tMat),
                        c(rep(0,d),1))
      itMat <- try(solve(tMatStar))
      if(!inherits(itMat,"try-error")){
        itMat[1:d,d+1] <- -itMat[1:d,d+1]   ## sign fix for minus intercept
        beta0 <- object$beta[iter,,]
        tmpBeta <- beta0%*%itMat
        newBeta[iter,,] <- tmpBeta
        if(debug){
          cat("inverse transformation matrix:\n")
          print(itMat)
        }
      }
      
      if(debug){
        muPP <- pnorm(cbind(newX[iter,,],-1)%*%t(tmpBeta))
        mu <- pnorm(cbind(object$x[iter,,],-1)%*%t(beta0))
        cat("sanity check, comparison of predictions from original and post-processed:\n")
        print(summary(as.vector(mu-muPP)))
      }
    } 
  }
  
  ## new ideal object
  newObject <- object

  ## new ideal point samples
  newObject$x <- newX
  dimnames(newObject$x) <- dimnames(object$x)
  ## ideal point posterior means
  newObject$xbar <- getMean(keep,newObject$x)
  
  ## for beta?
  if(haveBeta){
    newObject$beta <- newBeta
    dimnames(newObject$beta) <- dimnames(object$beta)
    newObject$betabar <- getMean(keep,newObject$beta)
  }
  
  return(newObject)
}


## implementConstraints
implementConstraints <- function(object,tMat,debug){
  haveBeta <- eval(object$call$store.item)
  if(haveBeta){
    d <- dim(tMat)[2]
    tMatStar <- rbind(t(tMat),
                      c(rep(0,d),1))
    itMat <- try(solve(tMatStar))
    if(inherits(itMat,"try-error"))
      stop("could not compute normalizing transformation for item parameters\n") 
    newBeta <- array(NA,dim(object$beta))
  }

  ## get burnin
  ##if(is.symbol(object$call$burnin)){
  ##  burnin <- eval(object$call$burnin)
  ##} else {
  ##  burnin <- object$call$burnin
  ##}
  keep <- checkBurnIn(object,
                      burnin=eval(object$call$burnin))
  
  nSavedIters <- dim(object$x)[1]
  newX <- array(NA,dim(object$x))
  newObject <- object ## copy ideal object
  
  ## loop over iterations, implementing transformation
  for(iter in 1:nSavedIters){
    thisX <- cbind(newObject$x[iter,,],1)  ## add intercept for translation
    newX[iter,,] <- thisX%*%tMat           ## transformation
    
    ## now transform beta (and alpha), if available
    if(haveBeta){
      beta0 <- object$beta[iter,,]
      tmpBeta <- beta0%*%itMat
      newBeta[iter,,] <- tmpBeta
      
      if(debug){
        muPP <- pnorm(cbind(newX[iter,,],-1)%*%t(tmpBeta))
        mu <- pnorm(cbind(object$x[iter,,],-1)%*%t(beta0))
        cat("sanity check, comparison of predictions from original and post-processed:\n")
        print(summary(as.vector(mu-muPP)))
      }
      
    }
  }

  ## gather up for new ideal object
  newObject$x <- newX
  dimnames(newObject$x) <- dimnames(object$x)
  ## new posterior means
  newObject$xbar <- getMean(keep,newObject$x)
  
  ## for Beta?
  if(haveBeta){
    newObject$beta <- newBeta
    dimnames(newObject$beta) <- dimnames(object$beta)
    newObject$betabar <- getMean(keep,newObject$beta)
  }

  return(newObject)
}

odTest <- function(glmobj,
                   alpha=.05,
                   digits=max(3,getOption("digits")-3))
{
  if(!inherits(glmobj, "negbin"))
    stop("this function only works for objects of class negbin\n")

  if(alpha>1 | alpha<0)
    stop("invalid value for alpha\n")
  
  poissonGLM <- glm(formula=eval(glmobj$call$formula),
                    data=eval(glmobj$call$data),
                    family="poisson")
  ## require(stats)
  llhPoisson <- logLik(poissonGLM)
  llhNB <- logLik(glmobj)
  
  d <- 2*(llhNB - llhPoisson)

  ## n.b., distribution of test-statistics is non-standard
  ## see Cameron and Trivedi 1998 p78
  critval <- qchisq(1-(2*alpha), df = 1)
  pval <- pchisq(d, df = 1, lower.tail=FALSE)/2
  
  cat("Likelihood ratio test of H0: Poisson, as restricted NB model:\n")
  cat("n.b., the distribution of the test-statistic under H0 is non-standard\n")
  cat("e.g., see help(odTest) for details/references\n\n")
  
  cat(paste("Critical value of test statistic at the alpha=",
            round(alpha,digits),
            "level:",
            round(critval,digits),
            "\n"))
  cat(paste("Chi-Square Test Statistic = ",
            round(d,digits),
            "p-value =",
            format.pval(pval,digits=digits),
            "\n"))
  
  invisible(NULL)
}

## function to set restrictions on certain legislators
constrain.legis <- function(obj,
                            dropList=list(codes="notInLegis",lop=0),
                            x, d=1){
  options(warn=-1)
  if(!inherits(obj, "rollcall"))
    stop("object must be of class rollcall")
  
  ## check dimensions of x list items
  if (!is.list(x))
    stop("x must be a list")
  if (length(x)<=d)
    stop("at least d+1 legislators must be constrained")
  options(warn=0)

  ## get working version of roll call object
  cat("constrain.legis: calling dropRollCall to get working version of rollcall object\n")
  rc <- dropRollCall(obj,dropList)
  rc$legis.names <- dimnames(rc$votes)[[1]]
  rc$vote.names <- dimnames(rc$votes)[[2]]
  v <- convertCodes(rc)
  
  n <- nrow(v)
  m <- ncol(v)
  xp <- matrix(rep(0, n*d), nrow=n)
  xpv <- matrix(rep(.01, n*d), nrow=n)
  bp <- matrix(rep(0,m*(d+1)), nrow=m)
  bpv <- matrix(rep(.01, m*(d+1)), nrow=m)

  cat("constrain.legis: generating start values for legislators\n")
  xstart <- x.startvalues(v,d=d)
  xcnst <- xstart*NA
  cat("constrain.legis: implementing constraints\n")
  ## loop over constraints
  for (i in 1:length(x)){
    thisLegis <- names(x)[i]
    if (length(x[[i]])!=d)
      stop("Each element of x must be of length d (dimension of model to be fitted).")
    ind <- pmatch(thisLegis,rc$legis.names)
    if (is.na(ind))
      stop(paste(thisLegis,"was not found in legis.names"))
    cat(paste("matching supplied name",
              thisLegis,"with",
              rc$legis.names[ind],"\n"))
    xp[ind,] <- x[[i]]
    xpv[ind,] <- rep(1e12,d)
    xstart[ind,] <- x[[i]]
    xcnst[ind,] <- x[[i]]
  }
  cat("constrain.legis: re-generating start values for legislators, with constraints\n")
  xstart <- x.startvalues(v,d=d,constraint=xcnst)

  options(warn=-1)
  cat("constrain.legis: generating start values for bill parameters,\n")
  cat("conditional on start values for legislators\n")
  bstart <- b.startvalues(v,xstart,d=d)
  bstart <- ifelse(abs(bstart - bp) < 2/sqrt(bpv),
                   bstart,
                   bp + 2*sign(bstart-bp)/sqrt(bpv))
  options(warn=0)

  return(list(xp=xp,xpv=xpv,bp=bp,bpv=bpv,x=xstart,b=bstart))
}


constrain.items <- function(obj,
                            dropList=list(codes="notInLegis",lop=0),
                            x, d=1){
  options(warn=-1)
  if(!inherits(obj, "rollcall"))
    stop("object must be of class rollcall")
  options(warn=0)

  rc <- dropRollCall(obj,dropList)
  rc$legis.names <- dimnames(rc$votes)[[1]]
  rc$vote.names <- dimnames(rc$votes)[[2]]
  v <- convertCodes(rc)
  
  n <- nrow(v)
  m <- ncol(v)
  xp <- matrix(rep(0, n*d), nrow=n)
  xpv <- matrix(rep(1, n*d), nrow=n)
  bp <- matrix(rep(0,m*(d+1)), nrow=m)
  bpv <- matrix(rep(0.01, m*(d+1)), nrow=m)

  ## check dimensions of x list items
  if (!is.list(x))
    stop("x must be a list")
  options(warn=-1)
  xstart <- x.startvalues(v,d=d)
  bstart <- b.startvalues(v,xstart,d=d)
  options(warn=0)
  for (i in 1:length(x)) {
    if (length(x[[i]])!=(d))
      stop("Each element of x must be of length d (dimension of model to be fitted).")
    ind <- pmatch(names(x)[i],rc$vote.names)
    if (is.na(ind))
      stop(paste(names(x)[i]," was not found in rc$vote.names"))
     cat(paste("matching supplied name",
              names(x)[i],"with",
              rc$vote.names[ind],"\n"))
    bp[ind,] <- c(x[[i]],0)
    bpv[ind,] <- c(rep(1e12,d),0.01)
    bstart[ind,1:d] <- x[[i]]
  }
  return(list(xp=xp,xpv=xpv,bp=bp,bpv=bpv,xstart=xstart,bstart=bstart))
}

  

"vuong" <- function(m1,m2,digits=getOption("digits")){
  ## get predicted probabilities for both models
  m1y <- m1$y
  m2y <- m2$y
  m1n <- length(m1y)
  m2n <- length(m2y)
  if(m1n==0 | m2n==0)
    stop("Could not extract dependent variables from models.")
  
  if(m1n != m2n)
    stop(paste("Models appear to have different numbers of observations.\n",
               "Model 1 has ",m1n," observations.\n",
               "Model 2 has ",m2n," observations.\n",
               sep="")
    )
  
  if(any(m1y != m2y)){
    stop(paste("Models appear to have different values on dependent variables.\n"))
  }
  
  p1 <- predprob(m1)   ## likelihood contributions, model 1, cond on MLEs
  p2 <- predprob(m2)   ## likelihood contributions, model 2
  if(!all(colnames(p1)==colnames(p2))){
    stop("Models appear to have different values on dependent variables.\n")
  }
  

  whichCol <- match(m1y,colnames(p1))  ## which column, matrix of predicted probs
  whichCol2 <- match(m2y,colnames(p2))
  if(!all(whichCol==whichCol2)){
    stop("Models appear to have different values on dependent variables.\n")
  }
                     
  m1p <- rep(NA,m1n)
  m2p <- rep(NA,m2n)
  for(i in 1:m1n){
    m1p[i] <- p1[i,whichCol[i]]  ## pick off correct predicted probabilty, for observed y
    m2p[i] <- p2[i,whichCol[i]]
  }
  
  
  ## gather up degrees of freedom
  k1 <- length(coef(m1))
  k2 <- length(coef(m2))
  
  lm1p <- log(m1p)
  lm2p <- log(m2p)
  m <- lm1p - lm2p  ## vector of log likelihood ratios (diffs of log probabilities)
  
  bad1 <- is.na(lm1p) | is.nan(lm1p) | is.infinite(lm1p)
  bad2 <- is.na(lm2p) | is.nan(lm2p) | is.infinite(lm2p)
  bad3   <- is.na(m) | is.nan(m) | is.infinite(m)
  bad <- bad1 | bad2 | bad3
  neff <- sum(!bad)

  if(any(bad)){
    cat("NA or numerical zeros or ones encountered in fitted probabilities\n")
    cat(paste("dropping these",sum(bad),"cases, but proceed with caution\n"))
  }
  
  aic.factor <- (k1-k2)/neff        
  bic.factor <- (k1-k2)/(2*neff) * log(neff)   
  
  ## 3 test statistics
  v <- rep(NA,3)
  arg1 <- matrix(m[!bad],nrow=neff,ncol=3,byrow=FALSE)
  arg2 <- matrix(c(0,aic.factor,bic.factor),nrow=neff,ncol=3,byrow=TRUE)
  num <- arg1 - arg2
  s <- apply(num,2,sd)
  numsum <- apply(num,2,sum)
  v <- numsum/(s*sqrt(neff))     ## Vuong  
  names(v) <- c("Raw","AIC-corrected","BIC-corrected")
  ##print(v)
  ##print(s)
  ##print(num)
  
  ## bundle up for output
  pval <- rep(NA,3)
  msg <- rep("",3)
  for(j in 1:3){
    if(v[j]>0){
      pval[j] <- 1 - pnorm(v[j])
      msg[j] <- "model1 > model2"
    } else {
      pval[j] <- pnorm(v[j])
      msg[j] <- "model2 > model1"
    }
  }
  out <- data.frame(v,msg,format.pval(pval))
  names(out) <- c("Vuong z-statistic","H_A","p-value")

  ## output
  cat(paste("Vuong Non-Nested Hypothesis Test-Statistic:",
            "\n"))
  cat("(test-statistic is asymptotically distributed N(0,1) under the\n")
  cat(" null that the models are indistinguishible)\n")
  cat("-------------------------------------------------------------\n")
  
  print(out)

  return(invisible(NULL))
}

## seats vote curve class
seatsVotes <- function(x,
                       desc=NULL,
                       method="uniformSwing"){

  xok <- x[!is.na(x)]
  if(length(xok)==0)
    stop("no data to analyze after deleting missings")
  
  if(any(!is.numeric(xok)))
    stop("svCurve only defined for numeric data\n")
  if(any(xok<0))
    stop("negative vote shares not permitted\n")

  if(any(xok>1) & !any(xok>100)){
    cat("proceeding assuming supplied votes are percentages")
    xLocal <- xok/100
  }
  else
    xLocal <- xok

  cl <- match.call()
  if(is.null(cl$method))
    cl$method <- method
  
  m <- 1001
  d0 <- seq(from=-1,to=1,length=m)
  x0 <- rep(NA,m)
  y0 <- rep(NA,m)
  for(i in 1:m){
    x0[i] <- mean(xLocal - d0[i],na.rm=TRUE)
    y0[i] <- mean(xLocal - d0[i] > .5,
                  na.rm=TRUE)
  }

  inBounds <- x0 >= 0 & x0 <= 1
  out <- list(s=y0[inBounds],
              v=x0[inBounds],
              x=xLocal,
              desc=desc,
              call=cl)

  class(out)  <- "seatsVotes"
  out
}

print.seatsVotes <- function(x,...){
  if(!inherits(x,"seatsVotes"))
    cat("print.svCurve only defined for objects of class seatsVotes\n")

  if(is.null(match.call()$digits))
    digits <- .Options$digits
  else
    digits <- match.call()$digits

  if(!is.null(x$desc))
    cat("Seats-Votes Curve:",x$desc,"\n")

  cat("\nSummary of",
      length(x$x),
      "non-missing vote shares:\n")
  print(summary(x$x))

  closestToFifty <- which.min(abs(x$v-.5))
  biasAtFifty <- x$s[closestToFifty] - .5

  cat("Bias at Average Vote Share = .5 is",
      round(biasAtFifty,digits),
      "\n")

  invisible(NULL)
}

summary.seatsVotes <- function(object,...){
  if(!inherits(object,"seatsVotes"))
    cat("summary.svCurve only defined for objects of class seatsVotes\n")
  object
}

plot.seatsVotes <- function(x,
                            type=c("seatsVotes","density"),
                            legend="bottomright",
                            transform=FALSE,
                            ...)
{
  if(!inherits(x,"seatsVotes"))
    cat("plot.svCurve only defined for objects of class seatsVotes\n")

  type <- match.arg(type)
  cl <- match.call()


  ## seats vote curve
  if(type=="seatsVotes"){
    oldpar <- par()
    par(mar=c(4.2,4,5,1),
        las=1)

    if(is.null(cl$xlab))
      xlab <- "Average District Vote"
    else
      xlab <- cl$xlab

    if(is.null(cl$ylab))
      ylab <- "Proportion of Seats Won"
    else
      ylab <- cl$ylab
    
    if(is.null(cl$xlim))
      xlim <- c(0,1)
    else
      xlim <- cl$xlim

    if(is.null(cl$ylim))
      ylim <- c(0,1)
    else
      ylim <- cl$ylim

    if(is.null(cl$xaxs))
      xaxs <- "i"
    else
      xaxs <- cl$xaxs

    if(is.null(cl$yaxs))
      yaxs <- "i"
    else
      yaxs <- cl$yaxs

    plot(x$v,x$s,type="l",
         lwd=3,
         axes=FALSE,
         xaxs=xaxs,
         yaxs=yaxs,
         xlim=xlim,
         ylim=ylim,
         xlab=xlab,
         ylab=ylab,
         ...)
    axis(1,at=seq(0,1,by=.25))
    axis(2,at=seq(0,1,by=.25))
    abline(h=.5,lty=2)
    abline(v=.5,lty=2)
    mtext(side=3,
          at=mean(x$x,na.rm=TRUE),
          "Actual\nResult",cex=.65)
    abline(v=mean(x$x,na.rm=TRUE),lty=2)
    abline(0,1,col=gray(.45),lwd=2)
    
    methodString <- switch(x$call$method,
                           "uniformSwing" = "Uniform Swing")
    
    if(!is.null(x$desc))
      title(paste("Simulated Seats-Votes Curve Using ",methodString,
                  "\n",
                  x$desc,sep=""))
    else
      title(paste("Simulated Seats-Votes Curve Using",methodString))
    
    if(!is.null(legend))
      legend(x=legend,
             col=gray(.45),
             lwd=2,
             lty=1,
             bty="n",
             cex=.65,
             legend="Proportional Representation\n(45 degree line)")
    
    par(oldpar)
  }

  if(type=="density"){
    if(is.null(cl$title)){
      if(is.null(x$desc))
        titleString <- "Density"
      else
        titleString <- paste("Density,",x$desc)
    }
    else
      titleString <- cl$title
    
    if(is.null(cl$xlab))
      xlab <- "Vote Shares"
    else
      xlab <- cl$xlab

    if(is.null(cl$ylab))
      ylab <- ""
    else
      ylab <- cl$ylab

    if(transform){
      transFunc <- function(x){
        v <- log(x/(1-x))
        beta <- sqrt(3)
        xstar <- v*beta
        exp(xstar)/(1+exp(xstar))
      }
      xLocal <- transFunc(x$x)
    }
    else
      xLocal <- x$x
    
    plot(density(xLocal,
                 na.rm=TRUE,
                 from=min(xLocal,na.rm=TRUE),
                 to=max(xLocal,na.rm=TRUE),
                 ...
                 ),
         xlab=xlab,
         ylab=ylab,
         main=titleString,
         axes=FALSE)
    if(transform){
      tcks <- pretty(x$x)
      tcks <- transFunc(tcks)
      axis(1,at=tcks,labels=pretty(x$x))
    }
    else
      axis(1)
    rug(xLocal)
  }
  invisible(NULL)
}

## pseudo r2 for binary, ordinal and multinomial models

pR2 <- function(object,...){
  UseMethod("pR2")
}

pR2Work <- function(llh,llhNull,n=NULL){
  if (is.null(n))
    n <- nobs(llh)
  McFadden <- 1 - llh/llhNull
  G2 <- -2*(llhNull-llh)
  r2ML <- 1 - exp(-G2/n)
  r2ML.max <- 1 - exp(llhNull*2/n)
  r2CU <- r2ML/r2ML.max
  out <- c(llh=llh,
           llhNull=llhNull,
           G2=G2,
           McFadden=McFadden,
           r2ML=r2ML,
           r2CU=r2CU)
  out      
}

pR2.default <- function(object,...){
  llh <- logLik(object)
  cat("fitting null model for pseudo-r2\n")
  objectNull <- update(object, ~ 1, data=model.frame(object))
  llhNull <- logLik(objectNull)
  pR2Work(llh,llhNull)
}

## ideal helper functions

## check validity of a burnin number
## return logical of valid iters
checkBurnIn <- function(object, burnin) {
  theIters <- as.numeric(dimnames(object$x)[[1]])
  if (as.numeric(burnin)>max(theIters))
    stop("burnin greater than number of iterations")
  return (theIters > burnin)
}

checkD <- function(x,d) {
  if ((d<1)||(d>x$d))
    stop("d must be equal to one of the dimensions in the roll call object")
}

checkCI <- function(conf.int) {
  if((conf.int<=0)||(conf.int>=1))
      stop("conf.int must be between 0 and 1")
}

getMean <- function(keep,x){
  xbar <- apply(x[keep,,,drop=FALSE],
                c(2,3),
                mean)
  dimnames(xbar) <- list(dimnames(x)[[2]],
                         dimnames(x)[[3]])
  return(xbar)
}

hurdle <- function(formula, data, subset, na.action, weights, offset,
                   dist = c("poisson", "negbin", "geometric"),
                   zero.dist = c("binomial", "poisson", "negbin", "geometric"),
                   link = c("logit", "probit", "cloglog", "cauchit", "log"),
		   control = hurdle.control(...),
		   model = TRUE, y = TRUE, x = FALSE, ...)
{
  ## set up likelihood components
  zeroPoisson <- function(parms) {
    ## mean
    mu <- as.vector(exp(Z %*% parms + offsetz))
    ## log-likelihood
    loglik0 <- -mu ## = dpois(0, lambda = mu, log = TRUE)
    ## collect and return
    loglik <- sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * log(1 - exp(loglik0[Y1])))
    loglik
  }

  countPoisson <- function(parms) {
    ## mean
    mu <- as.vector(exp(X %*% parms + offsetx))[Y1]
    ## log-likelihood
    loglik0 <- -mu ## = dpois(0, lambda = mu, log = TRUE)
    loglik1 <- dpois(Y[Y1], lambda = mu, log = TRUE)
    ## collect and return
    loglik <- sum(weights[Y1] * loglik1) - sum(weights[Y1] * log(1 - exp(loglik0)))
    loglik
  }

  zeroNegBin <- function(parms) {
    ## parameters
    mu <- as.vector(exp(Z %*% parms[1:kz] + offsetz))
    theta <- exp(parms[kz+1])
    ## log-likelihood
    loglik0 <- suppressWarnings(dnbinom(0, size = theta, mu = mu, log = TRUE))
    ## collect and return
    loglik <- sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * log(1 - exp(loglik0[Y1])))
    loglik
  }

  countNegBin <- function(parms) {
    ## parameters
    mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))[Y1]
    theta <- exp(parms[kx+1])
    ## log-likelihood
    loglik0 <- suppressWarnings(dnbinom(0, size = theta, mu = mu, log = TRUE))
    loglik1 <- suppressWarnings(dnbinom(Y[Y1], size = theta, mu = mu, log = TRUE))
    ## collect and return
    loglik <- sum(weights[Y1] * loglik1) - sum(weights[Y1] * log(1 - exp(loglik0)))
    loglik
  }

  zeroGeom <- function(parms) zeroNegBin(c(parms, 0))
  
  countGeom <- function(parms) countNegBin(c(parms, 0))

  zeroBinom <- function(parms) {
    ## mean
    mu <- as.vector(linkinv(Z %*% parms + offsetz))
    ## log-likelihood
    loglik <- sum(weights[Y0] * log(1 - mu[Y0])) + sum(weights[Y1] * log(mu[Y1]))
    loglik  
  }

  countGradPoisson <- function(parms) {
    eta <- as.vector(X %*% parms + offsetx)[Y1]
    mu <- exp(eta)
    colSums(((Y[Y1] - mu) - exp(ppois(0, lambda = mu, log.p = TRUE) -
      ppois(0, lambda = mu, lower.tail = FALSE, log.p = TRUE) + eta)) * weights[Y1] * X[Y1, , drop = FALSE])
  }
  
  countGradGeom <- function(parms) {
    eta <- as.vector(X %*% parms + offsetx)[Y1]
    mu <- exp(eta)      
    colSums(((Y[Y1] - mu * (Y[Y1] + 1)/(mu + 1)) -
      exp(pnbinom(0, mu = mu, size = 1, log.p = TRUE) -
        pnbinom(0, mu = mu, size = 1, lower.tail = FALSE, log.p = TRUE) -
	log(mu + 1) + eta)) * weights[Y1] * X[Y1, , drop = FALSE])
  }

  countGradNegBin <- function(parms) {
    eta <- as.vector(X %*% parms[1:kx] + offsetx)[Y1]
    mu <- exp(eta)      
    theta <- exp(parms[kx+1])
    logratio <- pnbinom(0, mu = mu, size = theta, log.p = TRUE) -
        pnbinom(0, mu = mu, size = theta, lower.tail = FALSE, log.p = TRUE)
    rval <- colSums(((Y[Y1] - mu * (Y[Y1] + theta)/(mu + theta)) -
      exp(logratio + log(theta) - log(mu + theta) + eta)) * weights[Y1] * X[Y1, , drop = FALSE])
    rval2 <- sum((digamma(Y[Y1] + theta) - digamma(theta) +    
      log(theta) - log(mu + theta) + 1 - (Y[Y1] + theta)/(mu + theta) +
      exp(logratio) * (log(theta) - log(mu + theta) + 1 - theta/(mu + theta))) * weights[Y1]) * theta
    c(rval, rval2)
  }  
  
  zeroGradPoisson <- function(parms) {
    eta <- as.vector(Z %*% parms + offsetz)
    mu <- exp(eta)
    colSums(ifelse(Y0, -mu, exp(ppois(0, lambda = mu, log.p = TRUE) -
      ppois(0, lambda = mu, lower.tail = FALSE, log.p = TRUE) + eta)) * weights * Z)
  }

  zeroGradGeom <- function(parms) {
    eta <- as.vector(Z %*% parms + offsetz)
    mu <- exp(eta)
    colSums(ifelse(Y0, -mu/(mu + 1), exp(pnbinom(0, mu = mu, size = 1, log.p = TRUE) -
      pnbinom(0, mu = mu, size = 1, lower.tail = FALSE, log.p = TRUE) - log(mu + 1) + eta)) * weights * Z)
  }

  zeroGradNegBin <- function(parms) {
    eta <- as.vector(Z %*% parms[1:kz] + offsetz)
    mu <- exp(eta)
    theta <- exp(parms[kz+1])
    logratio <- pnbinom(0, mu = mu, size = theta, log.p = TRUE) -
      pnbinom(0, mu = mu, size = theta, lower.tail = FALSE, log.p = TRUE)
    rval <- colSums(ifelse(Y0, -mu * theta/(mu + theta),
      exp(logratio + log(theta) - log(mu + theta) + eta)) * weights * Z)
    rval2 <- sum(ifelse(Y0, log(theta) - log(mu + theta) + 1 - theta/(mu + theta),
      -exp(logratio) * (log(theta) - log(mu + theta) + 1 - theta/(mu + theta))) * weights * theta)
    c(rval, rval2)
  }

  zeroGradBinom <- function(parms) {
    eta <- as.vector(Z %*% parms + offsetz)
    mu <- linkinv(eta)
    colSums(ifelse(Y0, -1/(1-mu), 1/mu)  * linkobj$mu.eta(eta) * weights * Z)  
  }

  ## collect likelihood components
  dist <- match.arg(dist)
  zero.dist <- match.arg(zero.dist)
  countDist <- switch(dist,
                     "poisson" = countPoisson,
		     "geometric" = countGeom,
		     "negbin" = countNegBin)
  zeroDist <- switch(zero.dist,
                     "poisson" = zeroPoisson,
		     "geometric" = zeroGeom,
		     "negbin" = zeroNegBin,
		     "binomial" = zeroBinom)
  countGrad <- switch(dist,
                     "poisson" = countGradPoisson,
		     "geometric" = countGradGeom,
		     "negbin" = countGradNegBin)
  zeroGrad <- switch(zero.dist,
                     "poisson" = zeroGradPoisson,
		     "geometric" = zeroGradGeom,
		     "negbin" = zeroGradNegBin,
		     "binomial" = zeroGradBinom)
  loglikfun <- function(parms) countDist(parms[1:(kx + (dist == "negbin"))]) +
    zeroDist(parms[(kx + (dist == "negbin") + 1):(kx + kz + (dist == "negbin") + (zero.dist == "negbin"))])
  gradfun <- function(parms) c(countGrad(parms[1:(kx + (dist == "negbin"))]),
    zeroGrad(parms[(kx + (dist == "negbin") + 1):(kx + kz + (dist == "negbin") + (zero.dist == "negbin"))]))


  ## binary link processing
  linkstr <- match.arg(link)
  linkobj <- make.link(linkstr)
  linkinv <- linkobj$linkinv

  if(control$trace) cat("Hurdle Count Model\n",
    paste("count model:", dist, "with log link\n"),
    paste("zero hurdle model:", zero.dist, "with", ifelse(zero.dist == "binomial", linkstr, "log"), "link\n"),
    sep = "")


  ## call and formula
  cl <- match.call()
  if(missing(data)) data <- environment(formula)
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("formula", "data", "subset", "na.action", "weights", "offset"), names(mf), 0)
  mf <- mf[c(1, m)]
  mf$drop.unused.levels <- TRUE
  
  ## extended formula processing
  if(length(formula[[3]]) > 1 && identical(formula[[3]][[1]], as.name("|")))
  {
    ff <- formula
    formula[[3]][1] <- call("+")
    mf$formula <- formula
    ffc <- . ~ .
    ffz <- ~ .
    ffc[[2]] <- ff[[2]]
    ffc[[3]] <- ff[[3]][[2]]
    ffz[[3]] <- ff[[3]][[3]]
    ffz[[2]] <- NULL
  } else {
    ffz <- ffc <- ff <- formula
    ffz[[2]] <- NULL
  }
  if(inherits(try(terms(ffz), silent = TRUE), "try-error")) {
    ffz <- eval(parse(text = sprintf( paste("%s -", deparse(ffc[[2]])), deparse(ffz) )))
  }

  ## call model.frame()
  mf[[1]] <- as.name("model.frame")
  mf <- eval(mf, parent.frame())
  
  ## extract terms, model matrices, response
  mt <- attr(mf, "terms")
  mtX <- terms(ffc, data = data)
  X <- model.matrix(mtX, mf)
  mtZ <- terms(ffz, data = data)
  mtZ <- terms(update(mtZ, ~ .), data = data)
  Z <- model.matrix(mtZ, mf)
  Y <- model.response(mf, "numeric")


  ## sanity checks
  if(length(Y) < 1) stop("empty model")
  if(all(Y > 0)) stop("invalid dependent variable, minimum count is not zero")  
  if(!isTRUE(all.equal(as.vector(Y), as.integer(round(Y + 0.001)))))
    stop("invalid dependent variable, non-integer values")
  Y <- as.integer(round(Y + 0.001))
  if(any(Y < 0)) stop("invalid dependent variable, negative counts")
  if(zero.dist == "negbin" & isTRUE(all.equal(as.vector(Z), rep.int(Z[1], length(Z)))))
    stop("negative binomial zero hurdle model is not identified with only an intercept")
  
  if(control$trace) {
    cat("dependent variable:\n")
    tab <- table(factor(Y, levels = 0:max(Y)), exclude = NULL)
    names(dimnames(tab)) <- NULL
    print(tab)
  }

  ## convenience variables
  n <- length(Y)
  kx <- NCOL(X)
  kz <- NCOL(Z)
  Y0 <- Y <= 0
  Y1 <- Y > 0

  ## weights and offset
  weights <- model.weights(mf)
  if(is.null(weights)) weights <- 1
  if(length(weights) == 1) weights <- rep.int(weights, n)
  weights <- as.vector(weights)
  names(weights) <- rownames(mf)
  
  offsetx <- model_offset_2(mf, terms = mtX, offset = TRUE)
  if(is.null(offsetx)) offsetx <- 0
  if(length(offsetx) == 1) offsetx <- rep.int(offsetx, n)
  offsetx <- as.vector(offsetx)
  offsetz <- model_offset_2(mf, terms = mtZ, offset = FALSE)
  if(is.null(offsetz)) offsetz <- 0
  if(length(offsetz) == 1) offsetz <- rep.int(offsetz, n)
  offsetz <- as.vector(offsetz)

  ## starting values
  start <- control$start
  if(!is.null(start)) {
    valid <- TRUE
    if(!("count" %in% names(start))) {
      valid <- FALSE
      warning("invalid starting values, count model coefficients not specified")
      start$count <- rep.int(0, kx)
    }
    if(!("zero" %in% names(start))) {
      valid <- FALSE
      warning("invalid starting values, zero-inflation model coefficients not specified")
      start$zero <- rep.int(0, kz)
    }
    if(length(start$count) != kx) {
      valid <- FALSE
      warning("invalid starting values, wrong number of count model coefficients")
    }
    if(length(start$zero) != kz) {
      valid <- FALSE
      warning("invalid starting values, wrong number of zero-inflation model coefficients")
    }
    if(dist == "negbin" | zero.dist == "negbin") {
      if(!("theta" %in% names(start))) start$theta <- c(1, 1)
      start <- list(count = start$count, zero = start$zero, theta = rep(start$theta, length.out = 2))
      if(is.null(names(start$theta))) names(start$theta) <- c("count", "zero")      
      if(dist != "negbin") start$theta <- start$theta["zero"]
      if(zero.dist != "negbin") start$theta <- start$theta["count"]
    } else {
      start <- list(count = start$count, zero = start$zero)
    }
    if(!valid) start <- NULL
  }
  
  if(is.null(start)) {
    if(control$trace) cat("generating starting values...")
    model_count <- glm.fit(X, Y, family = poisson(), weights = weights, offset = offsetx)
    model_zero <- switch(zero.dist,
      "poisson" = glm.fit(Z, Y, family = poisson(), weights = weights, offset = offsetz),
      "negbin" = glm.fit(Z, Y, family = poisson(), weights = weights, offset = offsetz),
      "geometric" = suppressWarnings(glm.fit(Z, factor(Y > 0), family = binomial(), weights = weights, offset = offsetz)),
      "binomial" = suppressWarnings(glm.fit(Z, factor(Y > 0), family = binomial(link = linkstr), weights = weights, offset = offsetz)))
    start <- list(count = model_count$coefficients, zero = model_zero$coefficients)
    start$theta <- c(count = if(dist == "negbin") 1 else NULL,
                     zero = if(zero.dist == "negbin") 1 else NULL)
    if(control$trace) cat("done\n")
  }

  ## model fitting
  ## control parameters
  method <- control$method
  hessian <- control$hessian
  separate <- control$separate
  ##ocontrol <- control
  control$method <- control$hessian <- control$separate <- control$start <- NULL

  ## ML estimation
  ## separate estimation of censored and truncated component...
  if(separate) {
    if(control$trace) cat("calling optim() for count component estimation:\n")
    fit_count <- optim(fn = countDist, gr = countGrad,
      par = c(start$count, if(dist == "negbin") log(start$theta["count"]) else NULL),
      method = method, hessian = hessian, control = control)

    if(control$trace) cat("calling optim() for zero hurdle component estimation:\n")
    fit_zero <- optim(fn = zeroDist, gr = zeroGrad,
      par = c(start$zero,  if(zero.dist == "negbin") log(start$theta["zero"]) else NULL),
      method = method, hessian = hessian, control = control)
    if(control$trace) cat("done\n")
    fit <- list(count = fit_count, zero = fit_zero)

    ## coefficients
    coefc <- fit_count$par[1:kx]
    coefz <- fit_zero$par[1:kz]
    theta <- c(count = if(dist == "negbin") as.vector(exp(fit_count$par[kx+1])) else NULL,
               zero = if(zero.dist == "negbin") as.vector(exp(fit_zero$par[kz+1])) else NULL)
    ## covariances
    vc_count <- tryCatch(-solve(as.matrix(fit_count$hessian)),
                         error=function(e) {
                             warning(e$message, call=FALSE)
                             k <- nrow(as.matrix(fit_count$hessian))
                             return(matrix(NA, k, k))
                         })
    vc_zero <- tryCatch(-solve(as.matrix(fit_zero$hessian)),
                        error=function(e) {
                            warning(e$message, call=FALSE)
                            k <- nrow(as.matrix(fit_zero$hessian))
                            return(matrix(NA, k, k))
                        })
    SE.logtheta <- list()
    if(dist == "negbin") {
      SE.logtheta$count <- as.vector(sqrt(diag(vc_count)[kx+1]))
      vc_count <- vc_count[-(kx+1), -(kx+1), drop = FALSE]
    }
    if(zero.dist == "negbin") {
      SE.logtheta$zero <- as.vector(sqrt(diag(vc_zero)[kz+1]))
      vc_zero <- vc_zero[-(kz+1), -(kz+1), drop = FALSE]
    }
    vc <- rbind(cbind(vc_count, matrix(0, kx, kz)), cbind(matrix(0, kz, kx), vc_zero))
    SE.logtheta <- unlist(SE.logtheta)
  } else {
  ## ...or joint.
    if(control$trace) cat("calling optim() for joint count and zero hurlde estimation:\n")
    fit <- optim(fn = loglikfun, gr = gradfun,
      par = c(start$count, if(dist == "negbin") log(start$theta["count"]) else NULL,
              start$zero,  if(zero.dist == "negbin") log(start$theta["zero"]) else NULL),
      method = method, hessian = hessian, control = control)
    if(fit$convergence > 0) warning("optimization failed to converge")
    if(control$trace) cat("done\n")

    ## coefficients
    coefc <- fit$par[1:kx]
    coefz <- fit$par[(kx + (dist == "negbin") + 1):(kx + kz + (dist == "negbin"))]
    ## covariances
    vc <- tryCatch(-solve(as.matrix(fit$hessian)),
                   error=function(e) {
                       warning(e$message, call=FALSE)
                       k <- nrow(as.matrix(fit$hessian))
                       return(matrix(NA, k, k))
                   })
    np <- c(if(dist == "negbin") kx+1 else NULL,
            if(zero.dist == "negbin") kx+kz+1+(dist == "negbin") else NULL)
    if(length(np) > 0) {
      theta <- as.vector(exp(fit$par[np]))
      SE.logtheta <- as.vector(sqrt(diag(vc)[np]))
      names(theta) <- names(SE.logtheta) <- c(if(dist == "negbin") "count" else NULL,
                                              if(zero.dist == "negbin") "zero" else NULL)
      vc <- vc[-np, -np, drop = FALSE]
    } else {
      theta <- NULL
      SE.logtheta <- NULL
    }

  }
  names(coefc) <- names(start$count) <- colnames(X)
  names(coefz) <- names(start$zero) <- colnames(Z)
  colnames(vc) <- rownames(vc) <- c(paste("count", colnames(X), sep = "_"),
                                    paste("zero",  colnames(Z), sep = "_"))

  ## fitted and residuals
  phi <- if(zero.dist == "binomial") linkinv(Z %*% coefz + offsetz)[,1] else exp(Z %*% coefz + offsetz)[,1]
  p0_zero <- switch(zero.dist,
  		    "binomial" = log(phi),
        	    "poisson" = ppois(0, lambda = phi, lower.tail = FALSE, log.p = TRUE),
        	    "negbin" = pnbinom(0, size = theta["zero"], mu = phi, lower.tail = FALSE, log.p = TRUE),
        	    "geometric" = pnbinom(0, size = 1, mu = phi, lower.tail = FALSE, log.p = TRUE))

  mu <- exp(X %*% coefc + offsetx)[,1]
  p0_count <- switch(dist,
        	    "poisson" = ppois(0, lambda = mu, lower.tail = FALSE, log.p = TRUE),
        	    "negbin" = pnbinom(0, size = theta["count"], mu = mu, lower.tail = FALSE, log.p = TRUE),
        	    "geometric" = pnbinom(0, size = 1, mu = mu, lower.tail = FALSE, log.p = TRUE))
  Yhat <- exp((p0_zero - p0_count) + log(mu))

  res <- sqrt(weights) * (Y - Yhat)

  ## effective observations
  nobs <- sum(weights > 0) ## = n - sum(weights == 0)

  rval <- list(coefficients = list(count = coefc, zero = coefz),
    residuals = res,
    fitted.values = Yhat,
    optim = fit,
    method = method,
    control = control,
    start = start,
    weights = if(identical(as.vector(weights), rep.int(1L, n))) NULL else weights,
    offset = list(count = if(identical(offsetx, rep.int(0, n))) NULL else offsetx,
      zero = if(identical(offsetz, rep.int(0, n))) NULL else offsetz),
    n = nobs,
    df.null = nobs - 2,
    df.residual = nobs - (kx + kz + (dist == "negbin") + (zero.dist == "negbin")),
    terms = list(count = mtX, zero = mtZ, full = mt),
    theta = theta,
    SE.logtheta = SE.logtheta,
    loglik = if(separate) fit_count$value + fit_zero$value else fit$value,
    vcov = vc,
    dist = list(count = dist, zero = zero.dist),
    link = if(zero.dist == "binomial") linkstr else NULL,
    linkinv = if(zero.dist == "binomial") linkinv else NULL,
    separate = separate,
    converged = if(separate) fit_count$convergence < 1 & fit_zero$convergence < 1 else fit$convergence < 1,
    call = cl,
    formula = ff,
    levels = .getXlevels(mt, mf),
    contrasts = list(count = attr(X, "contrasts"), zero = attr(Z, "contrasts"))
  )
  if(model) rval$model <- mf
  if(y) rval$y <- Y
  if(x) rval$x <- list(count = X, zero = Z)
      
  class(rval) <- "hurdle"
  return(rval)
}

hurdle.control <- function(method = "BFGS", maxit = 10000, trace = FALSE, separate = TRUE, start = NULL, ...) {
  rval <- list(method = method, maxit = maxit, trace = trace, separate = separate, start = start)
  rval <- c(rval, list(...))
  if(!is.null(rval$fnscale)) warning("fnscale must not be modified")
  rval$fnscale <- -1
  if(!is.null(rval$hessian)) warning("hessian must not be modified")
  rval$hessian <- TRUE
  if(is.null(rval$reltol)) rval$reltol <- .Machine$double.eps^(1/1.6)
  rval
}

coef.hurdle <- function(object, model = c("full", "count", "zero"), ...) {
  model <- match.arg(model)
  rval <- object$coefficients
  rval <- switch(model,
                 "full" = structure(c(rval$count, rval$zero),
		   .Names = c(paste("count", names(rval$count), sep = "_"),
                   paste("zero", names(rval$zero), sep = "_"))),
		 "count" = rval$count,
		 "zero" = rval$zero)
  rval
}

vcov.hurdle <- function(object, model = c("full", "count", "zero"), ...) {
  model <- match.arg(model)
  rval <- object$vcov
  if(model == "full") return(rval)

  cf <- object$coefficients[[model]]
  wi <- seq(along = object$coefficients$count)
  rval <- if(model == "count") rval[wi, wi, drop = FALSE] else rval[-wi, -wi, drop = FALSE]
  colnames(rval) <- rownames(rval) <- names(cf)
  return(rval)
}

logLik.hurdle <- function(object, ...) {
  structure(object$loglik, df = object$n - object$df.residual, nobs = object$n, class = "logLik")
}

print.hurdle <- function(x, digits = max(3, getOption("digits") - 3), ...)
{

  cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
  
  if(!x$converged) {
    cat("model did not converge\n")
  } else {
    cat(paste("Count model coefficients (truncated ", x$dist$count, " with log link):\n", sep = ""))
    print.default(format(x$coefficients$count, digits = digits), print.gap = 2, quote = FALSE)
    if(x$dist$count == "negbin") cat(paste("Theta =", round(x$theta["count"], digits), "\n"))
  
    zero_dist <- if(x$dist$zero != "binomial") paste("censored", x$dist$zero, "with log link")
      else paste("binomial with", x$link, "link")
    cat(paste("\nZero hurdle model coefficients (", zero_dist, "):\n", sep = ""))
    print.default(format(x$coefficients$zero, digits = digits), print.gap = 2, quote = FALSE)
    if(x$dist$zero == "negbin") cat(paste("Theta =", round(x$theta["zero"], digits), "\n"))
    cat("\n")
  }
  
  invisible(x)
}

summary.hurdle <- function(object,...)
{
  ## residuals
  object$residuals <- residuals(object, type = "pearson")
  
  ## compute z statistics
  kc <- length(object$coefficients$count)
  kz <- length(object$coefficients$zero)
  se <- sqrt(diag(object$vcov))
  coef <- c(object$coefficients$count, object$coefficients$zero)  
  if(object$dist$count == "negbin") {
    coef <- c(coef[1:kc], "Log(theta)" = as.vector(log(object$theta["count"])), coef[(kc+1):(kc+kz)])
    se <- c(se[1:kc], object$SE.logtheta["count"], se[(kc+1):(kc+kz)])
    kc <- kc+1
  }
  if(object$dist$zero == "negbin") {
    coef <- c(coef, "Log(theta)" = as.vector(log(object$theta["zero"])))
    se <- c(se, object$SE.logtheta["zero"])
    kz <- kz+1
  }
  zstat <- coef/se
  pval <- 2*pnorm(-abs(zstat))
  coef <- cbind(coef, se, zstat, pval)
  colnames(coef) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
  object$coefficients$count <- coef[1:kc,,drop = FALSE]
  object$coefficients$zero <- coef[(kc+1):(kc+kz),,drop = FALSE]
  
  ## number of iterations
  object$iterations <- if(!object$separate) tail(na.omit(object$optim$count), 1)
    else tail(na.omit(object$optim$count$count), 1) + tail(na.omit(object$optim$zero$count), 1)
  
  ## delete some slots
  object$fitted.values <- object$terms <- object$model <- object$y <-
    object$x <- object$levels <- object$contrasts <- object$start <- object$separate <- NULL

  ## return
  class(object) <- "summary.hurdle"
  object
}

print.summary.hurdle <- function(x, digits = max(3, getOption("digits") - 3), ...)
{

  cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
  
  if(!x$converged) {
    cat("model did not converge\n")
  } else {

    cat("Pearson residuals:\n")
    print(structure(quantile(x$residuals),
      names = c("Min", "1Q", "Median", "3Q", "Max")), digits = digits, ...)  

    cat(paste("\nCount model coefficients (truncated ", x$dist$count, " with log link):\n", sep = ""))
    printCoefmat(x$coefficients$count, digits = digits, signif.legend = FALSE)
  
    zero_dist <- if(x$dist$zero != "binomial") paste("censored", x$dist$zero, "with log link")
      else paste("binomial with", x$link, "link")
    cat(paste("Zero hurdle model coefficients (", zero_dist, "):\n", sep = ""))
    printCoefmat(x$coefficients$zero, digits = digits, signif.legend = FALSE)
    
    if(getOption("show.signif.stars") & any(rbind(x$coefficients$count, x$coefficients$zero)[,4] < 0.1, na.rm=TRUE))
      cat("---\nSignif. codes: ", "0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1", "\n")

    if(!is.null(x$theta)) cat(paste("\nTheta:", paste(names(x$theta), round(x$theta, digits), sep = " = ", collapse = ", ")))
    cat(paste("\nNumber of iterations in", x$method, "optimization:", x$iterations, "\n"))
    cat("Log-likelihood:", formatC(x$loglik, digits = digits), "on", x$n - x$df.residual, "Df\n")
  }
  
  invisible(x)
}

terms.hurdle <- function(x, model = c("count", "zero"), ...) {
  x$terms[[match.arg(model)]]
}

model.matrix.hurdle <- function(object, model = c("count", "zero"), ...) {
  model <- match.arg(model)
  if(!is.null(object$x)) rval <- object$x[[model]]
    else if(!is.null(object$model)) rval <- model.matrix(object$terms[[model]], object$model, contrasts = object$contrasts[[model]])
    else stop("not enough information in fitted model to return model.matrix")
  return(rval)
}

predict.hurdle <- function(object, newdata, type = c("response", "prob", "count", "zero"),
  na.action = na.pass, at = NULL, ...)
{
    type <- match.arg(type)

    ## if no new data supplied
    if(missing(newdata)) {
      if(type != "response") {
        if(!is.null(object$x)) {
	  X <- object$x$count
	  Z <- object$x$zero
	} else if(!is.null(object$model)) {
          X <- model.matrix(object$terms$count, object$model, contrasts = object$contrasts$count)
          Z <- model.matrix(object$terms$zero,  object$model, contrasts = object$contrasts$zero)	
	} else {
	  stop("predicted probabilities cannot be computed with missing newdata")
	}
	offsetx <- if(is.null(object$offset$count)) rep.int(0, NROW(X)) else object$offset$count
	offsetz <- if(is.null(object$offset$zero))  rep.int(0, NROW(Z)) else object$offset$zero
      } else {
        return(object$fitted.values)
      }
    } else {
      mf <- model.frame(delete.response(object$terms$full), newdata, na.action = na.action, xlev = object$levels)
      X <- model.matrix(delete.response(object$terms$count), mf, contrasts = object$contrasts$count)
      Z <- model.matrix(delete.response(object$terms$zero),  mf, contrasts = object$contrasts$zero)
      offsetx <- model_offset_2(mf, terms = object$terms$count, offset = FALSE)
      offsetz <- model_offset_2(mf, terms = object$terms$zero,  offset = FALSE)
      if(is.null(offsetx)) offsetx <- rep.int(0, NROW(X))
      if(is.null(offsetz)) offsetz <- rep.int(0, NROW(Z))
      if(!is.null(object$call$offset)) offsetx <- offsetx + eval(object$call$offset, newdata)
    }

    phi <- if(object$dist$zero == "binomial") object$linkinv(Z %*% object$coefficients$zero + offsetz)[,1]
      else exp(Z %*% object$coefficients$zero + offsetz)[,1]
    p0_zero <- switch(object$dist$zero,
                      "binomial" = log(phi),
		      "poisson" = ppois(0, lambda = phi, lower.tail = FALSE, log.p = TRUE),
		      "negbin" = pnbinom(0, size = object$theta["zero"], mu = phi, lower.tail = FALSE, log.p = TRUE),
		      "geometric" = pnbinom(0, size = 1, mu = phi, lower.tail = FALSE, log.p = TRUE))

    mu <- exp(X %*% object$coefficients$count + offsetx)[,1]
    p0_count <- switch(object$dist$count,
		      "poisson" = ppois(0, lambda = mu, lower.tail = FALSE, log.p = TRUE),
		      "negbin" = pnbinom(0, size = object$theta["count"], mu = mu, lower.tail = FALSE, log.p = TRUE),
		      "geometric" = pnbinom(0, size = 1, mu = mu, lower.tail = FALSE, log.p = TRUE))
    logphi <- p0_zero - p0_count
    
    if(type == "response") rval <- exp(logphi + log(mu))
    if(type == "count") rval <- mu
    if(type == "zero") rval <- exp(logphi)

    ## predicted probabilities
    if(type == "prob") {
      if(!is.null(object$y)) y <- object$y
        else if(!is.null(object$model)) y <- model.response(object$model)
	else stop("predicted probabilities cannot be computed for fits with y = FALSE and model = FALSE")

      yUnique <- if(is.null(at)) 0:max(y) else at
      nUnique <- length(yUnique)
      rval <- matrix(NA, nrow = length(mu), ncol = nUnique)
      dimnames(rval) <- list(rownames(X), yUnique)
      
      rval[,1] <- 1 - exp(p0_zero)
      switch(object$dist$count,
             "poisson" = {
               for(i in 2:nUnique) rval[,i] <- exp(logphi + dpois(yUnique[i], lambda = mu, log = TRUE))
	     },
	     "negbin" = {
               for(i in 2:nUnique) rval[,i] <- exp(logphi + dnbinom(yUnique[i], mu = mu, size = object$theta["count"], log = TRUE))
	     },
	     "geometric" = {
               for(i in 2:nUnique) rval[,i] <- exp(logphi + dnbinom(yUnique[i], mu = mu, size = 1, log = TRUE))
	     })
    }
   
    rval
}

fitted.hurdle <- function(object, ...) {
  object$fitted.values
}

residuals.hurdle <- function(object, type = c("pearson", "response"), ...) {

  type <- match.arg(type)
  res <- object$residuals

  switch(type,
  
  "response" = {
    return(res)
  },
  
  "pearson" = {
    mu <- predict(object, type = "count")
    phi <- predict(object, type = "zero")
    theta1 <- switch(object$dist$count,
      "poisson" = 0,
      "geometric" = 1,
      "negbin" = 1/object$theta["count"])
    vv <- object$fitted.values * (1 + ((1-phi) + theta1) * mu)
    return(res/sqrt(vv))
  })
}

predprob.hurdle <- function(obj, ...){
    predict(obj, type = "prob", ...)
}

extractAIC.hurdle <- function(fit, scale = NULL, k = 2, ...) {
  c(attr(logLik(fit), "df"), AIC(fit, k = k))
}

hurdletest <- function(object, ...) {
  stopifnot(inherits(object, "hurdle"))
  stopifnot(object$dist$count == object$dist$zero)
  stopifnot(all(sort(names(object$coefficients$count)) == sort(names(object$coefficients$zero))))
  stopifnot(requireNamespace("car"))
  nam <- names(object$coefficients$count)
  lh <- paste("count_", nam, " = ", "zero_", nam, sep = "")
  rval <- car::linearHypothesis(object, lh, ...)
  attr(rval, "heading")[1] <- "Wald test for hurdle models\n\nRestrictions:"
  return(rval)
}

## convenience helper function
model_offset_2 <- function(x, terms = NULL, offset = TRUE)
## allow optionally different terms
## potentially exclude "(offset)"
{
  if(is.null(terms)) terms <- attr(x, "terms")
  offsets <- attr(terms, "offset")
  if(length(offsets) > 0) {
    ans <- if(offset) x$"(offset)" else NULL
    if(is.null(ans)) ans <- 0
    for(i in offsets) ans <- ans + x[[deparse(attr(terms, "variables")[[i + 1]])]]
    ans
  }
  else {
    ans <- if(offset) x$"(offset)" else NULL
  }
  if(!is.null(ans) && !is.numeric(ans)) stop("'offset' must be numeric")
  ans
}

## hitmiss methods
hitmiss <- function(obj,
                    digits=max(3,getOption("digits")-3),
                    ...){
  UseMethod("hitmiss")
}

hitmiss.glm <- function(obj,
                        digits=max(3,getOption("digits")-3),
                        ...,
                        k=.5){
  if(!inherits(obj,"glm"))
    stop("hitmiss.glm only defined for objects of class glm\n")

  ## check to make sure if of class binomial
  if(family(obj)$family!="binomial")
    stop("hitmiss.glm only defined for binomial family glm objects\n")
  if(is.null(obj$y))
    y <- update(obj,y=TRUE)$y
  else
    y <- obj$y
  
  if(!all(y %in% c(0,1)))
    stop("hitmiss.glm only defined for models with binary responses")
  
  p <- predict(obj,type="response")
  tab <- matrix(NA,2,2)
  tab[1,1] <- sum(p<k & y==0,na.rm=T)   ## true negative
  tab[1,2] <- sum(p<k & y==1,na.rm=T)   ## false negative
  tab[2,1] <- sum(p>k & y==0,na.rm=T)   ## false positive
  tab[2,2] <- sum(p>k & y==1,na.rm=T)   ## true positive
  pcp <- sum(diag(tab))/sum(tab) * 100  ## overall PCP measure
  pcp0 <- tab[1,1]/sum(tab[1:2,1])*100  ## PCP | y = 0
  pcp1 <- tab[2,2]/sum(tab[1:2,2])*100  ## PCP | y = 1
  
  dimnames(tab) <- list(c("yhat=0","yhat=1"),
                        c("y=0","y=1"))
  cat(paste("Classification Threshold =",k,"\n"))
  print(tab)
  cat(paste("Percent Correctly Predicted = ",
            format(pcp,digits=digits),
            "%\n",
            sep=""))
  cat(paste("Percent Correctly Predicted = ",
            format(pcp0,digits=digits),
            "%, for y = 0\n",
            sep=""))
  cat(paste("Percent Correctly Predicted = ",
            format(pcp1,digits=digits),
            "%  for y = 1\n",
            sep=""))
  nullmodel <- max(c(sum(y==0),sum(y==1)))/sum(tab) * 100
  cat(paste("Null Model Correctly Predicts ",
            format(nullmodel,digits=digits),
            "%\n",
            sep=""))
  out <- c(pcp,pcp0,pcp1)
  out
}

hitmiss.polr <- function(obj,
                         digits=max(3,getOption("digits")-3),
                         ...){
  tmp <- obj
  p <- predict(tmp)

  if(is.null(tmp$model)){
    cat("refitting the model to extract responses...\n")
    tmp <- update(tmp,model=TRUE)
    cat("\n")
  }
  y <- tmp$model[,1]
  uniqueY <- sort(unique(y))
  J <- length(uniqueY)
  p <- factor(p,levels=uniqueY)
  y <- factor(y,levels=uniqueY)
  cat("Table of Actual (y) Against Predicted (p)\n")
  cat("Classification rule: outcome with highest probability.\n")
  tab <- table(y,p)
  dimnames(tab)[[1]] <- paste("y=",dimnames(tab)[[1]],sep="")
  dimnames(tab)[[2]] <- paste("p=",dimnames(tab)[[2]],sep="")
  n <- length(y)
  pcp <- sum(diag(tab))/n * 100
  tabY <- table(y)
  pcp0 <- tabY[which.max(tabY)]/n * 100

  pcpByRow <- rep(NA,J)
  for(i in 1:J){
    pcpByRow[i] <- tab[i,i]/sum(tab[i,]) * 100
  }
  tab <- cbind(tab,pcpByRow)
  dimnames(tab)[[2]][J+1] <- "Row PCP"

  print(tab,digits=digits)
  cat("\n")

  cat("Percent Correctly Predicted, Fitted Model: ",
      format(pcp,digits=digits),
      "%\n",sep="")
  cat("Percent Correctly Predicted, Null Model  : ",
      format(pcp0,digits=digits),
      "%\n",sep="")
  invisible(NULL)
}

hitmiss.multinom <- function(obj,
                             digits=max(3,getOption("digits")-3),
                             ...){
  hitmiss.polr(obj)
}

predprob <- function(obj, ...){
    UseMethod("predprob")
}

predprob.ideal <- function(obj, ...){
  if(!inherits(obj, "ideal"))
    stop("predprob.ideal only defined for objects of class ideal")
  else
    predict.ideal(obj,...)$pred.probs
}

\documentclass[nojss]{jss}

%% need no \usepackage{Sweave}
\usepackage{thumbpdf}

%% new commands
\newcommand{\class}[1]{``\code{#1}''}
\newcommand{\fct}[1]{\code{#1()}}

\author{Achim Zeileis\\Universit\"at Innsbruck \And
        Christian Kleiber\\Universit\"at Basel \And
	Simon Jackman\\Stanford University}
\Plainauthor{Achim Zeileis, Christian Kleiber, Simon Jackman}

\title{Regression Models for Count Data in \proglang{R}}
\Plaintitle{Regression Models for Count Data in R}

\Keywords{GLM, Poisson model, negative binomial model, hurdle model, zero-inflated model}

\Abstract{
  The classical Poisson, geometric and negative binomial regression models 
  for count data belong to the family of generalized linear models and
  are available at the core of the statistics toolbox in the \proglang{R}
  system for statistical computing. After reviewing the conceptual and
  computational features of these methods, a new implementation of
  hurdle and zero-inflated regression models in the functions \fct{hurdle}
  and \fct{zeroinfl} from the package \pkg{pscl} is introduced. It re-uses
  design and functionality of the basic \proglang{R} functions just
  as the underlying conceptual tools extend the classical models.
  Both hurdle and zero-inflated model, are able to incorporate over-dispersion and
  excess zeros---two problems
  that typically occur in count data sets in economics and the social
  sciences---better than their classical counterparts.
  Using cross-section data on the demand for medical care, it is illustrated
  how the classical as well as the zero-augmented models can be fitted,
  inspected and tested in practice.
}

\Address{
  Achim Zeileis\\
  Department of Statistics\\
  Universit\"at Innsbruck\\
  Universit\"atsstr.~15\\
  6020 Innsbruck, Austria\\
  E-mail: \email{Achim.Zeileis@R-project.org}\\
  URL: \url{http://statmath.wu-wien.ac.at/~zeileis/}
}

\begin{document}

\SweaveOpts{engine=R, eps=FALSE, keep.source = TRUE}
%\VignetteIndexEntry{Regression Models for Count Data in R}
%\VignetteDepends{sandwich,zoo,lmtest,MASS,car}
%\VignetteKeywords{GLM, Poisson model, negative binomial model, hurdle model, zero-inflated model}
%\VignettePackage{pscl}

<<preliminaries, echo=FALSE, results=hide>>=
library("sandwich")
library("lmtest")
library("MASS")
library("car")
library("pscl")
load("DebTrivedi.rda")

clog <- function(x) log(x + 0.5)
cfac <- function(x, breaks = NULL) {
  if(is.null(breaks)) breaks <- unique(quantile(x, 0:10/10))
  x <- cut(x, breaks, include.lowest = TRUE, right = FALSE)
  levels(x) <- paste(breaks[-length(breaks)], ifelse(diff(breaks) > 1,
    c(paste("-", breaks[-c(1, length(breaks))] - 1, sep = ""), "+"), ""), sep = "")
  return(x)
}

options(prompt = "R> ")
refit_models <- TRUE
@

\section{Introduction} \label{sec:intro}

Modeling count variables is a common task in economics and the social
sciences. The classical Poisson regression model for count
data is often of limited use in these disciplines because empirical
count data sets typically exhibit over-dispersion and/or an excess
number of zeros. The former issue can be addressed by extending 
the plain Poisson regression model in various directions: e.g.,
using sandwich covariances or estimating an additional dispersion
parameter (in a so-called quasi-Poisson model). Another more formal
way is to use a negative binomial (NB) regression. All of these models
belong to the family of generalized linear models 
\citep[GLMs, see][]{countreg:Nelder+Wedderburn:1972,countreg:McCullagh+Nelder:1989}.
However, although these models typically can capture over-dispersion
rather well, they are in many applications not sufficient for 
modeling excess zeros. Since \cite{countreg:Mullahy:1986} and
\cite{countreg:Lambert:1992} there is increased interest, both in the
econometrics and statistics literature, in zero-augmented models
that address this issue by a second model component capturing zero counts.
Hurdle models \citep{countreg:Mullahy:1986} combine a left-truncated
count component with a right-censored hurdle component. Zero-inflation
models \citep{countreg:Lambert:1992} take a somewhat different
approach: they are mixture models that combine a count component and a point mass at zero.
An overview of count data models in econometrics, including 
hurdle and zero-inflated models, is provided in
\cite{countreg:Cameron+Trivedi:1998,countreg:Cameron+Trivedi:2005}.

In \proglang{R} \citep{countreg:R:2008}, GLMs are provided
by the model fitting functions \fct{glm} \citep{countreg:Chambers+Hastie:1992}
in the \pkg{stats} package and \fct{glm.nb} in the \pkg{MASS}
package \citep{countreg:Venables+Ripley:2002} along with associated
methods for diagnostics and inference. Here, we discuss the implementation
of hurdle and zero-inflated models in the functions \fct{hurdle} and
\fct{zeroinfl} in the \pkg{pscl} package \citep{countreg:Jackman:2008},
available from the Comprehensive \proglang{R} Archive Network (CRAN)
at \url{http://CRAN.R-project.org/package=pscl}.
The design of both modeling functions as well as the methods operating
on the associated fitted model objects follows that of the base \proglang{R}
functionality so that the new software integrates easily into the
computational toolbox for modeling count data in \proglang{R}.

The remainder of this paper is organized as follows: Section~\ref{sec:software}
discusses both the classical and zero-augmented count data models and their
\proglang{R} implementations. In Section~\ref{sec:illustrations}, all
count regression models discussed are applied to a microeconomic
cross-section data set on the demand for medical care. The summary in
Section~\ref{sec:summary} concludes the main part of the paper; further
technical details are presented in the appendix.


\section{Models and software} \label{sec:software}

\begin{table}[b!]
\begin{center}
\begin{tabular}{|l|l|l|p{7.4cm}|}
\hline
Type & Distribution & Method & Description \\ \hline
GLM & Poisson & ML & Poisson regression: classical GLM, estimated by maximum likelihood (ML) \\
    &         & quasi & ``quasi-Poisson regression'':
                            same mean function, estimated by quasi-ML (QML) or equivalently
			    generalized estimating equations (GEE),
			    inference adjustment via estimated dispersion parameter \\
    &         & adjusted & ``adjusted Poisson regression'': same mean function,
                           estimated by QML/GEE,
			   inference adjustment via sandwich covariances\\
    & NB      & ML    & NB regression:
                        extended GLM, estimated by ML including additional shape parameter \\ \hline
zero-augmented & Poisson & ML & zero-inflated Poisson (ZIP), hurdle Poisson \\
               & NB      & ML & zero-inflated NB (ZINB), hurdle NB \\ \hline
\end{tabular}
\caption{\label{tab:overview} Overview of discussed count regression models. All GLMs use the
same log-linear mean function ($\log(\mu) = x^\top \beta$) but make different assumptions about
the remaining likelihood. The zero-augmented models extend the mean function by 
modifying (typically, increasing) the likelihood of zero counts.}
\end{center}
\end{table}

In this section, we briefly outline the theory and its implementation in 
\proglang{R} \citep{countreg:R:2008} for some basic count data regression
models as well as their zero-augmented extensions (see Table~\ref{tab:overview}
for an overview). The classical Poisson, geometric
and negative binomial models are described in a generalized linear model (GLM)
framework; they are implemented in \proglang{R} by the \fct{glm} function
\citep{countreg:Chambers+Hastie:1992} in the \pkg{stats} package and the
\fct{glm.nb} function in the \pkg{MASS} package \citep{countreg:Venables+Ripley:2002}.
The hurdle and zero-inflated extensions of these models are provided by the
functions \fct{hurdle} and \fct{zeroinfl} in package \pkg{pscl} \citep{countreg:Jackman:2008}.
The original implementation of \cite{countreg:Jackman:2008} was improved
by \cite{countreg:Kleiber+Zeileis:2008} for \pkg{pscl} to make the fitting functions and the
fitted model objects more similar to their \fct{glm} and \fct{glm.nb} counterparts.
The most important features of the new \fct{hurdle} and \fct{zeroinfl} functions
are discussed below while some technical aspects are deferred to the appendix.

An alternative implementation of zero-inflated count models is available in
the currently orphaned package \pkg{zicounts}
\citep{countreg:Mwalili:2007}. Another extension of zero-inflated Poisson models is available
in package \pkg{ZIGP} \citep{countreg:Erhardt:2008} which allows dispersion---in addition to
mean and zero-inflation level---to depend on regressors.
However, the interfaces of both packages are less standard with fewer (or no)
standard methods provided. Therefore, re-using generic inference tools is more
cumbersome and hence these packages are not discussed here. 

Two packages that embed zero-inflated models into more general implementations
of GLMs and GAMs (generalized additive models) are \pkg{gamlss} 
\citep{countreg:Stasinopoulos+Rigby:2007} and \pkg{VGAM} \citep{countreg:Yee:2008}.
The latter also provides hurdle models (under the name zero-altered models).
Both implementations allow specification of only one set of regressors.

In addition to zero-augmented models, there are many further extensions to
the classical Poisson model which are not discussed here. Some important model
classes include finite mixture models---implemented in \proglang{R} in package \pkg{flexmix}
\citep{countreg:Leisch:2004}---and generalized estimating equations
(GEE)---provided in \proglang{R} by package \pkg{geepack}
\citep{countreg:Halekoh+Hojsgaard+Yan:2006}---and mixed-effects models---available
in \proglang{R} in packages \pkg{lme4} and \pkg{nlme} \citep[see][]{countreg:Pinheiro+Bates:2000}.
Further information about the models and alternative \proglang{R} implementations
can be found in the respective references.

\subsection{Generalized linear models}

\subsubsection{Model frame}

The basic count data regression models can
be represented and understood using the GLM
framework that emerged in the statistical literature in the early 1970s
\citep{countreg:Nelder+Wedderburn:1972}. In the following, we briefly
sketch some important aspects relating to the unifying conceptual properties
and their implementation in \proglang{R}---for a detailed theoretical account of GLMs
see \cite{countreg:McCullagh+Nelder:1989}.

GLMs describe the dependence of a scalar variable $y_i$ ($i = 1, \dots, n$)
on a vector of regressors $x_i$. The conditional distribution of
$y_i | x_i$ is a linear exponential family with probability density function
\begin{equation} \label{eq:family}
f(y; \lambda, \phi) \quad = \quad
                              \exp \left( \frac{y \cdot \lambda - b(\lambda)}{\phi} +
                              c(y, \phi) \right),
\end{equation}
where $\lambda$ is the canonical parameter that depends on the regressors via
a linear predictor and $\phi$ is a dispersion parameter that is often known.
The functions $b(\cdot)$ and $c(\cdot)$ are known and determine which member
of the family is used, e.g., the normal, binomial or Poisson distribution.
Conditional mean and variance of $y_i$ are given by
$\E[y_i \, | \, x_i] = \mu_i = b'(\lambda_i)$ and
$\VAR[y_i \, | \, x_i] = \phi \cdot b''(\lambda_i)$. Thus, up to a scale
or dispersion parameter $\phi$, the distribution of $y_i$ is determined by
its mean. Its variance is proportional to $V(\mu) = b''(\lambda(\mu))$,
also called variance function.

The dependence of the conditional mean $\E[y_i \, | \, x_i] = \mu_i$
on the regressors $x_i$ is specified via
\begin{equation} \label{eq:mean}
g(\mu_i) \quad = \quad x_i^\top \beta,
\end{equation}
where $g(\cdot)$ is a known link function and $\beta$ is the vector of regression
coefficients which are typically estimated by maximum likelihood (ML)
using the iterative weighted least squares (IWLS) algorithm.

Instead of viewing GLMs as models for the full likelihood (as determined by
Equation~\ref{eq:family}), they can also be regarded as regression models for the
mean only (as specified in Equation~\ref{eq:mean}) where the estimating functions
used for fitting the model are derived from a particular family. As illustrated 
in the remainder of this section, the estimating function point of view is particularly
useful for relaxing the assumptions imposed by the Poisson likelihood.

\proglang{R} provides a very flexible implementation of the general GLM
framework in the function \fct{glm} \citep{countreg:Chambers+Hastie:1992}
contained in the \pkg{stats} package. Its most important arguments are
\begin{Soutput}
glm(formula, data, subset, na.action, weights, offset,
  family = gaussian, start = NULL, control = glm.control(...),
  model = TRUE, y = TRUE, x = FALSE, ...)
\end{Soutput}
where \code{formula} plus \code{data} is the now standard way of specifying
regression relationships in \proglang{R}/\proglang{S} introduced in
\cite{countreg:Chambers+Hastie:1992}. The remaining arguments in the first line
(\code{subset}, \code{na.action}, \code{weights}, and \code{offset}) are also standard 
for setting up formula-based regression models in \proglang{R}/\proglang{S}.
The arguments in the second line control aspects specific to GLMs while
the arguments in the last line specify which components are returned
in the fitted model object (of class \class{glm} which inherits from
\class{lm}). By default the model frame (\code{model}) and the vector
$(y_1, \dots, y_n)^\top$ (\code{y}) but not the model matrix (\code{x},
containing $x_1, \dots, x_n$ combined row-wise) are included. The
\code{family} argument specifies the link $g(\mu)$ and variance
function $V(\mu)$ of the model, \code{start} can be used to set 
starting values for $\beta$, and \code{control} contains control parameters for the IWLS
algorithm. For further arguments to \fct{glm} (including alternative specifications
of starting values) see \code{?glm}.
The high-level \fct{glm} interface relies on the function	
\fct{glm.fit} which carries out the actual model fitting (without
taking a formula-based input or returning classed output).

For \class{glm} objects, a set of standard methods (including \fct{print},
\fct{predict}, \fct{logLik} and many others) are provided. Inference can 
easily be performed using the \fct{summary} method for assessing the
regression coefficients via partial Wald tests or the \fct{anova} method
for comparing nested models via an analysis of deviance. These inference
functions are complemented by further generic inference functions in
contributed packages: e.g., \pkg{lmtest} \citep{countreg:Zeileis+Hothorn:2002}
provides a \fct{coeftest} function that also computes partial Wald tests
but allows for specification of alternative (robust) standard errors. Similarly,
\fct{waldtest} from \pkg{lmtest} and \fct{linearHypothesis} from \pkg{car}
\citep{countreg:Fox:2002} assess nested models via Wald tests (using
different specifications for the nested models). Finally, \fct{lrtest}
from \pkg{lmtest} compares nested models via likelihood ratio (LR) tests
based on an interface similar to \fct{waldtest} and \fct{anova}.


\subsubsection{Poisson model}

The simplest distribution used for modeling count data is the 
Poisson distribution with probability density function
\begin{equation} \label{eq:Poisson}
f(y; \mu) \quad = \quad \frac{\exp(-\mu) \cdot \mu^{y}}{y!},
\end{equation}
which is of type~(\ref{eq:family}) and thus Poisson regression
is a special case of the GLM framework. The canonical link is
$g(\mu) = \log(\mu)$ resulting in a log-linear relationship
between mean and linear predictor. The variance in the Poisson
model is identical to the mean, thus the dispersion is
fixed at $\phi = 1$ and the variance function is $V(\mu) = \mu$.

In \proglang{R}, this can easily be specified in the \fct{glm} 
call just by setting \code{family = poisson} (where the default log link
could also be changed in the \fct{poisson} call).

In practice, the Poisson model is often useful for describing the
mean $\mu_i$ but underestimates the variance in the data, rendering
all model-based tests liberal. One way of dealing with this is
to use the same estimating functions for the mean, but to base
inference on the more robust sandwich covariance matrix estimator.
In \proglang{R}, this estimator is provided by the \fct{sandwich}
function in the \pkg{sandwich} package \citep{countreg:Zeileis:2004,countreg:Zeileis:2006}.


\subsubsection{Quasi-Poisson model}

Another way of dealing with over-dispersion is to use the mean
regression function and the variance function from the Poisson GLM
but to leave the dispersion parameter $\phi$ unrestricted. Thus,
$\phi$ is not assumed to be fixed at $1$ but is estimated from
the data. This strategy leads
to the same coefficient estimates as the standard Poisson model
but inference is adjusted for over-dispersion. Consequently,
both models (quasi-Poisson and sandwich-adjusted Poisson)
adopt the estimating function view of the Poisson model and
do \emph{not} correspond to models with fully specified likelihoods.

In \proglang{R}, the quasi-Poisson model with estimated dispersion
parameter can also be fitted with the \fct{glm} function, simply
setting \code{family = quasipoisson}.

\subsubsection{Negative binomial models}

A third way of modeling over-dispersed count data is to assume
a negative binomial (NB) distribution for $y_i | x_i$ which can arise
as a gamma mixture of Poisson distributions. One parameterization of
its probability density function is
\begin{equation} \label{eq:negbin}
f(y; \mu, \theta) \quad = \quad \frac{\Gamma(y + \theta)}{\Gamma(\theta) \cdot y!} \cdot
                            \frac{\mu^{y} \cdot \theta^\theta}{(\mu + \theta)^{y + \theta}},
\end{equation}
with mean $\mu$ and shape parameter $\theta$; $\Gamma(\cdot)$ is the
gamma function. For every fixed
$\theta$, this is of type~(\ref{eq:family}) and thus is another
special case of the GLM framework. It also has $\phi = 1$
but with variance function $V(\mu) = \mu + \frac{\mu^2}{\theta}$.

Package \pkg{MASS} \citep{countreg:Venables+Ripley:2002} provides
the family function \fct{negative.binomial} that can directly
be plugged into \fct{glm} provided the argument \code{theta} is
specified. One application would be the geometric model, the
special case where $\theta = 1$, which can consequently be
fitted in \proglang{R} by setting
\code{family = negative.binomial(theta = 1)} in the \fct{glm}
call.

If $\theta$ is not known but to be estimated from the data,
the negative binomial model is not a special case of the general
GLM---however, an ML fit can easily be computed re-using GLM
methodology by iterating estimation of $\beta$ given $\theta$
and vice versa. This leads to ML estimates for both $\beta$
and $\theta$ which can be computed using the function \fct{glm.nb}
from the package \pkg{MASS}. It returns a model of class \class{negbin}
inheriting from \class{glm} for which appropriate methods
to the generic functions described above are again available.


\subsection{Hurdle models}

In addition to over-dispersion, many empirical count data sets
exhibit more zero observations than would be allowed for
by the Poisson model. One model class capable of capturing
both properties is the hurdle model, 
originally proposed by \cite{countreg:Mullahy:1986} in the econometrics literature
\citep[see][for an overview]{countreg:Cameron+Trivedi:1998,countreg:Cameron+Trivedi:2005}.
They are two-component models:
A truncated count component, such as Poisson, geometric or negative binomial,
is employed for positive counts, and
a hurdle component models zero vs.\ larger counts.
For the latter, either a binomial model or a censored count distribution can be employed.

More formally, the hurdle model combines a count data model 
$f_\mathrm{count}(y; x, \beta)$ (that is left-truncated at $y = 1$) and a 
zero hurdle model $f_\mathrm{zero}(y; z, \gamma)$ (right-censored at $y = 1$):
\begin{equation} \label{eq:hurdle}
f_\mathrm{hurdle}(y; x, z, \beta, \gamma) =
  \left\{
  \begin{array}{ll}
  f_\mathrm{zero}(0; z, \gamma) & \mbox{if } y = 0, \\
  (1 - f_\mathrm{zero}(0; z, \gamma)) \cdot
  f_\mathrm{count}(y; x, \beta)/(1 - f_\mathrm{count}(0; x, \beta)) & \mbox{if } y > 0
  \end{array}
  \right.
\end{equation}
The model parameters $\beta$, $\gamma$, and potentially one or two additional
dispersion parameters $\theta$
(if $f_\mathrm{count}$ or $f_\mathrm{zero}$ or both are negative binomial densities)
are estimated by ML, where the specification of the likelihood has the advantage that
the count and the hurdle component can be maximized separately. The corresponding
mean regression relationship is given by
\begin{equation} \label{eq:hurdle-mean}
\log(\mu_i) \quad = \quad x_i^\top \beta +
                    \log(1 - f_\mathrm{zero}(0; z_i, \gamma)) -
		    \log(1 - f_\mathrm{count}(0; x_i, \beta)),
\end{equation}
again using the canonical log link.
For interpreting the zero model as a hurdle, a binomial GLM is probably the most 
intuitive specification\footnote{Note that
binomial logit and censored geometric models as the hurdle part both lead to 
the same likelihood function and thus to the same coefficient estimates \citep{countreg:Mullahy:1986}.}.
Another useful interpretation arises if the same regressors $x_i = z_i$ are
used in the same count model in both components $f_\mathrm{count} = f_\mathrm{zero}$: 
A test of the hypothesis $\beta = \gamma$ then tests whether the hurdle is
needed or not.

In \proglang{R}, hurdle count data models can be fitted with the
\fct{hurdle} function from the \pkg{pscl} package \citep{countreg:Jackman:2008}.
Both its fitting function and the returned model objects of class \class{hurdle}
are modelled after the corresponding GLM functionality in \proglang{R}. The
arguments of \fct{hurdle} are given by

\begin{Soutput}
hurdle(formula, data, subset, na.action, weights, offset,
  dist = "poisson", zero.dist = "binomial", link = "logit",
  control = hurdle.control(...),
  model = TRUE, y = TRUE, x = FALSE, ...)
\end{Soutput}

where the first line contains the standard model-frame specifications,
the second and third lines have the arguments specific to hurdle models and
the arguments in the last line control some components of the return value.

If a \code{formula} of type \code{y ~ x1 + x2} is supplied, it not only
describes the count regression relationship of $y_i$ and $x_i$ but also implies that
the same set of regressors is used for the zero hurdle component $z_i = x_i$.
This is could be made more explicit by equivalently writing the formula as
\code{y ~ x1 + x2 | x1 + x2}. Of course, a different set of regressors
could be specified for the zero hurdle component, e.g.,
\code{y ~ x1 + x2 | z1 + z2 + z3}, giving the count data model \code{y ~ x1 + x2}
conditional on (\code{|}) the zero hurdle model \code{y ~ z1 + z2 + z3}.

The model likelihood can be specified by the \code{dist}, \code{zero.dist} and \code{link} arguments.
The count data distribution \code{dist} is \code{"poisson"} by default (it
can also be set to \code{"negbin"} or \code{"geometric"}), for which the canonical log link
is always used. The distribution for the zero hurdle model can be specified via
\code{zero.dist}. The default is a binomial model with \code{link} (defaulting
to \code{"logit"}, but all link functions of the \fct{binomial} family are also supported),
alternatively a right-censored count distribution (Poisson,
negative binomial or geometric, all with log link) could be specified.

ML estimation of all parameters employing analytical gradients is carried out
using \proglang{R}'s \fct{optim} with control options set in \fct{hurdle.control}.
Starting values can be user-supplied, otherwise they are estimated by \fct{glm.fit}
(the default). The covariance matrix estimate is derived numerically using
the Hessian matrix returned by \fct{optim}.
See Appendix~\ref{app:hurdle} for further technical details.

The returned fitted-model object of class \class{hurdle} is a list similar
to \class{glm} objects. Some of its elements---such as \code{coefficients} or
\code{terms}---are lists with a zero and count component,
respectively. For details see Appendix~\ref{app:hurdle}.

A set of standard extractor functions for fitted model objects is available for
objects of class \class{hurdle}, including the usual \fct{summary} method that
provides partial Wald tests for all coefficients. No \fct{anova} method is provided,
but the general \fct{coeftest}, \fct{waldtest} from \pkg{lmtest}, and \fct{linearHypothesis}
from \pkg{car} can be used for Wald tests and \fct{lrtest} from \pkg{lmtest}
for LR tests of nested models. The function \fct{hurdletest} is a convenience
interface to \fct{linearHypothesis} for testing for the presence of a hurdle
(which is only applicable if the same regressors and the same count distribution
are used in both components).


\subsection{Zero-inflated models}

Zero-inflated models \citep{countreg:Mullahy:1986,countreg:Lambert:1992}
are another model class capable of dealing with excess zero counts 
\citep[see][for an overview]{countreg:Cameron+Trivedi:1998,countreg:Cameron+Trivedi:2005}.
They are two-component mixture models
combining a point mass at zero with a count distribution such as
Poisson, geometric or negative binomial. Thus, there are two
sources of zeros: zeros may come from both the point mass and
from the count component. For modeling the unobserved state
(zero vs.\ count), a binary model is used: in the simplest case
only with an intercept but potentially containing regressors.

Formally, the zero-inflated density is a mixture of a point mass
at zero $I_{\{0\}}(y)$ and a count distribution $f_\mathrm{count}(y; x, \beta)$.
The probability of observing a zero count is inflated with probability
$\pi = f_\mathrm{zero}(0; z, \gamma)$:
\begin{equation} \label{eq:zeroinfl}
f_\mathrm{zeroinfl}(y; x, z, \beta, \gamma) \quad = \quad
  f_\mathrm{zero}(0; z, \gamma) \cdot I_{\{0\}}(y) \; + \;
  (1 - f_\mathrm{zero}(0; z, \gamma)) \cdot f_\mathrm{count}(y; x, \beta),
\end{equation}
where $I(\cdot)$ is the indicator function and the unobserved probability $\pi$
of belonging to the point mass component is modelled by a binomial GLM
$\pi =  g^{-1}(z^\top \gamma)$.
The corresponding regression equation for the mean is
\begin{equation} \label{eq:zeroinfl-mean}
\mu_i \quad = \quad \pi_i \cdot 0 \; + \; (1 - \pi_i) \cdot \exp(x_i^\top \beta),
\end{equation}
using the canonical log link.
The vector of regressors in the zero-inflation model $z_i$
and the regressors in the count component $x_i$ need not to be distinct---in
the simplest case, $z_i = 1$ is just
an intercept. The default link function $g(\pi)$ in binomial GLMs is the
logit link, but other links such as the probit are also available. The full
set of parameters of $\beta$, $\gamma$, and potentially the dispersion parameter $\theta$ (if 
a negative binomial count model is used) can be estimated by ML. Inference
is typically performed for $\beta$ and $\gamma$, while $\theta$ is treated
as a nuisance parameter even if a negative binomial model is used.

In \proglang{R}, zero-inflated count data models can be fitted with the
\fct{zeroinfl} function from the \pkg{pscl} package. Both 
the fitting function interface and the returned model objects of class \class{zeroinfl}
are almost identical to the corresponding \fct{hurdle} functionality and again
modelled after the corresponding GLM functionality in \proglang{R}. The
arguments of \fct{zeroinfl} are given by

\begin{Soutput}
zeroinfl(formula, data, subset, na.action, weights, offset,
  dist = "poisson", link = "logit", control = zeroinfl.control(...),
  model = TRUE, y = TRUE, x = FALSE, ...)
\end{Soutput}

where all arguments have almost the same meaning as for \fct{hurdle}. The main
difference is that there is no \code{zero.dist} argument: a binomial model is
always used for distribution in the zero-inflation component.

Again, ML estimates of all parameters are obtained from \fct{optim},
with control options set in \fct{zeroinfl.control} and employing analytical gradients.
Starting values can be user-supplied, estimated by the expectation maximization (EM)
algorithm, or by \fct{glm.fit} (the default).
The covariance matrix estimate is derived numerically using
the Hessian matrix returned by \fct{optim}. Using EM estimation for
deriving starting values is typically slower but can be numerically more stable.
It already maximizes the likelihood, but a single \fct{optim} iteration is used
for determining the covariance matrix estimate.
See Appendix~\ref{app:zeroinfl} for further technical details.

The returned fitted model object is of class \class{zeroinfl} whose structure
is virtually identical to that of \class{hurdle} models. As above,
a set of standard extractor functions for fitted model objects is available for
objects of class \class{zeroinfl}, including the usual \fct{summary} method that
provides partial Wald tests for all coefficients. Again, no \fct{anova} method is provided,
but the general functions \fct{coeftest} and \fct{waldtest} from \pkg{lmtest},
as well as \fct{linearHypothesis} from \pkg{car} can be used for Wald tests,
and \fct{lrtest} from \pkg{lmtest} for LR tests of nested models. 


\section{Application and illustrations} \label{sec:illustrations}

In the following, we illustrate all models described above by applying
them to a cross-sectional data set from health economics. Before the parametric models are fitted,
a basic exploratory analysis of the data set is carried out that addresses
some problems typically encountered when visualizing count data. At the
end of the section, all fitted models are compared highlighting that
the modelled mean function is similar but the fitted likelihood
is different and thus, the models differ with respect to explaining
over-dispersion and/or the number of zero counts.


\subsection{Demand for medical care by the elderly}

\cite{countreg:Deb+Trivedi:1997} analyze data on 4406
individuals, aged 66 and over, who are covered by Medicare, a public
insurance program. Originally obtained from the US National Medical
Expenditure Survey (NMES) for 1987/88, the data are available from the data archive of the
\textit{Journal of Applied Econometrics} at
\url{http://www.econ.queensu.ca/jae/1997-v12.3/deb-trivedi/}. It was
prepared for an \proglang{R} package accompanying
\cite{countreg:Kleiber+Zeileis:2008} and is also available as
\code{DebTrivedi.rda} in the \textit{Journal of Statistical Software}
together with \cite{countreg:Zeileis:2006}. The objective is to model
the demand for medical care---as captured by the number of physician/non-physician
office and hospital outpatient visits---by the covariates available
for the patients. Here, we adopt the number of physician office visits \code{ofp}
as the dependent variable and use the health status variables
\code{hosp} (number of hospital stays),
\code{health} (self-perceived health status),
\code{numchron} (number of chronic conditions),
as well as the socio-economic variables
\code{gender},
\code{school} (number of years of education), and
\code{privins} (private insurance indicator) as regressors. For convenience, we
select the variables used from the full data set:

<<dt>>=
dt <- DebTrivedi[, c(1, 6:8, 13, 15, 18)]
@

<<dt2, echo=FALSE>>=
dt2 <- DebTrivedi[, -(2:6)]
dt2$region <- relevel(dt2$region, "other")
@

To obtain a first overview of the dependent variable, we employ a histogram of the
observed count frequencies. In \proglang{R} various tools could be used, e.g.,
\code{hist(dt$ofp, breaks = 0:90 - 0.5)} for a histogram with rectangles or
<<ofp-plot, eval=FALSE>>=
plot(table(dt$ofp))
@
(see Figure~\ref{fig:ofp}) for a histogram with lines which brings out the extremely
large counts somewhat better. The histogram illustrates that the marginal distribution
exhibits both substantial variation and a rather large number of zeros.

\setkeys{Gin}{width=.5\textwidth} 
\begin{figure}[p]
\begin{center}
<<ofp-plot2, echo=FALSE, fig=TRUE, height=5, width=5>>=
plot(table(dt$ofp), xlab = "Number of physician office visits", ylab = "Frequency", axes = FALSE)
axis(2)
axis(1, at = 0:18 * 5, labels = FALSE)
axis(1, at = 0:9 * 10)
@
\caption{\label{fig:ofp} Frequency distribution for number of physician office visits.}
\end{center}
\end{figure}

\setkeys{Gin}{width=\textwidth} 
\begin{figure}[p]
\begin{center}
<<bad-good, fig=TRUE, height=5, width=10, echo=FALSE>>=
par(mfrow = c(1, 2))
plot(ofp ~ numchron, data = dt)
plot(clog(ofp) ~ cfac(numchron), data = dt)
@
\caption{\label{fig:bad-good} Bivariate explorative displays for number of physician office visits
  plotted against number of chronic conditions.}
\end{center}
\end{figure}

A natural second step in the exploratory analysis is to look at pairwise bivariate 
displays of the dependent variable against each of the regressors bringing out the
partial relationships. In \proglang{R}, such bivariate displays
can easily be generated with the \fct{plot} method for formulas, e.g., via \code{plot(y ~ x)}.
This chooses different types of displays depending on the combination of quantitative
and qualitative variables as dependent or regressor variable, respectively. However,
count variables are treated as all numerical variables and therefore the command

<<bad, eval=FALSE>>=
plot(ofp ~ numchron, data = dt)
@

produces a simple scatterplot as shown in the left panel of Figure~\ref{fig:bad-good}.
This is clearly not useful as both variables are count variables producing numerous
ties in the bivariate distribution and thus obscuring a large number of points in
the display. To overcome the problem, it is useful to group the number of chronic
conditions into a factor with levels `0', `1', `2', and `3 or more' and produce a
boxplot instead of a scatterplot. Furthermore, the picture is much clearer if the
dependent variable is log-transformed (just as all count regression models discussed
above also use a log link by default). As there are zero counts as well, we use a
convenience function \fct{clog} providing a continuity-corrected logarithm.

<<clog>>=
clog <- function(x) log(x + 0.5)
@

For transforming a count variable to a factor (for visualization purposes only),
we define another convenience function \fct{cfac}

<<cfac>>=
cfac <- function(x, breaks = NULL) {
  if(is.null(breaks)) breaks <- unique(quantile(x, 0:10/10))
  x <- cut(x, breaks, include.lowest = TRUE, right = FALSE)
  levels(x) <- paste(breaks[-length(breaks)], ifelse(diff(breaks) > 1,
    c(paste("-", breaks[-c(1, length(breaks))] - 1, sep = ""), "+"), ""),
    sep = "")
  return(x)
}
@

which by default tries to take an educated guess how to choose the breaks between the categories.
Clearly, the resulting exploratory display of the transformed variables produced by

<<good, eval=FALSE>>=
plot(clog(ofp) ~ cfac(numchron), data = dt)
@

(shown in the right panel of Figure~\ref{fig:bad-good}) brings out much better
how the number of doctor visits increases with the number of chronic conditions.

\setkeys{Gin}{width=\textwidth} 
\begin{figure}[p]
\begin{center}
<<ofp2-plot1, fig=TRUE, height=9, width=6.5, echo=FALSE>>=
par(mfrow = c(3, 2))
plot(clog(ofp) ~ health, data = dt, varwidth = TRUE,
  ylab = "Physician office visits (in clogs)", xlab = "Self-perceived health status", main = "health")
plot(clog(ofp) ~ cfac(numchron), data = dt,
  ylab = "Physician office visits (in clogs)", xlab = "Number of chronic conditions", main = "numchron")
plot(clog(ofp) ~ privins, data = dt, varwidth = TRUE,
  ylab = "Physician office visits (in clogs)", xlab = "Covered by private insurance", main = "privins")
plot(clog(ofp) ~ cfac(hosp, c(0:2, 8)), data = dt,
  ylab = "Physician office visits (in clogs)", xlab = "Number of hospital stays", main = "hosp")
plot(clog(ofp) ~ gender, data = dt, varwidth = TRUE,
  ylab = "Physician office visits (in clogs)", xlab = "Gender", main = "gender")
plot(cfac(ofp, c(0:2, 4, 6, 10, 100)) ~ school, data = dt, breaks = 9,
  ylab = "Physician office visits (number of visits)", xlab = "Number of years of education", main = "school")
@
\caption{\label{fig:ofp2} Number of physician office visits plotted against regressors used.}
\end{center}
\end{figure}

Analogous displays for the number of physician office visits against all regressors
can be produced via
<<ofp2, eval=FALSE>>=
plot(clog(ofp) ~ health, data = dt, varwidth = TRUE)
plot(clog(ofp) ~ cfac(numchron), data = dt)
plot(clog(ofp) ~ privins, data = dt, varwidth = TRUE)
plot(clog(ofp) ~ cfac(hosp, c(0:2, 8)), data = dt)
plot(clog(ofp) ~ gender, data = dt, varwidth = TRUE)
plot(cfac(ofp, c(0:2, 4, 6, 10, 100)) ~ school, data = dt, breaks = 9)
@
and are shown (with slightly enhanced labeling) in Figure~\ref{fig:ofp2}. 
The last plot uses a different type of display. Here, the dependent count variable
is not log-transformed but grouped into a factor and then a spinogram
is produced. This also groups the regressor (as in a histogram) and then
produces a highlighted mosaic plot. All displays show that the number of
doctor visits increases or decreases with the regressors as expected: \code{ofp}
decreases with the general health status but increases with the number of
chronic conditions or hospital stays. The median number of visits is also
slightly higher for patients with a private insurance and higher level of 
education. It is slightly lower for male compared to female patients.
The overall impression from all displays is that the changes in the mean
can only explain a modest amount of variation in the data.

<<models, echo=FALSE, results=hide>>=
if(refit_models & file.exists("countreg-models.rda")) file.remove("countreg-models.rda")
if(file.exists("countreg-models.rda")) {
  load("countreg-models.rda")
} else {
  fm_pois   <-      glm(ofp ~ ., data = dt, family = poisson)
  fm_qpois  <-      glm(ofp ~ ., data = dt, family = quasipoisson)
  fm_nbin   <-   MASS::glm.nb(ofp ~ ., data = dt)
  fm_zinb0  <- zeroinfl(ofp ~ ., data = dt, dist = "negbin")
  fm_zinb   <- zeroinfl(ofp ~ . | hosp + numchron + privins + school + gender, data = dt, dist = "negbin")
  fm_hurdle0<-   hurdle(ofp ~ ., data = dt, dist = "negbin")
  fm_hurdle <-   hurdle(ofp ~ . | hosp + numchron + privins + school + gender, data = dt, dist = "negbin")
  fm_hurdle2<-   hurdle(ofp ~ ., data = dt2, dist = "negbin")
  if(!refit_models) save(fm_pois, fm_qpois, fm_nbin, fm_zinb0, fm_zinb, fm_hurdle0, fm_hurdle, fm_hurdle2, file = "countreg-models.rda")
}
@

\subsection{Poisson regression}

As a first attempt to capture the relationship between the number of 
physician office visits and all regressors---described in \proglang{R} by the
formula \code{ofp ~ .}---in a parametric
regression model, we fit the basic Poisson regression model
<<poisson, eval=FALSE>>=
fm_pois <- glm(ofp ~ ., data = dt, family = poisson)
@
and obtain the coefficient estimates along with associated partial Wald tests
<<summary-poisson>>=
summary(fm_pois)
@
All coefficient estimates confirm the results from the exploratory analysis
in Figure~\ref{fig:ofp2}. All coefficients are highly significant with the
health variables leading to somewhat larger Wald statistics compared to the
socio-economic variables. However, the Wald test
results might be too optimistic due to a misspecification of the likelihood.
As the exploratory analysis suggested that over-dispersion is present in this data
set, we re-compute the Wald tests using sandwich standard errors via
<<coeftest-poisson>>=
coeftest(fm_pois, vcov = sandwich)
@
All regressors are still significant but the standard errors seem to be more
appropriate. This will also be confirmed by the following models that
deal with over-dispersion (and excess zeros) in a more formal way.

\subsection{Quasi-Poisson regression}

The quasi-Poisson model
<<quasipoisson, eval=FALSE>>=
fm_qpois <- glm(ofp ~ ., data = dt, family = quasipoisson)
@
leads to an estimated dispersion of
$\hat \phi = \Sexpr{round(summary(fm_qpois)$dispersion, digits = 3)}$ which
is clearly larger than $1$ confirming that over-dispersion is present in
the data.\footnote{Alternatively, over-dispersion can be confirmed by comparison
of the log-likelihoods of the Poisson and negative binomial model.}
The resulting partial Wald tests of the coefficients
are rather similar to the results obtained from the Poisson regression with
sandwich standard errors, leading to the same conclusions. As before, they
can be obtained via
<<summary-quasipoisson, eval=FALSE>>=
summary(fm_qpois)
@
The output is suppressed here and is presented in tabular form in Table~\ref{tab:summary}.


\subsection{Negative binomial regression}

A more formal way to accommodate over-dispersion in a count data regression
model is to use a negative binomial model, as in
<<nbin, eval=FALSE>>=
fm_nbin <- MASS::glm.nb(ofp ~ ., data = dt)
summary(fm_nbin)
@
As shown in Table~\ref{tab:summary},
both regression coefficients and standard errors are rather similar
to the quasi-Poisson and the sandwich-adjusted Poisson results above. Thus,
in terms of predicted means all three models give very similar results;
the associated partial Wald tests also lead to the same conclusions.

One advantage of the negative binomial model is that it is associated with a formal
likelihood so that information criteria are readily available. Furthermore, the expected
number of zeros can be computed from the fitted densities via 
$\sum_i f(0, \hat \mu_i, \hat \theta)$. 


\subsection{Hurdle regression}

The exploratory analysis conveyed the impression that there might be more zero observations
than explained by the basic count data distributions, hence a negative
binomial hurdle model is fitted via
<<hurdle0, eval=FALSE>>=
fm_hurdle0 <- hurdle(ofp ~ ., data = dt, dist = "negbin")
@
This uses the same type of count data model as in the preceding section
but it is now truncated for \code{ofp < 1} and has an additional hurdle
component modeling zero vs.\ count observations. By default, the hurdle
component is a binomial GLM with logit link which contains all regressors used in the count
model. The associated coefficient estimates and partial Wald tests for
both model components are displayed via
<<summary-hurdle0>>=
summary(fm_hurdle0)
@
The coefficients in the count component resemble those from the previous
models, but the increase in the log-likelihood (see also Table~\ref{tab:summary})
conveys that the model has improved by including the hurdle component.
However, it might be possible to omit the \code{health} variable from
the hurdle model. To test this hypothesis, the reduced model is fitted
via
<<hurdle, eval=FALSE>>=
fm_hurdle <- hurdle(ofp ~ . | hosp + numchron + privins + school + gender,
  data = dt, dist = "negbin")
@
and can then be compared to the full model in a Wald test
<<waldtest-hurdle>>=
waldtest(fm_hurdle0, fm_hurdle)
@
or an LR test
<<lrtest-hurdle, eval=FALSE>>=
lrtest(fm_hurdle0, fm_hurdle)
@
which leads to virtually identical results.

\begin{table}[p]
\begin{center}
\begin{tabular}{|l|rrrr|rr|} \hline
Type & \multicolumn{4}{|c|}{GLM} & \multicolumn{2}{|c|}{zero-augmented} \\
Distribution & \multicolumn{3}{|c}{Poisson} & \multicolumn{1}{c|}{NB} &
  \multicolumn{1}{|c}{Hurdle-NB} & \multicolumn{1}{c|}{ZINB} \\
Method & \multicolumn{1}{|c}{ML} & \multicolumn{1}{c}{adjusted} & \multicolumn{1}{c}{quasi} &
  \multicolumn{1}{c|}{ML} & \multicolumn{1}{|c}{ML} & \multicolumn{1}{c|}{ML} \\ 
Object & \code{fm_pois} & \code{fm_pois} & \code{fm_qpois} & \code{fm_nbin} & \code{fm_hurdle} & \code{fm_zinb} \\ \hline
<<summary-table, echo=FALSE, results=tex>>=
fm <- list("ML-Pois" = fm_pois, "Adj-Pois" = fm_pois, "Quasi-Pois" = fm_qpois, "NB" = fm_nbin,
  "Hurdle-NB" = fm_hurdle, "ZINB" = fm_zinb)
fm_summary <- matrix(character(6 * 33), ncol = 6)
colnames(fm_summary) <- names(fm)
rownames(fm_summary) <- c(as.vector(rbind(names(coef(fm_hurdle, model = "count")), "")),
  as.vector(rbind(names(coef(fm_hurdle, model = "zero")), "")),
  "no.\\ parameters", "$\\log L$", "AIC", "BIC", "$\\sum_i \\hat f_i(0)$")
rownames(fm_summary)[1:28] <- ifelse(rownames(fm_summary)[1:28] == "", "", 
  paste("\\code{", rownames(fm_summary)[1:28], "}", sep = ""))
fm_summary[1:8 * 2 - 1,] <- sapply(fm,
  function(x) paste("$", format(round(coef(x)[1:8], digits = 3)), "$\\phantom{)}", sep = ""))
fm_summary[1:8 * 2,] <- sapply(
  c(list("ML-Pois" = vcov(fm_pois), "Adj-Pois" = sandwich(fm_pois)),
  lapply(fm[-(1:2)], function(x) vcov(x))), function(x)
  paste("(", format(round(sqrt(diag(x))[1:8], digits = 3)), ")", sep = ""))
fm_summary[1:6 * 2 + 15,] <- cbind(NA, NA, NA, NA, sapply(fm[5:6], function(x)
  paste("$", format(round(coef(x, model = "zero"), digits = 3)), "$\\phantom{)}", sep = "")))
fm_summary[1:6 * 2 + 16,] <- cbind(NA, NA, NA, NA, sapply(fm[5:6],
  function(x) paste("(", format(round(sqrt(diag(vcov(x)))[-(1:8)], digits = 3)),  ")", sep = "")))
fm_summary[29,] <- sapply(fm, function(x) attr(logLik(x), "df"))
fm_summary[30,] <- paste("$", format(sapply(fm, function(x) round(logLik(x), digits = 1))), "$", sep = "")
fm_summary[31,] <- format(round(sapply(fm, AIC), digits = 1))
fm_summary[32,] <- format(round(sapply(fm, AIC, k = log(nrow(dt))), digits = 1))
fm_summary[33,] <- round(c("ML-Pois" = sum(dpois(0, fitted(fm_pois))),
  "Adj-Pois" = NA,
  "Quasi-Pois" = NA,
  "NB" = sum(dnbinom(0, mu = fitted(fm_nbin), size = fm_nbin$theta)),
  "NB-Hurdle" = sum(predict(fm_hurdle, type = "prob")[,1]),
  "ZINB" = sum(predict(fm_zinb, type = "prob")[,1])))
fm_summary[30:33,2:3] <- NA
fm_summary[is.na(fm_summary)] <- " "
fm_summary <- paste(apply(cbind(rownames(fm_summary), fm_summary), 1, paste, collapse = " & "), "\\\\")
fm_summary[c(16, 28, 33)] <- paste(fm_summary[c(16, 28, 33)], "\\hline")
writeLines(fm_summary)
@
\end{tabular}
\caption{\label{tab:summary} Summary of fitted count regression models for NMES data:
 coefficient estimates from count model, zero-inflation model (both with standard errors
 in parentheses), number of estimated parameters, maximized log-likelihood, AIC, BIC
 and expected number of zeros (sum of fitted densities evaluated at zero). The observed
 number of zeros is \Sexpr{sum(dt$ofp < 1)} in \Sexpr{nrow(dt)} observations.}
\end{center}
\end{table}


\subsection{Zero-inflated regression}

A different way of augmenting the negative binomial count model \code{fm_nbin}
with additional probability weight for zero counts is a zero-inflated
negative binomial (ZINB) regression. The default model is fitted via
<<zinb0, eval=FALSE>>=
fm_zinb0 <- zeroinfl(ofp ~ ., data = dt, dist = "negbin")
@
As for the hurdle model above, all regressors from the count model are also
used in the zero-inflation model. Again, we can modify the regressors in 
the zero-inflation part, e.g., by fitting a second model
<<zinb, eval=FALSE>>=
fm_zinb <- zeroinfl(ofp ~ . | hosp + numchron + privins + school + gender,
  data = dt, dist = "negbin")
@
that has the same variables in the zero-inflation part as the hurdle
component in \code{fm_hurdle}. By omitting the \code{health} variable, the
fit does not change significantly which can again be brought out by a Wald test
<<waldtest-zinb>>=
waldtest(fm_zinb0, fm_zinb)
@
or an LR test \code{lrtest(fm_zinb0, fm_zinb)} that produces virtually identical
results. The chosen fitted model can again be inspected via
<<summary-zinb, eval=FALSE>>=
summary(fm_zinb)
@
See Table~\ref{tab:summary} for a more concise summary.

\subsection{Comparison}

Having fitted several count data regression models to the demand for medical care
in the NMES data, it is, of course, of interest to understand what these models have
in common and what their differences are. In this section, we show how to compute
the components of Table~\ref{tab:summary} and provide some further comments and
interpretations. 

As a first comparison, it is of natural
interest to inspect the estimated regression coefficients in the count data model

<<coef-count, results=hide>>=
fm <- list("ML-Pois" = fm_pois, "Quasi-Pois" = fm_qpois, "NB" = fm_nbin,
  "Hurdle-NB" = fm_hurdle, "ZINB" = fm_zinb)
sapply(fm, function(x) coef(x)[1:8])
@

The result (see Table~\ref{tab:summary}) shows that there are some small differences,
especially between the GLMs and the
zero-augmented models. However, the zero-augmented models have to be interpreted 
slightly differently: While the GLMs all have the same mean function (\ref{eq:mean}),
the zero-augmentation also enters the mean function, see (\ref{eq:zeroinfl-mean}) and
(\ref{eq:hurdle-mean}). Nevertheless,
the overall impression is that the estimated mean functions are rather similar.
Moreover, the associated estimated standard errors
are very similar as well (see Table~\ref{tab:summary}):
<<se-count, results=hide>>=
cbind("ML-Pois" = sqrt(diag(vcov(fm_pois))),
  "Adj-Pois" = sqrt(diag(sandwich(fm_pois))),
  sapply(fm[-1], function(x) sqrt(diag(vcov(x)))[1:8]))
@
The only exception are the model-based standard errors for the Poisson model,
when treated as a fully specified model, which is obviously not appropriate for this data set.

In summary, the models are not too different with respect to their fitted 
mean functions. The differences become obvious if not only the mean but the
full likelihood is considered:
<<logLik>>=
rbind(logLik = sapply(fm, function(x) round(logLik(x), digits = 0)),
  Df = sapply(fm, function(x) attr(logLik(x), "df")))
@
The ML Poisson model is clearly inferior to all other fits. The quasi-Poisson
model and the sandwich-adjusted Poisson model are not associated with a fitted
likelihood. The negative binomial already improves the fit dramatically but
can in turn be improved by the hurdle and zero-inflated models which give
almost identical fits. This also reflects that the over-dispersion in the data
is captured better by the negative-binomial-based models than the plain
Poisson model. Additionally, it is of interest how the zero counts
are captured by the various models. Therefore, the observed zero counts are
compared to the expected number of zero counts for the likelihood-based models:
<<zero-counts>>=
round(c("Obs" = sum(dt$ofp < 1),
  "ML-Pois" = sum(dpois(0, fitted(fm_pois))),
  "NB" = sum(dnbinom(0, mu = fitted(fm_nbin), size = fm_nbin$theta)),
  "NB-Hurdle" = sum(predict(fm_hurdle, type = "prob")[,1]),
  "ZINB" = sum(predict(fm_zinb, type = "prob")[,1])))
@
Thus, the ML Poisson model is again not appropriate whereas the negative-binomial-based
models are much better in modeling the zero counts. By construction, the
expected number of zero counts in the hurdle model matches the observed
number.

In summary, the hurdle and zero-inflation models lead to the best 
results (in terms of likelihood) on this data set. Above, their mean function for the
count component was already shown to be very similar, below we take a
look at the fitted zero components:
<<coef-zero>>=
t(sapply(fm[4:5], function(x) round(x$coefficients$zero, digits = 3)))
@
This shows that the absolute values are rather different---which is not
surprising as they pertain to slightly different ways of modeling zero
counts---but the signs of the coefficients match, i.e., are just inversed.
For the hurdle model, the zero hurdle component describes the probability
of observing a positive count whereas, for the ZINB model, the zero-inflation
component predicts the probability of observing a zero count from the
point mass component. Overall, both models lead to the same qualitative
results and very similar model fits. Perhaps the hurdle model is slightly
preferable because it has the nicer interpretation: there is one process
that controls whether a patient sees a physician or not, and a second 
process that determines how many office visits are made.

\section{Summary} \label{sec:summary}

The model frame for basic count data models from the GLM framework as well
as their implementation in the \proglang{R} system for statistical computing
is reviewed. Starting from these basic tools, it is presented how
hurdle and zero-inflated models extend the classical models and how
likewise their \proglang{R} implementation in package \pkg{pscl} re-uses
design and functionality of the corresponding \proglang{R} software.
Hence, the new functions \fct{hurdle} and \fct{zeroinfl} are straightforward
to apply for model fitting. Additionally, standard methods for diagnostics 
are provided and generic inference tools from other packages can easily be
re-used.


\section*{Computational details}

The results in this paper were obtained using
\proglang{R}~\Sexpr{paste(R.Version()[6:7], collapse = ".")} with the packages
\pkg{MASS}~\Sexpr{gsub("-", "--", packageDescription("MASS")$Version)},
\pkg{pscl}~\Sexpr{gsub("-", "--", packageDescription("pscl")$Version)},
\pkg{sandwich}~\Sexpr{gsub("-", "--", packageDescription("sandwich")$Version)},
\pkg{car}~\Sexpr{gsub("-", "--", packageDescription("car")$Version)},
\pkg{lmtest}~\Sexpr{gsub("-", "--", packageDescription("lmtest")$Version)}.
\proglang{R} itself and all packages used are available from
CRAN at \url{http://CRAN.R-project.org/}.

\bibliography{countreg}

\newpage

\begin{appendix}

\section{Technical details for hurdle models} \label{app:hurdle}

The fitting of hurdle models via ML in \fct{hurdle} is controlled by
the arguments in the \fct{hurdle.control} wrapper function:

\begin{Soutput}
hurdle.control(method = "BFGS", maxit = 10000, trace = FALSE,
  separate = TRUE, start = NULL, ...)
\end{Soutput}

This modifies some default arguments passed on to the optimizer \fct{optim},
such as \code{method}, \code{maxit} and \code{trace}. The latter is also
used within \fct{hurdle} and can be set to produce more verbose output
concerning the fitting process. The argument \code{separate} controls
whether the two components of the model are optimized separately (the default)
or not. This is possible because there are no mixed sources for the zeros
in the data (unlike in zero-inflation models). The argument \code{start}
controls the choice of starting values for calling \fct{optim}, all remaining
arguments passed through \code{...} are directly passed on to \fct{optim}.

By default, starting values are estimated by calling \fct{glm.fit} for
both components of the model separately, once for the counts and once
for zero vs.\ non-zero counts. If starting values are supplied,
\code{start} needs to be set to a named list with the parameters for the
\code{$count} and \code{$zero} part of the model (and potentially a
\code{$theta} dispersion parameter if a negative binomial distribution is used).

The fitted model object of class \class{hurdle} is similar to \class{glm}
objects and contains sufficient information on all aspects of the fitting
process. In particular, the estimated parameters and associated covariances
are included as well as the result from the \fct{optim} call. Furthermore,
the call, formula, terms structure etc.\ is contained, potentially also the
model frame, dependent variable and regressor matrices.

Following \fct{glm.nb}, the $\theta$ parameter of the negative binomial
distribution is treated as a nuisance parameter. Thus, the \code{$coefficients}
component of the fitted model object just contains estimates of
$\beta$ and $\gamma$ while the estimate of $\theta$ and its standard deviation 
(on a log scale) are kept in extra list elements \code{$theta} and \code{$SE.logtheta}.


\section{Technical details for zero-inflated models} \label{app:zeroinfl}

Both the interface of the \fct{zeroinfl} function as well as its fitted
model objects are virtually identical to the corresponding \class{hurdle}
functionality. Hence, we only provide some additional information for
those aspects that differ from those discussed above. The details of the
ML optimization are again provided by a \fct{zeroinfl.control} wrapper:

\begin{Soutput}
zeroinfl.control(method = "BFGS", maxit = 10000, trace = FALSE,
  EM = FALSE, start = NULL, ...)
\end{Soutput}

The only new argument here is the argument \code{EM} which allows for EM
estimation of the starting values. Instead of calling \fct{glm.fit} only once for
both components of the model, this process can be iterated until convergence of the
parameters to the ML estimates. The optimizer is still called subsequently
(for a single iteration) to obtain the Hessian matrix from which the estimated
covariance matrix can be computed.


\section{Methods for fitted zero-inflated and hurdle models} \label{app:methods}

Users typically should not need to compute on the internal structure of
\class{hurdle} or \class{zeroinfl} objects because a set of standard extractor functions is provided,
an overview is given in Table~\ref{tab:methods}.
This includes methods to the generic functions \fct{print} and \fct{summary} which
print the estimated coefficients along with further information. The \fct{summary} in particular
supplies partial Wald tests based on the coefficients and the covariance matrix.
As usual, the \fct{summary} method returns an object of class \class{summary.hurdle}
or \class{summary.zeroinfl}, respectively, containing the relevant summary statistics
which can subsequently be printed using the associated \fct{print} method.

The methods for \fct{coef} and \fct{vcov} by default
return a single vector of coefficients and their associated covariance matrix,
respectively, i.e., all coefficients are concatenated. By setting their \code{model}
argument, the estimates for a single component can be extracted. Concatenating
the parameters by default and providing a matching covariance matrix estimate
(that does not contain the covariances of further nuisance parameters) facilitates
the application of generic inference functions such as \fct{coeftest}, \fct{waldtest},
and \fct{linearHypothesis}. All of these compute Wald tests for which coefficient
estimates and associated covariances is essentially all information required and 
can therefore be queried in an object-oriented way with the \fct{coef} and \fct{vcov}
methods.

Similarly, the \fct{terms} and \fct{model.matrix} extractors can
be used to extract the relevant information for either component of the model.
A \fct{logLik} method is provided, hence \fct{AIC}
can be called to compute information criteria and \fct{lrtest} for conducting
LR tests of nested models.

The \fct{predict} method computes predicted means (default) or probabilities (i.e.,
likelihood contributions) for observed or new data. Additionally, the means
from the count and zero component, respectively, can be predicted. For the
count component, this is the predicted count mean (without hurdle/inflation):
$\exp(x_i^\top \beta)$. For the zero component, this is the  the ratio of
probabilities $(1 - f_\mathrm{zero}(0; z_i, \gamma))/(1 - f_\mathrm{count}(0; x_i, \beta))$
of observing non-zero counts in hurdle models. In zero-inflation models, it is
the probability $f_\mathrm{zero}(0; z_i, \gamma)$ of observing a zero
from the point mass component in zero-inflated models

Predicted means for the observed data can also be obtained by the \fct{fitted} method. Deviations
between observed counts $y_i$ and predicted means $\hat \mu_i$ can be obtained
by the \fct{residuals} method returning either raw residuals $y_i - \hat \mu_i$
or the Pearson residuals (raw residuals standardized by square root of the variance function)
with the latter being the default.

\begin{table}[t!]
\begin{center}
\begin{tabular}{|l|p{8.7cm}|}
\hline
Function & Description \\ \hline
\fct{print} & simple printed display with coefficient estimates\\
\fct{summary} & standard regression output (coefficient estimates, standard errors, partial Wald tests);
                returns an object of class ``\code{summary.}\textit{class}''
                containing the relevant summary statistics (which has a \fct{print} method) \\ 	\hline
\fct{coef} & extract coefficients of model (full or components), a single vector of all coefficients by default \\
\fct{vcov} & associated covariance matrix (with matching names) \\
\fct{predict} & predictions (means or probabilities) for new data \\
\fct{fitted} & fitted means for observed data \\
\fct{residuals} & extract residuals (response or Pearson) \\ \hline
\fct{terms} & extract terms of model components \\
\fct{model.matrix} & extract model matrix of model components \\
\fct{logLik} & extract fitted log-likelihood \\ \hline
\fct{coeftest} & partial Wald tests of coefficients \\
\fct{waldtest} & Wald tests of nested models \\
\fct{linearHypothesis} & Wald tests of linear hypotheses \\
\fct{lrtest} & likelihood ratio tests of nested models \\
\fct{AIC} & compute information criteria (AIC, BIC, \dots) \\ \hline
\end{tabular}
\caption{\label{tab:methods} Functions and methods for objects of class \class{zeroinfl}
  and \class{hurdle}. The first three blocks refer to methods, the last block contains generic
  functions whose default methods work because of the information supplied by the methods above.}
\end{center}
\end{table}

\section{Replication of textbook results} \label{app:replication}

\citet[p.~204]{countreg:Cameron+Trivedi:1998} use a somewhat extended version
of the model employed above. Because not all variables in that extended
model are significant, a reduced set of variables was used throughout the main
paper. Here, however, we use the full model to show that the tools in \pkg{pscl}
reproduce the results of \cite{countreg:Cameron+Trivedi:1998}.

After omitting the responses other than \code{ofp} and setting \code{"other"}
as the reference category for \code{region} using
<<dt2a, eval=FALSE>>=
dt2 <- DebTrivedi[, -(2:6)]
dt2$region <- relevel(dt2$region, "other")
@
we fit a model that contains all explanatory variables, both in the
count model and the zero hurdle model:
<<hurdle2, eval=FALSE>>=
fm_hurdle2 <- hurdle(ofp ~ ., data = dt2, dist = "negbin")
@

The resulting coefficient estimates are virtually identical to those published
in \citet[p.~204]{countreg:Cameron+Trivedi:1998}. The associated Wald statistics
are also very similar provided that sandwich standard errors are used
\citep[which is not stated explicitely in][]{countreg:Cameron+Trivedi:1998}.
<<hurdle2-summary>>=
cfz <- coef(fm_hurdle2, model = "zero")
cfc <- coef(fm_hurdle2, model = "count")
se <- sqrt(diag(sandwich(fm_hurdle2)))
round(cbind(zero = cfz, zero_t = cfz/se[-seq(along = cfc)], 
  count = cfc, count_t = cfc/se[seq(along = cfc)]),
  digits = 3)[c(3, 2, 4, 5, 7, 6, 8, 9:17, 1),]
logLik(fm_hurdle2)
1/fm_hurdle2$theta
@
There are some small and very few larger deviations in the Wald statistics
which are probably explicable by different approximations to the gradient
of $\theta$ (or $1/\theta$ or $\log(\theta)$) and the usage of different
non-linear optimizers (and at least ten years of software development).

More replication exercises are performed in the example sections of
\pkg{AER} \citep{countreg:Zeileis+Kleiber:2008}, the software package
accompanying \cite{countreg:Kleiber+Zeileis:2008}.

\end{appendix}

\end{document}

\name{predprob.glm}
\alias{predprob.glm}

\title{Predicted Probabilities for GLM Fits}

\description{
  Obtains predicted probabilities from a fitted generalized linear model
  object. 
}

\usage{
\method{predprob}{glm}(obj, newdata = NULL, at = NULL, ...)
}

\arguments{
  \item{obj}{a fitted object of class inheriting from \code{"glm"}}
  \item{newdata}{optionally, a data frame in which to look for variables
    with which to predict.  If omitted, the fitted linear predictors are
    used.}
  \item{at}{an optional numeric vector at which the probabilities are
    evaluated. By default \code{0:max(y)} where \code{y} is the original
    observed response.}
  \item{...}{arguments passed to or from other methods}
}

\details{
  This method is only defined for glm objects with
  \code{family=\link{binomial}} or \code{family=\link{poisson}}, or negative
  binomial count models fit with the \code{\link[MASS:glm.nb]{glm.nb}} function in \code{library(MASS)}.
}

\value{
  A matrix of predicted probabilities.  Each row in the matrix is a
  vector of probabilities, assigning predicted probabilities over the range of
  responses actually observed in the data.  For instance, for models
  with \code{family=binomial}, the matrix has two columns for the "zero"
  (or failure) and "one" (success) outcomes, respectively, and
  trivially, each row in the matrix sums to 1.0. 
  For counts fit with \code{family=poisson} or via \code{glm.nb}, the
  matrix has \code{length(0:max(y))} columns.  Each observation
  used in fitting the model generates a row to the returned matrix; alternatively, if
  \code{newdata} is supplied, the returned matrix will have as many rows
  as in \code{newdata}.
}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\seealso{\code{\link{predict.glm}}}
\examples{
data(bioChemists)
glm1 <- glm(art ~ .,
            data=bioChemists,
            family=poisson,
            trace=TRUE)  ## poisson GLM
phat <- predprob(glm1)
apply(phat,1,sum)                    ## almost all 1.0
}
\keyword{models}% at least one, from doc/KEYWORDS
\keyword{regression}% __ONLY ONE__ keyword per line

\name{AustralianElectionPolling}
\alias{AustralianElectionPolling}
\docType{data}
\title{
	Political opinion polls in Australia, 2004-07
}
\description{
	The results of 239 published opinion polls measuring vote intentions (1st preference vote intention in a House of Representatives election) between the 2004 and 2007 Australian Federal elections, from 4 survey houses.
}
\usage{data(AustralianElectionPolling)}
\format{
  A data frame with 239 observations on the following 14 variables.
  \describe{
    \item{\code{ALP}}{a numeric vector, percentage of respondents reported as intending to vote for the Australian Labor Party}
    \item{\code{Lib}}{a numeric vector, percentage of respondents reported as intending to vote for the Liberal Party}
    \item{\code{Nat}}{a numeric vector, percentage of respondents reported as intending to vote for the National Party}
    \item{\code{Green}}{a numeric vector, percentage of respondents reported as intending to vote for the Greens}
    \item{\code{FamilyFirst}}{a numeric vector, percentage of respondents reported as intending to vote for the Family First party}
    \item{\code{Dems}}{a numeric vector, percentage of respondents reported as intending to vote for the Australian Democrats}
    \item{\code{OneNation}}{a numeric vector, percentage of respondents reported as intending to vote for One Nation}
    \item{\code{DK}}{a numeric vector, percentage of respondents reported as expressing no preference or a \dQuote{don't know} response}
    \item{\code{sampleSize}}{a numeric vector, reported sample size of the poll}
    \item{\code{org}}{a factor with levels \code{Galaxy}, \code{Morgan, F2F}, \code{Newspoll}, \code{Nielsen} and \code{Morgan, Phone}, indicating the survey house and/or mode of the poll}
    \item{\code{startDate}}{a Date, reported start of the field period}
    \item{\code{endDate}}{a Date, reported end of the field period}
    \item{\code{source}}{a character vector, source of the poll report}
    \item{\code{remark}}{a character vector, remarks noted by author and/or research assistant coders}
  }
}
\details{Morgan uses two modes: phone and face-to-face.  

		The 2004 Australian election was on October 9; the ALP won 37.6\% of the 1st preferences cast in elections for the House of Representatives.  The ALP won the 2007 election (November 24) with 43.4\% of 1st preferences.  

The ALP changed leaders twice in the 2004-07 inter-election period spanned by these data: (1) Mark Latham resigned the ALP leadership on January 18 2005 and was replaced by Kim Beazley; (2) Beazley lost the ALP leadership to Kevin Rudd on December 4, 2006.  

   The then Prime Minister, John Howard, announced the November 2007 election on October 14, 2007.     
}
\source{See the \code{source} variable.  Andrea Abel assisted with the data collection.
}
\references{
	Jackman, Simon. 2009.  \emph{Bayesian Analysis for the Social Sciences}.  Wiley: Hoboken, New Jersey.  Example 9.3.
}
\examples{
data(AustralianElectionPolling)
if(require(lattice)) {
    lattice::xyplot(ALP ~ startDate | org, 
       data=AustralianElectionPolling,
       layout=c(1,5),
       type="b",
       xlab="Start Date",
       ylab="ALP")
}

## test for house effects
y <- AustralianElectionPolling$ALP/100
v <- y*(1-y)/AustralianElectionPolling$sampleSize
w <- 1/v
m1 <- mgcv::gam(y ~ s(as.numeric(startDate)),
          weight=w,	      
          data=AustralianElectionPolling)
m2 <- update(m1, ~ . + org)
anova(m1,m2)
}
\keyword{datasets}

\name{UKHouseOfCommons}
\alias{UKHouseOfCommons}
\docType{data}
\title{1992 United Kingdom electoral returns}
\description{Electoral returns, selected constituencies, 1992 general
  election for the British House of Commons}
\usage{data(UKHouseOfCommons)}
\format{
  A data frame with 521 observations on the following 12 variables.
  \describe{
    \item{\code{constituency}}{a character vector, name of the House of Commons constituency}
    \item{\code{county}}{a character vector, county of the House of Commons constituency}
    \item{\code{y1}}{a numeric vector, log-odds of Conservative to LibDem vote share}
    \item{\code{y2}}{a numeric vector, log-odds of Labor to LibDem vote share}
    \item{\code{y1lag}}{a numeric vector, \code{y1} from previous election}
    \item{\code{y2lag}}{a numeric vector, \code{y2} from previous election}
    \item{\code{coninc}}{a numeric vector, 1 if the incumbent is a
        Conservative, 0 otherwise}
    \item{\code{labinc}}{a numeric vector, 1 if the incumbent is from
        the Labor Party, 0 otherwise}
    \item{\code{libinc}}{a numeric vector, 1 if the incumbent is from
        the LibDems, 0 otherwise}
    \item{\code{v1}}{a numeric vector, Conservative vote share
        (proportion of 3 party vote)}
    \item{\code{v2}}{a numeric vector, Labor vote share  (proportion of 3 party vote)}
    \item{\code{v3}}{a numeric vector, LibDem vote share (proportion of 3 party vote)}
  }
}
\details{
  These data span only 521 of the 621 seats in the House of
  Commons at the time of 1992 election.  Seats missing either a Conservative,
  Labor, or a LibDem candidate appear to have been dropped.   

  The original Katz and King data set does not have case labels.  I
  used matches to an additional data source to recover a set of constituency labels for
  these data; labels could not recovered for two of the constituencies.
}
\source{
   Jonathan Katz; Gary King. 1999. "Replication data for: A Statistical Model of Multiparty Electoral Data", \url{http://hdl.handle.net/1902.1/QIGTWZYTLZ}
  
  %%Martin Baxter.  \url{http://www.electoralcalculus.co.uk/electdata_1992ob.txt}
}
\references{
  Katz, Jonathan and Gary King. 1999. ``A Statistical Model for
  Multiparty Electoral Data''. \emph{American Political Science
    Review}. 93(1): 15-32.
  
  Jackman, Simon. 2009.  \emph{Bayesian Analysis for the Social
    Sciences}.  Wiley: Chichester.  Example 6.9.
}
\examples{
data(UKHouseOfCommons)
tmp <- UKHouseOfCommons[,c("v1","v2","v3")] 
summary(apply(tmp,1,sum))

col <- rep("black",dim(tmp)[1])
col[UKHouseOfCommons$coninc==1] <- "blue"
col[UKHouseOfCommons$labinc==1] <- "red"
col[UKHouseOfCommons$libinc==1] <- "orange"

library(vcd)
vcd::ternaryplot(tmp,
                 dimnames=c("Cons","Lab","Lib-Dem"),
                 labels="outside",
                 col=col,
                 pch=1,
                 main="1992 UK House of Commons Election",
                 cex=.75)
}
\keyword{datasets}

\name{dropRollCall}
\alias{dropRollCall}
\title{drop user-specified elements from a rollcall object}

\description{
  Drop user-specified elements of rollcall object, returning a roll
  call object. 
}

\usage{
dropRollCall(object, dropList,debug=FALSE)
}

\arguments{
  \item{object}{an object of class \code{\link{rollcall}}}

  \item{dropList}{a \code{\link{list}} (or \code{\link{alist}}) with
    some (or all) of the following components:
    \describe{
      \item{codes}{character or numeric, possibly a vector.  If
	character, it should match the names of \code{object$codes},
	indicating the set of entries in \code{object$votes} to be set to
	\code{NA}.  If numeric, then \code{codes} indicates the entries in
	\code{object$votes} that will be set to \code{NA}.}
      
      \item{lop}{numeric, non-negative integer, less than number of
	legislators represented in \code{object}.  Roll calls with \code{lop}
	or fewer legislators voting in the minority are dropped.}
      
      \item{legisMin}{numeric, non-negative integer, less than
	number of roll calls represented in \code{object}.  Legislators with
	\code{legisMin} or fewer votes are dropped.}
      
      \item{dropLegis}{an \code{\link{expression}} that evaluates to
	mode \code{logical}, vector of length equal to the number of
	legislators represented in \code{object}.  The expression is evaluated
	in the \code{legis.data} component of the rollcall \code{object}.
	Legislators for whom the expression evaluates to \code{TRUE} are
	dropped.}
      
      \item{dropVotes}{an \code{\link{expression}} that evaluates to
	mode \code{logical}, vector of length equal to the number of
	rollcalls represented in \code{object}.  The expression is evaluated
	in the \code{vote.data} component of the rollcall \code{object}.
	Rollcalls for which the expression evaluates to \code{TRUE} are
	dropped.}
    }
  }
  \item{debug}{\code{logical}, set to \code{TRUE} to see messages
      printed to the console as inspection and subsetting of the
      \code{rollcall} object takes place}  
}
\details{It is often desirable to restrict the analysis of roll call
      data in various ways.  For one thing, unanimous votes provide no
      information discriminating among legislators: hence, 
      summary and analysis should almost always use \code{dropList=list(lop=0)}.
      See the examples for other possibilities, limited only by the
      information supplied in \code{legis.data} and \code{votes.data}.
}
\value{
  An object of class \code{\link{rollcall}} with components modified/added by
  the subsetting indicated in the \code{dropList}.
  }
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\note{
  With the exception of \code{codes}, each component of
  \code{dropList} generates a vector of mode \code{\link{logical}},
  either with respect to legislators or votes.  These logical
  vectors are then combined element-wise, such that if any one of
  the subsetting restrictions is \code{TRUE} for a particular
  legislator or vote, then that legislator or vote is dropped.  Some
  summaries are reported to the console along the way if \code{debug=TRUE}.

  \code{dropRollCall} adds a component
  named \code{dropInfo} to the \code{rollcall} object it returns.  This
  component is itself a list containing named components
  \describe{
    \item{legislators}{a vector of mode \code{logical}, with each
      element \code{TRUE} if the legislator is retained in the returned
      \code{rollcall} object.}
    \item{votes}{a vector of mode \code{logical}, with each element
      \code{TRUE} if the corresponding is retained in the returned
      \code{rollcall} object.}
    \item{dropList}{the \code{dropList} supplied as input to
  \code{dropRollCall}.} 
  }
  If the input \code{rollcall} object is itself the product of a call to
  \code{dropRollCall}, the \code{dropInfo} component on output is a list
  with named components 
  \describe{
    \item{previous}{the \code{dropInfo} component of the input
      \code{rollcall} object.}
    \item{new}{the \code{dropInfo} list created by the current call to
      \code{dropRollCall}.}
  }
  Functions like \code{summary.rollcall} try to handle this information
  sensibly.

  When \code{dropList} uses the \code{dropLegis} or \code{dropVotes}
  components then \code{dropList} should be constructed via the
  \code{\link{alist}} command; this ensures that the
  \code{dropLegis} and \code{dropVotes} components of
  \code{dropList} are objects of mode \code{\link{expression}}, and
  \code{\link{eval}}uated to mode \code{\link{logical}} in the
  \code{legis.data} and \code{vote.data} \code{\link{environment}s} by
  the function, if possible (rather than being evaluated immediately in
  the environment calling \code{dropRollCall} or constructing
  \code{dropList}).  See the examples.  This is not entirely
  satisfactory, and behavior more like the \code{subset} argument in
  function \code{\link{lm}} would be preferable.
}

\seealso{\code{\link{dropUnanimous}}, \code{\link{summary.rollcall}},
  \code{\link{ideal}}, \code{\link{alist}}.} 

\examples{
data(s109)
s109.working <- dropRollCall(s109,
                             dropList=list(lop=0))
summary(s109.working)

s109.working <- dropRollCall(s109,
                             dropList=list(lop=0,
                               code="notInLegis"))
summary(s109.working)

s109.working <- dropRollCall(s109,
                             dropList=list(lop=3,
                               code="notInLegis"))
summary(s109.working)

## note use of alist, since dropLegis is an expression
dropList <- alist(lop=3,
                 dropLegis=party!="D",
                 code="notInLegis")
s109.working <- dropRollCall(s109,dropList=dropList,debug=TRUE)
summary(s109.working)

s109.working <- dropRollCall(s109.working,dropList=list(legisMin=25))
summary(s109.working)


\dontrun{
## read 102nd House from Poole web site
h102 <- readKH("ftp://voteview.ucsd.edu/dtaord/hou102kh.ord")

## drop President from roll call matrix
h102 <- dropRollCall(h102,
                     dropList=alist(dropLegis=state=="USA"))
summary(h102)
}
}
\keyword{manip}

\name{postProcess}
\alias{postProcess}
\title{remap MCMC output via affine transformations}

\description{
  Remap the MCMC iterates in an \code{\link{ideal}} object via an affine
  transformation, imposing identifying restrictions ex post (aka
  post-processing).} 

\usage{
postProcess(object, constraints="normalize", debug = FALSE)
}

\arguments{ \item{object}{an object of class \code{\link{ideal}}}
  \item{constraints}{list of length \code{d+1}, each component providing
  a set of \code{d} restrictions, where \code{d} is the dimension of the
  fitted \code{\link{ideal}} model; or the character string
  \code{normalize} (default).  If a list, the name of each component should
  uniquely match a legislator/subject's name.  See Details.}
  \item{debug}{logical flag for verbose output, used for debugging} }

\details{
  Item-response models are unidentified without restrictions on the
  underlying parameters.  Consider the \code{d=1} dimensional case. The
  model is
  \deqn{P(y_{ij} = 1) = F(x_i \beta_j - \alpha_j)}{%
    Pr(y_[ij] = 1) = F(x_i b_j - a_j).
  }
  Any
  linear transformation of the latent traits, say,
  \deqn{x^* = mx + c}{%
    x* = mx + c
  }
  can be exactly offset by applying the appropriate linear
  transformations to the item/bill parameters, meaning that there is no
  unique set of values for the model parameters that will maximize the
  likelihood function.  In higher dimensions,
  the latent traits can also be transformed via any arbitrary rotation,
  dilation and translation, with offsetting transformations applied to
  the item/bill parameters.

  One strategy in MCMC is to ignore the lack of identification at run
  time, but apply identifying restrictions ex post,
  \dQuote{post-processing} the MCMC output, iteration-by-iteration.  In
  a \code{d}-dimensional IRT model, a sufficient condition for global
  identification is to fix \code{d+1} latent traits, provided the
  constrained latent traits span the \code{d} dimensional latent space.
  This function implements this strategy.  The user supplies a set of
  constrained ideal points in the \code{constraints} list.  The function
  then processes the MCMC output in the \code{\link{ideal}}
  \code{object}, finding the transformation that maps the current
  iteration's sampled values for \code{x} (latent traits/ideal points)
  into the sub-space of identified parameters defined by the fixed
  points in \code{constraints}; i.e., what is the affine transformation
  that maps the unconstrained ideal points into the constraints?  Aside
  from minuscule numerical inaccuracies resulting from matrix inversion
  etc, this transformation is exact: after post-processing, the
  \code{d+1} constrained points do not vary over the MCMC iterations.
  The remaining \code{n-d-1} ideal points are subject to (posterior)
  uncertainty; the \dQuote{random tour} of the joint parameter space of
  these parameters produced by the MCMC algorithm has been mapped into a
  subspace in which the parameters are globally identified.

  If the \code{\link{ideal}} object was produced with \code{store.item}
  set to \code{TRUE}, then the item parameters are also post-processed,
  applying the inverse transformation.  Specifically, recall that the
  IRT model is \deqn{P(y_{ij} = 1) = F(x_i'\beta_j)}{%
    Pr(y_[ij] = 1) = F(x_i' b_j)}
  where in this formulation \eqn{x_i} is a vector of
  length \code{d+1}, including a \code{-1} to put a constant term into
  the model (i.e., the intercept or difficulty parameter
  is part of \eqn{\beta_j}{beta_j}).  Let \eqn{A} denote the
  non-singular, \code{d+1}-by-\code{d+1} matrix that maps the \eqn{x}
  into the space of identified parameters.  Recall that this
  transformation is computed iteration by iteration.  Then each
  \eqn{x_i} is transformed to \eqn{x^*_i = Ax_i}{x*_i = Ax_i} and
  \eqn{\beta_j}{b_j} is transformed to \eqn{\beta_j^* = A^{-1}
    \beta_j}{b_j^* = A^(-1) b_j}, \eqn{i = 1, \ldots, n; j = 1,
    \ldots, m}.
  
  Local identification can be obtained for a one-dimensional model by
  simply imposing a normalizing restriction on the ideal points: this
  normalization (mean zero, standard deviation one) is the default
  behavior, but (a) is only sufficient for local identification when
  the \code{rollcall} object was fit with \code{d=1}; (b) is not
  sufficient for even local identification when \code{d>1}, with
  further restrictions required so as to rule out other forms of
  invariance (e.g., translation, or "dimension-switching", a
  phenomenon akin to label-switching in mixture modeling).

  The default is to impose dimension-by-dimension normalization with
  respect to the means of the marginal posterior densities of the
  ideal points, such that the these means (the usual Bayes estimates
  of the ideal points) have mean zero and standard deviation one
  across legislators.  An offsetting transformation is applied to the
  items parameters as well, if they are saved in the \code{ideal} object.  

  Specifically, in one-dimension, the
  two-parameter IRT model is \deqn{P(y_{ij} = 1) = F(x_i \beta_j -
  \alpha_j).}{Pr(y_[ij] = 1) = F(x_i b_j - a_j).}  If we normalize the
  \eqn{x_i} to \eqn{x*_i = (x_i - c)/m} then the offsetting
  transformations for the item/bill parameters are \eqn{\beta_j^* =
  \beta_j m}{b*_j = b_j m} and \eqn{\alpha_j^* = \alpha_j -
  c\beta_j}{a*_j = a_j - cb_j}.
}

\value{
  An object of class \code{\link{ideal}}, with components suitably
  transformed and recomputed (i.e., \code{x} is transformed and
  \code{xbar} recomputed, and if the
  \code{\link{ideal}} object was fit with \code{store.item=TRUE},
  \code{beta} is transformed and \code{betabar} is recomputed).
}

\references{
Hoff, Peter, Adrian E. Raftery and Mark S. Handcock. 2002.
Latent Space Approaches to Social Network Analysis. \emph{Journal
  of the American Statistical Association} 97:1090--1098.

Edwards, Yancy D. and Greg M. Allenby. 2003.
Multivariate Analysis of Multiple Response Data. \emph{Journal of
  Marketing Research} 40:321--334.

Rivers, Douglas. 2003.  \dQuote{Identification of Multidimensional
  Item-Response Models.} Typescript.  Department of Political Science,
Stanford University.
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\note{Applying transformations to obtain identification can
  sometimes lead to surprising results.  Each data point makes the same
  likelihood contributions with either the identified or unidentified
  parameters.  But, in general, predictions generated with the
  parameters set to their posterior means will differ depending on
  whether one uses the identified subset of parameters or the
  unidentified parameters.  For this reason, caution should be used when
  using a function such as \code{\link{predict}} after post-processing
  output from \code{\link{ideal}}.  A better strategy is to compute the
  estimand of interest at each iteration and then take averages over
  iterations.

  When specifying a value of \code{burnin} different from that used in
  fitting the \code{\link{ideal}} object, note a distinction between
  the iteration numbers of the stored iterations, and the number of
  stored iterations.  That is, the \code{n}-th iteration stored in an
  \code{\link{ideal}} object will not be iteration \code{n} if the
  user specified \code{thin>1} in the call to \code{\link{ideal}}.
  Here, iterations are tagged with their iteration number.  Thus, if
  the user called \code{\link{ideal}} with \code{thin=10} and
  \code{burnin=100} then the stored iterations are numbered \code{100,
    110, 120, ...}.  Any future subsetting via a \code{burnin} refers to
  this iteration number.
}
  
\examples{
data(s109)
f = system.file("extdata",package="pscl","id1.rda")
load(f)

id1Local <- postProcess(id1)    ## default is to normalize
summary(id1Local)

id1pp <- postProcess(id1,
                     constraints=list(BOXER=-1,INHOFE=1))
summary(id1pp)

## two-dimensional fit
f = system.file("extdata",package="pscl","id2.rda")
load(f)

id2pp <- postProcess(id2,
                     constraints=list(BOXER=c(-1,0),
                       INHOFE=c(1,0),
                       CHAFEE=c(0,.25)))

tracex(id2pp,d=1:2,
       legis=c("BOXER","INHOFE","COLLINS","FEINGOLD","COLEMAN",
         "CHAFEE","MCCAIN","KYL"))
}
\keyword{models}

\name{predprob.ideal}
\alias{predprob.ideal}
\title{predicted probabilities from fitting ideal to rollcall data}
\description{
  Computes predicted probabilities of a \dQuote{Yea} vote conditional on
  the posterior means of the legislators' ideal points and
  vote-specific parameters.
}
\usage{
\method{predprob}{ideal}(obj, ...)
}
\arguments{
  \item{obj}{An object of class \code{\link{ideal}}}
  \item{\dots}{Arguments to be passed to other functions}
}
\details{
  This is a wrapper function to \code{\link{predict.ideal}}, extracting
  just the predicted probabilities component of the object returned by
  that function.  Predicted probabilities can and are generated for each
  voting decision, irrespective of whether the legislator actually voted
  on any particular roll call.
}
\value{
  A \code{\link{matrix}} of dimension \code{n} (number of legislators)
  by \code{m} (number of roll call votes).
}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\seealso{\code{\link{ideal}}, \code{\link{predprob}}, \code{\link{predict.ideal}}}
\examples{
f <- system.file("extdata","id1.rda",package="pscl")
load(f)
phat <- predprob(id1)
dim(phat)
}
\keyword{methods}% at least one, from doc/KEYWORDS
\keyword{models}% __ONLY ONE__ keyword per line

\name{idealToMCMC}
\alias{idealToMCMC}
\title{convert an object of class ideal to a coda MCMC object}
\description{
  Converts the x element of an \code{ideal} object to an MCMC object,
  as used in the \pkg{coda} package.
}

\usage{
idealToMCMC(object, burnin=NULL)
}

\arguments{
  \item{object}{an object of class \code{\link{ideal}}.}

 \item{burnin}{of the recorded MCMC samples, how many to discard as
    burnin?  Default is \code{NULL}, in which case the value of
    \code{burnin} in the \code{\link{ideal}} object is used.}
}

\value{A \code{\link[coda:mcmc]{mcmc}} object as used by the
  \pkg{coda} package, starting at
  iteration \code{start}, drawn from the \code{x} component of the
  \code{\link{ideal}} object.}

\note{When specifying a value of \code{burnin} different from that used
  in fitting the \code{\link{ideal}} object, note a distinction
  between the iteration numbers of the stored iterations, and the
  number of stored iterations.  That is, the \code{n}-th iteration
  stored in an \code{\link{ideal}} object will not be iteration
  \code{n} if the user specified \code{thin>1} in the call to
  \code{\link{ideal}}.  Here, iterations are tagged with their
  iteration number.  Thus, if the user called \code{\link{ideal}} with
  \code{thin=10} and \code{burnin=100} then the stored iterations are
  numbered \code{100, 110, 120, ...}.  Any future subsetting via a
  \code{burnin} refers to this iteration number.}

\seealso{\code{\link{ideal}}, \code{\link[coda:mcmc]{mcmc}}}

\examples{
data(s109)
f = system.file("extdata",package="pscl","id1.rda")
load(f)
id1coda <- idealToMCMC(id1)
summary(id1coda)  
}
\keyword{classes}

\name{predict.ideal}
\alias{predict.ideal}
\alias{print.predict.ideal}
\title{predicted probabilities from an ideal object}

\description{Compute predicted probabilities from an \code{\link{ideal}}
  object.  This predict method uses the posterior mean values of \eqn{x}
  and \eqn{\beta}{beta} to make predictions.}

\usage{
\method{predict}{ideal}(object,
                        cutoff=.5,
                        burnin=NULL,
                        ...)

\method{print}{predict.ideal}(x,digits=2,...)

}
\arguments{
  \item{object}{an object of class \code{ideal} (produced by
    \code{\link{ideal}}) with item parameters (beta) stored; i.e.,
    \code{store.item=TRUE} was set when the \code{ideal} object was
    fitted} 
  \item{cutoff}{numeric, a value between 0 and 1, the threshold to be
    used for classifying predicted probabilities of a Yea 
    votes as predicted Yea and Nay votes.}
  \item{burnin}{of the recorded MCMC samples, how many to discard as
    burnin?  Default is \code{NULL}, in which case the value of
    \code{burnin} in the \code{\link{ideal}} object is used.}
  \item{x}{object of class \code{predict.ideal}}
  \item{digits}{number of digits in printed object}
  \item{...}{further arguments passed to or from other methods.}  
}

\details{
  Predicted probabilities are computed using the mean of the posterior
  density of
  of \eqn{x} (ideal points, or latent ability) and \eqn{\beta} (bill or
  item parameters).  The percentage correctly predicted
  are determined by counting the percentages of votes with predicted
  probabilities of a Yea vote greater than or equal to the \code{cutoff} as the
  threshold.
}

\value{
  An object of class \code{predict.ideal}, containing:
  \item{pred.probs}{the calculated predicted probability for each
    legislator for each vote.}
  \item{prediction}{the calculated prediction (0 or 1) for each
    legislator for each vote.}
  \item{correct}{for each legislator for each vote, whether the
    prediction was correct.}
  \item{legis.percent}{for each legislator, the percent of votes
    correctly predicted.}
  \item{vote.percent}{for each vote, the percent correctly predicted.}
  \item{yea.percent}{the percent of yea votes correctly predicted.}
  \item{nay.percent}{the percent of nay votes correctly predicted.}
  \item{party.percent}{the average value of the percent correctly
    predicted by legislator, separated by party, if party information
    exists in the \code{rollcall} object used for \code{ideal}.  If no
    party information is available, \code{party.percent = NULL}.}
  \item{overall.percent}{the total percent of votes correctly
    predicted.}
  \item{ideal}{the name of the \code{\link{ideal}} object, which can be
    later \code{\link{eval}}uated}
  \item{desc}{string, the descriptive text from the
  \code{\link{rollcall}} object passed to \code{\link{ideal}}}
}

\note{When specifying a value of \code{burnin} different from that used
    in fitting the \code{\link{ideal}} object, note a distinction
    between the iteration numbers of the stored iterations, and the
    number of stored iterations.  That is, the \code{n}-th iteration
    stored in an \code{\link{ideal}} object will not be iteration
    \code{n} if the user specified \code{thin>1} in the call to
    \code{\link{ideal}}.  Here, iterations are tagged with their
    iteration number.  Thus, if the user called \code{\link{ideal}} with
    \code{thin=10} and \code{burnin=100} then the stored iterations are
    numbered \code{100, 110, 120, ...}.  Any future subsetting via a
    \code{burnin} refers to this iteration number.}

\seealso{\code{\link{ideal}}, \code{\link{summary.ideal}}, \code{\link{plot.predict.ideal}}}

\examples{
data(s109)

f <- system.file("extdata","id1.rda",package="pscl")
load(f)
phat <- predict(id1)
phat         ## print method
}

\keyword{classes}

\name{pR2}
\alias{pR2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{compute various pseudo-R2 measures}
\description{
  compute various pseudo-R2 measures for various GLMs
}
\usage{
pR2(object, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{a fitted model object for which \code{logLik},
    \code{update}, and \code{model.frame} methods exist (e.g.,
    an object of class \code{glm}, \code{polr}, or \code{mulitnom})}
  \item{\dots}{additional arguments to be passed to or from functions}
}

\details{Numerous pseudo r-squared measures have been proposed for
  generalized linear models, involving a comparison of the
  log-likelihood for the fitted model against the log-likelihood of a
  null/restricted model with no predictors, normalized to run from zero
  to one as the fitted model provides a better fit to the data
  (providing a rough analogue to the computation of r-squared in a
  linear regression).  }

\value{
  A vector of length 6 containing
  \item{llh}{The log-likelihood from the fitted model}
  \item{llhNull}{The log-likelihood from the intercept-only restricted
    model}
  \item{G2}{Minus two times the difference in the log-likelihoods}
  \item{McFadden}{McFadden's pseudo r-squared}
  \item{r2ML}{Maximum likelihood pseudo r-squared}
  \item{r2CU}{Cragg and Uhler's pseudo r-squared}
}
\references{Long, J. Scott. 1997. \emph{Regression Models for Categorical and
  Limited Dependent Variables}.  Sage.  pp104-106.}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\seealso{\code{\link{extractAIC}}, \code{\link{logLik}}}
\examples{
data(admit)
## ordered probit model
op1 <- MASS::polr(score ~ gre.quant + gre.verbal + ap + pt + female,
            Hess=TRUE,
            data=admit,
            method="probit")
pR2(op1)   
}
\keyword{models}

\name{rollcall}
\alias{rollcall}
\title{create an object of class rollcall}
\description{
  Create a \code{rollcall} object, used for the analysis of legislative
  voting or, equivalently, item-response modeling of binary data
  produced by standardized tests, etc.
}

\usage{
rollcall(data,
         yea=1, nay=0, missing=NA, notInLegis=9,
         legis.names=NULL, vote.names=NULL,
         legis.data=NULL, vote.data=NULL,
         desc=NULL, source=NULL)
}

\arguments{

  \item{data}{voting decisions (for roll calls), or test results (for
    IRT).  Can be in one of two forms.  First, \code{data} may be a
    \code{\link{matrix}}, with rows corresponding to legislators
    (subjects) and columns to roll calls (test items).  \code{data} can
    also be a \code{\link{list}} with an element named \code{votes}
    containing the matrix described above.}
  
  \item{yea}{numeric, possibly a vector, code(s) for a Yea vote in the
    rollcall context, or a correct answer in the educational testing
    context. Default is 1.}
  
  \item{nay}{numeric, possibly a vector, code(s) for a Nay vote in the
    rollcall context, or an incorrect answer in the educational testing
    context.  Default is 0.}
  
  \item{missing}{numeric or \code{NA}, possibly a vector, code(s) for
    missing data.  Default is \code{NA}.}
  
  \item{notInLegis}{numeric or \code{NA}, possibly a vector, code(s) for
    the legislator not being in the legislature when a particular roll
    call was recorded (e.g., deceased, retired, yet to be elected).}
  
  \item{legis.names}{a vector of names of the legislators or
    individuals.  If \code{data} is a \code{list} or \code{data.frame}
    and has a component named \code{legis.names}, then this will be
    used.  Names will be generated if not supplied, or if there are
    fewer unique names supplied than legislators/subjects (rows of the
    roll call matrix).}
  
  \item{vote.names}{a vector of names or labels for the votes or items.
    If \code{data} is a \code{list} or \code{data.frame} and has a
    component named \code{vote.names}, then this will be used.  Names
    will be generated if not supplied, or if there are fewer unique
    names supplied than votes/test-items (columns of the roll call
    matrix).}
  
  \item{legis.data}{a \code{\link{matrix}} or \code{\link{data.frame}}
    containing covariates specific to each legislator/test-taker; e.g.,
    party affiliation, district-level covariates.  If this object does
    not have the same number of rows as \code{data}, an error is
    returned.}
  
  \item{vote.data}{a \code{\link{matrix}} or \code{\link{data.frame}}
    containing covariates specific to each roll call vote or test item;
    e.g., a timestamp, the bill sponsor, descriptive text indicating
    the type of vote.  If this object does not have the same number of
    row as the number of columns in \code{data}, an error is returned.}
  
  \item{desc}{character, a string providing an (optional) description of
    the data being used.  If \code{data} is a list and contains an
    element named \code{desc}, then this will be used.}
  
  \item{source}{character, a string providing an (optional) description
    of where the roll call data originated (e.g., a URL or a short-form
    reference).  Used in print and summary methods.} 
}

\details{See below for methods that operate on objects of class
  \code{rollcall}.} 

\value{
  An object of class \code{rollcall}, a list with the following components:
  
  \item{votes}{a \code{\link{matrix}} containing voting decisions, with
    rows corresponding to legislators (test subjects) and columns to
    roll call votes (test items). Legislators (test subjects)
    and items (or votes) have been labeled in the
    \code{\link{dimnames}} attribute of this matrix, using
    the \code{legis.names} and/or \code{vote.names} arguments, respectively.}

  \item{codes}{a \code{\link{list}} with named components \code{yea},
    \code{nay}, \code{notInLegis} and \code{missing}, each component a
    numeric vector (possibly of length 1 and possibly \code{NA}),
    indicating how entries in the \code{votes} component of the
    \code{rollcall} object should be considered.  This list simply gathers
    up the values in the \code{yea}, \code{nay}, \code{notInLegis} and
    \code{missing} arguments passed to the function.}
  
  \item{n}{numeric, number of legislators, equal to \code{dim(votes)[1]}}

  \item{m}{numeric, number of votes, equal to \code{dim(votes)[2]}}

  \item{legis.data}{user-supplied data on legislators/test-subjects.}
  
  \item{vote.data}{user-supplied data on rollcall votes/test-items.}
  
  \item{desc}{any user-supplied description.  If no description was provided,
    defaults \code{desc} defaults to \code{NULL}.}
  
  \item{source}{any user-supplied source information (e.g., a url or a
    short-form reference).  If no description is provided, \code{source}
    defaults to \code{NULL}.}
  
}

\seealso{
  \code{\link{readKH}} for creating objects from files (possibly over
  the web), in the format used for data from the United States Congress
  used by Keith Poole and Howard Rosenthal (and others).

  \code{\link{summary.rollcall}}, \code{\link{ideal}} for model fitting.
}

\examples{
## generate some fake roll call data
set.seed(314159265)
fakeData <- matrix(sample(x=c(0,1),size=5000,replace=TRUE),
                   50,100)
rc <- rollcall(fakeData)
is(rc,"rollcall")        ## TRUE
rc                       ## print the rollcall object on screen

data(sc9497)             ## Supreme Court example data
rc <- rollcall(data=sc9497$votes,
               legis.names=sc9497$legis.names,
               desc=sc9497$desc)
summary(rc,verbose=TRUE)                      

\dontrun{
## s107
## could use readKH for this
dat <- readLines("sen107kh.ord")
dat <- substring(dat,37)
mat <- matrix(NA,ncol=nchar(dat[1]),nrow=length(dat))
for(i in 1:103){
  mat[i,] <- as.numeric(unlist(strsplit(dat[i],
                                        split=character(0))))
}

s107 <- rollcall(mat,
                 yea=c(1,2,3),
                 nay=c(4,5,6),
                 missing=c(7,8,9),
                 notInLegis=0,
                 desc="107th U.S. Senate",
                 source="http://voteview.ucsd.edu")
summary(s107)
}
}
\keyword{manip}

\name{state.info}
\alias{state.info}
\docType{data}
\title{information about the American states needed for U.S. Congress}
\description{
  Numeric codes and names of 50 states and the District of Columbia,
  required to parse Keith Poole
  and Howard Rosenthal's collections of U.S. Congressional roll calls.
}
\usage{data(state.info)
}
\format{
  \describe{
    \item{\code{icpsr}}{integer, numeric code for state used by the
      Inter-university Consortium for Political and Social Research}
    \item{\code{state}}{character, name of state or \code{Washington
	D.C.}}
    \item{\code{year}}{numeric or \code{NA}, year of statehood}
    }
  }

\details{The function \code{\link{readKH}} converts the integer ICPSR codes
  into strings, via a table lookup in this data frame.  Another table
  lookup in \code{\link{state.abb}} provides the 2-letter abbreviation
  commonly used in identifying American legislators, e.g.,
  \code{KENNEDY, E (D-MA)}.} 
\seealso{\code{\link{state}}
}
\source{Various ICPSR codebooks. e.g., \doi{10.3886/ICPSR00002.v5}}
\keyword{datasets}

\name{zeroinfl}
\alias{zeroinfl}
\alias{print.zeroinfl}

\title{Zero-inflated Count Data Regression}
\description{
  Fit zero-inflated regression models for count data via maximum likelihood.
}

\usage{
zeroinfl(formula, data, subset, na.action, weights, offset,
  dist = c("poisson", "negbin", "geometric"),
  link = c("logit", "probit", "cloglog", "cauchit", "log"),
  control = zeroinfl.control(\dots),
  model = TRUE, y = TRUE, x = FALSE, \dots)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data, subset, na.action}{arguments controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights.}
  \item{offset}{optional numeric vector with an a priori known component to be
    included in the linear predictor of the count model. See below for more
    information on offsets.}
  \item{dist}{character specification of count model family (a log link is 
    always used).}
  \item{link}{character specification of link function in the binary
    zero-inflation model (a binomial family is always used).}
  \item{control}{a list of control arguments specified via
    \code{\link{zeroinfl.control}}.}
  \item{model, y, x}{logicals. If \code{TRUE} the corresponding components
    of the fit (model frame, response, model matrix) are returned.}
  \item{\dots}{arguments passed to \code{\link{zeroinfl.control}} in the
    default setup.}
}

\details{
  Zero-inflated count models are two-component mixture models
  combining a point mass at zero with a proper count distribution.
  Thus, there are two sources of zeros: zeros may come from 
  both the point mass and from the count component. Usually the count model 
  is a Poisson or negative binomial regression (with log link). 
  The geometric distribution is a special case of the negative binomial
  with size parameter equal to 1.
  For modeling the unobserved state (zero vs. count), a binary model is used
  that captures the probability of zero inflation.
  in the simplest case only with an intercept but potentially containing regressors.
  For this zero-inflation model, a binomial model with different links can be
  used, typically logit or probit.
  
  The \code{formula} can be used to specify both components of the model:
  If a \code{formula} of type \code{y ~ x1 + x2} is supplied, then the same
  regressors are employed in both components. This is equivalent to
  \code{y ~ x1 + x2 | x1 + x2}. Of course, a different set of regressors
  could be specified for the count and zero-inflation component, e.g.,
  \code{y ~ x1 + x2 | z1 + z2 + z3} giving the count data model \code{y ~ x1 + x2}
  conditional on (\code{|}) the zero-inflation model \code{y ~ z1 + z2 + z3}.
  A simple inflation model where all zero counts have the same
  probability of belonging to the zero component can by specified by the formula
  \code{y ~ x1 + x2 | 1}.

  Offsets can be specified in both components of the model pertaining to count and
  zero-inflation model: \code{y ~ x1 + offset(x2) | z1 + z2 + offset(z3)}, where
  \code{x2} is used as an offset (i.e., with coefficient fixed to 1) in the
  count component and \code{z3} analogously in the zero-inflation component. By the rule
  stated above \code{y ~ x1 + offset(x2)} is expanded to
  \code{y ~ x1 + offset(x2) | x1 + offset(x2)}. Instead of using the
  \code{offset()} wrapper within the \code{formula}, the \code{offset} argument
  can also be employed which sets an offset only for the count model. Thus,
  \code{formula = y ~ x1} and \code{offset = x2} is equivalent to
  \code{formula = y ~ x1 + offset(x2) | x1}.  
  
  All parameters are estimated by maximum likelihood using \code{\link[stats]{optim}},
  with control options set in \code{\link{zeroinfl.control}}.
  Starting values can be supplied, estimated by the EM (expectation maximization)
  algorithm, or by \code{\link[stats]{glm.fit}} (the default). Standard errors
  are derived numerically using the Hessian matrix returned by \code{\link[stats]{optim}}.
  See \code{\link{zeroinfl.control}} for details.
  
  The returned fitted model object is of class \code{"zeroinfl"} and is similar
  to fitted \code{"glm"} objects. For elements such as \code{"coefficients"} or
  \code{"terms"} a list is returned with elements for the zero and count component,
  respectively. For details see below.
  
  A set of standard extractor functions for fitted model objects is available for
  objects of class \code{"zeroinfl"}, including methods to the generic functions
  \code{\link[base]{print}}, \code{\link[base]{summary}}, \code{\link[stats]{coef}}, 
  \code{\link[stats]{vcov}}, \code{\link[stats]{logLik}}, \code{\link[stats]{residuals}}, 
  \code{\link[stats]{predict}}, \code{\link[stats]{fitted}}, \code{\link[stats]{terms}},
  \code{\link[stats]{model.matrix}}. See \code{\link{predict.zeroinfl}} for more details
  on all methods.
}

\value{
  An object of class \code{"zeroinfl"}, i.e., a list with components including
  \item{coefficients}{a list with elements \code{"count"} and \code{"zero"}
    containing the coefficients from the respective models,}
  \item{residuals}{a vector of raw residuals (observed - fitted),}
  \item{fitted.values}{a vector of fitted means,}
  \item{optim}{a list with the output from the \code{optim} call for
    minimizing the negative log-likelihood,}
  \item{control}{the control arguments passed to the \code{optim} call,}
  \item{start}{the starting values for the parameters passed to the \code{optim} call,}
  \item{weights}{the case weights used,}
  \item{offset}{a list with elements \code{"count"} and \code{"zero"}
    containing the offset vectors (if any) from the respective models,}
  \item{n}{number of observations (with weights > 0),}
  \item{df.null}{residual degrees of freedom for the null model (= \code{n - 2}),}
  \item{df.residual}{residual degrees of freedom for fitted model,}
  \item{terms}{a list with elements \code{"count"}, \code{"zero"} and
    \code{"full"} containing the terms objects for the respective models,}
  \item{theta}{estimate of the additional \eqn{\theta}{theta} parameter of the
    negative binomial model (if a negative binomial regression is used),}
  \item{SE.logtheta}{standard error for \eqn{\log(\theta)}{log(theta)},}
  \item{loglik}{log-likelihood of the fitted model,}
  \item{vcov}{covariance matrix of all coefficients in the model (derived from the
    Hessian of the \code{optim} output),}
  \item{dist}{character string describing the count distribution used,}
  \item{link}{character string describing the link of the zero-inflation model,}
  \item{linkinv}{the inverse link function corresponding to \code{link},}
  \item{converged}{logical indicating successful convergence of \code{optim},}
  \item{call}{the original function call,}
  \item{formula}{the original formula,}
  \item{levels}{levels of the categorical regressors,}
  \item{contrasts}{a list with elements \code{"count"} and \code{"zero"}
    containing the contrasts corresponding to \code{levels} from the
    respective models,}
  \item{model}{the full model frame (if \code{model = TRUE}),}
  \item{y}{the response count vector (if \code{y = TRUE}),}
  \item{x}{a list with elements \code{"count"} and \code{"zero"}
    containing the model matrices from the respective models
    (if \code{x = TRUE}),}
}

\references{
Cameron, A. Colin and Pravin K. Trevedi. 1998. \emph{Regression Analysis of Count 
Data.} New York: Cambridge University Press.

Cameron, A. Colin and Pravin K. Trivedi. 2005. \emph{Microeconometrics: Methods and Applications}.
Cambridge: Cambridge University Press.

Lambert, Diane. 1992. \dQuote{Zero-Inflated Poisson Regression,
with an Application to Defects in Manufacturing.} \emph{Technometrics}. \bold{34}(1):1-14. \doi{10.2307/1269547} 

Zeileis, Achim, Christian Kleiber and Simon Jackman 2008.
\dQuote{Regression Models for Count Data in R.} 
\emph{Journal of Statistical Software}, \bold{27}(8).
URL \url{https://www.jstatsoft.org/v27/i08/}.
}

\author{Achim Zeileis <Achim.Zeileis@R-project.org>}

\seealso{\code{\link{zeroinfl.control}}, \code{\link[stats]{glm}},
  \code{\link[stats]{glm.fit}}, \code{\link[MASS]{glm.nb}},
  \code{\link{hurdle}}
}

\examples{
## data
data("bioChemists", package = "pscl")

## without inflation
## ("art ~ ." is "art ~ fem + mar + kid5 + phd + ment")
fm_pois <- glm(art ~ ., data = bioChemists, family = poisson)
fm_qpois <- glm(art ~ ., data = bioChemists, family = quasipoisson)
fm_nb <- MASS::glm.nb(art ~ ., data = bioChemists)

## with simple inflation (no regressors for zero component)
fm_zip <- zeroinfl(art ~ . | 1, data = bioChemists)
fm_zinb <- zeroinfl(art ~ . | 1, data = bioChemists, dist = "negbin")

## inflation with regressors
## ("art ~ . | ." is "art ~ fem + mar + kid5 + phd + ment | fem + mar + kid5 + phd + ment")
fm_zip2 <- zeroinfl(art ~ . | ., data = bioChemists)
fm_zinb2 <- zeroinfl(art ~ . | ., data = bioChemists, dist = "negbin")
}

\keyword{regression}

\name{vote92}
\alias{vote92}
\docType{data}
\title{
	Reports of voting in the 1992 U.S. Presidential election.
}
\description{
	Survey data containing self-reports of vote choice in the 1992 U.S. Presidential election, with numerous covariates, from the 1992 American National Election Studies.
}
\usage{data(vote92)}
\format{
  A data frame with 909 observations on the following 10 variables.
  \describe{
    \item{\code{vote}}{a factor with levels \code{Perot} \code{Clinton} \code{Bush}}
    \item{\code{dem}}{a numeric vector, 1 if the respondent reports identifying with the Democratic party, 0 otherwise.}
    \item{\code{rep}}{a numeric vector, 1 if the respondent reports identifying with the Republican party, 0 otherwise}
    \item{\code{female}}{a numeric vector, 1 if the respondent is female, 0 otherwise}
    \item{\code{persfinance}}{a numeric vector, -1 if the respondent reports that their personal financial situation has gotten worse over the last 12 months, 0 for no change, 1 if better}
    \item{\code{natlecon}}{a numeric vector, -1 if the respondent reports that national economic conditions have gotten worse over the last 12 months, 0 for no change, 1 if better}
    \item{\code{clintondis}}{a numeric vector, squared difference between respondent's self-placement on a scale measure of political ideology and the respondent's placement of the Democratic candidate, Bill Clinton}
    \item{\code{bushdis}}{a numeric vector, squared ideological distance of the respondent from the Republican candidate, President George H.W. Bush}
    \item{\code{perotdis}}{a numeric vector, squared ideological distance of the respondent from the Reform Party candidate, Ross Perot}
  }
}
\details{These data are unweighted.  Refer to the original data source for weights that purport to correct for non-representativeness and non-response.}
\source{
	Alvarez, R. Michael and Jonathan Nagler. 1995. Economics, issues and the Perot candidacy: Voter choice in the 1992 Presidential election.  \emph{American Journal of Political Science}. 39:714-44.

	Miller, Warren E., Donald R. Kinder, Steven J. Rosenstone and the National Election Studies.  1999.  \emph{National Election Studies, 1992: Pre-/Post-Election Study}.  Center for Political Studies, University of Michigan: Ann Arbor, Michigan.

	 Inter-University Consortium for Political and Social Research.  Study Number 1112.  \doi{10.3886/ICPSR01112}.   	
}
\references{
	Jackman, Simon. 2009.  \emph{Bayesian Analysis for the Social Sciences}.
	Wiley: Hoboken, New Jersey.  Examples 8.7 and 8.8.
}
\examples{
data(vote92)
summary(vote92)
}
\keyword{datasets}

\name{ideal}
\alias{ideal}
\title{analysis of educational testing data and roll call data with IRT models, via Markov
  chain Monte Carlo methods}

\description{
  Analysis of \code{rollcall} data via the spatial voting model;
  equivalent to a 2 parameter item-response model to educational testing data.  Model fitting via Markov chain Monte Carlo (MCMC).
}

\usage{
ideal(object, codes = object$codes,
      dropList = list(codes = "notInLegis", lop = 0),
      d = 1, maxiter = 10000, thin = 100, burnin = 5000,
      impute = FALSE,
      normalize = FALSE,
      meanzero = normalize,
      priors = NULL, startvals = "eigen",
      store.item = FALSE, file = NULL,
      verbose=FALSE, use.voter=NULL)
}

\arguments{
  \item{object}{an object of class \code{\link{rollcall}}}
  
  \item{codes}{a \code{\link{list}} describing the types of voting
    decisions in the roll call matrix (the \code{votes} component of the
    \code{\link{rollcall}} \code{object}); defaults to 
    \code{object$codes}, the codes in the rollcall object.} 
  
  \item{dropList}{a \code{\link{list}} (or \code{\link{alist}})
    listing voting decisions, legislators and/or votes to be dropped
    from the analysis; see \code{\link{dropRollCall}} for details.}
  
  \item{d}{numeric, (small) positive integer (default = 1), dimensionality of the ability space (or "policy space" in the rollcall context).}

  \item{maxiter}{numeric, positive integer, multiple of \code{thin}, number of MCMC iterations}

  \item{thin}{numeric, positive integer, thinning interval used for
    recording MCMC iterations.}

  \item{burnin}{number of MCMC iterations to run before recording.  The
    iteration numbered \code{burnin} will be recorded.  Must be a
    multiple of \code{thin}.}

  \item{impute}{\code{\link{logical}}, whether to treat missing entries
    of the rollcall matrix as missing at random, sampling from the
    predictive density of the missing entries at each MCMC iteration.}
  	    
  \item{normalize}{\code{\link{logical}}, impose identification with
    the constraint that the ideal points have mean zero and
    standard deviation one, in each dimension.  For one dimensional models this option is sufficient to
    locally identify the model parameters.
    See Details.}

  \item{meanzero}{to be deprecated/ignored; use \code{normalize} instead.}
  
  \item{priors}{a \code{list} of parameters (means and variances)
    specifying normal priors for the legislators' ideal points. The
    default is \code{NULL}, in which case the normal priors used have mean zero and
    precision 1 for the ideal points (ability parameters) and mean zero and
    precision .04 (variance 25) for the bill parameters (item discrimination and difficulty parameters).  If not \code{NULL}, \code{priors} must be a
    \code{list} with as many as four named components \code{xp, xpv, bp,
      bpv}:
    \describe{
    \item{\code{xp}}{a \code{n} by \code{d} matrix
      of prior \emph{means} for the legislators' ideal points;
      or alternatively, a scalar, which will be replicated to fill a \code{n} by \code{d} matrix.}
    \item{\code{xpv}}{a \code{n} by \code{d} matrix of prior
      \emph{precisions} (inverse variances);
      or alternatively, a scalar, which will be replicated to fill a \code{n}
      by \code{d} matrix.} 
    \item{\code{bp}}{a \code{m} by \code{d+1} matrix of prior means for the
      item parameters (with the item difficulty parameter coming last);
      or alternatively, a scalar, which will be replicated to fill a \code{m}
      by \code{d+1} matrix.}
    \item{\code{bpv}}{a \code{m} by \code{d+1} matrix of prior
      precisions for the item parameters;
      or alternatively, a scalar, which will be replicated to fill a \code{m}
      by \code{d+1} matrix.}
    }
    None of the components should contain \code{NA}.  If any
    of the four possible components are not provided, then the
    corresponding component of \code{priors} is assigned using the default
    values described above.}

  \item{startvals}{either a string naming a method for generating start
    values, valid options are \code{"eigen"} (the default),
    \code{"random"} or a \code{list} containing start values for
    legislators' ideal points and item parameters.  See Details.}

  \item{store.item}{\code{\link{logical}}, whether item discrimination
    parameters should be stored.  Storing item discrimination parameters
    can consume a large amount of memory.  These need to be stored for
    prediction; see \code{\link{predict.ideal}}.}

  \item{file}{string, file to write MCMC output.  Default is
    \code{NULL}, in which case MCMC output is stored in memory.  Note
    that post-estimation commands like \code{plot} will not work unless
    MCMC output is stored in memory.}

  \item{verbose}{logical, default
    is \code{FALSE}, which generates relatively little output to the R
    console during execution.}

  \item{use.voter}{A vector of logicals of length \code{n} controlling
    which legislators' vote data informs item parameter
    estimates. Legislators corresponding to \code{FALSE} entries will
    not have their voting data included in updates of the item
    parameters. The default value of \code{NULL} will run the standard
    ideal-point model, which uses all legislators in updating item
    parameters. See Jessee (2016).}
}

\details{The function fits a \code{d}+1 parameter item-response model to
  the roll call data object, so in one dimension the model reduces
  to the two-parameter item-response model popular in educational testing.
  See References.
  
  \strong{Identification}: The model parameters are \strong{not
    identified} without the user supplying some restrictions on the
    model parameters; i.e., translations, rotations and re-scalings of
    the ideal points are observationally equivalent, via offsetting
    transformations of the item parameters.  It is the user's
    responsibility to impose these identifying restrictions if
    desired. The following brief discussion provides some guidance.

  For one-dimensional models (i.e., \code{d=1}), a simple route to
  identification is the \code{normalize} option, by imposing the restriction that the means of the posterior densities of the ideal points (ability parameters) have mean zero and standard deviation one, across legislators (test-takers).  This normalization supplies
  \emph{local} identification (that is, identification up to a 180 degree rotation of
  the recovered dimension). 
  
  Near-degenerate \dQuote{spike} priors
  (priors with arbitrarily large precisions) or the
  \code{constrain.legis} option on any two legislators' ideal points
  ensures \emph{global} identification in one dimension.

  Identification in higher dimensions can be obtained by supplying
  fixed values for \code{d+1} legislators' ideal points, provided the
  supplied fixed points span a \code{d}-dimensional space (e.g., three
  supplied ideal points form a triangle in \code{d=2} dimensions), via
  the \code{\link{constrain.legis}} option. In this case the function
  defaults to vague normal priors on the unconstrained ideal points, but at each iteration the sampled
  ideal points are transformed back into the space of identified
  parameters, applying the linear transformation that maps the
  \code{d+1} fixed ideal points from their sampled values to their
  fixed values.  Alternatively, one can impose
  restrictions on the item parameters via
  \code{\link{constrain.items}}. See the examples in the documentation
  for the \code{\link{constrain.legis}} and
  \code{\link{constrain.items}}.

  Another route to identification is via \emph{post-processing}.  That
  is, the user can run \code{ideal} without any identification
  constraints. This does not pose any formal/technical problem in a
  Bayesian analysis.  The fact that the posterior density may have
  multiple modes doesn't imply that the posterior is improper or that
  it can't be explored via MCMC methods. -- but then use the function
  \code{\link{postProcess}} to map the MCMC output from the space of
  unidentified parameters into the subspace of identified parameters.
  See the example in the documentation for the
  \code{\link{postProcess}} function.

  When the \code{normalize} option is set to \code{TRUE}, an
  unidentified model is run, and the \code{ideal} object is
  post-processed with the \code{normalize} option, and then returned
  to the user (but again, note that the \code{normalize} option is
  only implemented for unidimensional models).
  
  \strong{Start values}.  Start values can be supplied by the user, or
  generated by the function itself. 

  The default method, corresponding to \code{startvals="eigen"}, first
  forms a \code{n}-by-\code{n} correlation matrix from the
  double-centered roll call matrix (subtracting row means, and column
  means, adding in the grand mean), and then extracts the first
  \code{d} principal components (eigenvectors), scaling the
  eigenvectors by the square root of their corresponding eigenvector.
  If the user is imposing constraints on ideal points (via
  \code{\link{constrain.legis}}), these constraints are applied to the
  corresponding elements of the start values generated from the
  eigen-decomposition.  Then, to generate start 
  values for the rollcall/item parameters, a series of
  \code{\link[=family]{binomial}} \code{\link[=glm]{glms}} are
  estimated (with a probit \code{\link[=make.link]{link}}), one for
  each rollcall/item, \eqn{j = 1, \ldots, m}.  The votes on the
  \eqn{j}-th rollcall/item are binary responses (presumed to be
  conditionally independent given each legislator's latent
  preference), and the (constrained or unconstrained) start values for
  legislators are used as predictors. The estimated coefficients from
  these probit models are used as start values for the item
  discrimination and difficulty parameters (with the intercepts from
  the probit GLMs multiplied by -1 so as to make those coefficients
  difficulty parameters).

  The default \code{eigen} method generates extremely good start
  values for low-dimensional models fit to recent U.S. congresses,
  where high rates of party line voting result in excellent fits from
  low dimensional models.  The \code{eigen} method may be
  computationally expensive or lead to memory errors for
  \code{rollcall} objects with large numbers of legislators.

  The \code{random} method generates start values via iid sampling
  from a N(0,1) density, via \code{\link{rnorm}}, imposing any
  constraints that may have been supplied via
  \code{\link{constrain.legis}}, and then uses the probit method
  described above to get start values for the rollcall/item
  parameters.
  
  If \code{startvals} is a \code{list}, it must contain the named
  components \code{x} and/or \code{b}, or named components that
  (uniquely) begin with the letters \code{x} and/or \code{b}.  The
  component \code{x} must be a vector or a matrix of dimensions equal to
  the number of individuals (legislators) by \code{d}.  If supplied,
  \code{startvals$b} must be a matrix with dimension number of items
  (votes) by \code{d}+1.  The \code{x} and \code{b} components cannot
  contain \code{NA}.  If \code{x} is not supplied when \code{startvals}
  is a list, then start values are generated using the default
  \code{eiegn} method described above, and start values for the
  rollcall/item parameters are regenerated using the probit method,
  ignoring any user-supplied values in \code{startvals$b}.  That is,
  user-supplied values in \code{startvals$b} are only used when
  accompanied by a valid set of start values for the ideal points in
  \code{startvals$x}.

  \strong{Implementation via Data Augmentation}.  The MCMC algorithm
  for this problem consists of a Gibbs sampler for the ideal points
  (latent traits)
  and item parameters, conditional on latent data \eqn{y^*}, generated
  via a data augmentation (DA) step.  That is, following Albert (1992) and
  Albert and Chib (1993), if \eqn{y_{ij} = 1} we sample from the
  truncated normal density
  \deqn{y_{ij}^* \sim N(x_i' \beta_j - \alpha_j, 1)\mathcal{I}(y_{ij}^* \geq 0)}
  and for \eqn{y_{ij}=0} we sample 
  \deqn{y_{ij}^* \sim N(x_i' \beta_j - \alpha_j, 1)\mathcal{I}(y_{ij}^* < 0)}
  where \eqn{\mathcal{I}} is an indicator function evaluating to one
  if its argument is true and zero otherwise.  Given the
  latent \eqn{y^*}, the conditional distributions for \eqn{x} and
  \eqn{(\beta,\alpha)} are extremely simple to sample from; see the
  references for details.  

  This data-augmented Gibbs sampling strategy is easily implemented,
  but can sometimes require many thousands of samples in order to
  generate tolerable explorations of the posterior densities of the
  latent traits, particularly for legislators with short and/or
  extreme voting histories (the equivalent in the educational testing
  setting is a test-taker who gets almost every item right or wrong).

  % The MCMC algorithm can generate better performance
  % via a parameter expansion strategy usually referred to as \emph{marginal
  % data augmentation} (e.g., van Dyk and Meng 2001).  The idea is to
  % introduce a additional working parameter into the MCMC sampler that
  % has the effect of improving the performance of the sampler in the
  % sub-space of parameters of direct interest.  In this case we
  % introduce a variance parameter \eqn{\sigma^2} for the latent data;
  % in the DA algorithm of Albert and Chib (1993) --- and in any conventional
  % probit analysis --- this parameter is set
  % to 1.0 for identification.  In the MDA approach we carry this
  % (unidentified) parameter into the DA stage of the algorithm with an
  % improper prior, \eqn{p(\sigma^2) \propto \sigma^{-2}},
  % generating \eqn{y^*} that exhibit bigger moves from iteration to
  % iteration, such that in turn the MCMC algorithm displays better
  % mixing with respect to the identified parameters of direct interest,
  % \eqn{x} and \eqn{(\beta,\alpha)} than the mixing obtained from
  % the Gibbs-with-DA MCMC algorithm.   The MDA algorithm is the default
  % in \code{ideal}, but Gibbs-with-DA can be implemented by setting
  % \code{mda=FALSE} in the call to \code{ideal}.
}
  
\value{a \code{\link{list}} of class \code{ideal} with named components
  
  \item{n}{\code{\link{numeric}}, integer, number of legislators in the
    analysis, after any subsetting via processing the \code{dropList}.}

  \item{m}{\code{\link{numeric}}, integer, number of rollcalls in roll
    call matrix, after any subsetting via processing the \code{dropList}.}

  \item{d}{\code{\link{numeric}}, integer, number of dimensions
    fitted.}

  \item{x}{a three-dimensional \code{\link{array}} containing the MCMC
    output with respect to the 
    the ideal point of each legislator in each dimension.
    The three-dimensional array is in iteration-legislator-dimension
    order.  The iterations run from \code{burnin} to \code{maxiter}, at
    an interval of \code{thin}.}

   \item{beta}{a three-dimensional \code{\link{array}} containing the
    MCMC output for the item parameters.  The three-dimensional array
    is in iteration-rollcall-parameter order.  The iterations run from
    \code{burnin} to \code{maxiter}, at an interval of \code{thin}.
    Each rollcall has \code{d+1} parameters, with the
    item-discrimination parameters stored first, in the first \code{d}
    components of the 3rd dimension of the \code{beta} array; the
    item-difficulty parameter follows in the final \code{d+1}
    component of the 3rd dimension of the \code{beta} array.}

  \item{xbar}{a \code{n} by \code{d} \code{\link{matrix}} containing the means of the
    MCMC samples for the ideal point of each legislator in each dimension,
    using iterations \code{burnin} to \code{maxiter}, at an interval of
    \code{thin}.}
  
  \item{betabar}{a \code{m} by \code{d+1} \code{\link{matrix}} containing the means of
    the MCMC samples for the item-specific parameters, using iterations
    \code{burnin} to \code{maxiter}, at an interval of \code{thin}.}

	\item{args}{calling arguments, evaluated in the frame calling \code{ideal}.}

  \item{call}{an object of class \code{\link{call}}, containing
    the arguments passed to \code{ideal} as unevaluated expressions or values (for functions arguments that evaluate to scalar integer or logical such as \code{maxiter}, \code{burnin}, etc).}
}

\references{
  Albert, James. 1992. Bayesian Estimation of normal ogive item
  response curves using Gibbs sampling. \emph{Journal of Educational
  Statistics}. 17:251-269.

  Albert, James H. and Siddhartha Chib.  1993.  Bayesian Analysis of
  Binary and Polychotomous Response Data.  \emph{Journal of the
  American Statistical Association}. 88:669-679.

  Clinton, Joshua, Simon Jackman and Douglas Rivers. 2004. The
  Statistical Analysis of Roll Call Data.  \emph{American Political
  Science Review}.  98:335-370.

  Jackman, Simon. 2009.  \emph{Bayesian Analysis for the Social
  Sciences}.  Wiley: Hoboken, New Jersey.

  Jessee, Stephen. 2016. (How) Can We Estimate the Ideology
  of Citizens and Political Elites on the Same Scale?
  \emph{American Journal of Political Science}.

  Patz, Richard J. and Brian W. Junker. 1999.  A Straightforward
  Approach to Markov Chain Monte Carlo Methods for Item Response
  Models. \emph{Journal of Education and Behavioral
  Statistics}. 24:146-178.
  
  Rivers, Douglas. 2003.  \dQuote{Identification of Multidimensional
  Item-Response Models.} Typescript.  Department of Political Science,
  Stanford University.

  van Dyk, David A and Xiao-Li Meng.  2001.  The art of data
  augmentation (with discussion).  \emph{Journal of Computational and
  Graphical Statistics}. 10(1):1-111.
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}, with help from Christina
  Maimone and Alex Tahk.}

\seealso{
  \code{\link{rollcall}}, \code{\link{summary.ideal}},
  \code{\link{plot.ideal}}, \code{\link{predict.ideal}}.
  \code{\link{tracex}} for graphical display of MCMC iterative
  history.

  \code{\link{idealToMCMC}} converts the MCMC iterates in an
  \code{ideal} object to a form that can be used by the \code{coda} library.

  \code{\link{constrain.items}} and
  \code{\link{constrain.legis}} for implementing identifying
  restrictions.

  \code{\link{postProcess}} for imposing identifying restrictions
  \emph{ex post}.
  
  \code{\link[MCMCpack:MCMCirt1d]{MCMCirt1d}} and 
  \code{\link[MCMCpack:MCMCirtKd]{MCMCirtKd}} in the \pkg{MCMCpack}
  package provide similar functionality to \code{ideal}.
  
}

\examples{
\dontrun{
## long run, many iterations
data(s109)
n <- dim(s109$legis.data)[1]
x0 <- rep(0,n)
x0[s109$legis.data$party=="D"] <- -1
x0[s109$legis.data$party=="R"] <- 1

id1 <- ideal(s109,
             d=1,
             startvals=list(x=x0),
             normalize=TRUE,
             store.item=TRUE,
             maxiter=260E3,
             burnin=10E3,
             thin=100)  
}
}
\keyword{models}

\name{plot.ideal}
\alias{plot.ideal}
\alias{plot1d}
\alias{plot2d}
\title{plots an ideal object}
\description{
  Plot of the results of an ideal point estimation contained
  in an object of class \code{ideal}.
}

\usage{
\method{plot}{ideal}(x, conf.int=0.95, burnin=NULL, ...)

plot1d(x, d=1, conf.int=0.95, burnin=NULL,
       showAllNames = FALSE, ...)

plot2d(x, d1=1, d2=2, burnin=NULL,
       overlayCuttingPlanes=FALSE, ...)
}

\arguments{
  \item{x}{an object of class \code{\link{ideal}}}
  \item{conf.int}{for "ideal" objects with 1 dimension estimated, the
    level of the confidence interval to plot around the
    posterior mean for each legislator.  If 2 or more dimensions were
    estimated, \code{conf.int} is ignored.}
  
  \item{d}{integer, which dimension to display in a 1d plot, if the
    object is a multidimensional ideal object}.
  
    \item{burnin}{of the recorded MCMC samples, how many to discard as
    burnin?  Default is \code{NULL}, in which case the value of
    \code{burnin} in the \code{\link{ideal}} object is used.}

  \item{showAllNames}{\code{\link{logical}}, if \code{TRUE}, the
    vertical axis will the names of all legislators. Default is
    \code{FALSE} to reduce clutter on typical-sized graph.}
  
  \item{d1}{integer, the number of the first dimension to plot when
    plotting multi-dimensional \code{\link{ideal}} objects.  This
    dimension will appear on the horizontal (x) axis.}
  
  \item{d2}{integer, the number of the second dimension to plot when
    plotting multi-dimensional \code{\link{ideal}} objects.  This dimension
    will appear on the vertical (y) axis.}
  
  \item{overlayCuttingPlanes}{logical, if \code{TRUE}, overlay the
    estimated bill-specific cutting planes}
  
  \item{...}{other parameters to be passed through to plotting functions.}
}


\details{ If the \code{ideal} object comes from fitting a \code{d=1}
  dimensional model, then \code{plot.ideal} plots the mean of the
  posterior density over each legislator's ideal point, accompanied by a
  \code{conf.int} confidence interval.  In this case, \code{plot.ideal}
  is simply a wrapper function to \code{\link{plot1d}}.

  If the \code{ideal} object has \code{d=2} dimensions, then
  \code{\link{plot2d}} is called, which plots the (estimated) mean of
  the posterior density of each legislator's ideal point (i.e., the
  ideal point/latent trait is a point in 2-dimensional Euclidean space,
  and the posterior density for each ideal point is a bivariate
  density).  Single dimension summaries of the estimated ideal points
  (latent traits) can be obtained for multidimensional
  \code{\link{ideal}} objects by passing the \code{\link{ideal}} object
  directly to \code{plot1d} with \code{d} set appropriately.

  If the \code{\link{ideal}} object has \code{d>2} dimensions, a
  scatterplot matrix is produced via \code{\link{pairs}}, with the
  posterior means of the ideal points (latent traits) plotted against
  one another, dimension by dimension.
  
  For unidimensional and two-dimensional models, if party information is
  available in the \code{rollcall} object contained in the \code{ideal}
  object, legislators from different parties are plotted in different
  colors.  If the \code{ideal} object has more than 2 dimensions,
  \code{plot.ideal()} produces a matrix of plots of the mean ideal
  points of each dimension against the posterior mean ideal points of
  the other dimensions.

}

\note{When specifying a value of \code{burnin} different from that used
  in fitting the \code{\link{ideal}} object, note a distinction
  between the iteration numbers of the stored iterations, and the
  number of stored iterations.  That is, the \code{n}-th iteration
  stored in an \code{\link{ideal}} object will not be iteration
  \code{n} if the user specified \code{thin>1} in the call to
  \code{\link{ideal}}.  Here, iterations are tagged with their
  iteration number.  Thus, if the user called \code{\link{ideal}} with
  \code{thin=10} and \code{burnin=100} then the stored iterations are
  numbered \code{100, 110, 120, ...}.  Any future subsetting via a
  \code{burnin} refers to this iteration number.}

\seealso{\code{\link{ideal}};  
  \code{\link{tracex}} for trace plots, a graphical aid useful in
diagnosing convergence of the MCMC algorithms.} 

\examples{
\dontrun{
data(s109)
id1 <- ideal(s109,
             d=1,
             normalize=TRUE,
             store.item=TRUE,
             maxiter=500,   ## short run for examples
             burnin=100,
             thin=10)  

plot(id1)


id2 <- ideal(s109,
             d=2,
             store.item=TRUE,
             maxiter=11e2,
             burnin=1e2,
             verbose=TRUE,
             thin=25)

plot(id2,overlayCuttingPlanes=TRUE)

id2pp <- postProcess(id2,
                     constraints=list(BOXER=c(-1,0),
                       INHOFE=c(1,0),
                       CHAFEE=c(0,.25)))

plot(id2pp,overlayCuttingPlanes=TRUE)
}
} 
\keyword{hplot}

\name{admit}
\alias{admit}
\docType{data}
\title{Applications to a Political Science PhD Program}
\description{
  Ordinal ratings (faculty evaluations) of applicants to a Political Science PhD
  Program.}
\usage{data(admit)}
\format{
  A data frame with 106 observations on the following 6 variables.
  \describe{
    \item{\code{score}}{an ordered factor with levels \code{1} < \code{2} < \code{3} < \code{4} < \code{5}}
    \item{\code{gre.quant}}{applicant's score on the quantitative
      section of the GRE; the maximum score is 800}
    \item{\code{gre.verbal}}{applicant's score on the verbal section of
      the GRE; the maximum score is 800}
    \item{\code{ap}}{1 if the applicant indicated an interest in
      American politics; 0 otherwise}
    \item{\code{pt}}{1 if the applicant indicated an interest in
      Political Theory; 0 otherwise}
    \item{\code{female}}{1 for female applicants; 0 otherwise}
  }
}

\references{Jackman, Simon.  2004.  "What Do We Learn From Graduate
  Admissions Committees?: A Multiple-Rater, Latent Variable Model, with
  Incomplete Discrete and Continuous Indicators."  \emph{Political
  Analysis}.  12(4):400-424.  }

\examples{
data(admit)
summary(admit)
## ordered probit model
op1 <- MASS::polr(score ~ gre.quant + gre.verbal + ap + pt + female,
            Hess=TRUE,
            data=admit,
            method="probit")
summary(op1)
hitmiss(op1)
logLik(op1)
pR2(op1)   
}
\keyword{datasets}

\name{computeMargins}
\alias{computeMargins}
\title{add information about voting outcomes to a rollcall
  object}

\description{
  Add summaries of each roll call vote to a \code{\link{rollcall}} object.
}

\usage{
computeMargins(object, dropList = NULL)
}

\arguments{
  \item{object}{an object of class \code{\link{rollcall}}}
  \item{dropList}{a \code{\link{list}} (or \code{\link{alist}})
      listing voting decisions, legislators and/or votes to be dropped
      from the analysis; see \code{\link{dropRollCall}} for details.}
}

\details{ The subsetting implied by the \code{dropList} is first applied
  to the \code{\link{rollcall}} object, via \code{\link{dropRollCall}}.
  Then, for each remaining roll call vote, the number of legislators
  voting \dQuote{Yea}, \dQuote{Nay}, and not voting are computed, using
  the encoding information in the \code{codes} component of the
  \code{\link{rollcall}} object via the \code{\link{convertCodes}}
  function.  The matrix of vote counts are added to the \code{\link{rollcall}}
  object as a component \code{voteMargins}.
}

\value{
  An object of class \code{\link{rollcall}}, with a component
  \code{voteMargins} that is a matrix with four columns: 
  \item{Yea}{number of legislators voting \dQuote{Yea}}
  \item{Nay}{number of legislators voting \dQuote{Nay}}
  \item{NA}{number of legislators not voting \dQuote{Nay}}
  \item{Min}{the number of legislators voting on the losing side of the
  roll call}
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\seealso{\code{\link{dropRollCall}} on specifying a \code{dropList}.
  The vote-specific marginals produced by this function are used by as
  \code{\link{dropRollCall}}, \code{\link{summary.ideal}} and
  \code{\link{predict.ideal}}.}

\examples{
data(s109)
tmp <- computeMargins(s109)
dim(tmp$voteMargins)   ## 645 by 4

tmp <- computeMargins(s109,
                     dropList=list(codes="notInLegis",lop=0))
dim(tmp$voteMargins)   ## 544 by 4
}

\keyword{manip} 

\name{hitmiss}
\alias{hitmiss}
\alias{hitmiss.glm}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Table of Actual Outcomes against Predicted Outcomes for discrete
  data models}

\description{ Cross-tabulations of actual outcomes against predicted
  outcomes for discrete data models, with summary statistics such as
  percent correctly predicted (PCP) under fitted and null models.  For
  models with binary responses (generalized linear models with
  \code{family=binomial}), the user can specific a classification
  threshold for the predicted probabilities.  }

\usage{
hitmiss(obj, digits = max(3, getOption("digits") - 3), ...)

\method{hitmiss}{glm}(obj,digits=max(3,getOption("digits")-3),
            ...,
            k=.5)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{obj}{a fitted model object, such as a \code{glm} with
    \code{family=binomial}, a \code{polr} model for ordinal responses,
    or a \code{multinom} model for unordered/multinomial outcomes} 
  \item{digits}{number of digits to display in on-screen output}
  \item{\dots}{additional arguments passed to or from other functions}
  \item{k}{classification threshold for binary models}
}

\details{For models with binary responses, the user can specify a
  parameter 0 < \code{k} < 1; if the predicted probabilities exceed this
  threshold then the model is deemed to have predicted y=1, and
  otherwise to have predicted y=0.  Measures like percent correctly
  predicted are crude summaries of model fit; the cross-tabulation of
  actual against predicted is somewhat more informative, providing a
  little more insight as to where the model fits less well.
}

\value{
  For \code{hitmiss.glm}, a vector of length 3:
  \item{pcp}{Percent Correctly Predicted}
  \item{pcp0}{Percent Correctly Predicted among y=0}
  \item{pcp1}{Percent Correctly Predicted among y=1}
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\note{To-do: The \code{glm} method should also handle binomial data presented
  as two-vector success/failures counts; and count data with
  \code{family=poisson}, the \code{glm.nb} models and \code{zeroinfl}
  and \code{hurdle} etc.   We should also make the output a class with
  prettier print methods, i.e., save the cross-tabulation in the
  returned object etc.
}

\seealso{\code{\link{pR2}} for pseudo r-squared;  \code{\link{predict}};
  \code{\link{extractAIC}}.  See also the \pkg{ROCR} package and the \code{lroc}  function in the \pkg{epicalc} package for ROC computations for assessing binary classifications.} 

\examples{
data(admit)
## ordered probit model
op1 <- MASS::polr(score ~ gre.quant + gre.verbal + ap + pt + female,
            Hess=TRUE,
            data=admit,
            method="probit")
hitmiss(op1)
}

\keyword{models}

\name{hurdle.control}
\alias{hurdle.control}

\title{Control Parameters for Hurdle Count Data Regression}
\description{
  Various parameters that control fitting of hurdle regression models
  using \code{\link{hurdle}}.
}

\usage{
hurdle.control(method = "BFGS", maxit = 10000, trace = FALSE,
  separate = TRUE, start = NULL, \dots)
}

\arguments{
  \item{method}{characters string specifying the \code{method} argument
    passed to \code{\link[stats]{optim}}.}
  \item{maxit}{integer specifying the \code{maxit} argument (maximal number
    of iterations) passed to \code{\link[stats]{optim}}.}
  \item{trace}{logical or integer controlling whether tracing information on  
    the progress of the optimization should be produced (passed to \code{\link[stats]{optim}}).}
  \item{separate}{logical. Should the estimation of the parameters in the truncated
    count component and hurdle zero component be carried out separately? See details.}
  \item{start}{an optional list with elements \code{"count"} and \code{"zero"}
    (and potentially \code{"theta"}) containing the coefficients for the corresponding component.}
  \item{\dots}{arguments passed to \code{\link[stats]{optim}}.}
}

\details{
  All parameters in \code{\link{hurdle}} are estimated by maximum likelihood
  using \code{\link[stats]{optim}} with control options set in \code{\link{hurdle.control}}.
  Most arguments are passed on directly to \code{optim}, only \code{trace} is also
  used within \code{hurdle} and \code{separate}/\code{start} control how \code{optim}
  is called.

  Starting values can be supplied via \code{start} or estimated by
  \code{\link[stats]{glm.fit}} (default).
  If \code{separate = TRUE} (default) the likelihoods of the truncated count component and the
  hurdle zero component will be maximized separately, otherwise the joint likelihood is
  set up and maximized. Standard errors are derived numerically using
  the Hessian matrix returned by \code{\link[stats]{optim}}. To supply starting 
  values, \code{start} should be a list with elements \code{"count"} and \code{"zero"}
  and potentially \code{"theta"} (a named vector, for models with negative binomial components 
  only) containing the starting values for the coefficients of the corresponding component 
  of the model.
}

\value{
  A list with the arguments specified.
}

\author{Achim Zeileis <Achim.Zeileis@R-project.org>}

\seealso{\code{\link{hurdle}}}

\examples{
data("bioChemists", package = "pscl")

## default start values
fm1 <- hurdle(art ~ fem + ment, data = bioChemists,
              dist = "negbin", zero = "negbin")

## user-supplied start values and other options
fm2 <- hurdle(art ~ fem + ment, data = bioChemists,
              dist = "negbin",
              zero = "negbin",
              trace=TRUE,
              separate=FALSE,
              start = list(count = c(0.3, -0.2, 0),
                           zero = c(4, -2, 0.8),
                           theta = c(count = 2, zero = 0.1)))

}

\keyword{regression}

\name{constrain.items}
\alias{constrain.items}
\title{constrain item parameters in analysis of roll call data}

\description{
  Sets constraints on specified item parameters in Bayesian analysis of
  roll call data by generating appropriate priors and start values for
  Markov chain Monte Carlo iterations.
}

\usage{
constrain.items(obj, dropList = list(codes = "notInLegis", lop = 0),
                x, d = 1)
}

\arguments{
  \item{obj}{an object of class \code{\link{rollcall}}.}
  \item{dropList}{a \code{\link{list}} (or \code{\link{alist}})
    indicating which voting decisions, legislators and/or roll calls are
    to be excluded from the subsequent analysis; see
    \code{\link{dropRollCall}} for details.}
  \item{x}{a \code{\link{list}} containing elements with names matching
    votes found in \code{dimnames(object$votes)[[2]]} (but after any
    subsetting specified by \code{dropList}).  Each component of the
    list must be a vector containing \code{d} elements, specifying the
    value to which the item discrimination parameters should be
    constrained, in each of the \code{d} dimensions.  The intercept or
    item difficultly parameter will not be constrained.}
  \item{d}{numeric, positive integer, the number of dimensions for which
    to set up the priors and start values.}
}

\details{\code{constrain.items} and its cousin,
  \code{\link{constrain.legis}} are usefully thought of as
  \dQuote{pre-processor} functions, generating priors \emph{and} start
  values for both the item parameters and the ideal points.  For the
  items specified in \code{x}, the prior mean for each dimension is set
  to the value given in \code{x}, and the prior precision for each
  dimension is set to \code{1e12} (i.e., a near-degenerate
  \dQuote{spike} prior).  For the other items, the priors are set to a
  mean of 0 and precision 0.01.  All of the ideal points are given
  normal priors with mean 0, precision 1.

  Start values are also generated for both ideal points and item
  parameters.  The start values for the items specified in \code{x} are
  set to the values specified in \code{x}.  The list resulting from
  \code{constrain.items} can then be given as the value for the
  parameters \code{priors} and \code{startvals} when \code{\link{ideal}}
  is run. The user is responsible for ensuring that a sufficient number
  of items are constrained such that when \code{\link{ideal}} is run,
  the model parameters are identified.

  \code{\link{dropRollCall}} is first called to generate the desired
  roll call matrix.  The entries of the roll call matrix are mapped to
  \code{c(0,1,NA)} using the \code{codes} component of the
  \code{\link{rollcall}} \code{object}.  See the discussion in the
  documentation of \code{\link{ideal}} for details on the generation of
  start values.
}

\value{
  a list with elements:
  \item{xp}{prior means for ideal points.  A matrix of dimensions number
    of legislators in \code{obj} by \code{d}.}
  \item{xpv}{prior meansprecisions for ideal points.  A matrix of dimensions number
    of legislators in \code{obj} by \code{d}.}
  \item{bp}{prior means for item parameters.  A matrix of dimensions number
    of items or votes in \code{obj} by \code{d+1}.}
  \item{bpv}{prior meansprecisions for item parameters.  A matrix of dimensions number
    of items or votes in \code{obj} by \code{d+1}.}
  \item{xstart}{start values for ideal points.  A matrix of dimensions number
    of legislators in \code{obj} by \code{d}.}
  \item{bstart}{start values for ideal points.  A matrix of dimensions number
    of items or votes in \code{obj} by \code{d+1}.}
}

\seealso{
  \code{\link{rollcall}}, \code{\link{ideal}},
  \code{\link{constrain.legis}}
}
\examples{
\dontrun{
data(s109)
f <- system.file("extdata","id1.rda",package="pscl")
load(f)
id1sum <- summary(id1,include.beta=TRUE)
suspect1 <- id1sum$bSig[[1]]=="95% CI overlaps 0"
close60 <- id1sum$bResults[[1]][,"Yea"] < 60
close40 <- id1sum$bResults[[1]][,"Yea"] > 40
suspect <- suspect1 & close60 & close40
id1sum$bResults[[1]][suspect,]
suspectVotes <- dimnames(id1sum$bResults[[1]][suspect,])[[1]]


## constraints on 2d model,
## close rollcall poorly fit by 1d model
## serves as reference item for 2nd dimension

cl <- constrain.items(s109,
                      x=list("2-150"=c(0,7),
                        "2-169"=c(7,0)),
                      d=2)

id1Constrained <- ideal(s109,
                        d=2,
                        meanzero=TRUE,
                        priors=cl,
                        startvals=cl,
                        maxiter=1e5,
                        burnin=1e3,
                        thin=1e2)
summary(id1Constrained,include.beta=TRUE)
}
}
\keyword{datagen}

\name{summary.rollcall}
\alias{summary.rollcall}
\alias{print.summary.rollcall}
\title{summarize a rollcall object}

\description{
  Provides a summary of the information about votes, legislators, etc in
  a \code{\link{rollcall}} object. 
}

\usage{
\method{summary}{rollcall}(object,
                 dropList=NULL,  
                 verbose=FALSE,debug=FALSE,...)

\method{print}{summary.rollcall}(x, digits=1, ...)
}

\arguments{
  \item{object}{an \code{\link{rollcall}} object.}
  \item{dropList}{a \code{\link{list}} or \code{\link{alist}}, listing voting
    decisions, legislators and/or votes to be dropped 
    from the summary; see \code{\link{dropRollCall}} for details.}
  \item{verbose}{logical, if \code{TRUE}, compute legislator-specific
    and vote-specific Yea/Nay/\code{NA} summaries}
  \item{debug}{logical, if \code{TRUE}, print messages to console during
  processing of the \code{rollcall} object}
  \item{x}{an object of class \code{summary.rollcall}}
  \item{digits}{number of decimal places in printed display}
  \item{...}{further arguments passed to or from other methods.}
}

\value{
  An object of class \code{summary.rollcall} with the following elements
  (depending on the logical flag \code{verbose}):

  \item{n}{number of legislators in the \code{\link{rollcall}} object,
    after processing the \code{dropList}}

  \item{m}{number of roll call votes in the \code{\link{rollcall}}
    object, after processing the \code{dropList}}
  
  \item{codes}{a \code{\link{list}} that describes how the voting
    decisions in the \code{\link{rollcall}} matrix (\code{object$votes})
    map into \dQuote{Yea} and \dQuote{Nay} etc, after processing the
    \code{dropList}; see \code{\link{rollcall}} for more details}
  
  \item{allVotes}{a matrix containing a tabular breakdown of all votes
    in the \code{\link{rollcall}} matrix (\code{object$votes}), after
    processing the \code{dropList}}

  \item{partyTab}{a tabular breakdown of the legislators' party
    affiliations, after processing the \code{dropList}, and only if
    party affiliations are supplied as\code{object$legis.data$party};
    see \code{\link{rollcall}} for details}
  
  \item{lopSided}{a tabular summary of the frequency of lop-sided roll
      call votes in the \code{\link{rollcall}} object, again, after
      processing the \code{dropList}}
    
    \item{legisTab}{a tabular summary of each legislators' voting
      history}

    \item{partyLoyalty}{the proportion of times that each legislator
    votes the way that a majority of his or her fellow partisans did,
    provided party affiliations are available}

  \item{voteTab}{a tabular summary of each rollcall's votes}

  \item{call}{the \code{\link[=match.call]{matched call}} used to invoke
    \code{summary.rollcall}}
    
}

\seealso{\code{\link{rollcall}}}

\examples{
set.seed(314159265)
fakeData <- matrix(sample(x=c(0,1),size=1000,replace=TRUE),
                   10,100)
rc <- rollcall(fakeData)
rc

data(sc9497)
rc <- rollcall(sc9497)
summary(rc)

data(s109)
summary(s109)
summary(s109,verbose=TRUE)
}

\keyword{classes}

\name{plot.predict.ideal}
\alias{plot.predict.ideal}

\title{plot methods for predictions from ideal objects}

\description{
  Plot classification success rates by legislators, or by roll calls,
  using predictions from ideal.
}

\usage{
\method{plot}{predict.ideal}(x, type = c("legis", "votes"),...)
}

\arguments{
  \item{x}{an object of class \code{\link{predict.ideal}}.}
  \item{type}{string; one of \code{legis} or \code{votes}.}
  \item{...}{further arguments passed to or from other methods.}
}

\details{
  \code{type="legis"} produces a plot of the \dQuote{percent correctly
    predicted} for each legislator/subject
  (using the classification threshold set in
  \code{\link{predict.ideal}}) against the estimated ideal point of each
  legislator/subject (the estimated mean of the posterior density of the
  ideal point), dimension at
  a time.  If the legislators' party affiliations are available in the
  \code{\link{rollcall}} object that was passed to \code{\link{ideal}},
  then legislators from the same party are plotted with a unique color.

  \code{type="votes"} produces a plot of classification rates for each
  roll call, by the percentage of legislators voting for the losing
  side.  The \code{x}-ordinate is jittered for clarity.
}
\value{
  After drawing plots on the current device, exits silently returning \code{invisible(NULL)}.
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\seealso{\code{\link{predict.ideal}} \code{\link{ideal}} }

\examples{
data(s109)
f = system.file("extdata","id1.rda",package="pscl")
load(f)
phat <- predict(id1)
plot(phat,type="legis")
plot(phat,type="votes")
}
\keyword{hplot}% __ONLY ONE__ keyword per line

\name{absentee}
\alias{absentee}
\docType{data}
\title{Absentee and Machine Ballots in Pennsylvania State Senate Races}
\description{
  Absentee ballot outcomes contrasted with machine ballots, cast
  in Pennsylvania State Senate elections, selected districts,
  1982-1993.  
}
\usage{data(absentee)}
\format{
  A data frame with 22 observations on the following 8 variables.
  \describe{
    \item{\code{year}}{a numeric vector, year of election, 19xx}
    \item{\code{district}}{a numeric vector, Pennsylvania State Senate district}
    \item{\code{absdem}}{a numeric vector, absentee ballots cast for the
    Democratic candidate}
    \item{\code{absrep}}{a numeric vector, absentee ballots cast for the
    Republican candidate}
    \item{\code{machdem}}{a numeric vector, votes cast on voting
      machines for the Democratic candidate}
    \item{\code{machrep}}{a numeric vector, votes cast on voting
      machines for the Republican candidate}
    \item{\code{dabs}}{a numeric vector, Democratic margin among
      absentee ballots}
    \item{\code{dmach}}{a numeric vector, Democratic margin among
      ballots case on voting machines}
  }
}

\details{In November 1993, the state of Pennsylvania conducted
  elections for its state legislature. The result in the Senate election
  in the 2nd district (based in Philadelphia) was challenged in court,
  and ultimately overturned. The Democratic candidate won 19,127 of
  the votes cast by voting machine, while the Republican won 19,691
  votes cast by voting machine, giving the Republican a lead of 564
  votes. However, the Democrat won 1,396 absentee ballots, while the
  Republican won just 371 absentee ballots, more than offsetting the
  Republican lead based on the votes recorded by machines on election
  day. The Republican candidate sued, claiming that many of the absentee
  ballots were fraudulent. The judge in the case solicited expert
  analysis from Orley Ashenfelter, an economist at Princeton
  University. Ashenfelter examined the relationship between absentee
  vote margins and machine vote margins in 21 previous Pennsylvania
  Senate elections in seven districts in the Philadelphia area over the
  preceding decade.}

\source{
  Ashenfelter, Orley.  1994.  Report on Expected Absentee Ballots.
  Typescript.  Department of Economics, Princeton University.
}

\references{
  Ashenfelter, Orley, Phillip Levine and David Zimmerman.  2003.
  \emph{Statistics and Econometrics: Methods and Applications}.  New
  York: John Wiley and Sons.

  Jackman, Simon.  2009.  \emph{Bayesian Analysis for the Social Sciences}. 
  Wiley: Hoboken, New Jersey.  Examples 2.13, 2.14, 2.15.
}
\examples{
data(absentee)
summary(absentee)

denom <- absentee$absdem + absentee$absrep
y <- (absentee$absdem - absentee$absrep)/denom * 100
denom <- absentee$machdem + absentee$machrep
x <- (absentee$machdem - absentee$machrep)/denom *100

ols <- lm(y ~ x,
          subset=c(rep(TRUE,21),FALSE)  ## drop data point 22
          )

## predictions for disputed absentee point
yhat22 <- predict(ols,
                  newdata=list(x=x[22]),
                  se.fit=TRUE,
                  interval="prediction")
tstat <- (y[22]-yhat22$fit[,"fit"])/yhat22$se.fit
cat("tstat on actual outcome for obs 22:",tstat,"\n")
cat(paste("Pr(t>",round(tstat,2),") i.e., one-sided:\n",sep=""))
cat(1-pt(tstat,df=yhat22$df),"\n")

## make a picture
xseq <- seq(min(x)-.1*diff(range(x)),
            max(x)+.1*diff(range(x)),
            length=100)
yhat <- predict(ols,interval="prediction",
                newdata=list(x=xseq))
plot(y~x,
     type="n",
     axes=FALSE,
     ylim=range(yhat,y),
     xlim=range(xseq),xaxs="i",
     xlab="Democratic Margin, Machine Ballots (Percentage Points)",
     ylab="Democratic Margin, Absentee Ballots (Percentage Points)")
polygon(x=c(xseq,rev(xseq)),  ## overlay 95% prediction CI
        y=c(yhat[,"lwr"],rev(yhat[,"upr"])),
        border=FALSE,
        col=gray(.85))
abline(ols,lwd=2)           ## overlay ols
points(x[-22],y[-22],pch=1) ## data
points(x[22],y[22],pch=16)  ## disputed data point

text(x[22],y[22],
     "Disputed\nElection",
     cex=.75,
     adj=1.25)
axis(1)
axis(2)

}
\keyword{datasets}

\name{dropUnanimous}
\alias{dropUnanimous}
%\alias{dropUnanimous.rollcall}
%\alias{dropUnanimous.matrix}

\title{drop unanimous votes from rollcall objects and matrices}

\description{
  Drop unanimous votes from rollcall objects and rollcall matrices.
}

\usage{dropUnanimous(obj, lop = 0)}
\arguments{
  \item{obj}{object, either of class \code{\link{rollcall}} or
    \code{\link{matrix}}} 
  \item{lop}{numeric, non-negative integer, less than number of
    legislators represented in \code{obj}.  Roll calls with \code{lop}
    or fewer legislators voting in the minority are dropped.  Default is
  0, meaning that unanimous votes are dropped.}
}

\details{
  Unanimous votes are the
  equivalent of test items that all subjects score \dQuote{correct} (or
  all subjects scores \dQuote{incorrect}); since there is no variation
  among the legislators/subjects, these votes/items provide no
  information as to latent traits (ideology, preferences, ability).  A
  reasonably large number of rollcalls in any contemporary U.S. Congress
  are unanimous.

  Specific methods are provided for objects of class
  \code{\link{rollcall}} or \code{\link{matrix}}.
}
\value{
  A \code{\link{rollcall}} object or a \code{\link{matrix}} depending on
  the class of \code{object}.
}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\seealso{\code{\link{dropRollCall}},
  \code{\link{rollcall}}, \code{\link{summary.rollcall}}, \code{\link{ideal}}}
\examples{
data(s109)
s109.working <- dropUnanimous(s109)
summary(s109.working)
}
\keyword{manip}

\name{zeroinfl.control}
\alias{zeroinfl.control}

\title{Control Parameters for Zero-inflated Count Data Regression}
\description{
  Various parameters that control fitting of zero-inflated regression models
  using \code{\link{zeroinfl}}.
}

\usage{
zeroinfl.control(method = "BFGS", maxit = 10000, trace = FALSE,
  EM = FALSE, start = NULL, \dots)
}

\arguments{
  \item{method}{characters string specifying the \code{method} argument
    passed to \code{\link[stats]{optim}}.}
  \item{maxit}{integer specifying the \code{maxit} argument (maximal number
    of iterations) passed to \code{\link[stats]{optim}}.}
  \item{trace}{logical or integer controlling whether tracing information on 
    the progress of the optimization should be produced (passed to \code{\link[stats]{optim}}).}
  \item{EM}{logical. Should starting values be estimated by the EM (expectation
    maximization) algorithm? See details.}
  \item{start}{an optional list with elements \code{"count"} and \code{"zero"}
    (and potentially \code{"theta"}) containing the coefficients for the corresponding component.}
  \item{\dots}{arguments passed to \code{\link[stats]{optim}}.}
}

\details{
  All parameters in \code{\link{zeroinfl}} are estimated by maximum likelihood
  using \code{\link[stats]{optim}} with control options set in \code{\link{zeroinfl.control}}.
  Most arguments are passed on directly to \code{optim}, only \code{trace} is also
  used within \code{zeroinfl} and \code{EM}/\code{start} control the choice
  of starting values for calling \code{optim}.

  Starting values can be supplied, estimated by the EM (expectation maximization)
  algorithm, or by \code{\link[stats]{glm.fit}} (the default). Standard errors are
  derived numerically using
  the Hessian matrix returned by \code{\link[stats]{optim}}. To supply starting 
  values, \code{start} should be a list with elements \code{"count"} and \code{"zero"}
  and potentially \code{"theta"} (for negative binomial components only) containing
  the starting values for the coefficients of the corresponding component of the
  model.
}

\value{
  A list with the arguments specified.
}

\author{Achim Zeileis <Achim.Zeileis@R-project.org>}

\seealso{\code{\link{zeroinfl}}}

\examples{
\dontrun{
data("bioChemists", package = "pscl")

## default start values
fm1 <- zeroinfl(art ~ ., data = bioChemists)

## use EM algorithm for start values
fm2 <- zeroinfl(art ~ ., data = bioChemists, EM = TRUE)

## user-supplied start values
fm3 <- zeroinfl(art ~ ., data = bioChemists,
  start = list(count = c(0.7, -0.2, 0.1, -0.2, 0, 0), zero = -1.7))
}
}

\keyword{regression}

\name{constrain.legis}
\alias{constrain.legis}
\title{constrain legislators' ideal points in analysis of roll call data}

\description{
  Sets constraints on specified legislators for ideal
  point estimation by generating appropriate priors and start values.
}

\usage{
constrain.legis(obj, dropList = list(codes = "notInLegis", lop = 0),
                 x, d = 1)

}

\arguments{
  \item{obj}{an object of class \code{\link{rollcall}}.}

  \item{dropList}{a \code{\link{list}} (or \code{\link{alist}})
    indicating which voting decisions, legislators and/or roll calls are
    to be excluded from the subsequent analysis; see
    \code{\link{dropRollCall}} for details.}

  \item{x}{a \code{\link{list}} containing elements with names partially
    matching legislators found in \code{dimnames(object$votes)[[1]]} (but after
    any sub-setting specified by \code{dropList}).  Each element must be a
    vector containing \code{d} elements, specifying the value to which
    the ideal point should be constrained in each of \code{d}
    dimensions. \code{x} must have at least \code{d+1} components; i.e.,
    supplying a necessary (but not sufficient) set of constraints for
    global identification of the parameters of a \code{d}-dimensional
    item-response model, see Details.}

  \item{d}{the number of dimensions for which to set up the priors and
    start values.}
}

\details{
  \code{constrain.items} and its cousin,
  \code{\link{constrain.legis}} are usefully thought of as
  \dQuote{pre-processor} functions, implementing identification
  constraints for the ideal point model by generating priors \emph{and} start
  values for both the item parameters and the ideal points.

  For the legislators specified in \code{x}, the prior mean for each
  dimension is set to the specified value and the prior precision for
  each dimension is set to \code{1e12} (i.e., a near-degenerate
  \dQuote{spike} prior, and, for all practical purposes, constraining
  that parameter to a fixed value).  For the other legislators, the
  priors on their ideal points are set to a mean of 0 and a small
  precision of .01, corresponding to a prior variance of 100, or a prior
  95 percent confidence interval of -20 to 20.  All of the item
  parameter priors are set to mean 0, precision 0.01.

  Start values are also generated for both ideal points and item
  parameters.  The start values for the legislators named in
  \code{x} are set to the values specified in \code{x}.  The list
  resulting from \code{constrain.legis} can then be given as the value
  for the parameters \code{priors} and \code{startvals} when
  \code{\link{ideal}} is run. \code{constrain.legis} requires that
  \code{d+1} constraints be specified; if the constrained ideal points
  points are linearly independent, then the parameters of the
  item-response model are (at least locally) identified.  For instance,
  when fitting a 1 dimensional model, constraining the ideal points of
  two legislators is sufficient to globally identify the model
  parameters.

 \code{\link{dropRollCall}} is first called to generate the desired
  roll call matrix.  The entries of the roll call matrix are mapped to
  \code{c(0,1,NA)} using the \code{codes} component of the
  \code{\link{rollcall}} \code{object}.  See the discussion in the
  documentation of \code{\link{ideal}} for details on the generation of
  start values.

}


\value{ a list with elements:

  \item{xp}{prior means for ideal points.  A matrix of dimensions number
    of legislators in \code{rc} by \code{d}.}
  
  \item{xpv}{prior meansprecisions for ideal points.  A matrix of
    dimensions number of legislators in \code{rc} by \code{d}.}
  
  \item{bp}{prior means for item parameters.  A matrix of
    dimensions number of items or votes in \code{rc} by \code{d+1}.}
  
  \item{bpv}{prior meansprecisions for item parameters.  A matrix of
    dimensions number of items or votes in \code{rc} by \code{d+1}.}
  
  \item{x}{start values for ideal points.  A matrix of dimensions
    number of legislators in \code{rc} by \code{d}.}
  
  \item{b}{start values for ideal points.  A matrix of dimensions
    number of items or votes in \code{rc} by \code{d+1}.}

}

\seealso{
  \code{\link{rollcall}}, \code{\link{ideal}},
  \code{\link{constrain.items}}.
  See \code{\link{pmatch}} on how supplied names are matched against the
  names in the \code{\link{rollcall}} object.
} 


\examples{
data(s109)
cl <- constrain.legis(s109,
                      x=list("KENNEDY"=-1,
                        "ENZI"=1),
                      d=1)

\dontrun{
## too long for examples
id1Constrained <- ideal(s109,
                       d=1,
                       priors=cl,      ## use cl
                       startvals=cl,   ## use cl
                       maxiter=5000,
                       burnin=500,
                       thin=25)
summary(id1Constrained)

cl2 <- constrain.legis(s109,
                       x=list("KENNEDY"=c(-1,0),
                         "ENZI"=c(1,0),
                         "CHAFEE"=c(0,-.5)),
                       d=2)


id2Constrained <- ideal(s109,
                        d=2,
                        priors=cl2,      ## priors (w constraints)
                        startvals=cl2,   ## start value (w constraints)
                        store.item=TRUE,
                        maxiter=5000,
                        burnin=500,
                        thin=25)
summary(id2Constrained,include.items=TRUE)
}
}
\keyword{datagen}

\name{hurdletest}
\alias{hurdletest}

\title{Testing for the Presence of a Zero Hurdle}

\description{
  Wald test of the null hypothesis that no zero hurdle
  is required in hurdle regression models for count data.
}

\usage{
hurdletest(object, \dots)
}

\arguments{
  \item{object}{A fitted model object of class \code{"hurdle"}
    as returned by \code{\link{hurdle}}, see details for more information.}
  \item{\dots}{arguments passed to \code{\link[car]{linearHypothesis}}.}
}

\details{
  If the same count distribution and the same set of regressors is used in
  the hurdle model for both, the count component and the zero hurdle component,
  then a test of pairwise equality between all coefficients from the two
  components assesses the null hypothesis that no hurdle is needed in the
  model.
  
  The function \code{hurdletest} is a simple convenience interface to
  the function \code{\link[car]{linearHypothesis}} from the \pkg{car}
  packages that can be employed to carry out a Wald test for this 
  hypothesis.
}

\value{
  An object of class \code{"anova"} as returned by \code{\link[car]{linearHypothesis}}.
}

\references{
Cameron, A. Colin and Pravin K. Trivedi. 1998. \emph{Regression Analysis of Count 
Data}. New York: Cambridge University Press.

Cameron, A. Colin and Pravin K. Trivedi 2005. \emph{Microeconometrics: Methods and Applications}.
Cambridge: Cambridge University Press.
}

\author{Achim Zeileis <Achim.Zeileis@R-project.org>}

\seealso{\code{\link{hurdle}}, \code{\link[car]{linearHypothesis}}}

\examples{
data("bioChemists", package = "pscl")
fm <- hurdle(art ~ ., data = bioChemists, dist = "negbin", zero = "negbin")
hurdletest(fm)
}

\keyword{regression}

\name{AustralianElections}
\alias{AustralianElections}
\docType{data}
\title{elections to Australian House of
  Representatives, 1949-2016}

\description{
  Aggregate data on the 24 elections to Australia's House of
  Representatives, 1949 to 2016. 
}

\usage{data(AustralianElections)}

\format{
 A data frame with the following variables:	
  \describe{
    \item{\code{date}}{date of election, stored using the
      \code{\link{Date}} class}
    \item{\code{Seats}}{numeric, number of seats in the House of
      Representatives} 
    \item{\code{Uncontested}}{numeric, number of uncontested seats}
    \item{\code{ALPSeats}}{numeric, number of seats won by the
      Australian Labor Party}
    \item{\code{LPSeats}}{numeric, number of seats won by the Liberal Party} 
    \item{\code{NPSeats}}{numeric, number of seats won by the National
      Party (previously known as the Country Party)}
    \item{\code{OtherSeats}}{numeric, number of seats won by other
      parties and/or independent candidates}
    \item{\code{ALP}}{numeric, percentage of first preference votes cast for
      Australian Labor Party candidates}
    \item{\code{ALP2PP}}{numeric, percentage of the two-party preferred
      vote won by Australian Labor Party candidates}
    \item{\code{LP}}{numeric, percent of first preference votes cast for
      Liberal Party candidates}
    \item{\code{NP}}{numeric, percent of first preference votes cast for
      National Party (Country Party) candidates}
    \item{\code{DLP}}{numeric, percent of first preference votes cast
      for Democratic Labor Party candidates}
    \item{\code{Dem}}{numeric, percent of first preference votes cast
      for Australian Democrat candidates}
    \item{\code{Green}}{numeric, percent of first preference votes cast
      for Green Party candidates}
    \item{\code{Hanson}}{numeric, percent of first preference votes cast
      for candidates from Pauline Hanson's One Nation party}
    \item{\code{Com}}{numeric, percent of first preference votes cast
      for Communist Party candidates}
    \item{\code{AP}}{numeric, percent of first preference votes cast for
      Australia Party candidates}
    \item{\code{Informal}}{numeric, percent of ballots cast that are
      spoiled, blank, or otherwise uncountable (usually because of
      errors in enumerating preferences)}
    \item{\code{Turnout}}{numeric, percent of enrolled voters recorded
      as having turned out to vote (Australia has compulsory voting)}
  }
}

\note{The Liberal National Party of Queensland formed in 2008 after a merger of the Liberal Party and the National Party. In all elections following 2008, they have been categorised under \code{LP}.}

\source{Australian Electoral Commission. \url{http://www.aec.gov.au}.
}

\references{Jackman, Simon. 2009. \emph{Bayesian Analysis for the Social Sciences}.  Wiley: Hoboken, New Jersey.  Example 3.5.
} 
 
\examples{
data(AustralianElections)
attach(AustralianElections)
alpSeatShare <- ALPSeats/Seats
alpVoteShare <- ALP2PP/100

## log-odds transforms
x <- log(alpVoteShare/(1-alpVoteShare))
y <- log(alpSeatShare/(1-alpSeatShare))

ols <- lm(y~x)   ## Tufte-style seats-votes regression

xseq <- seq(-4.5,4.5,length=500)
yhat <- coef(ols)[1] + coef(ols)[2]*xseq
yhat <- exp(yhat)/(1+exp(yhat))
xseq <- exp(xseq)/(1+exp(xseq))

## seats vote curve
plot(x=alpVoteShare,
     y=alpSeatShare,
     xlab="ALP Vote Share",
     ylab="ALP Seat Share")
lines(xseq,yhat,lwd=2)
abline(h=.5,lty=2)
abline(v=.5,lty=2)
}
\keyword{datasets}

\name{simpi}
\alias{simpi}

\title{Monte Carlo estimate of pi (3.14159265...)}
\description{
  Monte Carlo estimation of pi
}
\usage{
simpi(n)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n}{integer, number of Monte Carlo samples, defaults to 1000}
}

\details{
  A crude Monte Carlo estimate of \eqn{\pi}{pi} can be formed as
  follows.  Sample from the unit square many times (i.e., each sample is
  formed with two independent draws from a uniform density on the unit
  interval).  Compute the proportion \eqn{p}{p} of sampled points that
  lie inside a unit circle centered on the origin; such points
  \eqn{(x,y)}{(x,y)} have distance from the origin \eqn{d = \sqrt{x^2 +
  y^2}}{d=sqrt(x^2 + y^2)} less than 1.  Four times \eqn{p}{p} is a
  Monte Carlo estimate of \eqn{\pi}{pi}.  This function is a wrapper to
  a simple C function, bringing noticeable speed gains and memory
  efficiencies over implementations in native R.

  Contrast this Monte Carlo method with Buffon's needle and refinements thereof (see
  the discussion in Ripley (1987, 193ff). 
}
  
\value{the Monte Carlo estimate of \eqn{\pi}{pi}}

\references{Ripley, Brain D. 1987 [2006].  \emph{Stochastic Simulation}.
  Wiley: Hoboken, New Jersey.}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\examples{
seed <- round(pi*10000)  ## hah hah hah
m <- 6
z <- rep(NA,m)
lim <- rep(NA,m)
for(i in 1:m){
  cat(paste("simulation for ",i,"\n"))
  set.seed(seed)
  timings <- system.time(z[i] <- simpi(10^i))
  print(timings)
  cat("\n")
  lim[i] <- qbinom(prob=pi/4,size=10^i,.975)/10^i * 4
}

## convert to squared error
z <-(z - pi)^2
lim <- (lim - pi)^2

plot(x=1:m,
     y=z,
     type="b",
     pch=16,
     log="y",
     axes=FALSE,
     ylim=range(z,lim),
     xlab="Monte Carlo Samples",
     ylab="Log Squared Error")
lines(1:m,lim,col="blue",type="b",pch=1)
legend(x="topright",
       legend=c("95\% bound",
         "Realized"),
       pch=c(1,16),
       lty=c(1,1),
       col=c("blue","black"),
       bty="n")
axis(1,at=1:m,
     labels=c(expression(10^{1}),
       expression(10^{2}),
       expression(10^{3}),
       expression(10^{4}),
       expression(10^{5}),
       expression(10^{6})))
axis(2)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{misc}

\name{summary.ideal}
\alias{summary.ideal}
\title{summary of an ideal object}

\description{
  Provides a summary of the output from ideal point estimation contained
  in an object of class \code{ideal}.
}

\usage{
\method{summary}{ideal}(object, prob=.95,
              burnin=NULL,
              sort=TRUE,
              include.beta=FALSE,...)
}

\arguments{

  \item{object}{an object of class \code{\link{ideal}}.}

  \item{prob}{scalar, a proportion between 0 and 1, the content of the highest posterior density (HPD) interval to compute for the parameters}
  
  \item{burnin}{of the recorded MCMC samples, how many to discard as
    burnin?  Default is \code{NULL}, in which case the value of
    \code{burnin} in the \code{\link{ideal}} object is used.}
  
  \item{sort}{logical, default is \code{TRUE}, indicating that the
    summary of the ideal points be sorted by the estimated posterior means
    (lowest to highest)}
  
  \item{include.beta}{whether or not to calculate summary statistics of
    beta, if beta is available.  If the item parameters were not stored
    in the \code{ideal} object, then \code{include.beta} is ignored.}
  
  \item{...}{further arguments passed to or from other functions}
  
}

\value{
  An item of class \code{summary.ideal} with elements:
  \item{object}{the name of the ideal object as an
    \code{\link[=eval]{unevaluated}} \code{\link{expression}}, produced
    by \code{match.call()$object}}
  \item{xm}{\code{n} by \code{d} matrix of posterior means for the ideal points}
  \item{xsd}{\code{n} by \code{d} matrix of posterior means for the ideal points}
  \item{xHDR}{\code{n} by 2 by \code{d} array of HDRs for the ideal points}  
  \item{bm}{\code{m} by \code{d+1} matrix of posterior means for the
      item parameters}
  \item{bsd}{\code{m} by \code{d+1} matrix of posterior standard deviation for the
      item parameters}
  \item{bHDR}{\code{m} by 2 by \code{d+1} array of HDRs for the item parameters}  
  \item{bSig}{a \code{\link{list}} of length \code{d}, each component a
    vector of length \code{m}, of mode \code{logical}, equal to
    \code{TRUE} if the corresponding discrimination parameter is
    distinguishable from zero; see Details.  If \code{store.item} was
    set to \code{FALSE} when \code{\link{ideal}} was invoked, then
    \code{bSig} is a list of length zero.}   
  \item{party.quant}{if party information is available through the
    \code{rollcall} object that was used to run \code{ideal}, then
    \code{party.quant} gives the posterior mean of the legislators'
    ideal points by party, by dimension.  If no party information is
    available, then \code{party.quant=NULL}.}
}

\details{The test of whether a given discrimination parameter is
  distinguishable from zero first checks to see if the two most extreme
  \code{quantiles} are symmetric around .5 (e.g., as are the default
  value of .025 and .975).  If so, the corresponding quantiles of the
  MCMC samples for each discrimination parameter are inspected to see if
  they have the same sign. If they do, then the corresponding
  discrimination parameter is flagged as distinguishable from zero;
  otherwise not.
}

\note{When specifying a value of \code{burnin} different from that used
  in fitting the \code{\link{ideal}} object, note a distinction
  between the iteration numbers of the stored iterations, and the
  number of stored iterations.  That is, the \code{n}-th iteration
  stored in an \code{\link{ideal}} object will not be iteration
  \code{n} if the user specified \code{thin>1} in the call to
  \code{\link{ideal}}.  Here, iterations are tagged with their
  iteration number.  Thus, if the user called \code{\link{ideal}} with
  \code{thin=10} and \code{burnin=100} then the stored iterations are
  numbered \code{100, 110, 120, ...}.  Any future subsetting via a
  \code{burnin} refers to this iteration number.}

\seealso{\code{\link{ideal}}}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\examples{
f <- system.file("extdata","id1.rda",package="pscl")
load(f)
summary(id1)

\dontrun{
data(s109)
cl2 <- constrain.legis(s109,
                       x=list("KENNEDY (D MA)"=c(-1,0),
                         "ENZI (R WY)"=c(1,0),
                         "CHAFEE (R RI)"=c(0,-.5)),
                       d=2)
id2Constrained <- ideal(s109,
                        d=2,
                        priors=cl2,      ## priors (w constraints)
                        startvals=cl2,   ## start value (w constraints)
                        store.item=TRUE,
                        maxiter=5000,
                        burnin=500,
                        thin=25)

summary(id2Constrained,
        include.items=TRUE)
}
}

\keyword{classes}

\name{prussian}
\alias{prussian}
\docType{data}

\title{Prussian army horse kick data}

\description{
  Deaths by year, by corp, from horse kicks.
}

\usage{data(prussian)}

\format{
  A data frame with 280 observations on the following 3 variables.
  \describe{
    \item{\code{y}}{a numeric vector, count of deaths}
    \item{\code{year}}{a numeric vector, 18XX, year of observation}
    \item{\code{corp}}{a \code{\link{factor}}, corp of Prussian Army generating
      observation}
  }
}

\source{
  von Bortkiewicz, L. 1898. \emph{Das Gesetz der Kleinen Zahlen.} Leipzig: Teubner. 
}

\examples{
data(prussian)
corpP <- glm(y ~ corp, family=poisson,data=prussian)
summary(corpP)
}
\keyword{datasets}

\name{s109}
\alias{s109}
\docType{data}
\title{rollcall object, 109th U.S. Senate (2005-06).}

\description{A sample rollcall object, generated using a
  collection of the rollcalls of the 109th U.S. Senate (2005-2006).}

\usage{data(s109)}

\format{
  A \code{\link{rollcall}} object containing the recorded votes of the 109th
  U.S. Senate, plus information identifying the legislators and the
  rollcalls.
}

\details{Note the coding scheme used by Poole and Rosenthal; Yea (1),
  Nay (6) etc.}

\source{
  Keith Poole's web site:
  \url{https://legacy.voteview.com/senate109.htm}

  Originally scraped from the Senate's web site by Jeff Lewis (UCLA).
  
  Information identifying the votes is available at
  \url{https://voteview.com/static/data/out/rollcalls/S109_rollcalls.csv}
}

\examples{
require(pscl)
data(s109)
is(s109,"rollcall")    ## TRUE
s109                   ## print method for class rollcall
summary(s109)          ## summary method
summary(s109,verbose=TRUE)
\dontrun{
## how s109 was created
require(pscl)
s109 <- readKH("https://voteview.com/static/data/out/votes/S109_votes.ord",
               desc="109th U.S. Senate",
               debug=TRUE)
url <- "https://voteview.com/static/data/out/rollcalls/S109_rollcalls.csv"

s109$vote.data <- data.frame(read.csv(file=url,header=TRUE))
s109$vote.data$date <- as.Date(s109$vote.data$date,
                               format="%Y-%m-%d")
dimnames(s109$votes)[[2]] <- paste(s109$vote.data$session,
                                   s109$vote.data$number,sep="-")
}
}
\keyword{datasets}

\name{bioChemists}
\alias{bioChemists}
\docType{data}
\title{article production by graduate students in biochemistry Ph.D. programs}
\description{
  A sample of 915 biochemistry graduate students.
}
\usage{data(bioChemists)}
\format{
  \describe{
    \item{\code{art}}{count of articles produced during last 3 years of Ph.D.}
    \item{\code{fem}}{factor indicating gender of student, with levels Men and Women}
    \item{\code{mar}}{factor indicating marital status of student, with levels
      Single and Married}
    \item{\code{kid5}}{number of children aged 5 or younger}
    \item{\code{phd}}{prestige of Ph.D. department}
    \item{\code{ment}}{count of articles produced by Ph.D. mentor during last 3 years}
    }
  }
%%\source{found in Stata format at \url{http://www.indiana.edu/~jslsoc/stata/socdata/couart2.dta}}
\references{Long, J. Scott. 1990.  The origins of sex differences in
  science. \emph{Social Forces}. 68(3):1297-1316.

  Long, J. Scott. 1997.  \emph{Regression Models for Categorical and
    Limited Dependent Variables}. Thousand Oaks, California: Sage.
}
\keyword{datasets}

\name{nj07}
\alias{nj07}
\docType{data}
\title{rollcall object, National Journal key votes of 2007}

\description{A rollcall object containing 99 rollcalls from the 2nd session of the 110th U.S. Senate,  
  designated by \emph{National Journal} as the "key votes" of 2007.  These data were used to by \emph{National Journal} to rate (then Senator) Barack Obama was the "most liberal senator" in 2007.} 

\usage{data(nj07)}

\format{
  A \code{\link{rollcall}} object containing the recorded votes,
  plus information identifying the legislators and the
  rollcalls.
}

\details{Note the coding scheme used by Poole and Rosenthal; Yea (1,2,3),
  Nay (4,5,6) etc.}

\source{
  Keith Poole's web site:
  \url{http://legacy.voteview.com/senate110.htm}

  Originally scraped from the Senate's web site by Jeff Lewis.

  Josh Clinton compiled the list of \emph{National Journal} key votes.  
}

\references{
	Clinton, Joshua and Simon Jackman.  2009. To Simulate or NOMINATE?  
	\emph{Legislative Studies Quarterly}. V34(4):593-621.

	Jackman, Simon.  2009. \emph{Bayesian Analysis for the Social Sciences}. Wiley: Hoboken, New Jersey. Example 9.2. 
}

\examples{
require(pscl)
data(nj07)
is(nj07,"rollcall")    ## TRUE
nj07                   ## print method for class rollcall
names(nj07)
names(nj07$vote.data)
table(nj07$vote.data$policyArea)
summary(nj07)          ## summary method
summary(nj07,verbose=TRUE)
}
\keyword{datasets}

\name{predprob}
\alias{predprob}

\title{compute predicted probabilities from fitted models}

\description{
  Compute predicted probabilities from fitted models, optionally at new
  covariate values.
}

\usage{
predprob(obj, \dots)
}

\arguments{
  \item{obj}{fitted model object}
  \item{\dots}{other arguments}
}

\details{
  See documentation for specific methods.
}

\value{
  A matrix of predicted probabilities, each row a vector of
  predicted probabilities over the range of responses seen in the data
  (i.e., \code{min(y):max(y)}), conditional on the values of covariates. 
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\seealso{\code{\link{predprob.glm}}, \code{\link{predprob.zeroinfl}}}

\examples{
\dontrun{
data("bioChemists")
zip <- zeroinfl(art ~ . | ., data = bioChemists, EM = TRUE)
phat <- predprob(zip)

newdata <- expand.grid(list(fem="Men",mar="Married",
                            kid5=1,phd=3.103,
                            ment=0:77))
phat <- predprob(zip, newdata = newdata)
}
}
\keyword{regression}
\keyword{models}

\name{plot.seatsVotes}
\alias{plot.seatsVotes}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{plot seats-votes curves}
\description{
  Plot seats-votes curves produced by \code{seatsVotes}
}
\usage{
\method{plot}{seatsVotes}(x, type = c("seatsVotes", "density"),
                legend = "bottomright", transform=FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{an object of class \code{\link{seatsVotes}}}
  \item{type}{character, partially matching the options above; see details}
  \item{legend}{where to put the legend when plotting with
    \code{type="seatsVotes"}}
  \item{transform}{logical, whether to transform the vote shares for
    \code{type="density"}; see Details} 
  \item{\dots}{arguments passed to or from other functions (e.g.,
    options for the \code{\link{density}} function, when
    \code{type="density"})}  
}

\details{A seats-votes curve (with various annotations) is produced
  with option \code{type="seatsVotes"}.

  A density plot of the vote shares is
  produced with \code{type="density"}. A bimodal density corresponds to
  an electoral system with a proliferation of safe seats for both
  parties, and a seats-votes curve that is relatively flat (or
  \dQuote{unresponsive}) in the neighborhood of average district-level
  vote shares of 50 percent.  The density is fitted using the defaults
  in the \code{\link{density}} function, but with the density constrained
  to fall to zero at the extremes of the data, via the \code{from} and
  \code{to} options to density.  A \code{\link{rug}} is added to the
  density plot.

  If \code{transform=TRUE}, the vote shares are transformed prior to
  plotting, so as to reduce the extent to which extreme vote shares
  close to zero or 1 determine the shape of the density (i.e., this
  option is available only for plots of \code{type="density"}).  The
  transformation is a sinusoidal function, a scaled
  \dQuote{log-odds/inverse-log-odds} function mapping from (0,1) to
  (0,1): i.e., \eqn{f(x) = g(k\cdot h(x))}{f(x) = g(kh(x))} where
  \eqn{h(x)} is the log-odds transformation \eqn{h(x) =
  \log(x/(1-x))}{h(x) = log[x/(1-x)]}, \eqn{k} is a scaling parameter
  set to \eqn{\sqrt{3}}{sqrt(3)}, and \eqn{g(x)} is the inverse-log-odds
  transformation \eqn{g(x) = \exp(x)/(1+\exp(x))}{g(x) =
  exp(x)/[1+exp(x)]}.  Note that this transformation is cosmetic, with
  the effect of assigning more of the graphing region to be devoted to
  marginal seats.
  
}

\value{
  The function performs the requested plots and exits silently with
  \code{invisible{NULL}}.
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\seealso{\code{\link{density}}, \code{\link{rug}}}
\examples{
data(ca2006)
x <- ca2006$D/(ca2006$D+ca2006$R)
sv <- seatsVotes(x,
                 desc="Democratic Vote Shares, California 2006 congressional elections")

plot(sv)
plot(sv,type="density")
plot(sv,type="density",transform=TRUE)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{hplot}

\name{igamma}
\alias{igamma}
\alias{densigamma}
\alias{pigamma}
\alias{qigamma}
\alias{rigamma}
\alias{igammaHDR}
\title{inverse-Gamma distribution}

\description{Density, distribution function, quantile function, and highest density region calculation for the inverse-Gamma distribution with parameters \code{alpha} and \code{beta}.}

\usage{
	densigamma(x,alpha,beta)
  	pigamma(q,alpha,beta)
	qigamma(p,alpha,beta)
	rigamma(n,alpha,beta)
	igammaHDR(alpha,beta,content=.95,debug=FALSE)
}

\arguments{
	\item{x,q}{vector of quantiles}
	\item{p}{vector of probabilities}
	\item{n}{number of random samples in \code{rigamma}} 
	\item{alpha,beta}{rate and shape parameters of the
	inverse-Gamma density, both positive}
	\item{content}{scalar, 0 < \code{content} < 1, volume of
	highest density region}
	\item{debug}{logical; if TRUE, debugging information from the
	search for the HDR is printed} 
    }
    
\details{
  The inverse-Gamma density arises frequently in Bayesian
  analysis of normal data, as the (marginal) conjugate prior for
  the unknown variance parameter.  The inverse-Gamma density
  for \eqn{x>0} with parameters \eqn{\alpha>0} and \eqn{\beta>0}
  is
  \deqn{f(x) = \frac{\beta^\alpha}{\Gamma(\alpha)} x^{-\alpha-1}
    \exp(-\beta/x)}{%
    (beta^alpha)/Gamma(alpha) x^(-alpha-1) exp(-beta/x)
  }
  where \eqn{\Gamma(x)} is the \code{\link{gamma}} function
  \deqn{\Gamma(a) = \int_0^\infty t^{a-1} \exp(-t) dt}{%
    Gamma(a) = int_0^infty t^(a-1) exp(-t) dt
  }
  and so ensures \eqn{f(x)} integrates to one.  The
  inverse-Gamma density has a mean at \eqn{\beta/(\alpha-1)}{beta/(alpha-1)} for
  \eqn{\alpha>1}{alpha>1} and has variance \eqn{\beta^2/((\alpha-1)^2
    (\alpha-2))}{beta^2/((alpha-1)^2 (alpha-2))} for
  \eqn{\alpha>2}{alpha>2}.  The inverse-Gamma density has a unique mode at
  \eqn{\beta/(\alpha+1)}{beta/(alpha+1)}.  
  
  The evaluation of the density,
  cumulative distribution function and quantiles is done by
  calls to the \code{dgamma}, \code{pgamma} and \code{igamma}
  functions, with the arguments appropriately transformed.  That
  is, note
  that if \eqn{x \sim IG(\alpha,\beta)}{x ~ IG(alpha,beta} then \eqn{1/x \sim
    G(\alpha,\beta)}{1/x ~ G(alpha,beta)}.  
  
  \emph{Highest Density Regions}. In general, suppose \eqn{x}
  has a density \eqn{f(x)}, where \eqn{x \in \Theta}.  Then a
  highest density region (HDR) for \eqn{x} with content \eqn{p
    \in (0,1]} is a region (or set of regions) \eqn{\mathcal{Q}
    \subseteq \Theta} such that:
  \deqn{\int_\mathcal{Q} f(x) dx = p}{%
    int_Q f(x) dx = p
  }
  and
  \deqn{f(x) > f(x^*) \, \forall\ x \in \mathcal{Q},
    x^* \not\in \mathcal{Q}.}{%
    f(x) > f(x*) for all x in Q and all x* not in Q.
  }
  For a continuous, unimodal
  density defined with respect to a single parameter (like the
  inverse-Gamma case considered here with parameters \eqn{0 <
    \alpha < \infty, \,\, 0 < \beta < \infty}), a HDR region \eqn{Q}
  of content \eqn{p} (with \eqn{0 < p < 1}) is a unique, closed
  interval on the real half-line.
  
  This function uses numerical methods to solve for the
  boundaries of a HDR with \code{content} \eqn{p} for the
  inverse-Gamma density, via repeated calls the functions
  \code{densigamma}, \code{pigamma} and
  \code{qigamma}. In particular, the function \code{\link{uniroot}} is used to
  find points \eqn{v} and \eqn{w} such that \deqn{f(v) = f(w)} subject
  to the constraint
  \deqn{\int_v^w f(x; \alpha, \beta) d\theta = p.}{%
    int_v^w f(x; alpha, beta) d theta = p.
    }
}

\value{\code{densigamma} gives the density, \code{pigamma} the
  distribution function, \code{qigamma} the quantile function,
  \code{rigamma} generates random samples, and \code{igammaHDR} gives
  the lower and upper limits of the HDR, as defined above (\code{NA}s if
  the optimization is not successful).
}

\note{The \code{densigamma} is named so as not to conflict with the
  \code{\link{digamma}} function in the R \code{\link{base}} package
  (the derivative of the \code{gamma} function).
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\seealso{\code{\link{gamma}}, \code{\link{dgamma}},
\code{\link{pgamma}}, \code{\link{qgamma}}, \code{\link{uniroot}}}

\examples{
alpha <- 4
beta <- 30
summary(rigamma(n=1000,alpha,beta))

xseq <- seq(.1,30,by=.1)
fx <- densigamma(xseq,alpha,beta)
plot(xseq,fx,type="n",
     xlab="x",
     ylab="f(x)",
     ylim=c(0,1.01*max(fx)),
     yaxs="i",
     axes=FALSE)
axis(1)
title(substitute(list(alpha==a,beta==b),list(a=alpha,b=beta)))
q <- igammaHDR(alpha,beta,debug=TRUE)
xlo <- which.min(abs(q[1]-xseq))
xup <- which.min(abs(q[2]-xseq))
plotZero <- par()$usr[3]
polygon(x=xseq[c(xlo,xlo:xup,xup:xlo)],
        y=c(plotZero,
          fx[xlo:xup],
          rep(plotZero,length(xlo:xup))),
        border=FALSE,
        col=gray(.45))
lines(xseq,fx,lwd=1.25)


\dontrun{
alpha <- beta <- .1
xseq <- exp(seq(-7,30,length=1001))
fx <- densigamma(xseq,alpha,beta)
plot(xseq,fx,
     log="xy",
     type="l",
     ylim=c(min(fx),1.01*max(fx)),
     yaxs="i",
     xlab="x, log scale",
     ylab="f(x), log scale",
     axes=FALSE)
axis(1)

title(substitute(list(alpha==a,beta==b),list(a=alpha,b=beta)))
q <- igammaHDR(alpha,beta,debug=TRUE)
xlo <- which.min(abs(q[1]-xseq))
xup <- which.min(abs(q[2]-xseq))
plotZero <- min(fx)
polygon(x=xseq[c(xlo,xlo:xup,xup:xlo)],
        y=c(plotZero,
          fx[xlo:xup],
          rep(plotZero,length(xlo:xup))),
        border=FALSE,
        col=gray(.45))
lines(xseq,fx,lwd=1.25)
}
}
\keyword{distribution}

\name{ntable}
\alias{ntable}
\title{nicely formatted tables}
\description{
  Nicely formatted tables, with row or column marginals etc.
}
\usage{
ntable(x,y=NULL,
       percent=1,digits=2,
       row=FALSE,col=FALSE)
}

\arguments{
  \item{x}{vector or \code{\link{factor}}}
  \item{y}{vector of \code{\link{factor}}}
  \item{percent}{integer, 1 for row percentages (default), 2 for column
    percentages}
  \item{digits}{integer, digits to print after decimal place (default is 2)}
  \item{row}{logical, if \code{TRUE}, print row marginals}
  \item{col}{logical, if \code{TRUE}, print column marginals}
}

\details{A wrapper function to \code{\link{prop.table}} that produces prettier
  looking results.
}

\value{
  nothing returned; the function prints the table and
  exits silently.
}
\seealso{\code{\link{prop.table}}, \code{\link{table}}}

\author{Jim Fearon \email{jfearon@stanford.edu}}

\examples{
data(bioChemists)
attach(bioChemists)
ntable(fem)
ntable(fem,mar,row=TRUE)
ntable(fem,mar,per=2,col=TRUE)
ntable(fem,mar,per=2,row=TRUE,col=TRUE)
}
\keyword{print}

\name{tracex}
\alias{tracex}
\title{trace plot of MCMC iterates, posterior density of legislators'
  ideal points} 

\description{
  Produces a trace plot of the MCMC samples from the posterior density of
  legislators' \code{\link{ideal}} points.
}

\usage{
tracex(object, legis=NULL, d=1, conf.int=0.95,
       multi = FALSE, burnin=NULL,span=.25,
       legendLoc="topright")
}

\arguments{
  \item{object}{an object of class \code{ideal}.}
  \item{legis}{a vector of either the names of legislators (or
  \code{\link[=pmatch]{partial matches}} of the names as given in the
  \code{\link{dimnames}} of \code{object$x}.}
  \item{d}{numeric, either a scalar or a vector of length two, the
  dimension(s) to be traced.} 
  \item{conf.int}{numeric, the level of the confidence interval on the
  posterior mean to be plotted.}
  \item{multi}{logical, multiple plotting panels, one per legislators? If \code{FALSE} (default) and \code{length(d)==2},
  display traces for all selected legislators' ideal points on
  the one plot.}
 \item{burnin}{of the recorded MCMC samples, how many to discard as
    burnin?  Default is \code{NULL}, in which case the value of
    \code{burnin} in the \code{\link{ideal}} object is used.}
  \item{span}{numeric, a proportion, the \code{span} to be used when
    calling \code{loess} to generate a moving average for trace plots when
    \code{d=1}}
  \item{legendLoc}{numeric or character, and possibly a vector,
  specifying where to place the legend when \code{d=1}; setting
  \code{legendLoc=NULL} will suppress the legend for all requested trace
  plots}
}

\details{Produces a trace plot showing the history of the MCMC
  iterations for the ideal point of each of the legislators (partially)
  named in \code{legis}.  For \code{d=1}, each trace plot includes a
  trace over iterations, the cumulative mean, a moving average, the
  MCMC-based estimate of the mean of the posterior, and a confidence
  interval (specified by \code{conf.int}) around the mean of the
  posterior (using the estimated \code{\link[=quantile]{quantiles}}) of
  the respective MCMC iterates).  All of these values are calculated
  discarding the initial \code{burnin} iterations.

  When \code{d} is a vector of length two, a 2-dimensional trace plot is
  displayed, with the \code{d[1]} dimension on the horizontal axis, and
  the \code{d[2]} dimension on the vertical axis.

  When \code{d=1}, a legend will be placed on the plot; the option
  \code{legendLoc} controls the placing of the legend.  \code{legendLoc}
  may be a vector, specifying a unique legend location for each
  requested trace plots.  If
  \code{legendLoc} is of length 1, it will be \code{\link{rep}}licated
  to have length equal to the number of requested trace plots.
}



\seealso{\code{\link{ideal}}; \code{\link{pmatch}} for matching
  legislators' names.  See \code{\link{legend}} for valid options to
  \code{legendLoc}.} 

\examples{
data(s109)
f <- system.file("extdata","id1.rda",package="pscl")
load(f)
tracex(id1,legis="KENN")

## n.b., no such legislator named Thomas Bayes
tracex(id1,legis=c("KENN","BOX","KYL","Thomas Bayes"))


f <- system.file("extdata","id2.rda",package="pscl")
load(f)

tracex(id2,d=1,legis=c("KENNEDY","BOXER","KYL","Thomas Bayes"))
tracex(id2,d=2,legis=c("KENNEDY","BOXER","KYL","Thomas Bayes"))
tracex(id2,d=1:2,
       legis=c("KENNEDY","BOXER","KYL","Thomas Bayes"))

## partial matching
tracex(id2,d=1:2,
       legis=c("KENN","BOX","BID","SNO","SPEC","MCCA","KYL",
               "Thomas Bayes"),
       multi=TRUE)
}

\keyword{hplot}

\name{RockTheVote}
\alias{RockTheVote}
\docType{data}
\title{Voter turnout experiment, using Rock The Vote ads}
\description{
  Voter turnout data spanning 85 cable TV systems, randomly allocated to
  a voter mobilization experiment targeting 18-19 year olds with "Rock
  the Vote" television advertisements 
}
\usage{data(RockTheVote)}
\format{
  A data frame with 85 observations on the following 6 variables.
  \describe{
    \item{\code{strata}}{numeric, experimental strata}
    \item{\code{treated}}{numeric, 1 if a treated cable system, 0 otherwise}
    \item{\code{r}}{numeric, number of 18 and 19 year olds turning out}
    \item{\code{n}}{numeric, number of 19 and 19 year olds registered}
    \item{\code{p}}{numeric, proportion of 18 and 19 year olds turning out}
    \item{\code{treatedIndex}}{numeric, a counter indexing the 42 treated units}
  }
}

\details{Green and Vavreck (2008) implemented a cluster-randomized
 experimental design in assessing the effects of a voter mobilization
 treatment in the 2004 U.S. Presidential election.  The clusters in this
 design are geographic areas served by a single cable television system.
 So as to facilitate analysis, the researchers restricted their
 attention to small cable systems whose reach is limited to a single zip
 code.  Further, since the experiment was fielded during the last week
 of the presidential election, the researchers restricted their search
 to cable systems that were not in the 16 hotly-contested
 \dQuote{battleground} states (as designated by the \emph{Los Angeles
   Times}).

 Eighty-five cable systems were available for randomization and were
 assigned to treatment after stratification on previous turnout levels
 in presidential elections (as determined from analysis of the
 corresponding states' voter registration files).  Each cable system was
 matched with one or sometimes two other cable systems in the same
 state, yielding 40 strata.  Then within each strata, cable systems were
 randomly assigned to treatment and control conditions.  Strata 3, 8 and
 25 have two control cable systems and 1 treated system each, while
 strata 6 and 20 have two treated cable systems and one control system.
 The remaining 35 strata have 1 treated cable system and 1 control
 system.  In this way there are 38 + 4 = 42 treated systems, spanning 40
 experiment strata.

  The treatment involved researchers purchasing prime-time advertising
  spots on four channels in the respective cable system in which the
  researchers aired voter mobilization ads.  The ads were produced by
  \emph{Rock the Vote}, targeted at younger voters, and aired four times
  per night, per channel, over the last eight days of the election
  campaign.  After the election, public records were consulted to
  assemble data on turnout levels in the treated and control cable
  systems.  In the analysis reported in Green and Vavreck (2008), the
  researchers focused on turnout among registered voters aged 18 and 19
  years old.
}

\references{Green, Donald P. and Lynn Vavreck. 2008. Analysis of
Cluster-Randomized Experiments: A Comparison of Alternative Estimation
Approaches. \emph{Political Analysis} 16:138-152.

Jackman, Simon, 2009. \emph{Bayesian Analysis for the Social Sciences}.  
Wiley: Hoboken, New Jersey.  Example 7.9.
}

\examples{
data(RockTheVote)
## estimate MLEs of treatment effects
deltaFunction <- function(data){
  model <- glm(cbind(r,n-r)~treated,
               data=data,
               family=binomial)
  c(coef(model)[2],
    confint(model)[2,])
}


tmp <- by(RockTheVote,
          as.factor(RockTheVote$strata),
          deltaFunction)

tmp <- matrix(unlist(tmp),ncol=3,byrow=TRUE)

indx <- order(tmp[,1])

plot(y=1:40,
     x=tmp[indx,1],
     pch=16,cex=1.25,
     xlim=range(tmp),
     ylab="",
     axes=FALSE,
     xlab="Estimated Treatment Effect (MLEs, Logit Scale)")
text(y=1:40,
     x=par()$usr[1],
     pos=4,
     as.character((1:40)[indx]),
     cex=.5)
segments(x0=tmp[indx,2],
         x1=tmp[indx,3],
         y0=1:40,
         y1=1:40)
axis(1)
axis(3)
abline(v=0)
}

\keyword{datasets}

\name{unionDensity}
\alias{unionDensity}
\docType{data}
\title{cross national rates of trade union density}
\description{
  Cross-national data on relative size of the trade unions and
  predictors, in 20 countries.  Two of the predictors are highly
  collinear, and are the source of a debate between Stephens and
  Wallerstein (1991), later reviewed by Western and Jackman (1994).
}
\usage{data(unionDensity)
}
\format{
  \describe{
    \item{\code{union}}{numeric, percentage of the total number of wage
      and salary earners plus the unemployed who are union members,
      measured between 1975 and 1980, with most of the data drawn from 1979}
    \item{\code{left}}{numeric, an index tapping the extent to which
      parties of the left have controlled governments since 1919, due to
      Wilensky (1981).}
    \item{\code{size}}{numeric, log of labor force size, defined as the
      number of wage and salary earners, plus the unemployed}
    \item{\code{concen}}{numeric, percentage of employment, shipments, or
      production accounted for by the four largest enterprises in a
      particular industry, averaged over industries (with weights
      proportional to the size of the industry) and the resulting measure
      is normalized such that the United States scores a 1.0, and is due
      to Pryor (1973). Some of the scores on this variable are imputed
      using procedures described in Stephens and Wallerstein (1991, 945).}
  }
}

\source{Pryor, Frederic. 1973. \emph{Property and Industrial
    Organization in Communist and Capitalist Countries}. Bloomington:
    Indiana University Press.  
  
  Stephens, John and Michael Wallerstein. 1991. Industrial
  Concentration, Country Size and Trade Union Membership.
  \emph{American Political Science Review} 85:941-953.
  
  Western, Bruce and Simon Jackman. 1994. Bayesian Inference for
  Comparative Research.  \emph{American Political Science Review}
  88:412-423. 
  
  Wilensky, Harold L. 1981. Leftism, Catholicism, Democratic
  Corporatism: The Role of Political Parties in Recent Welfare State
  Development. In \emph{The Development of Welfare States in Europe
  and America}, ed. Peter Flora and Arnold J. Heidenheimer. New
  Brunswick: Transaction Books. 
}

\references{Jackman, Simon. 2009.  \emph{Bayesian Analysis for the Social Sciences}. Wiley: Hoboken, New Jersey.}
  
\examples{
data(unionDensity)
summary(unionDensity)
pairs(unionDensity,
      labels=c("Union\nDensity",
        "Left\nGovernment",
        "log Size of\nLabor Force",
        "Economic\nConcentration"),
      lower.panel=function(x,y,digits=2){
        r <- cor(x,y)
        par(usr=c(0,1,0,1))
        text(.5,.5,
             format(c(r,0.123456789),digits=digits)[1],
             cex=1.5)
      }
      )
ols <- lm(union ~ left + size + concen,
          data=unionDensity)
summary(ols)
}
\keyword{datasets}

\name{iraqVote}
\alias{iraqVote}
\docType{data}
\title{
	U.S. Senate vote on the use of force against Iraq, 2002.	
}
\description{
	On October 11, 2002, the United States Senate
  voted 77-23 to authorize the use of military force against Iraq.  This data set lists the \dQuote{Ayes} and \dQuote{Nays} for each Senator and some covariates.
}
\usage{data(iraqVote)}
\format{
  A data frame with 100 observations on the following 6 variables.
  \describe{
    \item{\code{y}}{a numeric vector, the recorded vote (1 if Aye, 0 if Nay)}
    \item{\code{state.abb}}{two letter abbreviation for each state}
    \item{\code{name}}{senator name, party and state, e.g., \code{AKAKA (D HI)}}
    \item{\code{rep}}{logical, \code{TRUE} for Republican senators}
    \item{\code{state.name}}{name of state}
    \item{\code{gorevote}}{numeric, the vote share recorded by Al Gore in the corresponding state in the 2000 Presidential election}
  }
}
\details{The only Republican to vote against the resolution was Lincoln
  Chafee (Rhode Island); Democrats split 29-22 in favor of the
  resolution.
}
\source{
	Keith Poole, 107th Senate Roll Call Data. \url{https://voteview.com/static/data/out/votes/S107_votes.ord}  The Iraq vote is vote number 617.

	David Leip's Atlas of U.S. Presidential Elections.  \url{http://uselectionatlas.org} 
}
\references{
	Jackman, Simon. 2009.  \emph{Bayesian Analysis for the Social Sciences}.  Wiley: Chichester.  Example 8.3.
}
\examples{
data(iraqVote)
## probit model
glm1 <- glm(y ~ gorevote + rep,
            data=iraqVote,
            family=binomial(link=probit))
}
\keyword{datasets}

\name{convertCodes}
\alias{convertCodes}
\title{convert entries in a rollcall matrix to binary form}

\description{
  Convert roll call matrix to binary form using encoding information.
}

\usage{
convertCodes(object, codes = object$codes)
}

\arguments{
  \item{object}{\code{\link{rollcall}} object}
  \item{codes}{list, mapping entries in the \code{votes} component of
    \code{rollcall} object to 0 (\sQuote{Nay}), 1 (\sQuote{Yea}) and
    \code{NA} (missing, abstentions, etc). Defaults to the \code{codes}
    component of the \code{rollcall} object.} 
}

\details{
  See \code{\link{rollcall}} for details on the form of the
  \code{codes} list.
}
\value{a \code{\link{matrix}} with dimensions equal to the dimensions of
  the \code{votes} component of the \code{rollcall} object.} 
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\note{Any entries in the \code{votes} matrix that can
  not be mapped into \code{c(0,1,NA)} using the information in
  \code{codes} are mapped to \code{NA}, with an informative message
  sent to the console.
}
\seealso{\code{\link{rollcall}}}
\examples{
  data(s109)
  mat <- convertCodes(s109)
  table(mat,exclude=NULL)
}  
\keyword{manip}

\name{ca2006}
\alias{ca2006}
\docType{data}
\title{California Congressional Districts in 2006}
\description{
  Election returns and identifying information, California's 53
  congressional districts in the 2006 Congressional elections.
}
\usage{data(ca2006)}
\format{
  A data frame with 53 observations on the following 11 variables.
  \describe{
    \item{\code{district}}{numeric, number of Congressional district}
    \item{\code{D}}{numeric, number of votes for the Democratic candidate}
    \item{\code{R}}{numeric, votes for the Republican candidate}
    \item{\code{Other}}{numeric, votes for other candidates}
    \item{\code{IncParty}}{character, party of the incumbent (or retiring member),
      \code{D} or \code{R}}
    \item{\code{IncName}}{character, last name of the incumbent, character
      \code{NA} if no incumbent running}
    \item{\code{open}}{logical, \code{TRUE} if no incumbent running}
    \item{\code{contested}}{logical, \code{TRUE} if both major parties
      ran candidates}
    \item{\code{Bush2004}}{numeric, votes for George W. Bush (R) in the district in
      the 2004 presidential election}
    \item{\code{Kerry2004}}{numeric, votes for John Kerry (D) in 2004}
    \item{\code{Other2004}}{numeric votes for other candidates in 2004}
    \item{\code{Bush2000}}{numeric, votes for George W. Bush in 2000}
    \item{\code{Gore2000}}{numeric, votes for Al Gore (D) in 2000}
  }
}

\source{2006 data from the California Secretary of State's web site,
  \url{https://www.sos.ca.gov/elections/prior-elections/statewide-election-results/general-election-november-7-2006/statement-vote}. 
  %%Excel data at \url{http://elections.cdn.sos.ca.gov/sov/2006-general/congress.xls}. 
   2004 and 2000 presidential vote in congressional districts from the 2006 \emph{Almanac of American Politics}.

Thanks to Arthur Aguirre for the updated links, above.
}

\references{
  Michael Baraon and Richard E. Cohen.  2006. \emph{The Almanac of American
  Politics, 2006.} National Journal Group: Washington, D.C.
}
\examples{
data(ca2006)

## 2006 CA congressional vote against 2004 pvote
y <- ca2006$D/(ca2006$D+ca2006$R)
x <- ca2006$Kerry2004/(ca2006$Kerry2004+ca2006$Bush2004)

pch <- rep(19,length(y))
pch[ca2006$open] <- 1
col <- rep("black",length(y))
col[11] <- "red"    ## Pembo (R) loses to McNerney (D)
plot(y~x,pch=pch,
     col=col,
     xlim=range(x,y,na.rm=TRUE),
     ylim=range(x,y,na.rm=TRUE),
     xlab="Kerry Two-Party Vote, 2004",
     ylab="Democratic Two-Party Vote Share, 2006")
abline(0,1)
abline(h=.5,lty=2)
abline(v=.5,lty=2)
legend(x="topleft",
       bty="n",
       col=c("red","black","black"),
       pch=c(19,19,1),
       legend=c("Seat Changing Hands",
         "Seat Retained by Incumbent Party",
         "Open Seat (no incumbent running)")
       )
}
\keyword{datasets}

\name{readKH}
\alias{readKH}
\title{read roll call data in Poole-Rosenthal KH format}
\description{
  Creates a \code{rollcall} object from the flat file format for roll
  call data used by
  Keith Poole and Howard Rosenthal.
}

\usage{
readKH(file,
       dtl=NULL,
       yea=c(1,2,3),
       nay=c(4,5,6),
       missing=c(7,8,9),
       notInLegis=0,
       desc=NULL,
       debug=FALSE)
}

\arguments{
  \item{file}{string, name of a file or URL holding KH data}
  \item{dtl}{string, name of a file or URL holding KH \code{dtl} file
    (information about votes); default is \code{NULL}, indicating no
    \code{dtl} file} 
  \item{yea}{numeric, possibly a vector, code(s) for a Yea vote in the
    rollcall context (or a correct answer in the educational testing
    context). Default is \code{c(1,2,3)}, which corresponds to Yea,
    Paired Yea, and Announced Yea in Poole/Rosenthal data files.}
  \item{nay}{numeric, possibly a vector, code(s) for a Nay vote in the
    rollcall context (or an incorrect answer in the educational testing
    context).  Default is \code{c(4,5,6)}, which corresponds to
    Announced Nay, Paired Nay, and Nay in Poole/Rosenthal data files.}
  \item{missing}{numeric and/or \code{NA}, possible a vector, code(s)
    for missing data.  Default is \code{c(0,7,8,9,NA)}; the first four
    codes correspond to Not Yet a Member, Present (some Congresses),
    Present (some Congresses), and Not Voting.}
  \item{notInLegis}{numeric or \code{NA}, possibly a vector, code(s) for
    the legislator not being in the legislature when a particular roll
    call was recorded (e.g., deceased, retired, yet to be
    elected). Default is \code{0} for Poole/Rosenthal data files.}
  \item{desc}{string, describing the data, e.g., \code{82nd U.S. House
      of Representatives}; default is \code{NULL}}
  \item{debug}{logical, print debugging information for net connection}
}

\value{an object of class \code{\link{rollcall}}, with components created
  using the identifying information in the Poole/Rosenthal files.  If
  the function can not read the file (e.g., the user specified a URL and
  the machine is not connected to the Internet), the function fails with
  an error message (set \code{debug=TRUE} to help resolve these issues).
}

\details{Keith Poole and Howard Rosenthal have gathered an impressive
  collection of roll call data, spanning every roll call cast in
  the United States Congress.  This effort continues now as a real-time
  exercise, via a collaboration with Jeff Lewis (109th Congress
  onwards).  Nolan McCarty collaborated on the compilation of roll call
  data for the 102nd through 108th Congress.

  This function relies on some hard-coded features of Poole-Rosenthal
  flat files, and assumes that the \code{file} being supplied has the following
  structure (variable, start-end columns):
  \describe{
    \item{ICPSR legislator unique ID}{4-8}
    \item{ICPSR state ID}{9-10}
    \item{Congressional District}{11-12}
    \item{state name}{13-20}
    \item{party code}{21-23}
    \item{legislator name}{26-36}
    \item{roll-call voting record}{37 to end-of-record}
    }
  This function reads data files in that format, and creates a
  \code{\link{rollcall}}, for which there are useful methods such as
  \code{\link{summary.rollcall}}.  The \code{legis.data} component of the
  \code{\link{rollcall}} object is a \code{\link{data.frame}} which
  contains:
  \describe{
    \item{\code{state}}{a 2-character string abbreviation of each
      legislator' state}
    \item{\code{icpsrState}}{a 2-digit numeric code for each
      legislator's state, as used by the Inter-university Consortium for
      Political and Social Research (ICPSR)}
    \item{\code{cd}}{numeric, the number of each legislator's
      congressional district within each state; this is always 0 for
      members of the Senate}    
    \item{\code{icpsrLegis}}{a unique numeric identifier for each
      legislator assigned by the ICPSR, as corrected by Poole and
      Rosenthal.}
    \item{\code{partyName}}{character string, the name of each legislator's
      political party}
    \item{\code{party}}{numeric, code for each legislator's political
      party; see \url{http://legacy.voteview.com/PARTY3.HTM}}
  }
  The \code{\link{rownames}} attribute of this data frame is a
  concatenation of the legislators' names, party abbreviations (for
  Democrats and Republicans) and state, and (where appropriate), a
  district number; e.g., \code{Bonner (R AL-1)}.  This tag is also
  provided in the \code{legis.name} component of the returned rollcall
  object.

  Poole and Rosenthal also make \code{dtl} files available for
  Congresses 1 through 106. These files contain information about the
  votes themselves, in a multiple-line per vote \code{ascii} format, and
  reside in the \code{dtl} director of Poole's web site, e.g.,
  \url{https://legacy.voteview.com/k7ftp/dtl/102s.dtl} is the \code{dtl}
  file for the 102nd Senate.  The default is to presume that no such
  file exists.  When a \code{dtl} file is available, and is read, the
  \code{votes.data} attribute of the resulting \code{\link{rollcall}}
  object is a \code{\link{data.frame}} with one record per vote, with
  the following variables:
  \describe{
    \item{\code{date}}{vector of class \code{\link{Date}}, date of the
    rollcall, if available; otherwise \code{NULL}}
    \item{\code{description}}{vector of mode \code{character}, descriptive
      text}
  }
  The \code{dtl} files are presumed to have the date of the rollcall in
  the first line of text for each roll call, and lines 3 onwards contain
  descriptive text.
  
  Finally, note also that the Poole/Rosenthal data sets often include
  the U.S. President as a pseudo-legislator, adding the announced
  positions of a president or the administration to the roll call
  matrix. This adds an extra \dQuote{legislator} to the data set and can
  sometimes produce surprising results (e.g., a U.S. Senate of 101
  senators), and a \dQuote{legislator} with a surprisingly low party loyalty
  score (since the President/administration only announces positions on
  a relatively small fraction of all Congressional roll calls).   
}

\references{Poole, Keith and Howard Rosenthal.  1997.  \emph{Congress: A
  Political-Economic History of Roll Call Voting}.  New York: Oxford
  University Press.  

Poole, Keith.  \url{http://legacy.voteview.com}

Rosenthal, Howard L. and Keith T. Poole.  \emph{United States Congressional
Roll Call Voting Records, 1789-1990: Reformatted Data [computer
file].} 2nd ICPSR release. Pittsburgh, PA: Howard L. Rosenthal and Keith
T. Poole, Carnegie Mellon University, Graduate School of Industrial
Administration [producers], 1991. Ann Arbor, MI: Inter-university
Consortium for Political and Social Research [distributor], 2000.  
\url{http://www.icpsr.umich.edu/icpsrweb/ICPSR/studies/09822}
}

\seealso{\code{\link{rollcall}}
}

\examples{
\dontrun{
h107 <- readKH("https://voteview.com/static/data/out/votes/H107_votes.ord",
                desc="107th U.S. House of Representatives")

s107 <- readKH("https://voteview.com/static/data/out/votes/S107_votes.ord",
                desc="107th U.S. Senate")
}
}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\keyword{datasets}

\name{seatsVotes}
\alias{seatsVotes}
%\alias{print.seatsVotes}
%\alias{summary.seatsVotes}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{A class for creating seats-votes curves}
\description{
  Convert a vector of vote shares into a seats-vote curve object,
  providing estimates of partisan bias.
}
\usage{
seatsVotes(x, desc = NULL, method = "uniformSwing")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{a vector of vote shares for a specific party (either
    proportions or percentages)}
  \item{desc}{descriptive text}
  \item{method}{how to simulate a seats-vote curve; the only supported
    method at this stage is \code{uniformSwing}.}
}

\details{Simulation methods are required to induce a seats-votes curve
  given a vector of vote shares from one election.  The uniform swing
  method simply slides the empirical distribution function of the vote
  shares \dQuote{up} and \dQuote{down}, computing the proportion of
  the vote shares that lie above .5 (by construction, the winning
  percentage in a two-party election) for each new location of the
  vector of vote shares.  That is, as the empirical CDF of the
  observed vote shares slides up or down, more or less seats cross the
  .5 threshold.  A seats-votes curve is formed by plotting the seat
  share above .5 as a function of the average district-level vote
  share (a weakly monotone function, since the empirical CDF
  constitutes a set of sufficient statistics for this problem).  The
  simulation is run so as to ensure that average district-level vote
  shares range between 0 and 1.

  The extent to which the seats-votes curve departs from symmetry is
  known as bias.  More specifically, the vertical displacement of the
  seats-votes curve from .5 when average district-level vote share is .5
  is conventionally reported as an estimate of the bias of the electoral
  system.  

  Different methods produce different estimates of seats-votes curves
  and summary estimands such as bias.  The
  uniform swing method is completely deterministic and does not
  produce any uncertainty assessment (e.g., confidence intervals etc).
}

\value{
  An object of class \code{seatsVotes}, with components
  \item{s}{Estimated seat shares over the range of simulated average,
    district-level vote shares}
  \item{v}{Simulated average district-level vote shares}
  \item{x}{observed seat shares, with missing data removed}
  \item{desc}{user-supplied descriptive character string}
  \item{call}{a list of class \code{\link{call}}, the call to the function}
}

\references{
   Tufte, Edward R. 1973. The Relationship Between Seats and Votes in
   Two-Party Systems. \emph{American Political Science
     Review}. 67(2):540-554.

   Gelman, Andrew and Gary King.  1990.  Estimating the Consequences of
   Electoral Redistricting.  \emph{Journal of the American Statistical
     Association}. 85:274-282.

   Jackman, Simon. 1994.  Measuring Electoral Bias: Australia,
   1949-93. \emph{British Journal of Political Science}. 24(3):319-357.
}

\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\note{Additional methods to come later.}

\seealso{\code{\link{plot.seatsVotes}} for plotting methods.}

\examples{
data(ca2006)
x <- ca2006$D/(ca2006$D+ca2006$R)
sv <- seatsVotes(x,
                 desc="Democratic Vote Shares, California 2006 congressional elections")
}

\keyword{misc}

\name{odTest}
\alias{odTest}

\title{likelihood ratio test for over-dispersion in count data}

\description{
  Compares the log-likelihoods of a negative binomial regression model
  and a Poisson regression model.
}

\usage{
odTest(glmobj, alpha=.05, digits = max(3, getOption("digits") - 3))
}

\arguments{
   \item{glmobj}{an object of class \code{negbin} produced by
     \code{\link[MASS:glm.nb]{glm.nb}}}  
   \item{alpha}{significance level of over-dispersion test}
   \item{digits}{ number of digits in printed output}
}

\details{The negative binomial model relaxes the assumption in the
  Poisson model that the (conditional) variance equals the (conditional)
  mean, by estimating one extra parameter.  A likelihood ratio (LR) test
  can be used to test the null hypothesis that the restriction implicit
  in the Poisson model is true. The LR test-statistic has a non-standard
  distribution, even asymptotically, since the negative binomial
  over-dispersion parameter (called \code{theta} in \code{glm.nb}) is
  restricted to be positive.  The asymptotic distribution of the LR
  (likelihood ratio) test-statistic has probability mass of one half at
  zero, and a half \eqn{\chi^2_1}{chi-square (1)} distribution above
  zero.  This means that if testing at the \eqn{\alpha}{alpha} = .05
  level, one should not reject the null unless the LR test statistic
  exceeds the critical value associated with the \eqn{2\alpha}{2 alpha}
  = .10 level; this LR test involves just one parameter restriction, so
  the critical value of the test statistic at the \eqn{p}{p} = .05 level
  is 2.7, instead of the usual 3.8 (i.e., the .90 quantile of the
  \eqn{\chi^2_1}{chi-square (1)} distribution, versus the .95 quantile).

  A Poisson model is run using \code{\link{glm}} with family set to
  \code{link{poisson}}, using the \code{\link{formula}} in the negbin
  model object passed as input.  The \code{\link{logLik}} functions are
  used to extract the log-likelihood for each model.
}

\value{None; prints results and returns silently}

\references{A. Colin Cameron and Pravin K. Trivedi (1998) \emph{Regression
    analysis of count data}. New York: Cambridge University Press.

  Lawless, J. F. (1987) Negative Binomial and Mixed Poisson
  Regressions. \emph{The Canadian Journal of Statistics}. 15:209-225.
 }

  \author{Simon Jackman \email{simon.jackman@sydney.edu.au}.  John Fox noted an
  error in an earlier version.}

\seealso{\code{\link[MASS:glm.nb]{glm.nb}}, \code{\link{logLik}}}

\examples{
data(bioChemists)
modelnb <- MASS::glm.nb(art ~ .,
                 data=bioChemists,
                 trace=TRUE)
odTest(modelnb)
}
\keyword{regression}% at least one, from doc/KEYWORDS

\name{extractRollCallObject}
\alias{extractRollCallObject}
\title{return the roll call object used in fitting an ideal model}
\description{
  Given a fitted model of class \code{\link{ideal}}, return the
  \code{\link{rollcall}} object that was used in the model fitting
  (i.e., apply all subsetting and recoding implied by the \code{dropList}
  passed to \code{\link{ideal}}).
}
\usage{
extractRollCallObject(object)
}
\arguments{
  \item{object}{an object of class \code{\link{ideal}}}
}
\details{
  This function is used by many post-estimation commands that operate on
  objects of class \code{\link{ideal}}.  The function inspects the
  \code{call} attribute of the 
  \code{\link{ideal}} object, extracting the name of the
  \code{\link{rollcall}} object and the \code{dropList}, then hands them
  over to \code{\link{dropRollCall}}.
}
\value{
  An object of class \code{\link{rollcall}}
}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\seealso{\code{\link{rollcall}}; see \code{\link{dropRollCall}} for
  details on the form of a \code{dropList}.}
\examples{
data(s109)
f = system.file("extdata","id1.rda",package="pscl")
load(f)
tmp <- extractRollCallObject(id1)
summary(tmp)
v <- convertCodes(tmp)         ## roll call matrix per se  
}
\keyword{models}

\name{vuong}
\alias{vuong}
\title{Vuong's non-nested hypothesis test}
\description{
  Compares two models fit to the same data that do not nest via Vuong's
  non-nested test.
}

\usage{
vuong(m1, m2, digits = getOption("digits"))
}

\arguments{
  \item{m1}{model 1, an object inheriting from class \code{glm},
    \code{negbin} or \code{zeroinfl}}
  \item{m2}{model 2, as for model 1}
  \item{digits}{significant digits in printed result}
}
\details{
  The Vuong non-nested test is based on a comparison of the predicted
  probabilities of two models that do not nest.  Examples include
  comparisons of zero-inflated count models with their non-zero-inflated
  analogs (e.g., zero-inflated Poisson versus ordinary Poisson, or
  zero-inflated negative-binomial versus ordinary negative-binomial).  A
  large, positive test statistic provides evidence of
  the superiority of model 1 over model 2, while a large, negative test statistic is
  evidence of the superiority of model 2 over model 1.  Under the null
  that the models are indistinguishable, the test
  statistic is asymptotically distributed standard normal.
  
  Let \eqn{p_i = \hat{Pr}(y_i | M_1)} be the predicted probabilities from model 1, evaluated conditional on the estimated MLEs.   Let \eqn{q_i} be the corresponding probabilities from model 2.   Then the Vuong statistic is \eqn{\sqrt{N} \bar{m}/s_m} where \eqn{m_i = log(p_i) - log(q_i)} and \eqn{s_m} is the sample standard deviation of \eqn{m_i}.
  
  Two finite sample corrections are often considered, based on the Akaike (AIC) and Schwarz (BIC) penalty terms, based on the complexity of the two models. These corrections sometimes generate conflicting conclusions. 

  The function will fail if the models do not contain identical values
  in their respective components named \code{y} (the value of the
  response being modeled).
  
  
}
\value{
  nothing returned, prints 3 test statistics and \eqn{p} values and
  exits silently.
}
\references{Vuong, Q.H. 1989. Likelihood ratio tests for model
  selection and non-nested hypotheses. \emph{Econometrica}. 57:307-333.}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}

\examples{
\dontrun{
data("bioChemists")
## compare Poisson GLM and ZIP
glm1 <- glm(art ~ ., data = bioChemists, family = poisson)
zip <- zeroinfl(art ~ . | ., data = bioChemists, EM = TRUE)
vuong(glm1, zip)


## compare negbin with zero-inflated negbin
nb1 <- MASS::glm.nb(art ~ ., data=bioChemists)
zinb <- zeroinfl(art ~ . | ., data = bioChemists, dist = "negbin", EM = TRUE)
vuong(nb1, zinb)
}
}
\keyword{models}

\name{hurdle}
\alias{hurdle}
\alias{print.hurdle}

\title{Hurdle Models for Count Data Regression}

\description{
  Fit hurdle regression models for count data via maximum likelihood. 
}

\usage{
hurdle(formula, data, subset, na.action, weights, offset,
  dist = c("poisson", "negbin", "geometric"),
  zero.dist = c("binomial", "poisson", "negbin", "geometric"),
  link = c("logit", "probit", "cloglog", "cauchit", "log"),
  control = hurdle.control(\dots),
  model = TRUE, y = TRUE, x = FALSE, \dots)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data, subset, na.action}{arguments controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights.}
  \item{offset}{optional numeric vector with an a priori known component to be
    included in the linear predictor of the count model. See below for more
    information on offsets.}
  \item{dist}{character specification of count model family.}
  \item{zero.dist}{character specification of the zero hurdle model family.}
  \item{link}{character specification of link function in the binomial
    zero hurdle (only used if \code{zero.dist = "binomial"}.}
  \item{control}{a list of control arguments specified via
    \code{\link{hurdle.control}}.}
  \item{model, y, x}{logicals. If \code{TRUE} the corresponding components
    of the fit (model frame, response, model matrix) are returned.}
  \item{\dots}{arguments passed to \code{\link{hurdle.control}} in the
    default setup.}
}

\details{
  Hurdle count models are two-component models with a truncated count
  component for positive counts and a hurdle component that models the
  zero counts. Thus, unlike zero-inflation models, there are \emph{not} two
  sources of zeros: the count model is only employed if the hurdle for
  modeling the occurrence of zeros is exceeded. The count model is typically
  a truncated Poisson or negative binomial regression (with log link).
  The geometric distribution is a special case of the negative binomial with
  size parameter equal to 1. For modeling the hurdle, either a binomial model
  can be employed or a censored count distribution. The outcome of the hurdle
  component of the model is the occurrence of a non-zero (positive) count.
  Thus, for most models, positive coefficients in the hurdle component indicate
  that an increase in the regressor increases the probability of a non-zero count.
  Binomial logit and censored geometric models as the hurdle part both lead to 
  the same likelihood function and thus to the same coefficient estimates.
  A censored negative binomial model for the zero hurdle is only identified
  if there is at least one non-constant regressor with (true) coefficient different
  from zero (and if all coefficients are close to zero the model can be poorly
  conditioned).
  
  The \code{formula} can be used to specify both components of the model:
  If a \code{formula} of type \code{y ~ x1 + x2} is supplied, then the same
  regressors are employed in both components. This is equivalent to
  \code{y ~ x1 + x2 | x1 + x2}. Of course, a different set of regressors
  could be specified for the zero hurdle component, e.g.,
  \code{y ~ x1 + x2 | z1 + z2 + z3} giving the count data model \code{y ~ x1 + x2}
  conditional on (\code{|}) the zero hurdle model \code{y ~ z1 + z2 + z3}.
  
  Offsets can be specified in both parts of the model pertaining to count and
  zero hurdle model: \code{y ~ x1 + offset(x2) | z1 + z2 + offset(z3)}, where
  \code{x2} is used as an offset (i.e., with coefficient fixed to 1) in the
  count part and \code{z3} analogously in the zero hurdle part. By the rule
  stated above \code{y ~ x1 + offset(x2)} is expanded to
  \code{y ~ x1 + offset(x2) | x1 + offset(x2)}. Instead of using the
  \code{offset()} wrapper within the \code{formula}, the \code{offset} argument
  can also be employed which sets an offset only for the count model. Thus,
  \code{formula = y ~ x1} and \code{offset = x2} is equivalent to
  \code{formula = y ~ x1 + offset(x2) | x1}.
  
  All parameters are estimated by maximum likelihood using \code{\link[stats]{optim}},
  with control options set in \code{\link{hurdle.control}}.
  Starting values can be supplied, otherwise they are estimated by \code{\link[stats]{glm.fit}}
  (the default). By default, the two components of the model are estimated separately
  using two \code{optim} calls. Standard errors are derived numerically using
  the Hessian matrix returned by \code{\link[stats]{optim}}. See
  \code{\link{hurdle.control}} for details.
  
  The returned fitted model object is of class \code{"hurdle"} and is similar
  to fitted \code{"glm"} objects. For elements such as \code{"coefficients"} or
  \code{"terms"} a list is returned with elements for the zero and count components,
  respectively. For details see below.
  
  A set of standard extractor functions for fitted model objects is available for
  objects of class \code{"hurdle"}, including methods to the generic functions
  \code{\link[base]{print}}, \code{\link[base]{summary}}, \code{\link[stats]{coef}}, 
  \code{\link[stats]{vcov}}, \code{\link[stats]{logLik}}, \code{\link[stats]{residuals}}, 
  \code{\link[stats]{predict}}, \code{\link[stats]{fitted}}, \code{\link[stats]{terms}},
  \code{\link[stats]{model.matrix}}. See \code{\link{predict.hurdle}} for more details
  on all methods.
}

\value{
  An object of class \code{"hurdle"}, i.e., a list with components including
  \item{coefficients}{a list with elements \code{"count"} and \code{"zero"}
    containing the coefficients from the respective models,}
  \item{residuals}{a vector of raw residuals (observed - fitted),}
  \item{fitted.values}{a vector of fitted means,}
  \item{optim}{a list (of lists) with the output(s) from the \code{optim} call(s) for
    minimizing the negative log-likelihood(s),}
  \item{control}{the control arguments passed to the \code{optim} call,}
  \item{start}{the starting values for the parameters passed to the \code{optim} call(s),}
  \item{weights}{the case weights used,}
  \item{offset}{a list with elements \code{"count"} and \code{"zero"}
    containing the offset vectors (if any) from the respective models,}
  \item{n}{number of observations (with weights > 0),}
  \item{df.null}{residual degrees of freedom for the null model (= \code{n - 2}),}
  \item{df.residual}{residual degrees of freedom for fitted model,}
  \item{terms}{a list with elements \code{"count"}, \code{"zero"} and
    \code{"full"} containing the terms objects for the respective models,}
  \item{theta}{estimate of the additional \eqn{\theta}{theta} parameter of the
    negative binomial model(s) (if negative binomial component is used),}
  \item{SE.logtheta}{standard error(s) for \eqn{\log(\theta)}{log(theta)},}
  \item{loglik}{log-likelihood of the fitted model,}
  \item{vcov}{covariance matrix of all coefficients in the model (derived from the
    Hessian of the \code{optim} output(s)),}
  \item{dist}{a list with elements \code{"count"} and \code{"zero"} with character
    strings describing the respective distributions used,}
  \item{link}{character string describing the link if a binomial zero hurdle model
    is used,}
  \item{linkinv}{the inverse link function corresponding to \code{link},}
  \item{converged}{logical indicating successful convergence of \code{optim},}
  \item{call}{the original function call,}
  \item{formula}{the original formula,}
  \item{levels}{levels of the categorical regressors,}
  \item{contrasts}{a list with elements \code{"count"} and \code{"zero"}
    containing the contrasts corresponding to \code{levels} from the
    respective models,}
  \item{model}{the full model frame (if \code{model = TRUE}),}
  \item{y}{the response count vector (if \code{y = TRUE}),}
  \item{x}{a list with elements \code{"count"} and \code{"zero"}
    containing the model matrices from the respective models
    (if \code{x = TRUE}).}
}

\references{
Cameron, A. Colin and Pravin K. Trivedi. 1998. \emph{Regression Analysis of Count 
Data}. New York: Cambridge University Press.

Cameron, A. Colin and Pravin K. Trivedi 2005. \emph{Microeconometrics: Methods and Applications}.
Cambridge: Cambridge University Press.

Mullahy, J. 1986. Specification and Testing of Some Modified Count Data Models.
  \emph{Journal of Econometrics}. \bold{33}:341--365.

Zeileis, Achim, Christian Kleiber and Simon Jackman 2008.
\dQuote{Regression Models for Count Data in R.} 
\emph{Journal of Statistical Software}, \bold{27}(8).
URL \url{http://www.jstatsoft.org/v27/i08/}.
}

\author{Achim Zeileis <Achim.Zeileis@R-project.org>}

\seealso{\code{\link{hurdle.control}}, \code{\link[stats]{glm}},
  \code{\link[stats]{glm.fit}}, \code{\link[MASS]{glm.nb}},
  \code{\link{zeroinfl}}
}

\examples{
## data
data("bioChemists", package = "pscl")

## logit-poisson
## "art ~ ." is the same as "art ~ . | .", i.e.
## "art ~ fem + mar + kid5 + phd + ment | fem + mar + kid5 + phd + ment"
fm_hp1 <- hurdle(art ~ ., data = bioChemists)
summary(fm_hp1)

## geometric-poisson
fm_hp2 <- hurdle(art ~ ., data = bioChemists, zero = "geometric")
summary(fm_hp2)

## logit and geometric model are equivalent
coef(fm_hp1, model = "zero") - coef(fm_hp2, model = "zero")

## logit-negbin
fm_hnb1 <- hurdle(art ~ ., data = bioChemists, dist = "negbin")
summary(fm_hnb1)

## negbin-negbin
## (poorly conditioned zero hurdle, note the standard errors)
fm_hnb2 <- hurdle(art ~ ., data = bioChemists, dist = "negbin", zero = "negbin")
summary(fm_hnb2)
}

\keyword{regression}

\name{partycodes}
\alias{partycodes}
\docType{data}
\title{political parties appearing in the U.S. Congress}

\description{
  Numeric codes and names of 85 political parties appearing in Poole
  and Rosenthal's collection of U.S. Congressional roll calls.
}

\usage{data(partycodes)}

\format{
  \describe{
    \item{\code{code}}{integer, numeric code for legislator appearing in Poole and
      Rosenthal rollcall data files}
    \item{\code{party}}{character, name of party}
    }
  }

\details{The function \code{\link{readKH}} converts the integer codes
  into strings, via a table lookup in this data frame.}
\seealso{\code{\link{readKH}}
}
\source{Keith Poole's website: \url{https://legacy.voteview.com/PARTY3.HTM}}
\keyword{datasets}

\name{EfronMorris}
\alias{EfronMorris}
\docType{data}
\title{Batting Averages for 18 major league baseball players, 1970}

\description{Batting averages for 18 major league baseball players,
  first 45 at bats of the 1970 season.}

\usage{data(EfronMorris)}

\format{
  \describe{
	\item{\code{name}}{character, name of player}
	\item{\code{team}}{character, team of player, abbreviated}
	\item{\code{league}}{character, National League or American League}
  	\item{\code{r}}{numeric, hits in 1st 45 at bats}
	\item{\code{y}}{numeric, \code{r}/45, batting average over 1st
  45 at bats}
  \item{\code{n}}{numeric, number of at bats, remainder of 1970
  season}
  \item{\code{p}}{numeric, batting average over remainder of 1970 season}
  }
}

\source{Efron, Bradley and Carl Morris. 1975. Data Analysis Using
Stein's Estimator and Its Generalizations.  \emph{Journal of the American
Statistical Association}. 70:311-319.
}
  
\examples{
data(EfronMorris)
attach(EfronMorris)
plot(p~y,
     xlim=range(p,y),
     ylim=range(p,y),
     xlab="Batting Average, 1st 45 at bats",
     ylab="Batting Average, Remainder of Season")
abline(0,1)
}
\keyword{datasets}

\name{betaHPD}
\alias{betaHPD}

\title{compute and optionally plot beta HDRs}

\description{Compute and optionally plot highest density regions for the Beta
  distribution.}

\usage{
   betaHPD(alpha,beta,p=.95,plot=FALSE,xlim=NULL,debug=FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{alpha}{scalar, first shape parameter of the Beta density.  Must be
    greater than 1, see details}
  \item{beta}{scalar, second shape parameter of the Beta density.  Must be
    greater than 1, see details}
  \item{p}{scalar, content of HPD, must lie between 0 and 1}
  \item{plot}{logical flag, if \code{TRUE} then plot the density and
    show the HDR}
  \item{xlim}{numeric vector of length 2, the limits of the density's
   support to show when plotting; the default is \code{NULL}, in which
   case the function will confine plotting to where the density is
   non-negligible} 
  \item{debug}{logical flag, if \code{TRUE} produce messages to the
   console}
}
\details{The Beta density arises frequently in Bayesian models of
  binary events, rates, and proportions, which take on values in the
  open unit interval.  For instance, the Beta density is a conjugate prior
  for the unknown success probability in binomial trials.  With shape
  parameters \eqn{\alpha > 1} and \eqn{\beta > 1}, the Beta density is
  unimodal.

  In general, suppose \eqn{\theta \in \Theta \subseteq R^k}
  is a random variable with density \eqn{f(\theta)}.  A highest
  density region (HDR) of \eqn{f(\theta)} with content \eqn{p \in
  (0,1]} is a set \eqn{\mathcal{Q} \subseteq \Theta} with the
  following properties:
  \deqn{\int_\mathcal{Q} f(\theta) d\theta = p}
  and
  \deqn{f(\theta) > f(\theta^*) \, \forall\
    \theta \in \mathcal{Q},
    \theta^* \not\in \mathcal{Q}.}
  For a unimodal
  Beta density (the class of Beta densities handled by this function),
  a HDR of content \eqn{0 < p < 1} is simply an interval \eqn{\mathcal{Q} \in (0,1)}.
  
  This function uses numerical methods to solve for the
  end points of a HDR for a Beta density with user-specified shape
  parameters, via repeated calls to the functions \code{\link{dbeta}},
  \code{\link{pbeta}} and \code{\link{qbeta}}. The function
  \code{\link{optimize}} is used to find points \eqn{v} and \eqn{w}
  such that \deqn{f(v) = f(w)} subject to the constraint
  \deqn{\int_v^w f(\theta; \alpha, \beta) d\theta = p,}
  where \eqn{f(\theta; \alpha, \beta)} is a Beta density with shape
  parameters \eqn{\alpha} and \eqn{\beta}. 

  In the special case of \eqn{\alpha = \beta > 1}, the end points
  of a HDR  with content \eqn{p} are given by the \eqn{(1 \pm p)/2}
  quantiles of the Beta density, and are computed with the
  \code{\link{qbeta}} function. 

  Again note that the function will only compute a HDR for a unimodal
  Beta density, and exit with an error if \code{alpha<=1 | beta <=1}.
  Note that the uniform density results with \eqn{\alpha = \beta = 1},
  which does not have a unique HDR with content \eqn{0 < p <
  1}.  With shape parameters \eqn{\alpha<1} and \eqn{\beta>1} (or
  vice-versa, respectively), the Beta density is infinite at 0 (or 1,
  respectively), but still integrates to one, and so a HDR is still
  well-defined (but not implemented here, at least not yet).
  Similarly, with \eqn{0 < \alpha, \beta < 1} the Beta density is
  infinite at both 0 and 1, but integrates to one, and again a HDR of
  content \eqn{p<1} is well-defined in this case, but will be a set of
  two disjoint intervals (again, at present, this function does not
  cover this case).
}

\value{
  If the numerical optimization is successful an vector of length 2,
  containing \eqn{v} and \eqn{w}, defined above.    If the optimization
  fails for whatever reason, a vector of \code{NAs} is returned.
  
  The function will also produce a plot of the density with area under
  the density supported by the HDR shaded, if the user calls the
  function with \code{plot=TRUE}; the plot will appear on the current
  graphics device.

  Debugging messages are printed to the console if the \code{debug}
  logical flag is set to \code{TRUE}.  
}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}.  Thanks to John
  Bullock who discovered a bug in an earlier version.}
\seealso{\code{\link{pbeta}}, \code{\link{qbeta}},
  \code{\link{dbeta}}, \code{\link{uniroot}}}
\examples{
betaHPD(4,5)
betaHPD(2,120)
betaHPD(120,45,p=.75,xlim=c(0,1))
}
\keyword{distribution}% at least one, from doc/KEYWORDS

\name{politicalInformation}
\alias{politicalInformation}
\docType{data}
\title{Interviewer ratings of respondent levels of political information}
\description{
  Interviewers administering the 2000 American National Election Studies
  assigned an ordinal rating to each respondent's "general
  level of information" about politics and public affairs.
}

\usage{data(politicalInformation)}

\format{
  A data frame with 1807 observations on the following 8 variables.
  \describe{
    \item{\code{y}}{interviewer rating, a factor with levels \code{Very Low} \code{Fairly Low} \code{Average} \code{Fairly High} \code{Very High}}
    \item{\code{collegeDegree}}{a factor with levels \code{No} \code{Yes}}
    \item{\code{female}}{a factor with levels \code{No} \code{Yes}}
    \item{\code{age}}{a numeric vector, respondent age in years}
    \item{\code{homeOwn}}{a factor with levels \code{No} \code{Yes}}
    \item{\code{govt}}{a factor with levels \code{No} \code{Yes}}
    \item{\code{length}}{a numeric vector, length of ANES pre-election
      interview in minutes}
    \item{\code{id}}{a factor, unique identifier for each interviewer}
  }
}

\details{
  Seven respondents have missing data on the ordinal interviewer rating.
   The covariates \code{age} and \code{length} also have some missing data.
}

\source{
  The National Election Studies (www.electionstudies.org). THE 2000
  NATIONAL ELECTION STUDY [dataset]. Ann Arbor, MI: University of
  Michigan, Center for Political Studies [producer and distributor].  
}

\references{
  Jackman, Simon. 2009.  \emph{Bayesian Analysis for the Social
  Sciences}.  Wiley: Hoboken, New Jersey.
}

\examples{
data(politicalInformation)

table(politicalInformation$y,exclude=NULL)

op <- MASS::polr(y ~ collegeDegree + female + log(age) + homeOwn + govt + log(length),
           data=politicalInformation,
           Hess=TRUE,
           method="probit")
}
\keyword{datasets}

\name{vectorRepresentation}
\alias{vectorRepresentation}

\title{convert roll call matrix to series of vectors}

\description{
 Extract the information in a roll call matrix as a series of vectors
 with voting decision, a unique identifier for the legislator and a
 unique identifier for the roll call.
}

\usage{
vectorRepresentation(object, dropList = list(codes = c("missing", "notInLegis")))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{an object of class \code{\link{rollcall}}}
  \item{dropList}{a \code{dropList}; see \code{\link{dropRollCall}}}
}

\details{ It is often the case that roll call matrices are sparse, say,
  when the roll call matrix has an \dQuote{overlapping generations}
  structure; e.g., consider forming data by pooling across a long
  temporal sequence of legislatures such that relatively few of the
  legislators in the data set actually vote on any given roll call.  In
  such a case, representing the data as a roll call matrix is not
  particularly helpful nor efficient, either for data summaries or modeling.
}

\value{A \code{\link{matrix}} with \code{z} rows, where \code{z} is the
  number of non-missing entries in \code{object$votes}, with
  \sQuote{missingness} defined by the \code{codes} component of the
  \code{dropList}.  The matrix has 3 columns:
  \item{vote}{the voting decision, either a \code{1} if the
    corresponding element of the roll call matrix \code{object$votes}
    is in the \code{yea} component of \code{object$codes}, or a \code{0}
    if the corresponding element of the roll call matrix is in the
    \code{nay} component of \code{object$codes}.  Non-missing entries of
    the roll call matrix are not stored.}
  \item{i}{the row of the roll call matrix \code{object$votes} that
    supplied the voting decision; i.e., a unique identifier for the
    legislator generating this \code{vote}}
  \item{j}{the column of the roll call matrix \code{object$votes} that
    supplied the \code{vote}; i.e., a unique identifier for the vote.}
}
\author{Simon Jackman \email{simon.jackman@sydney.edu.au}}
\seealso{\code{\link{rollcall}}}
\examples{
data(s109)
y <- vectorRepresentation(s109)
apply(y,2,table,exclude=NULL)
}
\keyword{manip}
\keyword{utilities}

\name{presidentialElections}
\alias{presidentialElections}
\docType{data}
\title{elections for U.S. President, 1932-2016, by state}

\description{
  Democratic share of the
  presidential vote, 1932-2016, in each state and the District of Columbia.}

\usage{data(presidentialElections)}

\format{
  \itemize{
    \item{\code{state}}{character, name of state}
    \item{\item{demVote}}{numeric, percent of the vote for president won by the
      Democratic candidate}
    \item{\item{year}}{numeric, integer}
    \item{\item{south}}{logical, \code{TRUE} if state is one of the 11 states
      of the former Confederacy}
  }
}

\note{1,047 observations, unbalanced panel data in long format. Hawaii and Alaska contribute data
  from 1960 onwards the District of Columbia contributes data
  from 1964 onward; Alabama has missing data for 1948 and
  1964.}

\source{David Leip's Atlas of U.S. Presidential Elections \url{https://uselectionatlas.org}
}
  
\examples{
data(presidentialElections)
if(require(lattice)) {
    lattice::xyplot(demVote ~ year | state,
       panel=lattice::panel.lines,
       ylab="Democratic Vote for President (percent)",
       xlab="Year",
       data=presidentialElections,
       scales=list(y=list(cex=.6),x=list(cex=.35)),
       strip=strip.custom(par.strip.text=list(cex=.6)))
}

## Obama vs Kerry, except DC
y08 <- presidentialElections$year==2008
y04 <- presidentialElections$year==2004
tmpData <- merge(y=presidentialElections[y08,],
	         x=presidentialElections[y04,],
                 by="state")
tmpData <- tmpData[tmpData$state!="DC",]
xlim <- range(tmpData$demVote.x,tmpData$demVote.y)
col <- rep("black",dim(tmpData)[1])
col[tmpData$south.x] <- "red"

plot(demVote.y ~ demVote.x,
     xlab="Kerry Vote Share, 2004 (percent)",
     ylab="Obama Vote Share, 2008 (percent)",
     xlim=xlim,
     ylim=xlim,
     type="n",
     las=1,
     data=tmpData)
abline(0,1,lwd=2,col=gray(.65))
ols <- lm(demVote.y ~ demVote.x,
          data=tmpData)
abline(ols,lwd=2)	

text(tmpData$demVote.x,
     tmpData$demVote.y,
     tmpData$state,
     col=col,
     cex=.65)
legend(x="topleft",
       bty="n",
       lwd=c(2,2),
       col=c(gray(.65),"black"),
       legend=c("No Change from 2004","Regression"))
legend(x="bottomright",
       bty="n",
       text.col=c("red","black"),
       legend=c("South","Non-South"))

}
\keyword{datasets}

\name{sc9497}
\docType{data}
\alias{sc9497}
\title{votes from the United States Supreme Court, from 1994-1997}
\description{
  This data set provides information on the United States Supreme Court
  from 1994-1997.  Votes included are non-unanimous.
}

\usage{data(sc9497)}
\format{A list containing the elements:
  \describe{
    \item{votes}{a matrix of the votes, 0=Nay, 1=Yea,
    \code{NA}=Abstained or missing data.  The matrix columns are labeled with
    \code{vote.names} and the rows are labeled with \code{legis.names}.}
    \item{legis.names}{a vector of the names of the nine Justices
    sitting on the court at this time.}
    \item{party}{NULL; exists for consistency with House and Senate data sets.}
    \item{state}{NULL; exists for consistency with House and Senate data
    sets.}
    \item{district}{NULL; exists for consistency with House data sets.}
    \item{id}{NULL; exists for consistency with House and Senate data sets.}
    \item{vote.names}{a vector of strings numbering the cases simply to
      distinguish them from one another.}
    \item{desc}{a description of the data set.}
  }
}

\source{Harold J. Spaeth (1999). \emph{United States Supreme Court Judicial
Database, 1953-1997 Terms}.  Ninth edition.  Inter-university Consortium for Political and Social Research.  Ann Arbor, Michigan.  \doi{10.3886/ICPSR09422.v9}}

\keyword{datasets}

\name{predict.hurdle}
\alias{predict.hurdle}
\alias{residuals.hurdle}
\alias{terms.hurdle}
\alias{model.matrix.hurdle}
\alias{coef.hurdle}
\alias{vcov.hurdle}
\alias{summary.hurdle}
\alias{print.summary.hurdle}
\alias{logLik.hurdle}
\alias{fitted.hurdle}
\alias{predprob.hurdle}
\alias{extractAIC.hurdle}

\title{Methods for hurdle Objects}
\description{
  Methods for extracting information from fitted hurdle
  regression model objects of class \code{"hurdle"}.
}

\usage{
\method{predict}{hurdle}(object, newdata,
  type = c("response", "prob", "count", "zero"), na.action = na.pass,
  at = NULL, \dots)
\method{residuals}{hurdle}(object, type = c("pearson", "response"), \dots)

\method{coef}{hurdle}(object, model = c("full", "count", "zero"), \dots)
\method{vcov}{hurdle}(object, model = c("full", "count", "zero"), \dots)

\method{terms}{hurdle}(x, model = c("count", "zero"), \dots)
\method{model.matrix}{hurdle}(object, model = c("count", "zero"), \dots)
}

\arguments{
  \item{object, x}{an object of class \code{"hurdle"} as returned by
    \code{\link{hurdle}}.}
  \item{newdata}{optionally, a data frame in which to look for variables with
    which to predict. If omitted, the original observations are used.}
  \item{type}{character specifying the type of predictions or residuals,
    respectively. For details see below.}
  \item{na.action}{function determining what should be done with missing values
    in \code{newdata}. The default is to predict \code{NA}.}
  \item{at}{optionally, if \code{type = "prob"}, a numeric vector at which
    the probabilities are evaluated. By default \code{0:max(y)} is used
    where \code{y} is the original observed response.}
  \item{model}{character specifying for which component of the model the
    terms or model matrix should be extracted.}
  \item{\dots}{currently not used.}
}

\details{
  A set of standard extractor functions for fitted model objects is available for
  objects of class \code{"hurdle"}, including methods to the generic functions
  \code{\link[base]{print}} and \code{\link[base]{summary}} which print the estimated
  coefficients along with some further information. The \code{summary} in particular
  supplies partial Wald tests based on the coefficients and the covariance matrix
  (estimated from the Hessian in the numerical optimization of the log-likelihood).
  As usual, the \code{summary} method returns an object of class \code{"summary.hurdle"}
  containing the relevant summary statistics which can subsequently be printed
  using the associated \code{print} method.
  
  The methods for \code{\link[stats]{coef}} and \code{\link[stats]{vcov}} by default
  return a single vector of coefficients and their associated covariance matrix,
  respectively, i.e., all coefficients are concatenated. By setting the \code{model}
  argument, the estimates for the corresponding model component can be extracted.
  
  Both the \code{\link[stats]{fitted}} and \code{\link[stats]{predict}} methods can
  compute fitted responses. The latter additionally provides the predicted density
  (i.e., probabilities for the observed counts), the predicted mean from the count
  component (without zero hurdle) and the predicted ratio of probabilities for
  observing a non-zero count. The latter is the ratio of probabilities for a non-zero
  implied by the zero hurdle component and a non-zero count in the non-truncated
  count distribution. See also Appendix C in Zeileis et al. (2008).
  
  The \code{\link[stats]{residuals}} method can compute raw residuals
  (observed - fitted) and Pearson residuals (raw residuals scaled by
  square root of variance function).
  
  The \code{\link[stats]{terms}} and \code{\link[stats]{model.matrix}} extractors can
  be used to extract the relevant information for either component of the model.

  A \code{\link[stats]{logLik}} method is provided, hence \code{\link[stats]{AIC}}
  can be called to compute information criteria.
}

\references{
  Zeileis, Achim, Christian Kleiber and Simon Jackman 2008.
  \dQuote{Regression Models for Count Data in R.} 
  \emph{Journal of Statistical Software}, \bold{27}(8).
  URL \url{https://www.jstatsoft.org/v27/i08/}.
}

\author{Achim Zeileis <Achim.Zeileis@R-project.org>}

\seealso{\code{\link{hurdle}}}

\examples{
data("bioChemists", package = "pscl")
fm <- hurdle(art ~ ., data = bioChemists)

plot(residuals(fm) ~ fitted(fm))

coef(fm)
coef(fm, model = "zero")

summary(fm)
logLik(fm)
}

\keyword{regression}
